@@ -18,7 +18,7 @@
 #
 # build.sh - Unix Build Script for Apache Batik
 #
-# $Id$
+# $Id: build.sh 1372317 2012-08-13 08:42:09Z helder $
 # -----------------------------------------------------------------------------
 
 # ----- Verify and Set Required Environment Variables -------------------------
@@ -67,4 +67,4 @@ fi
 
 # $JAVA_HOME/bin/java -version
 
-"$JAVA_HOME"/bin/java $ANT_OPTS -classpath "$CP" org.apache.tools.ant.Main -emacs -Dant.home=$ANT_HOME $TARGET -Dargs="$*"
+"$JAVA_HOME/bin/java" $ANT_OPTS -classpath "$CP" org.apache.tools.ant.Main -emacs -Dant.home=$ANT_HOME $TARGET -Dargs="$*"
