@@ -1,11 +1,21 @@
-/*****************************************************************************
- * Copyright (C) The Apache Software Foundation. All rights reserved.        *
- * ------------------------------------------------------------------------- *
- * This software is published under the terms of the Apache Software License *
- * version 1.1, a copy of which has been included with this distribution in  *
- * the LICENSE file.                                                         *
- *****************************************************************************/
+/*
 
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
+
+       http://www.apache.org/licenses/LICENSE-2.0
+
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
+
+ */
 package org.apache.tools.ant.taskdefs.optional;
 
 // -- Ant classes ------------------------------------------------------------
@@ -52,7 +62,7 @@ import java.util.List;
  * @see org.apache.batik.apps.rasterizer.Main Main
  *
  * @author <a href="mailto:ruini@iki.fi">Henri Ruini</a>
- * @version $Id: RasterizerTask.java,v 1.2 2002/09/18 09:23:11 vhardy Exp $
+ * @version $Id: RasterizerTask.java 720465 2008-11-25 11:18:59Z deweese $
  */
 public class RasterizerTask extends MatchingTask {
 
@@ -252,7 +262,7 @@ public class RasterizerTask extends MatchingTask {
         this.language = language;
     }
 
-	/**
+    /**
      * Sets classname of an XML parser.
      * The attribute is optional.
      *
@@ -337,7 +347,7 @@ public class RasterizerTask extends MatchingTask {
 
         String[] sources;        // Array of input files.
 
-		// Store default XML parser information and set user class.
+        // Store default XML parser information and set user class.
         String defaultParser = XMLResourceDescriptor.getXMLParserClassName();
         // Throws BuildException.
         XMLResourceDescriptor.setXMLParserClassName(getParserClassName(readerClassName));
@@ -531,7 +541,7 @@ public class RasterizerTask extends MatchingTask {
      *
      * @return Result image type as an object or <code>null</code> if the parameter doesn't have corresponding object.
      */
-	protected DestinationType getResultType(String type) {
+    protected DestinationType getResultType(String type) {
         if(type.equals(DestinationType.PNG_STR)) {
             return DestinationType.PNG;
         } else if(type.equals(DestinationType.JPEG_STR)) {
@@ -542,7 +552,7 @@ public class RasterizerTask extends MatchingTask {
             return DestinationType.PDF;
         }
         return null;
-	}
+    }
 
     /**
      * Checks if the quality value can be set. Only result image type
@@ -673,7 +683,7 @@ public class RasterizerTask extends MatchingTask {
      */
     private String getParserClassName(final String className) {
         String name = className;
-        if(className.equals(JAXP_PARSER)) {
+        if ((className == null) || className.equals(JAXP_PARSER)) {
             // Set first JAXP parser.
             // Throws BuildException.
             XMLReader reader = JAXPUtils.getXMLReader();
@@ -696,7 +706,7 @@ public class RasterizerTask extends MatchingTask {
      * <p>See the Ant documentation for more information.</p>
      *
      * @author <a href="mailto:ruini@iki.fi">Henri Ruini</a>
-     * @version $Id: RasterizerTask.java,v 1.2 2002/09/18 09:23:11 vhardy Exp $
+     * @version $Id: RasterizerTask.java 720465 2008-11-25 11:18:59Z deweese $
      */
     public static class ValidImageTypes extends EnumeratedAttribute {
 
@@ -720,7 +730,7 @@ public class RasterizerTask extends MatchingTask {
      * <p>See the Ant documentation for more information.</p>
      *
      * @author <a href="mailto:ruini@iki.fi">Henri Ruini</a>
-     * @version $Id: RasterizerTask.java,v 1.2 2002/09/18 09:23:11 vhardy Exp $
+     * @version $Id: RasterizerTask.java 720465 2008-11-25 11:18:59Z deweese $
      */
     public static class ValidMediaTypes extends EnumeratedAttribute {
 
