@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2002  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -22,26 +23,26 @@ package org.apache.batik.apps.svgbrowser;
  * origins.
  *
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: ResourceOrigin.java,v 1.4 2004/08/18 07:12:27 vhardy Exp $
+ * @version $Id: ResourceOrigin.java 478160 2006-11-22 13:35:06Z dvholten $
  */
 public interface ResourceOrigin {
     /**
      * Any origin
      */
-    static final int ANY = 1;
+    int ANY = 1;
 
     /**
      * Same as document
      */
-    static final int DOCUMENT = 2;
+    int DOCUMENT = 2;
 
     /**
-     * Embeded into the document 
+     * Embeded into the document
      */
-    static final int EMBEDED = 4;
+    int EMBEDED = 4;
 
     /**
      * No origin is ok
      */
-    static final int NONE = 8;
+    int NONE = 8;
 }
