@@ -1,10 +1,11 @@
 /*
 
-   Copyright 1999-2003  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -45,7 +46,7 @@ import org.w3c.dom.Node;
  * in XML.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: XMLPreferenceManager.java,v 1.6 2004/10/30 18:38:04 deweese Exp $
+ * @version $Id: XMLPreferenceManager.java 662304 2008-06-02 01:37:54Z cam $
  */
 public class XMLPreferenceManager extends PreferenceManager {
     
@@ -168,7 +169,11 @@ public class XMLPreferenceManager extends PreferenceManager {
                 String v = (String)m.get(n);
                 
                 w.write("<property name=\"" + n + "\">");
-                w.write(DOMUtilities.contentToString(v));
+                try {
+                    w.write(DOMUtilities.contentToString(v, false));
+                } catch (IOException ex) {
+                    // unlikely to happen
+                }
                 w.write("</property>\n");
             }
 
