@@ -1,74 +1,55 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.bridge;
 
+import java.awt.Shape;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Rectangle2D;
+import java.lang.ref.SoftReference;
 
+import org.apache.batik.anim.dom.AnimatedLiveAttributeValue;
+import org.apache.batik.anim.dom.SVGOMAnimatedTransformList;
+import org.apache.batik.anim.dom.SVGOMElement;
 import org.apache.batik.css.engine.CSSEngineEvent;
 import org.apache.batik.css.engine.SVGCSSEngine;
+import org.apache.batik.dom.events.AbstractEvent;
+import org.apache.batik.dom.svg.AbstractSVGTransformList;
+import org.apache.batik.dom.svg.LiveAttributeException;
 import org.apache.batik.dom.svg.SVGContext;
-import org.apache.batik.dom.svg.SVGOMElement;
+import org.apache.batik.dom.svg.SVGMotionAnimatableElement;
+import org.apache.batik.ext.awt.geom.SegmentList;
 import org.apache.batik.gvt.CanvasGraphicsNode;
 import org.apache.batik.gvt.CompositeGraphicsNode;
 import org.apache.batik.gvt.GraphicsNode;
+
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
+import org.w3c.dom.events.DocumentEvent;
+import org.w3c.dom.events.EventTarget;
 import org.w3c.dom.events.MutationEvent;
 import org.w3c.dom.svg.SVGFitToViewBox;
+import org.w3c.dom.svg.SVGTransformable;
 
 /**
  * The base bridge class for SVG graphics node. By default, the namespace URI is
- * the SVG namespace. Override the <tt>getNamespaceURI</tt> if you want to add
- * custom <tt>GraphicsNode</tt> with a custom namespace.
+ * the SVG namespace. Override the <code>getNamespaceURI</code> if you want to add
+ * custom <code>GraphicsNode</code> with a custom namespace.
  *
  * <p>This class handles various attributes that are defined on most
  * of the SVG graphic elements as described in the SVG
@@ -84,18 +65,13 @@ import org.w3c.dom.svg.SVGFitToViewBox;
  * </ul>
  *
  * @author <a href="mailto:tkormann@apache.org">Thierry Kormann</a>
- * @version $Id: AbstractGraphicsNodeBridge.java,v 1.33 2003/10/06 00:56:00 deweese Exp $
+ * @version $Id: AbstractGraphicsNodeBridge.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
-public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
-    implements SVGContext, 
-               BridgeUpdateHandler, 
-               GraphicsNodeBridge, 
+public abstract class AbstractGraphicsNodeBridge extends AnimatableSVGBridge
+    implements SVGContext,
+               BridgeUpdateHandler,
+               GraphicsNodeBridge,
                ErrorConstants {
-    
-    /**
-     * The element that has been handled by this bridge.
-     */
-    protected Element e;
 
     /**
      * The graphics node constructed by this bridge.
@@ -103,9 +79,14 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
     protected GraphicsNode node;
 
     /**
-     * The bridge context to use for dynamic updates.
+     * Whether the document is an SVG 1.2 document.
      */
-    protected BridgeContext ctx;
+    protected boolean isSVG12;
+
+    /**
+     * The unit context for length conversions.
+     */
+    protected UnitProcessor.Context unitContext;
 
     /**
      * Constructs a new abstract bridge.
@@ -113,7 +94,7 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
     protected AbstractGraphicsNodeBridge() {}
 
     /**
-     * Creates a <tt>GraphicsNode</tt> according to the specified parameters.
+     * Creates a <code>GraphicsNode</code> according to the specified parameters.
      *
      * @param ctx the bridge context to use
      * @param e the element that describes the graphics node to build
@@ -126,14 +107,15 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
         }
 
         GraphicsNode node = instantiateGraphicsNode();
+
         // 'transform'
-        String s = e.getAttributeNS(null, SVG_TRANSFORM_ATTRIBUTE);
-        if (s.length() != 0) {
-            node.setTransform
-                (SVGUtilities.convertTransform(e, SVG_TRANSFORM_ATTRIBUTE, s));
-        }
+        setTransform(node, e, ctx);
+
         // 'visibility'
         node.setVisible(CSSUtilities.convertVisibility(e));
+
+        associateSVGContext(ctx, e, node);
+
         return node;
     }
 
@@ -166,9 +148,6 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
         node.setPointerEventType(CSSUtilities.convertPointerEvents(e));
 
         initializeDynamicSupport(ctx, e, node);
-
-        // Handle children elements such as <title>
-        SVGUtilities.bridgeChildren(ctx, e);
     }
 
     /**
@@ -179,27 +158,76 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
         return CSSUtilities.convertDisplay(e);
     }
 
+    /**
+     * Returns an {@link AffineTransform} that is the transformation to
+     * be applied to the node.
+     */
+    protected AffineTransform computeTransform(SVGTransformable te,
+                                               BridgeContext ctx) {
+        try {
+            AffineTransform at = new AffineTransform();
+
+            // 'transform'
+            SVGOMAnimatedTransformList atl =
+                (SVGOMAnimatedTransformList) te.getTransform();
+            if (atl.isSpecified()) {
+                atl.check();
+                AbstractSVGTransformList tl =
+                    (AbstractSVGTransformList) te.getTransform().getAnimVal();
+                at.concatenate(tl.getAffineTransform());
+            }
+
+            // motion animation
+            if (e instanceof SVGMotionAnimatableElement) {
+                SVGMotionAnimatableElement mae = (SVGMotionAnimatableElement) e;
+                AffineTransform mat = mae.getMotionTransform();
+                if (mat != null) {
+                    at.concatenate(mat);
+                }
+            }
+
+            return at;
+        } catch (LiveAttributeException ex) {
+            throw new BridgeException(ctx, ex);
+        }
+    }
+
+    /**
+     * Sets the graphics node's transform to the current animated transform
+     * value.
+     */
+    protected void setTransform(GraphicsNode n, Element e, BridgeContext ctx) {
+        n.setTransform(computeTransform((SVGTransformable) e, ctx));
+    }
+
+    /**
+     * Associates the {@link SVGContext} with the element.  This method should
+     * be called even for static documents, since some bridges will need to
+     * access animated attribute values even during the first build.
+     */
+    protected void associateSVGContext(BridgeContext ctx,
+                                       Element e,
+                                       GraphicsNode node) {
+        this.e = e;
+        this.node = node;
+        this.ctx = ctx;
+        this.unitContext = UnitProcessor.createContext(ctx, e);
+        this.isSVG12 = ctx.isSVG12();
+        ((SVGOMElement)e).setSVGContext(this);
+    }
+
     /**
      * This method is invoked during the build phase if the document
-     * is dynamic. The responsability of this method is to ensure that
+     * is dynamic. The responsibility of this method is to ensure that
      * any dynamic modifications of the element this bridge is
      * dedicated to, happen on its associated GVT product.
      */
     protected void initializeDynamicSupport(BridgeContext ctx,
                                             Element e,
                                             GraphicsNode node) {
-        if (!ctx.isInteractive())
-            return;
-
-        // Bind the nodes for interactive and dynamic
-        ctx.bind(e, node);
-
-        if (ctx.isDynamic()) {
-            // only set context for dynamic documents not interactive.
-            this.e = e;
-            this.node = node;
-            this.ctx = ctx;
-            ((SVGOMElement)e).setSVGContext(this);
+        if (ctx.isInteractive()) {
+            // Bind the nodes for interactive and dynamic.
+            ctx.bind(e, node);
         }
     }
 
@@ -209,47 +237,74 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
      * Invoked when an MutationEvent of type 'DOMAttrModified' is fired.
      */
     public void handleDOMAttrModifiedEvent(MutationEvent evt) {
-        String attrName = evt.getAttrName();
-        if (attrName.equals(SVG_TRANSFORM_ATTRIBUTE)) {
-            String s = evt.getNewValue();
-            AffineTransform at = GraphicsNode.IDENTITY;
-            if (s.length() != 0) {
-                at = SVGUtilities.convertTransform
-                    (e, SVG_TRANSFORM_ATTRIBUTE, s);
+    }
+
+    /**
+     * Invoked when the geometry of a graphical element has changed.
+     */
+    protected void handleGeometryChanged() {
+        node.setFilter(CSSUtilities.convertFilter(e, node, ctx));
+        node.setMask(CSSUtilities.convertMask(e, node, ctx));
+        node.setClip(CSSUtilities.convertClipPath(e, node, ctx));
+        if (isSVG12) {
+            if (!SVG_USE_TAG.equals(e.getLocalName())) {
+                // ShapeChange events get fired only for basic shapes and paths.
+                fireShapeChangeEvent();
             }
-            node.setTransform(at);
-            handleGeometryChanged();
+            fireBBoxChangeEvent();
         }
     }
 
     /**
-     * Invoked when the geometry of an graphical element has changed.
+     * Fires a ShapeChange event on the element this bridge is managing.
      */
-    protected  void handleGeometryChanged() {
-        node.setFilter(CSSUtilities.convertFilter(e, node, ctx));
-        node.setMask(CSSUtilities.convertMask(e, node, ctx));
-        node.setClip(CSSUtilities.convertClipPath(e, node, ctx));
+    protected void fireShapeChangeEvent() {
+        DocumentEvent d = (DocumentEvent) e.getOwnerDocument();
+        AbstractEvent evt = (AbstractEvent) d.createEvent("SVGEvents");
+        evt.initEventNS(SVG_NAMESPACE_URI,
+                        "shapechange",
+                        true,
+                        false);
+        try {
+            ((EventTarget) e).dispatchEvent(evt);
+        } catch (RuntimeException ex) {
+            ctx.getUserAgent().displayError(ex);
+        }
     }
 
     /**
      * Invoked when an MutationEvent of type 'DOMNodeInserted' is fired.
      */
     public void handleDOMNodeInsertedEvent(MutationEvent evt) {
-        // never called. The global listener on the document will
-        // invoke this method on the parent element.
+        if (evt.getTarget() instanceof Element) {
+            // Handle "generic" bridges.
+            Element e2 = (Element)evt.getTarget();
+            Bridge b = ctx.getBridge(e2);
+            if (b instanceof GenericBridge) {
+                ((GenericBridge) b).handleElement(ctx, e2);
+            }
+        }
     }
 
     /**
      * Invoked when an MutationEvent of type 'DOMNodeRemoved' is fired.
      */
     public void handleDOMNodeRemovedEvent(MutationEvent evt) {
+        Node parent = e.getParentNode();
+        if (parent instanceof SVGOMElement) {
+            SVGContext bridge = ((SVGOMElement) parent).getSVGContext();
+            if (bridge instanceof SVGSwitchElementBridge) {
+                ((SVGSwitchElementBridge) bridge).handleChildElementRemoved(e);
+                return;
+            }
+        }
         CompositeGraphicsNode gn = node.getParent();
         gn.remove(node);
         disposeTree(e);
     }
 
     /**
-     * Invoked when an MutationEvent of type 'DOMCharacterDataModified' 
+     * Invoked when an MutationEvent of type 'DOMCharacterDataModified'
      * is fired.
      */
     public void handleDOMCharacterDataModified(MutationEvent evt) {
@@ -262,21 +317,35 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
         SVGOMElement elt = (SVGOMElement)e;
         elt.setSVGContext(null);
         ctx.unbind(e);
+
+        bboxShape = null;
     }
 
+    /**
+     * Disposes all resources related to the specified node and its subtree.
+     */
+    protected static void disposeTree(Node node) {
+        disposeTree(node, true);
+    }
 
     /**
-     * Disposes all resources related to the specified node and its subtree
+     * Disposes all resources related to the specified node and its subtree,
+     * and optionally removes the nodes' {@link SVGContext}.
      */
-    static void disposeTree(Node node) {
+    protected static void disposeTree(Node node, boolean removeContext) {
         if (node instanceof SVGOMElement) {
             SVGOMElement elt = (SVGOMElement)node;
-            BridgeUpdateHandler h = (BridgeUpdateHandler)elt.getSVGContext();
-            if (h != null)
+            SVGContext ctx = elt.getSVGContext();
+            if (ctx instanceof BridgeUpdateHandler) {
+                BridgeUpdateHandler h = (BridgeUpdateHandler) ctx;
+                if (removeContext) {
+                    elt.setSVGContext(null);
+                }
                 h.dispose();
+            }
         }
-        for (Node n = node.getFirstChild(); n!=null; n = n.getNextSibling()) {
-            disposeTree(n);
+        for (Node n = node.getFirstChild(); n != null; n = n.getNextSibling()) {
+            disposeTree(n, removeContext);
         }
     }
 
@@ -285,9 +354,13 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
      */
     public void handleCSSEngineEvent(CSSEngineEvent evt) {
         try {
-            int [] properties = evt.getProperties();
-            for (int i=0; i < properties.length; ++i) {
-                handleCSSPropertyChanged(properties[i]);
+            SVGCSSEngine eng = (SVGCSSEngine) evt.getSource();
+            int[] properties = evt.getProperties();
+            for (int i = 0; i < properties.length; i++) {
+                int idx = properties[i];
+                handleCSSPropertyChanged(idx);
+                String pn = eng.getPropertyName(idx);
+                fireBaseAttributeListeners(pn);
             }
         } catch (Exception ex) {
             ctx.getUserAgent().displayError(ex);
@@ -321,14 +394,72 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
             if (!getDisplay(e)) {
                 // Remove the subtree.
                 CompositeGraphicsNode parent = node.getParent();
-                int idx = parent.indexOf(node);
                 parent.remove(node);
-                disposeTree(e);
+                disposeTree(e, false);
             }
             break;
         }
     }
 
+    /**
+     * Invoked when the animated value of an animatable attribute has changed.
+     */
+    public void handleAnimatedAttributeChanged
+            (AnimatedLiveAttributeValue alav) {
+        if (alav.getNamespaceURI() == null
+                && alav.getLocalName().equals(SVG_TRANSFORM_ATTRIBUTE)) {
+            setTransform(node, e, ctx);
+            handleGeometryChanged();
+        }
+    }
+
+    /**
+     * Invoked when an 'other' animation value has changed.
+     */
+    public void handleOtherAnimationChanged(String type) {
+        if (type.equals("motion")) {
+            setTransform(node, e, ctx);
+            handleGeometryChanged();
+        }
+    }
+
+    /**
+     * Checks if the bounding box of the node has changed, and if so,
+     * fires a bboxchange event on the element.
+     */
+    protected void checkBBoxChange() {
+        if (e != null) {
+            /*Rectangle2D oldBBox = bbox;
+            Rectangle2D newBBox = getBBox();
+            if (oldBBox != newBBox && newBBox != null) {
+                if (oldBBox == null ||
+                        oldBBox.getX() != bbox.getX()
+                        || oldBBox.getY() != bbox.getY()
+                        || oldBBox.getWidth() != bbox.getWidth()
+                        || oldBBox.getHeight() != bbox.getHeight()) {*/
+                    fireBBoxChangeEvent();
+                /*}
+            }*/
+        }
+    }
+
+    /**
+     * Fires an svg:bboxchange event on the element.
+     */
+    protected void fireBBoxChangeEvent() {
+        DocumentEvent d = (DocumentEvent) e.getOwnerDocument();
+        AbstractEvent evt = (AbstractEvent) d.createEvent("SVGEvents");
+        evt.initEventNS(SVG_NAMESPACE_URI,
+                        "RenderedBBoxChange",
+                        true,
+                        false);
+        try {
+            ((EventTarget) e).dispatchEvent(evt);
+        } catch (RuntimeException ex) {
+            ctx.getUserAgent().displayError(ex);
+        }
+    }
+
     // SVGContext implementation ///////////////////////////////////////////
 
     /**
@@ -345,9 +476,11 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
      */
     public float getPixelToMM() {
         return getPixelUnitToMillimeter();
-            
     }
 
+    protected SoftReference bboxShape = null;
+    protected Rectangle2D bbox = null;
+
     /**
      * Returns the tight bounding box in current user space (i.e.,
      * after application of the transform attribute, if any) on the
@@ -355,7 +488,20 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
      * stroke-width and filter effects).
      */
     public Rectangle2D getBBox() {
-        return node.getPrimitiveBounds();
+        if (node == null) {
+            return null;
+        }
+        Shape s = node.getOutline();
+
+        if ((bboxShape != null) && (s == bboxShape.get())) return bbox;
+        bboxShape = new SoftReference(s); // don't keep this live.
+        bbox = null;
+        if (s == null) return bbox;
+
+        // SegmentList.getBounds2D gives tight BBox.
+        SegmentList sl = new SegmentList(s);
+        bbox = sl.getBounds2D();
+        return bbox;
     }
 
     /**
@@ -399,7 +545,7 @@ public abstract class AbstractGraphicsNodeBridge extends AbstractSVGBridge
     }
 
     /**
-     * Returns the display transform.
+     * Sets the display transform.
      */
     public void setScreenTransform(AffineTransform at) {
         ctx.getUserAgent().setTransform(at);
