@@ -1,72 +1,52 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.bridge;
 
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.InputStreamReader;
+import java.io.PushbackInputStream;
 import java.io.Reader;
 import java.io.StringReader;
+import java.io.UnsupportedEncodingException;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.util.HashMap;
 import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
 import java.util.Set;
+import java.util.WeakHashMap;
 import java.util.jar.Manifest;
 
-import org.apache.batik.dom.svg.XMLBaseSupport;
+import org.apache.batik.dom.AbstractElement;
+import org.apache.batik.dom.events.AbstractEvent;
+import org.apache.batik.dom.events.NodeEventTarget;
 import org.apache.batik.dom.util.XLinkSupport;
 import org.apache.batik.script.Interpreter;
 import org.apache.batik.script.InterpreterException;
-import org.apache.batik.script.ScriptHandler;
+import org.apache.batik.script.ScriptEventWrapper;
 import org.apache.batik.util.ParsedURL;
 import org.apache.batik.util.SVGConstants;
+import org.apache.batik.util.XMLConstants;
+
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
@@ -83,23 +63,27 @@ import org.w3c.dom.svg.EventListenerInitializer;
  * This class is the base class for SVG scripting.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: BaseScriptingEnvironment.java,v 1.28 2004/02/19 20:32:07 vhardy Exp $
+ * @version $Id: BaseScriptingEnvironment.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class BaseScriptingEnvironment {
     /**
-     * Constant used to describe inline scripts
+     * Constant used to describe inline scripts.
+     * <pre>
      * {0} - URL of document containing script.
      * {1} - Element tag
      * {2} - line number of element.
+     * </pre>
      */
     public static final String INLINE_SCRIPT_DESCRIPTION
         = "BaseScriptingEnvironment.constant.inline.script.description";
 
     /**
-     * Constant used to describe inline scripts
+     * Constant used to describe inline scripts.
+     * <pre>
      * {0} - URL of document containing script.
      * {1} - Event attribute name
      * {2} - line number of element.
+     * </pre>
      */
     public static final String EVENT_SCRIPT_DESCRIPTION
         = "BaseScriptingEnvironment.constant.event.script.description";
@@ -107,7 +91,7 @@ public class BaseScriptingEnvironment {
     /**
      * Tells whether the given SVG document is dynamic.
      */
-    public static boolean isDynamicDocument(Document doc) {
+    public static boolean isDynamicDocument(BridgeContext ctx, Document doc) {
         Element elt = doc.getDocumentElement();
         if ((elt != null) &&
             SVGConstants.SVG_NAMESPACE_URI.equals(elt.getNamespaceURI())) {
@@ -135,23 +119,29 @@ public class BaseScriptingEnvironment {
                 (null, SVGConstants.SVG_ONZOOM_ATTRIBUTE).length() > 0) {
                 return true;
             }
-            return isDynamicElement(doc.getDocumentElement());
+            return isDynamicElement(ctx, doc.getDocumentElement());
         }
         return false;
     }
 
+    public static boolean isDynamicElement(BridgeContext ctx, Element elt) {
+        List bridgeExtensions = ctx.getBridgeExtensions(elt.getOwnerDocument());
+        return isDynamicElement(elt, ctx, bridgeExtensions);
+    }
+
     /**
      * Tells whether the given SVG element is dynamic.
      */
-    public static boolean isDynamicElement(Element elt) {
+    public static boolean isDynamicElement
+        (Element elt, BridgeContext ctx, List bridgeExtensions) {
+        Iterator i = bridgeExtensions.iterator();
+        while (i.hasNext()) {
+            BridgeExtension bridgeExtension = (BridgeExtension) i.next();
+            if (bridgeExtension.isDynamicElement(elt)) {
+                return true;
+            }
+        }
         if (SVGConstants.SVG_NAMESPACE_URI.equals(elt.getNamespaceURI())) {
-            String name = elt.getLocalName();
-            if (name.equals(SVGConstants.SVG_SCRIPT_TAG)) {
-                return true;
-            }
-            if (name.startsWith("animate") || name.equals("set")) {
-                return true;
-            }
             if (elt.getAttributeNS
                 (null, SVGConstants.SVG_ONKEYUP_ATTRIBUTE).length() > 0) {
                 return true;
@@ -208,14 +198,14 @@ public class BaseScriptingEnvironment {
                 (null, SVGConstants.SVG_ONMOUSEUP_ATTRIBUTE).length() > 0) {
                 return true;
             }
+        }
 
-            for (Node n = elt.getFirstChild();
-                 n != null;
-                 n = n.getNextSibling()) {
-                if (n.getNodeType() == Node.ELEMENT_NODE) {
-                    if (isDynamicElement((Element)n)) {
-                        return true;
-                    }
+        for (Node n = elt.getFirstChild();
+             n != null;
+             n = n.getNextSibling()) {
+            if (n.getNodeType() == Node.ELEMENT_NODE) {
+                if (isDynamicElement(ctx, (Element)n)) {
+                    return true;
                 }
             }
         }
@@ -223,8 +213,14 @@ public class BaseScriptingEnvironment {
     }
 
 
-    protected final static String EVENT_NAME = "event";
-    protected final static String ALTERNATE_EVENT_NAME = "evt";
+    protected static final String EVENT_NAME = "event";
+    protected static final String ALTERNATE_EVENT_NAME = "evt";
+
+    /**
+     * The 'application/ecmascript' MIME type.
+     */
+    protected static final String APPLICATION_ECMASCRIPT =
+        "application/ecmascript";
 
     /**
      * The bridge context.
@@ -253,6 +249,17 @@ public class BaseScriptingEnvironment {
      */
     protected Interpreter interpreter;
 
+    /**
+     * Map of {@link Interpreter} to {@link org.apache.batik.bridge.Window}
+     * objects.
+     */
+    protected Map windowObjects = new HashMap();
+
+    /**
+     * Set of &lt;script> elements that have already been executed.
+     */
+    protected WeakHashMap executedScripts = new WeakHashMap();
+
     /**
      * Creates a new BaseScriptingEnvironment.
      * @param ctx the bridge context
@@ -265,18 +272,35 @@ public class BaseScriptingEnvironment {
     }
 
     /**
-     * Creates a new Window object.
+     * Returns the Window object for the specified {@link Interpreter}.
      */
-    public org.apache.batik.script.Window createWindow
-        (Interpreter interp, String lang) {
-        return new Window(interp, lang);
+    public org.apache.batik.bridge.Window getWindow(Interpreter interp,
+                                                    String lang) {
+        org.apache.batik.bridge.Window w =
+            (org.apache.batik.bridge.Window) windowObjects.get(interp);
+        if (w == null) {
+            w = interp == null ? new Window(null, null)
+                               : createWindow(interp, lang);
+            windowObjects.put(interp, w);
+        }
+        return w;
+    }
+
+    /**
+     * Returns the Window object for scripting languages that have no
+     * {@link Interpreter} object.
+     */
+    public org.apache.batik.bridge.Window getWindow() {
+        return getWindow(null, null);
     }
 
     /**
      * Creates a new Window object.
      */
-    public org.apache.batik.script.Window createWindow() {
-        return createWindow(null, null);
+    protected org.apache.batik.bridge.Window createWindow(Interpreter interp,
+                                                          String lang) {
+        
+        return new Window(interp, lang);
     }
 
     /**
@@ -294,14 +318,11 @@ public class BaseScriptingEnvironment {
     public Interpreter getInterpreter(String lang) {
         interpreter = bridgeContext.getInterpreter(lang);
         if (interpreter == null) {
-            if (languages.contains(lang))
-                // Already issued warning so just return null;
+            if (languages.contains(lang)) {
+                // Already issued warning so just return null.
                 return null;
-
-            UserAgent ua = bridgeContext.getUserAgent();
-            if (ua != null) {
-                ua.displayError(new Exception("Unknown language: " + lang));
             }
+
             // So we know we have processed this interpreter.
             languages.add(lang);
             return null;
@@ -318,160 +339,240 @@ public class BaseScriptingEnvironment {
      * Initializes the environment of the given interpreter.
      */
     public void initializeEnvironment(Interpreter interp, String lang) {
-        interp.bindObject("window", createWindow(interp, lang));
+        interp.bindObject("window", getWindow(interp, lang));
     }
 
     /**
-     * Loads the scripts contained in the <script> elements.
+     * Loads the scripts contained in the &lt;script> elements.
      */
     public void loadScripts() {
-        org.apache.batik.script.Window window = null;
-
         NodeList scripts = document.getElementsByTagNameNS
             (SVGConstants.SVG_NAMESPACE_URI, SVGConstants.SVG_SCRIPT_TAG);
-        int len = scripts.getLength();
 
-        if (len == 0) {
+        int len = scripts.getLength();
+        for (int i = 0; i < len; i++) {
+            AbstractElement script = (AbstractElement) scripts.item(i);
+            loadScript(script);
+        }
+    }
+
+    /**
+     * Executes the specified &lt;script> element, if it hasn't been
+     * executed already.
+     */
+    protected void loadScript(AbstractElement script) {
+        // Don't execute <script> elements more than once.
+        if (executedScripts.containsKey(script)) {
             return;
         }
 
-        for (int i = 0; i < len; i++) {
-            Element script = (Element)scripts.item(i);
-            String type = script.getAttributeNS
-                (null, SVGConstants.SVG_TYPE_ATTRIBUTE);
-
-            if (type.length() == 0) {
-                type = SVGConstants.SVG_SCRIPT_TYPE_DEFAULT_VALUE;
-            }
-
-            //
-            // Java code invocation.
-            //
-            if (type.equals(SVGConstants.SVG_SCRIPT_TYPE_JAVA)) {
-                try {
-                    String href = XLinkSupport.getXLinkHref(script);
-                    ParsedURL purl = new ParsedURL
-                        (XMLBaseSupport.getCascadedXMLBase(script), href);
-
-                    checkCompatibleScriptURL(type, purl);
-
-                    DocumentJarClassLoader cll;
-                    URL docURL = null;
-                    try {
-                        docURL = new URL(docPURL.toString());
-                    } catch (MalformedURLException mue) {
-                        /* nothing just let docURL be null */
-                    }
-                    cll = new DocumentJarClassLoader
-                        (new URL(purl.toString()), docURL);
-
-                    // Get the 'Script-Handler' entry in the manifest.
-                    URL url = cll.findResource("META-INF/MANIFEST.MF");
-                    if (url == null) {
-                        continue;
-                    }
-                    Manifest man = new Manifest(url.openStream());
-
-                    String sh;
-
-                    sh = man.getMainAttributes().getValue("Script-Handler");
-                    if (sh != null) {
-                        // Run the script handler.
-                        ScriptHandler h;
-                        h = (ScriptHandler)cll.loadClass(sh).newInstance();
-
-                        if (window == null) {
-                            window = createWindow();
-                        }
-
-                        h.run(document, window);
-                    }
-
-                    sh = man.getMainAttributes().getValue("SVG-Handler-Class");
-                    if (sh != null) {
-                        // Run the initializer
-                        EventListenerInitializer initializer;
-                        initializer =
-                            (EventListenerInitializer)cll.loadClass(sh).newInstance();
-
-                        if (window == null) {
-                            window = createWindow();
-                        }
-
-                        initializer.initializeEventListeners((SVGDocument)document);
-                    }
-                } catch (Exception e) {
-                    if (userAgent != null) {
-                        userAgent.displayError(e);
-                    }
+        // Don't execute a <script> element that has been removed from
+        // the document.
+        {
+            Node n = script;
+            do {
+                n = n.getParentNode();
+                if (n == null) {
+                    return;
                 }
-                continue;
-            }
+            } while (n.getNodeType() != Node.DOCUMENT_NODE);
+        }
 
-            //
-            // Scripting language invocation.
-            //
-            Interpreter interpreter = getInterpreter(type);
-            if (interpreter == null)
-                // Can't find interpreter so just skip this script block.
-                continue;
+        String type = script.getAttributeNS
+            (null, SVGConstants.SVG_TYPE_ATTRIBUTE);
 
+        if (type.length() == 0) {
+            type = SVGConstants.SVG_SCRIPT_TYPE_DEFAULT_VALUE;
+        }
+
+        //
+        // Java code invocation.
+        //
+        if (type.equals(SVGConstants.SVG_SCRIPT_TYPE_JAVA)) {
             try {
                 String href = XLinkSupport.getXLinkHref(script);
-                String desc = null;
-                Reader reader;
+                ParsedURL purl = new ParsedURL(script.getBaseURI(), href);
 
-                if (href.length() > 0) {
-                    desc = href;
+                checkCompatibleScriptURL(type, purl);
 
-                    // External script.
-                    ParsedURL purl = new ParsedURL
-                        (XMLBaseSupport.getCascadedXMLBase(script), href);
+                DocumentJarClassLoader cll;
+                URL docURL = null;
+                try {
+                    docURL = new URL(docPURL.toString());
+                } catch (MalformedURLException mue) {
+                    /* nothing just let docURL be null */
+                }
+                cll = new DocumentJarClassLoader
+                    (new URL(purl.toString()), docURL);
 
-                    checkCompatibleScriptURL(type, purl);
-                    reader = new InputStreamReader(purl.openStream());
-                } else {
-                    checkCompatibleScriptURL(type, docPURL);
-                    DocumentLoader dl = bridgeContext.getDocumentLoader();
-                    Element e = script;
-                    SVGDocument d = (SVGDocument)e.getOwnerDocument();
-                    int line = dl.getLineNumber(script);
-                    desc = Messages.formatMessage
-                        (INLINE_SCRIPT_DESCRIPTION,
-                         new Object [] {d.getURL(),
-                                        "<"+script.getNodeName()+">", 
-                                        new Integer(line)});
-                    // Inline script.
-                    Node n = script.getFirstChild();
-                    if (n != null) {
-                        StringBuffer sb = new StringBuffer();
-                        while (n != null) {
-                            if (n.getNodeType() == Node.CDATA_SECTION_NODE
-                                || n.getNodeType() == Node.TEXT_NODE)
-                                sb.append(n.getNodeValue());
-                            n = n.getNextSibling();
-                        }
-                        reader = new StringReader(sb.toString());
-                    } else {
-                        continue;
+                // Get the 'Script-Handler' entry in the manifest.
+                URL url = cll.findResource("META-INF/MANIFEST.MF");
+                if (url == null) {
+                    return;
+                }
+                Manifest man = new Manifest(url.openStream());
+
+                String sh;
+
+                executedScripts.put(script, null);
+
+                sh = man.getMainAttributes().getValue("Script-Handler");
+                if (sh != null) {
+                    // Run the script handler.
+                    ScriptHandler h;
+                    h = (ScriptHandler)cll.loadClass(sh).newInstance();
+
+                    h.run(document, getWindow());
+                }
+
+                sh = man.getMainAttributes().getValue("SVG-Handler-Class");
+                if (sh != null) {
+                    // Run the initializer
+                    EventListenerInitializer initializer;
+                    initializer =
+                        (EventListenerInitializer)cll.loadClass(sh).newInstance();
+
+                    getWindow();
+
+                    initializer.initializeEventListeners((SVGDocument)document);
+                }
+
+            } catch (Exception e) {
+                if (userAgent != null) {
+                    userAgent.displayError(e);
+                }
+            }
+            return;
+        }
+
+        //
+        // Scripting language invocation.
+        //
+        Interpreter interpreter = getInterpreter(type);
+        if (interpreter == null) {
+            // Can't find interpreter so just skip this script block.
+            return;
+        }
+
+        try {
+            String href = XLinkSupport.getXLinkHref(script);
+            String desc = null;
+            Reader reader = null;
+
+            if (href.length() > 0) {
+                desc = href;
+
+                // External script.
+                ParsedURL purl = new ParsedURL(script.getBaseURI(), href);
+
+                checkCompatibleScriptURL(type, purl);
+                InputStream is = purl.openStream();
+                String mediaType = purl.getContentTypeMediaType();
+                String enc = purl.getContentTypeCharset();
+                if (enc != null) {
+                    try {
+                        reader = new InputStreamReader(is, enc);
+                    } catch (UnsupportedEncodingException uee) {
+                        enc = null;
                     }
                 }
+                if (reader == null) {
+                    if (APPLICATION_ECMASCRIPT.equals(mediaType)) {
+                        // No encoding was specified in the MIME type, so
+                        // infer it according to RFC 4329.
+                        if (purl.hasContentTypeParameter("version")) {
+                            // Future versions of application/ecmascript 
+                            // are not supported, so skip this script 
+                            // element if the version parameter is present.
+                            return;
+                        }
 
-                interpreter.evaluate(reader, desc);
+                        PushbackInputStream pbis =
+                            new PushbackInputStream(is, 8);
+                        byte[] buf = new byte[4];
+                        int read = pbis.read(buf);
+                        if (read > 0) {
+                            pbis.unread(buf, 0, read);
+                            if (read >= 2) {
+                                if (buf[0] == (byte)0xff &&
+                                        buf[1] == (byte)0xfe) {
+                                    if (read >= 4 && buf[2] == 0 &&
+                                            buf[3] == 0) {
+                                        enc = "UTF32-LE";
+                                        pbis.skip(4);
+                                    } else {
+                                        enc = "UTF-16LE";
+                                        pbis.skip(2);
+                                    }
+                                } else if (buf[0] == (byte)0xfe &&
+                                        buf[1] == (byte)0xff) {
+                                    enc = "UTF-16BE";
+                                    pbis.skip(2);
+                                } else if (read >= 3
+                                        && buf[0] == (byte)0xef 
+                                        && buf[1] == (byte)0xbb
+                                        && buf[2] == (byte)0xbf) {
+                                    enc = "UTF-8";
+                                    pbis.skip(3);
+                                } else if (read >= 4 && buf[0] == 0 &&
+                                        buf[1] == 0 &&
+                                        buf[2] == (byte)0xfe &&
+                                        buf[3] == (byte)0xff) {
+                                    enc = "UTF-32BE";
+                                    pbis.skip(4);
+                                }
+                            }
+                            if (enc == null) {
+                                enc = "UTF-8";
+                            }
+                        }
+                        reader = new InputStreamReader(pbis, enc);
+                    } else {
+                        reader = new InputStreamReader(is);
+                    }
+                }
+            } else {
+                checkCompatibleScriptURL(type, docPURL);
+                DocumentLoader dl = bridgeContext.getDocumentLoader();
+                Element e = script;
+                SVGDocument d = (SVGDocument)e.getOwnerDocument();
+                int line = dl.getLineNumber(script);
+                desc = Messages.formatMessage
+                    (INLINE_SCRIPT_DESCRIPTION,
+                     new Object [] {d.getURL(),
+                                    "<"+script.getNodeName()+">",
+                                    new Integer(line)});
+                // Inline script.
+                Node n = script.getFirstChild();
+                if (n != null) {
+                    StringBuffer sb = new StringBuffer();
+                    while (n != null) {
+                        if (n.getNodeType() == Node.CDATA_SECTION_NODE
+                            || n.getNodeType() == Node.TEXT_NODE)
+                            sb.append(n.getNodeValue());
+                        n = n.getNextSibling();
+                    }
+                    reader = new StringReader(sb.toString());
+                } else {
+                    return;
+                }
+            }
 
-            } catch (IOException e) {
-                if (userAgent != null) {
-                    userAgent.displayError(e);
-                }
-                return;
-            } catch (InterpreterException e) {
-                System.err.println("InterpExcept: " + e);
-                handleInterpreterException(e);
-                return;
-            } catch (SecurityException e) {
-                if (userAgent != null) {
-                    userAgent.displayError(e);
-                }
+            executedScripts.put(script, null);
+            interpreter.evaluate(reader, desc);
+
+        } catch (IOException e) {
+            if (userAgent != null) {
+                userAgent.displayError(e);
+            }
+            return;
+        } catch (InterpreterException e) {
+            System.err.println("InterpExcept: " + e);
+            handleInterpreterException(e);
+            return;
+        } catch (SecurityException e) {
+            if (userAgent != null) {
+                userAgent.displayError(e);
             }
         }
     }
@@ -492,6 +593,8 @@ public class BaseScriptingEnvironment {
     public void dispatchSVGLoadEvent() {
         SVGSVGElement root = (SVGSVGElement)document.getDocumentElement();
         String lang = root.getContentScriptType();
+        long documentStartTime = System.currentTimeMillis();
+        bridgeContext.getAnimationEngine().start(documentStartTime);
         dispatchSVGLoad(root, true, lang);
     }
 
@@ -509,11 +612,19 @@ public class BaseScriptingEnvironment {
             }
         }
 
-        Event ev;
         DocumentEvent de = (DocumentEvent)elt.getOwnerDocument();
-        ev = de.createEvent("SVGEvents");
-        ev.initEvent("SVGLoad", false, false);
-        EventTarget t = (EventTarget)elt;
+        AbstractEvent ev = (AbstractEvent) de.createEvent("SVGEvents");
+        String type;
+        if (bridgeContext.isSVG12()) {
+            type = "load";
+        } else {
+            type = "SVGLoad";
+        }
+        ev.initEventNS(XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                       type,
+                       false,
+                       false);
+        NodeEventTarget t = (NodeEventTarget)elt;
 
         final String s =
             elt.getAttributeNS(null, SVGConstants.SVG_ONLOAD_ATTRIBUTE);
@@ -544,14 +655,20 @@ public class BaseScriptingEnvironment {
         final String desc = Messages.formatMessage
             (EVENT_SCRIPT_DESCRIPTION,
              new Object [] {d.getURL(),
-                            SVGConstants.SVG_ONLOAD_ATTRIBUTE, 
+                            SVGConstants.SVG_ONLOAD_ATTRIBUTE,
                             new Integer(line)});
 
         EventListener l = new EventListener() {
                 public void handleEvent(Event evt) {
                     try {
-                        interp.bindObject(EVENT_NAME, evt);
-                        interp.bindObject(ALTERNATE_EVENT_NAME, evt);
+                        Object event;
+                        if (evt instanceof ScriptEventWrapper) {
+                            event = ((ScriptEventWrapper) evt).getEventObject();
+                        } else {
+                            event = evt;
+                        }
+                        interp.bindObject(EVENT_NAME, event);
+                        interp.bindObject(ALTERNATE_EVENT_NAME, event);
                         interp.evaluate(new StringReader(s), desc);
                     } catch (IOException io) {
                     } catch (InterpreterException e) {
@@ -559,30 +676,46 @@ public class BaseScriptingEnvironment {
                     }
                 }
             };
-        t.addEventListener("SVGLoad", l, false);
+        t.addEventListenerNS
+            (XMLConstants.XML_EVENTS_NAMESPACE_URI, type,
+             l, false, null);
         t.dispatchEvent(ev);
-        t.removeEventListener("SVGLoad", l, false);
+        t.removeEventListenerNS
+            (XMLConstants.XML_EVENTS_NAMESPACE_URI, type,
+             l, false);
     }
 
     /**
      * Method to dispatch SVG Zoom event.
      */
     protected void dispatchSVGZoomEvent() {
-        dispatchSVGDocEvent("SVGZoom");
+        if (bridgeContext.isSVG12()) {
+            dispatchSVGDocEvent("zoom");
+        } else {
+            dispatchSVGDocEvent("SVGZoom");
+        }
     }
 
     /**
      * Method to dispatch SVG Scroll event.
      */
     protected void dispatchSVGScrollEvent() {
-        dispatchSVGDocEvent("SVGScroll");
+        if (bridgeContext.isSVG12()) {
+            dispatchSVGDocEvent("scroll");
+        } else {
+            dispatchSVGDocEvent("SVGScroll");
+        }
     }
 
     /**
      * Method to dispatch SVG Resize event.
      */
     protected void dispatchSVGResizeEvent() {
-        dispatchSVGDocEvent("SVGResize");
+        if (bridgeContext.isSVG12()) {
+            dispatchSVGDocEvent("resize");
+        } else {
+            dispatchSVGDocEvent("SVGResize");
+        }
     }
 
     protected void dispatchSVGDocEvent(String eventType) {
@@ -592,8 +725,11 @@ public class BaseScriptingEnvironment {
         EventTarget t = root;
 
         DocumentEvent de = (DocumentEvent)document;
-        Event ev = de.createEvent("SVGEvents");
-        ev.initEvent(eventType, false, false);
+        AbstractEvent ev = (AbstractEvent) de.createEvent("SVGEvents");
+        ev.initEventNS(XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                       eventType,
+                       false,
+                       false);
         t.dispatchEvent(ev);
     }
 
@@ -619,7 +755,7 @@ public class BaseScriptingEnvironment {
     /**
      * Represents the window object of this environment.
      */
-    protected class Window implements org.apache.batik.script.Window {
+    protected class Window implements org.apache.batik.bridge.Window {
 
         /**
          * The associated interpreter.
@@ -641,7 +777,7 @@ public class BaseScriptingEnvironment {
 
         /**
          * Implements {@link
-         * org.apache.batik.script.Window#setInterval(String,long)}.
+         * org.apache.batik.bridge.Window#setInterval(String,long)}.
          */
         public Object setInterval(final String script, long interval) {
             return null;
@@ -649,7 +785,7 @@ public class BaseScriptingEnvironment {
 
         /**
          * Implements {@link
-         * org.apache.batik.script.Window#setInterval(Runnable,long)}.
+         * org.apache.batik.bridge.Window#setInterval(Runnable,long)}.
          */
         public Object setInterval(final Runnable r, long interval) {
             return null;
@@ -657,14 +793,14 @@ public class BaseScriptingEnvironment {
 
         /**
          * Implements {@link
-         * org.apache.batik.script.Window#clearInterval(Object)}.
+         * org.apache.batik.bridge.Window#clearInterval(Object)}.
          */
         public void clearInterval(Object interval) {
         }
 
         /**
          * Implements {@link
-         * org.apache.batik.script.Window#setTimeout(String,long)}.
+         * org.apache.batik.bridge.Window#setTimeout(String,long)}.
          */
         public Object setTimeout(final String script, long timeout) {
             return null;
@@ -672,7 +808,7 @@ public class BaseScriptingEnvironment {
 
         /**
          * Implements {@link
-         * org.apache.batik.script.Window#setTimeout(Runnable,long)}.
+         * org.apache.batik.bridge.Window#setTimeout(Runnable,long)}.
          */
         public Object setTimeout(final Runnable r, long timeout) {
             return null;
@@ -680,7 +816,7 @@ public class BaseScriptingEnvironment {
 
         /**
          * Implements {@link
-         * org.apache.batik.script.Window#clearTimeout(Object)}.
+         * org.apache.batik.bridge.Window#clearTimeout(Object)}.
          */
         public void clearTimeout(Object timeout) {
         }
@@ -695,12 +831,20 @@ public class BaseScriptingEnvironment {
             return null;
         }
 
+        /**
+         * Serializes the given node.
+         */
+        public String printNode(Node n) {
+            return null;
+        }
+
         /**
          * Gets data from the given URI.
          * @param uri The URI where the data is located.
          * @param h A handler called when the data is available.
          */
-        public void getURL(String uri, org.apache.batik.script.Window.GetURLHandler h) {
+        public void getURL(String uri, org.apache.batik.bridge.Window.URLResponseHandler h) {
+            getURL(uri, h, "UTF8");
         }
 
         /**
@@ -710,10 +854,30 @@ public class BaseScriptingEnvironment {
          * @param enc The character encoding of the data.
          */
         public void getURL(String uri,
-                           org.apache.batik.script.Window.GetURLHandler h,
+                           org.apache.batik.bridge.Window.URLResponseHandler h,
                            String enc) {
         }
 
+        public void postURL(String uri, String content,
+                            org.apache.batik.bridge.Window.URLResponseHandler h) {
+            postURL(uri, content, h, "text/plain", null);
+        }
+
+        public void postURL(String uri, String content,
+                            org.apache.batik.bridge.Window.URLResponseHandler h,
+                     String mimeType) {
+            postURL(uri, content, h, mimeType, null);
+        }
+
+        public void postURL(String uri,
+                            String content,
+                            org.apache.batik.bridge.Window.URLResponseHandler h,
+                            String mimeType,
+                            String fEnc) {
+        }
+
+
+
         /**
          * Displays an alert dialog box.
          */
@@ -755,5 +919,19 @@ public class BaseScriptingEnvironment {
             return interpreter;
         }
 
+        /**
+         * Returns the Location.
+         */
+        public org.w3c.dom.Location getLocation() {
+            return null;
+        }
+
+        /**
+         * Returns the parent Window object.
+         */
+        public org.w3c.dom.Window getParent() {
+            return null;
+        }
+
     }
 }
