@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.bridge;
 
 import java.awt.Point;
@@ -55,40 +23,45 @@ import java.awt.event.KeyEvent;
 import java.awt.geom.NoninvertibleTransformException;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
+import java.lang.ref.SoftReference;
 import java.text.AttributedCharacterIterator;
 import java.util.List;
 
 import org.apache.batik.dom.events.DOMKeyEvent;
+import org.apache.batik.dom.events.DOMMouseEvent;
+import org.apache.batik.dom.events.NodeEventTarget;
+import org.apache.batik.dom.util.DOMUtilities;
 import org.apache.batik.gvt.GraphicsNode;
-import org.apache.batik.gvt.TextNode;
 import org.apache.batik.gvt.event.EventDispatcher;
 import org.apache.batik.gvt.event.GraphicsNodeKeyEvent;
 import org.apache.batik.gvt.event.GraphicsNodeKeyListener;
 import org.apache.batik.gvt.event.GraphicsNodeMouseEvent;
 import org.apache.batik.gvt.event.GraphicsNodeMouseListener;
-import org.apache.batik.gvt.renderer.StrokingTextPainter;
 import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
-import org.apache.batik.gvt.text.TextHit;
-import org.apache.batik.gvt.text.TextSpanLayout;
 import org.apache.batik.util.SVGConstants;
+import org.apache.batik.util.XMLConstants;
+
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.events.DocumentEvent;
 import org.w3c.dom.events.Event;
 import org.w3c.dom.events.EventListener;
 import org.w3c.dom.events.EventTarget;
-import org.w3c.dom.events.MouseEvent;
 
 /**
  * This class is responsible of tracking GraphicsNodeMouseEvent and
  * fowarding them to the DOM as regular DOM MouseEvent.
  *
- * @author <a href="mailto:tkormann@ilog.fr>Thierry Kormann</a>
- * @version $Id: BridgeEventSupport.java,v 1.53 2004/02/05 02:53:44 deweese Exp $
+ * @author <a href="mailto:tkormann@ilog.fr">Thierry Kormann</a>
+ * @version $Id: BridgeEventSupport.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
-public class BridgeEventSupport implements SVGConstants {
+public abstract class BridgeEventSupport implements SVGConstants {
 
-    private BridgeEventSupport() {}
+    public static final
+        AttributedCharacterIterator.Attribute TEXT_COMPOUND_ID =
+        GVTAttributedCharacterIterator.TextAttribute.TEXT_COMPOUND_ID;
+
+    protected BridgeEventSupport() {}
 
     /**
      * Is called only for the root element in order to dispatch GVT
@@ -105,19 +78,49 @@ public class BridgeEventSupport implements SVGConstants {
                 // add an unload listener on the SVGDocument to remove
                 // that listener for dispatching events
                 EventListener l = new GVTUnloadListener(dispatcher, listener);
-                EventTarget target = (EventTarget)doc;
-                target.addEventListener("SVGUnload", l, false);
-                ctx.storeEventListener(target, "SVGUnload", l, false);
+                NodeEventTarget target = (NodeEventTarget)doc;
+                target.addEventListenerNS
+                    (XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                     "SVGUnload",
+                     l, false, null);
+                storeEventListenerNS
+                    (ctx, target,
+                     XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                     "SVGUnload",
+                     l, false);
             }
         }
     }
 
+    /**
+     * Calls storeEventListener on the given BridgeContext.
+     */
+    protected static void storeEventListener(BridgeContext ctx,
+                                             EventTarget e,
+                                             String t,
+                                             EventListener l,
+                                             boolean c) {
+        ctx.storeEventListener(e, t, l, c);
+    }
+
+    /**
+     * Calls storeEventListenerNS on the given BridgeContext.
+     */
+    protected static void storeEventListenerNS(BridgeContext ctx,
+                                               EventTarget e,
+                                               String n,
+                                               String t,
+                                               EventListener l,
+                                               boolean c) {
+        ctx.storeEventListenerNS(e, n, t, l, c);
+    }
+
     protected static class GVTUnloadListener implements EventListener {
 
         protected EventDispatcher dispatcher;
         protected Listener listener;
 
-        public GVTUnloadListener(EventDispatcher dispatcher, 
+        public GVTUnloadListener(EventDispatcher dispatcher,
                                  Listener listener) {
             this.dispatcher = dispatcher;
             this.listener = listener;
@@ -126,16 +129,19 @@ public class BridgeEventSupport implements SVGConstants {
         public void handleEvent(Event evt) {
             dispatcher.removeGraphicsNodeMouseListener(listener);
             dispatcher.removeGraphicsNodeKeyListener(listener);
-            evt.getTarget().removeEventListener("SVGUnload", this, false);
+            NodeEventTarget et = (NodeEventTarget) evt.getTarget();
+            et.removeEventListenerNS
+                (XMLConstants.XML_EVENTS_NAMESPACE_URI, "SVGUnload",
+                 this, false);
         }
     }
 
     /**
      * A GraphicsNodeMouseListener that dispatch DOM events accordingly.
      */
-    protected static class Listener 
-        implements GraphicsNodeMouseListener, GraphicsNodeKeyListener {
-        
+    protected static class Listener implements GraphicsNodeMouseListener,
+                                               GraphicsNodeKeyListener {
+
         protected BridgeContext context;
         protected UserAgent ua;
         protected Element lastTargetElement;
@@ -153,10 +159,16 @@ public class BridgeEventSupport implements SVGConstants {
          * @param evt the graphics node key event
          */
         public void keyPressed(GraphicsNodeKeyEvent evt) {
+            // XXX isDown is not preventing key repeats
             if (!isDown) {
                 isDown = true;
                 dispatchKeyEvent("keydown", evt);
             }
+            if (evt.getKeyChar() == KeyEvent.CHAR_UNDEFINED) {
+                // We will not get a KEY_TYPED event for this char
+                // so generate a keypress event here.
+                dispatchKeyEvent("keypress", evt);
+            }
         }
 
         /**
@@ -176,25 +188,28 @@ public class BridgeEventSupport implements SVGConstants {
             dispatchKeyEvent("keypress", evt);
         }
 
-        protected void dispatchKeyEvent(String eventType, 
+        /**
+         * Dispatch a DOM 2 Draft Key event.
+         */
+        protected void dispatchKeyEvent(String eventType,
                                         GraphicsNodeKeyEvent evt) {
             FocusManager fmgr = context.getFocusManager();
             if (fmgr == null) return;
 
             Element targetElement = (Element)fmgr.getCurrentEventTarget();
             if (targetElement == null) {
-                return;
+                targetElement = context.getDocument().getDocumentElement();
             }
             DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
             DOMKeyEvent keyEvt = (DOMKeyEvent)d.createEvent("KeyEvents");
-            keyEvt.initKeyEvent(eventType, 
-                                true, 
-                                true, 
-                                evt.isControlDown(), 
+            keyEvt.initKeyEvent(eventType,
+                                true,
+                                true,
+                                evt.isControlDown(),
                                 evt.isAltDown(),
-                                evt.isShiftDown(), 
+                                evt.isShiftDown(),
                                 evt.isMetaDown(),
-                                mapKeyCode(evt.getKeyCode()), 
+                                mapKeyCode(evt.getKeyCode()),
                                 evt.getKeyChar(),
                                 null);
 
@@ -214,7 +229,7 @@ public class BridgeEventSupport implements SVGConstants {
         protected final int mapKeyCode(int keyCode) {
             switch (keyCode) {
                 case KeyEvent.VK_ENTER:
-                    return DOMKeyEvent.DOM_VK_ENTER; 
+                    return DOMKeyEvent.DOM_VK_ENTER;
             case KeyEvent.VK_KANA_LOCK:
                 return DOMKeyEvent.DOM_VK_UNDEFINED;
             case KeyEvent.VK_INPUT_METHOD_ON_OFF:
@@ -239,21 +254,31 @@ public class BridgeEventSupport implements SVGConstants {
         }
 
         public void mouseEntered(GraphicsNodeMouseEvent evt) {
-            dispatchMouseEvent("mouseover", evt, true);
+            Point clientXY = evt.getClientPoint();
+            GraphicsNode node = evt.getGraphicsNode();
+            Element targetElement = getEventTarget(node, evt.getPoint2D());
+            Element relatedElement = getRelatedElement(evt);
+            dispatchMouseEvent("mouseover",
+                               targetElement,
+                               relatedElement,
+                               clientXY,
+                               evt,
+                               true);
         }
 
         public void mouseExited(GraphicsNodeMouseEvent evt) {
             Point clientXY = evt.getClientPoint();
             // Get the 'new' node for the DOM event.
             GraphicsNode node = evt.getRelatedNode();
-            Element targetElement = getEventTarget(node, clientXY);
+            Element targetElement = getEventTarget(node, evt.getPoint2D());
             if (lastTargetElement != null) {
-                dispatchMouseEvent("mouseout", 
+                dispatchMouseEvent("mouseout",
                                    lastTargetElement, // target
                                    targetElement,     // relatedTarget
                                    clientXY,
                                    evt,
                                    true);
+                lastTargetElement = null;
             }
         }
 
@@ -264,11 +289,11 @@ public class BridgeEventSupport implements SVGConstants {
         public void mouseMoved(GraphicsNodeMouseEvent evt) {
             Point clientXY = evt.getClientPoint();
             GraphicsNode node = evt.getGraphicsNode();
-            Element targetElement = getEventTarget(node, clientXY);
+            Element targetElement = getEventTarget(node, evt.getPoint2D());
             Element holdLTE = lastTargetElement;
             if (holdLTE != targetElement) {
                 if (holdLTE != null) {
-                    dispatchMouseEvent("mouseout", 
+                    dispatchMouseEvent("mouseout",
                                        holdLTE, // target
                                        targetElement,     // relatedTarget
                                        clientXY,
@@ -276,7 +301,7 @@ public class BridgeEventSupport implements SVGConstants {
                                        true);
                 }
                 if (targetElement != null) {
-                    dispatchMouseEvent("mouseover", 
+                    dispatchMouseEvent("mouseover",
                                        targetElement,     // target
                                        holdLTE, // relatedTarget
                                        clientXY,
@@ -284,7 +309,7 @@ public class BridgeEventSupport implements SVGConstants {
                                        true);
                 }
             }
-            dispatchMouseEvent("mousemove", 
+            dispatchMouseEvent("mousemove",
                                targetElement,     // target
                                null,              // relatedTarget
                                clientXY,
@@ -305,14 +330,13 @@ public class BridgeEventSupport implements SVGConstants {
                                           boolean cancelable) {
             Point clientXY = evt.getClientPoint();
             GraphicsNode node = evt.getGraphicsNode();
-            Element targetElement = getEventTarget
-                (node, new Point2D.Float(evt.getX(), evt.getY()));
+            Element targetElement = getEventTarget(node, evt.getPoint2D());
             Element relatedElement = getRelatedElement(evt);
-            dispatchMouseEvent(eventType, 
+            dispatchMouseEvent(eventType,
                                targetElement,
                                relatedElement,
-                               clientXY, 
-                               evt, 
+                               clientXY,
+                               evt,
                                cancelable);
         }
 
@@ -349,26 +373,27 @@ public class BridgeEventSupport implements SVGConstants {
 
             }
             */
-            short button = getButton(evt);
             Point screenXY = evt.getScreenPoint();
             // create the coresponding DOM MouseEvent
             DocumentEvent d = (DocumentEvent)targetElement.getOwnerDocument();
-            MouseEvent mouseEvt = (MouseEvent)d.createEvent("MouseEvents");
-            mouseEvt.initMouseEvent(eventType, 
-                                    true, 
-                                    cancelable, 
-                                    null,
-                                    evt.getClickCount(),
-                                    screenXY.x, 
-                                    screenXY.y,
-                                    clientXY.x,
-                                    clientXY.y,
-                                    evt.isControlDown(), 
-                                    evt.isAltDown(),
-                                    evt.isShiftDown(), 
-                                    evt.isMetaDown(),
-                                    button, 
-                                    (EventTarget)relatedElement);
+            DOMMouseEvent mouseEvt
+                = (DOMMouseEvent)d.createEvent("MouseEvents");
+            String modifiers
+                = DOMUtilities.getModifiersList(evt.getLockState(),
+                                                evt.getModifiers());
+            mouseEvt.initMouseEventNS(XMLConstants.XML_EVENTS_NAMESPACE_URI,
+                                      eventType,
+                                      true,
+                                      cancelable,
+                                      null,
+                                      evt.getClickCount(),
+                                      screenXY.x,
+                                      screenXY.y,
+                                      clientXY.x,
+                                      clientXY.y,
+                                      (short) (evt.getButton() - 1),
+                                      (EventTarget)relatedElement,
+                                      modifiers);
 
             try {
                 ((EventTarget)targetElement).dispatchEvent(mouseEvt);
@@ -393,21 +418,6 @@ public class BridgeEventSupport implements SVGConstants {
             return relatedElement;
         }
 
-        /**
-         * Returns the mouse event button.
-         *
-         * @param evt the GVT GraphicsNodeMouseEvent
-         */
-        protected short getButton(GraphicsNodeMouseEvent evt) {
-            short button = 1;
-            if ((GraphicsNodeMouseEvent.BUTTON1_MASK & evt.getModifiers()) != 0) {
-                button = 0;
-            } else if ((GraphicsNodeMouseEvent.BUTTON3_MASK & evt.getModifiers()) != 0) {
-                button = 2;
-            }
-            return button;
-        }
-
         /**
          * Returns the element that is the target of the specified
          * event or null if any.
@@ -415,34 +425,29 @@ public class BridgeEventSupport implements SVGConstants {
          * @param node the graphics node that received the event
          * @param coords the mouse coordinates in the GVT tree space
          */
-        protected Element getEventTarget(GraphicsNode node, Point2D coords) {
+        protected Element getEventTarget(GraphicsNode node, Point2D pt) {
             Element target = context.getElement(node);
             // Lookup inside the text element children to see if the target
             // is a tspan or textPath
-
             if (target != null && node instanceof TextNode) {
-		TextNode textNode = (TextNode)node;
-		List list = textNode.getTextRuns();
-                Point2D pt = (Point2D)coords.clone();
-                // place coords in text node coordinate system
-                try {
-                    node.getGlobalTransform().createInverse().transform(pt, pt);
-                } catch (NoninvertibleTransformException ex) {
-                }
+                TextNode textNode = (TextNode)node;
+                List list = textNode.getTextRuns();
                 if (list != null){
+                    float x = (float)pt.getX();
+                    float y = (float)pt.getY();
                     for (int i = 0 ; i < list.size(); i++) {
                         StrokingTextPainter.TextRun run =
                             (StrokingTextPainter.TextRun)list.get(i);
                         AttributedCharacterIterator aci = run.getACI();
                         TextSpanLayout layout = run.getLayout();
-                        float x = (float)pt.getX();
-                        float y = (float)pt.getY();
                         TextHit textHit = layout.hitTestChar(x, y);
                         Rectangle2D bounds = layout.getBounds2D();
-                        if ((textHit != null) && 
+                        if ((textHit != null) &&
                             (bounds != null) && bounds.contains(x, y)) {
-                            Object delimiter = aci.getAttribute
-                                (GVTAttributedCharacterIterator.TextAttribute.TEXT_COMPOUND_DELIMITER);
+                            SoftReference sr;
+                            sr =(SoftReference)aci.getAttribute
+                                (TEXT_COMPOUND_ID);
+                            Object delimiter = sr.get();
                             if (delimiter instanceof Element) {
                                 return (Element)delimiter;
                             }
