@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.bridge;
 
 import java.awt.AlphaComposite;
@@ -58,13 +26,12 @@ import java.awt.RenderingHints;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Rectangle2D;
 
+import org.apache.batik.anim.dom.SVGOMDocument;
 import org.apache.batik.css.engine.CSSEngine;
 import org.apache.batik.css.engine.CSSStylableElement;
 import org.apache.batik.css.engine.SVGCSSEngine;
 import org.apache.batik.css.engine.value.ListValue;
 import org.apache.batik.css.engine.value.Value;
-import org.apache.batik.css.engine.value.svg.ICCColor;
-import org.apache.batik.dom.svg.SVGOMDocument;
 import org.apache.batik.ext.awt.MultipleGradientPaint;
 import org.apache.batik.ext.awt.image.renderable.ClipRable;
 import org.apache.batik.ext.awt.image.renderable.Filter;
@@ -83,7 +50,7 @@ import org.w3c.dom.css.CSSValue;
  * concrete objects regarding to CSS properties.
  *
  * @author <a href="mailto:tkormann@apache.org">Thierry Kormann</a>
- * @version $Id: CSSUtilities.java,v 1.49 2003/08/09 16:58:37 deweese Exp $
+ * @version $Id: CSSUtilities.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public abstract class CSSUtilities
     implements CSSConstants, ErrorConstants, XMLConstants {
@@ -111,7 +78,7 @@ public abstract class CSSUtilities
     public static Value getComputedStyle(Element e, int property) {
         CSSEngine engine = getCSSEngine(e);
         if (engine == null) return null;
-        return engine.getComputedStyle((CSSStylableElement)e, 
+        return engine.getComputedStyle((CSSStylableElement)e,
                                        null, property);
     }
 
@@ -148,7 +115,8 @@ public abstract class CSSUtilities
                 case 's':
                     return GraphicsNode.VISIBLE_STROKE;
                 default:
-                    throw new Error(); // can't be reached
+                    // can't be reached
+                    throw new IllegalStateException("unexpected event, must be one of (p,f,s) is:" + s.charAt(7) );
                 }
             }
         case 'p':
@@ -162,7 +130,8 @@ public abstract class CSSUtilities
         case 'n':
             return GraphicsNode.NONE;
         default:
-            throw new Error(); // can't be reached
+            // can't be reached
+            throw new IllegalStateException("unexpected event, must be one of (v,p,f,s,a,n) is:" + s.charAt(0) );
         }
     }
 
@@ -195,7 +164,7 @@ public abstract class CSSUtilities
             return new Rectangle2D.Float(x, y, w, h);
 
         default:
-            throw new InternalError(); // Cannot happen
+            throw new IllegalStateException("Unexpected length:" + length ); // Cannot happen
         }
     }
 
@@ -242,8 +211,8 @@ public abstract class CSSUtilities
      * Checks if the cursor property on the input element is set to auto
      */
     public static boolean isAutoCursor(Element e) {
-        Value cursorValue = 
-            CSSUtilities.getComputedStyle(e, 
+        Value cursorValue =
+            CSSUtilities.getComputedStyle(e,
                                           SVGCSSEngine.CURSOR_INDEX);
 
         boolean isAuto = false;
@@ -262,9 +231,9 @@ public abstract class CSSUtilities
                        cursorValue.getLength() == 1) {
                 Value lValue = cursorValue.item(0);
                 if (lValue != null
-                    && 
+                    &&
                     lValue.getCssValueType() == CSSValue.CSS_PRIMITIVE_VALUE
-                    && 
+                    &&
                     lValue.getPrimitiveType() == CSSPrimitiveValue.CSS_IDENT
                     &&
                     lValue.getStringValue().charAt(0) == 'a') {
@@ -330,14 +299,16 @@ public abstract class CSSUtilities
      */
     public static RenderingHints convertShapeRendering(Element e,
                                                        RenderingHints hints) {
-        Value v = getComputedStyle(e, SVGCSSEngine.SHAPE_RENDERING_INDEX);
+        Value  v = getComputedStyle(e, SVGCSSEngine.SHAPE_RENDERING_INDEX);
         String s = v.getStringValue();
-        if (s.charAt(0) == 'a') { // auto
+        int    len = s.length();
+        if ((len == 4) && (s.charAt(0) == 'a')) // auto
             return hints;
-        }
-        if (hints == null) {
+        if (len < 10) return hints;  // Unknown.
+
+        if (hints == null)
             hints = new RenderingHints(null);
-        }
+
         switch(s.charAt(0)) {
         case 'o': // optimizeSpeed
             hints.put(RenderingHints.KEY_RENDERING,
@@ -414,12 +385,14 @@ public abstract class CSSUtilities
                                                       RenderingHints hints) {
         Value v = getComputedStyle(e, SVGCSSEngine.TEXT_RENDERING_INDEX);
         String s = v.getStringValue();
-        if (s.charAt(0) == 'a') { // auto
+        int    len = s.length();
+        if ((len == 4) && (s.charAt(0) == 'a')) // auto
             return hints;
-        }
-        if (hints == null) {
+        if (len < 13) return hints;  // Unknown.
+
+        if (hints == null)
             hints = new RenderingHints(null);
-        }
+
         switch(s.charAt(8)) {
         case 's': // optimizeSpeed
             hints.put(RenderingHints.KEY_RENDERING,
@@ -490,12 +463,14 @@ public abstract class CSSUtilities
                                                        RenderingHints hints) {
         Value v = getComputedStyle(e, SVGCSSEngine.IMAGE_RENDERING_INDEX);
         String s = v.getStringValue();
-        if (s.charAt(0) == 'a') { // auto
+        int    len = s.length();
+        if ((len == 4) && (s.charAt(0) == 'a')) // auto
             return hints;
-        }
-        if (hints == null) {
+        if (len < 13) return hints;  // Unknown.
+
+        if (hints == null)
             hints = new RenderingHints(null);
-        }
+
         switch(s.charAt(8)) {
         case 's': // optimizeSpeed
             hints.put(RenderingHints.KEY_RENDERING,
@@ -546,12 +521,14 @@ public abstract class CSSUtilities
                                                        RenderingHints hints) {
         Value v = getComputedStyle(e, SVGCSSEngine.COLOR_RENDERING_INDEX);
         String s = v.getStringValue();
-        if (s.charAt(0) == 'a') { // auto
+        int    len = s.length();
+        if ((len == 4) && (s.charAt(0) == 'a')) // auto
             return hints;
-        }
-        if (hints == null) {
+        if (len < 13) return hints;  // Unknown.
+
+        if (hints == null)
             hints = new RenderingHints(null);
-        }
+
         switch(s.charAt(8)) {
         case 's': // optimizeSpeed
             hints.put(RenderingHints.KEY_COLOR_RENDERING,
@@ -580,6 +557,9 @@ public abstract class CSSUtilities
      * @param e the element
      */
     public static boolean convertDisplay(Element e) {
+        if (!(e instanceof CSSStylableElement)) {
+            return true;
+        }
         Value v = getComputedStyle(e, SVGCSSEngine.DISPLAY_INDEX);
         return v.getStringValue().charAt(0) != 'n';
     }
@@ -603,7 +583,7 @@ public abstract class CSSUtilities
     // 'opacity'
     /////////////////////////////////////////////////////////////////////////
 
-    public final static Composite TRANSPARENT =
+    public static final Composite TRANSPARENT =
         AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0);
 
     /**
@@ -617,7 +597,7 @@ public abstract class CSSUtilities
         float f = v.getFloatValue();
         if (f <= 0f) {
             return TRANSPARENT;
-        } else if (f >= 1f) {
+        } else if (f >= 1.0f) {
             return AlphaComposite.SrcOver;
         } else {
             return AlphaComposite.getInstance(AlphaComposite.SRC_OVER, f);
@@ -651,7 +631,8 @@ public abstract class CSSUtilities
      */
     public static float[] convertClip(Element e) {
         Value v = getComputedStyle(e, SVGCSSEngine.CLIP_INDEX);
-        switch (v.getPrimitiveType()) {
+        int primitiveType = v.getPrimitiveType();
+        switch ( primitiveType ) {
         case CSSPrimitiveValue.CSS_RECT:
             float [] off = new float[4];
             off[0] = v.getTop().getFloatValue();
@@ -662,7 +643,8 @@ public abstract class CSSUtilities
         case CSSPrimitiveValue.CSS_IDENT:
             return null; // 'auto' means no offsets
         default:
-            throw new Error(); // can't be reached
+            // can't be reached
+            throw new IllegalStateException("Unexpected primitiveType:" + primitiveType );
         }
     }
 
@@ -671,7 +653,7 @@ public abstract class CSSUtilities
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Returns a <tt>Filter</tt> referenced by the specified element
+     * Returns a <code>Filter</code> referenced by the specified element
      * and which applies on the specified graphics node.
      * Handle the 'filter' property.
      *
@@ -684,7 +666,8 @@ public abstract class CSSUtilities
                                        GraphicsNode filteredNode,
                                        BridgeContext ctx) {
         Value v = getComputedStyle(filteredElement, SVGCSSEngine.FILTER_INDEX);
-        switch (v.getPrimitiveType()) {
+        int primitiveType = v.getPrimitiveType();
+        switch ( primitiveType ) {
         case CSSPrimitiveValue.CSS_IDENT:
             return null; // 'filter:none'
 
@@ -693,7 +676,7 @@ public abstract class CSSUtilities
             Element filter = ctx.getReferencedElement(filteredElement, uri);
             Bridge bridge = ctx.getBridge(filter);
             if (bridge == null || !(bridge instanceof FilterBridge)) {
-                throw new BridgeException(filteredElement,
+                throw new BridgeException(ctx, filteredElement,
                                           ERR_CSS_URI_BAD_TARGET,
                                           new Object[] {uri});
             }
@@ -702,8 +685,8 @@ public abstract class CSSUtilities
                                                        filteredElement,
                                                        filteredNode);
         default:
-            throw new InternalError(); // can't be reached
-            
+            throw new IllegalStateException("Unexpected primitive type:" + primitiveType ); // can't be reached
+
         }
     }
 
@@ -712,7 +695,7 @@ public abstract class CSSUtilities
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Returns a <tt>Clip</tt> referenced by the specified element and
+     * Returns a <code>Clip</code> referenced by the specified element and
      * which applies on the specified graphics node.
      * Handle the 'clip-path' property.
      *
@@ -725,7 +708,8 @@ public abstract class CSSUtilities
                                             BridgeContext ctx) {
         Value v = getComputedStyle(clippedElement,
                                    SVGCSSEngine.CLIP_PATH_INDEX);
-        switch (v.getPrimitiveType()) {
+        int primitiveType = v.getPrimitiveType();
+        switch ( primitiveType ) {
         case CSSPrimitiveValue.CSS_IDENT:
             return null; // 'clip-path:none'
 
@@ -734,7 +718,7 @@ public abstract class CSSUtilities
             Element cp = ctx.getReferencedElement(clippedElement, uri);
             Bridge bridge = ctx.getBridge(cp);
             if (bridge == null || !(bridge instanceof ClipBridge)) {
-                throw new BridgeException(clippedElement,
+                throw new BridgeException(ctx, clippedElement,
                                           ERR_CSS_URI_BAD_TARGET,
                                           new Object[] {uri});
             }
@@ -743,8 +727,7 @@ public abstract class CSSUtilities
                                                    clippedElement,
                                                    clippedNode);
         default:
-            throw new InternalError(); // can't be reached
-            
+            throw new IllegalStateException("Unexpected primitive type:" + primitiveType ); // can't be reached
         }
     }
 
@@ -766,7 +749,7 @@ public abstract class CSSUtilities
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Returns a <tt>Mask</tt> referenced by the specified element and
+     * Returns a <code>Mask</code> referenced by the specified element and
      * which applies on the specified graphics node.
      * Handle the 'mask' property.
      *
@@ -778,7 +761,8 @@ public abstract class CSSUtilities
                                    GraphicsNode maskedNode,
                                    BridgeContext ctx) {
         Value v = getComputedStyle(maskedElement, SVGCSSEngine.MASK_INDEX);
-        switch (v.getPrimitiveType()) {
+        int primitiveType = v.getPrimitiveType();
+        switch ( primitiveType ) {
         case CSSPrimitiveValue.CSS_IDENT:
             return null; // 'mask:none'
 
@@ -787,7 +771,7 @@ public abstract class CSSUtilities
             Element m = ctx.getReferencedElement(maskedElement, uri);
             Bridge bridge = ctx.getBridge(m);
             if (bridge == null || !(bridge instanceof MaskBridge)) {
-                throw new BridgeException(maskedElement,
+                throw new BridgeException(ctx, maskedElement,
                                           ERR_CSS_URI_BAD_TARGET,
                                           new Object[] {uri});
             }
@@ -796,8 +780,7 @@ public abstract class CSSUtilities
                                                    maskedElement,
                                                    maskedNode);
         default:
-            throw new InternalError(); // can't be reached
-            
+            throw new IllegalStateException("Unexpected primitive type:" + primitiveType ); // can't be reached
         }
     }
 
@@ -820,7 +803,7 @@ public abstract class CSSUtilities
 
     /**
      * Converts the color defined on the specified lighting filter element
-     * to a <tt>Color</tt>.
+     * to a <code>Color</code>.
      *
      * @param e the lighting filter element
      * @param ctx the bridge context
@@ -831,7 +814,7 @@ public abstract class CSSUtilities
             return PaintServer.convertColor(v, 1);
         } else {
             return PaintServer.convertRGBICCColor
-                (e, v.item(0), (ICCColor)v.item(1), 1, ctx);
+                (e, v.item(0), v.item(1), 1, ctx);
         }
     }
 
@@ -841,7 +824,7 @@ public abstract class CSSUtilities
 
     /**
      * Converts the color defined on the specified &lt;feFlood>
-     * element to a <tt>Color</tt>.
+     * element to a <code>Color</code>.
      *
      * @param e the feFlood element
      * @param ctx the bridge context
@@ -854,7 +837,7 @@ public abstract class CSSUtilities
             return PaintServer.convertColor(v, f);
         } else {
             return PaintServer.convertRGBICCColor
-                (e, v.item(0), (ICCColor)v.item(1), f, ctx);
+                (e, v.item(0), v.item(1), f, ctx);
         }
     }
 
@@ -864,7 +847,7 @@ public abstract class CSSUtilities
 
     /**
      * Converts the color defined on the specified &lt;stop> element
-     * to a <tt>Color</tt>.
+     * to a <code>Color</code>.
      *
      * @param e the stop element
      * @param opacity the paint opacity
@@ -880,7 +863,7 @@ public abstract class CSSUtilities
             return PaintServer.convertColor(v, opacity);
         } else {
             return PaintServer.convertRGBICCColor
-                (e, v.item(0), (ICCColor)v.item(1), opacity, ctx);
+                (e, v.item(0), v.item(1), opacity, ctx);
         }
     }
 
@@ -900,12 +883,12 @@ public abstract class CSSUtilities
     public static void computeStyleAndURIs(Element refElement,
                                            Element localRefElement,
                                            String  uri) {
-	// Pull fragement id off first...
+        // Pull fragement id off first...
         int idx = uri.indexOf('#');
         if (idx != -1)
             uri = uri.substring(0,idx);
 
-	// Only set xml:base if we have a real URL.
+        // Only set xml:base if we have a real URL.
         if (uri.length() != 0)
             localRefElement.setAttributeNS(XML_NAMESPACE_URI,
                                            "base",
@@ -913,7 +896,7 @@ public abstract class CSSUtilities
 
         CSSEngine engine    = CSSUtilities.getCSSEngine(localRefElement);
         CSSEngine refEngine = CSSUtilities.getCSSEngine(refElement);
-        
+
         engine.importCascadedStyleMaps(refElement, refEngine, localRefElement);
     }
 
