@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.bridge;
 
 import java.awt.BasicStroke;
@@ -55,12 +23,15 @@ import java.awt.Color;
 import java.awt.Paint;
 import java.awt.Shape;
 import java.awt.Stroke;
+import java.awt.color.ICC_Profile;
+import java.io.IOException;
 
 import org.apache.batik.css.engine.SVGCSSEngine;
-import org.apache.batik.css.engine.value.ListValue;
 import org.apache.batik.css.engine.value.Value;
 import org.apache.batik.css.engine.value.svg.ICCColor;
-import org.apache.batik.ext.awt.color.ICCColorSpaceExt;
+import org.apache.batik.css.engine.value.svg12.CIELabColor;
+import org.apache.batik.css.engine.value.svg12.DeviceColor;
+import org.apache.batik.css.engine.value.svg12.ICCNamedColor;
 import org.apache.batik.gvt.CompositeShapePainter;
 import org.apache.batik.gvt.FillShapePainter;
 import org.apache.batik.gvt.GraphicsNode;
@@ -71,18 +42,28 @@ import org.apache.batik.gvt.ShapePainter;
 import org.apache.batik.gvt.StrokeShapePainter;
 import org.apache.batik.util.CSSConstants;
 import org.apache.batik.util.SVGConstants;
+
+import org.apache.xmlgraphics.java2d.color.CIELabColorSpace;
+import org.apache.xmlgraphics.java2d.color.ColorSpaces;
+import org.apache.xmlgraphics.java2d.color.ColorWithAlternatives;
+import org.apache.xmlgraphics.java2d.color.DeviceCMYKColorSpace;
+import org.apache.xmlgraphics.java2d.color.ICCColorSpaceWithIntent;
+import org.apache.xmlgraphics.java2d.color.NamedColorSpace;
+import org.apache.xmlgraphics.java2d.color.profile.NamedColorProfile;
+import org.apache.xmlgraphics.java2d.color.profile.NamedColorProfileParser;
+
 import org.w3c.dom.Element;
 import org.w3c.dom.css.CSSPrimitiveValue;
 import org.w3c.dom.css.CSSValue;
 
 /**
- * A collection of utility methods to deliver <tt>java.awt.Paint</tt>,
- * <tt>java.awt.Stroke</tt> objects that could be used to paint a
+ * A collection of utility methods to deliver <code>java.awt.Paint</code>,
+ * <code>java.awt.Stroke</code> objects that could be used to paint a
  * shape. This class also provides additional methods the deliver SVG
  * Paint using the ShapePainter interface.
  *
  * @author <a href="mailto:tkormann@apache.org">Thierry Kormann</a>
- * @version $Id: PaintServer.java,v 1.13 2003/12/18 01:53:49 deweese Exp $
+ * @version $Id: PaintServer.java 1617286 2014-08-11 14:03:51Z lbernardo $
  */
 public abstract class PaintServer
     implements SVGConstants, CSSConstants, ErrorConstants {
@@ -98,7 +79,7 @@ public abstract class PaintServer
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Returns a <tt>ShapePainter</tt> defined on the specified
+     * Returns a <code>ShapePainter</code> defined on the specified
      * element and for the specified shape node.
      *
      * @param e the element with the marker CSS properties
@@ -132,7 +113,7 @@ public abstract class PaintServer
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Returns a <tt>Marker</tt> defined on the specified element by
+     * Returns a <code>Marker</code> defined on the specified element by
      * the specified value, and for the specified shape node.
      *
      * @param e the painted element
@@ -150,7 +131,7 @@ public abstract class PaintServer
             Element markerElement = ctx.getReferencedElement(e, uri);
             Bridge bridge = ctx.getBridge(markerElement);
             if (bridge == null || !(bridge instanceof MarkerBridge)) {
-                throw new BridgeException(e, ERR_CSS_URI_BAD_TARGET,
+                throw new BridgeException(ctx, e, ERR_CSS_URI_BAD_TARGET,
                                           new Object[] {uri});
             }
             return ((MarkerBridge)bridge).createMarker(ctx, markerElement, e);
@@ -162,7 +143,7 @@ public abstract class PaintServer
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Returns a <tt>ShapePainter</tt> defined on the specified element and
+     * Returns a <code>ShapePainter</code> defined on the specified element and
      * for the specified shape node, and using the specified bridge
      * context.
      *
@@ -173,25 +154,20 @@ public abstract class PaintServer
     public static ShapePainter convertFillAndStroke(Element e,
                                                     ShapeNode node,
                                                     BridgeContext ctx) {
-
-        Paint  fillPaint   = convertFillPaint  (e, node, ctx);
-        Paint  strokePaint = convertStrokePaint(e, node, ctx);
-        Stroke stroke      = convertStroke     (e);
-
         Shape shape = node.getShape();
         if (shape == null) return null;
 
-        if (stroke == null) {
-            FillShapePainter fp = new FillShapePainter(shape);
-            fp.setPaint(fillPaint);
-            return fp;
-        }
-
+        Paint  fillPaint   = convertFillPaint  (e, node, ctx);
         FillShapePainter fp = new FillShapePainter(shape);
         fp.setPaint(fillPaint);
 
+        Stroke stroke      = convertStroke     (e);
+        if (stroke == null)
+            return fp;
+
+        Paint  strokePaint = convertStrokePaint(e, node, ctx);
         StrokeShapePainter sp = new StrokeShapePainter(shape);
-        sp.setStroke(PaintServer.convertStroke(e));
+        sp.setStroke(stroke);
         sp.setPaint(strokePaint);
 
         CompositeShapePainter cp = new CompositeShapePainter(shape);
@@ -200,6 +176,24 @@ public abstract class PaintServer
         return cp;
     }
 
+
+    public static ShapePainter convertStrokePainter(Element e,
+                                                    ShapeNode node,
+                                                    BridgeContext ctx) {
+        Shape shape = node.getShape();
+        if (shape == null) return null;
+
+        Stroke stroke = convertStroke(e);
+        if (stroke == null)
+            return null;
+
+        Paint  strokePaint = convertStrokePaint(e, node, ctx);
+        StrokeShapePainter sp = new StrokeShapePainter(shape);
+        sp.setStroke(stroke);
+        sp.setPaint(strokePaint);
+        return sp;
+    }
+
     /////////////////////////////////////////////////////////////////////////
     // java.awt.Paint
     /////////////////////////////////////////////////////////////////////////
@@ -253,11 +247,10 @@ public abstract class PaintServer
     }
 
     /**
-     * Converts a Paint definition to a concrete <tt>java.awt.Paint</tt>
+     * Converts a Paint definition to a concrete <code>java.awt.Paint</code>
      * instance according to the specified parameters.
      *
      * @param paintedElement the element interested in a Paint
-     * @param decl the CSS declaration of the painted element
      * @param paintedNode the graphics node to paint (objectBoundingBox)
      * @param paintDef the paint definition
      * @param opacity the opacity to consider for the Paint
@@ -284,42 +277,45 @@ public abstract class PaintServer
                                        ctx);
 
             default:
-                throw new Error(); // can't be reached
+                throw new IllegalArgumentException
+                    ("Paint argument is not an appropriate CSS value");
             }
         } else { // List
             Value v = paintDef.item(0);
             switch (v.getPrimitiveType()) {
             case CSSPrimitiveValue.CSS_RGBCOLOR:
                 return convertRGBICCColor(paintedElement, v,
-                                          (ICCColor)paintDef.item(1),
+                                          paintDef.item(1),
                                           opacity, ctx);
 
-            case CSSPrimitiveValue.CSS_URI:
+            case CSSPrimitiveValue.CSS_URI: {
                 Paint result = silentConvertURIPaint(paintedElement,
                                                      paintedNode,
-                                                     v,
-                                                     opacity,
-                                                     ctx);
-                if (result == null) {
-                    v = paintDef.item(1);
-                    switch (v.getPrimitiveType()) {
-                    case CSSPrimitiveValue.CSS_IDENT:
-                        return null; // none
+                                                     v, opacity, ctx);
+                if (result != null) return result;
 
-                    case CSSPrimitiveValue.CSS_RGBCOLOR:
-                        if (paintDef.getLength() == 2) {
-                            return convertColor(v, opacity);
-                        } else {
-                            return convertRGBICCColor(paintedElement, v,
-                                                      (ICCColor)paintDef.item(2),
-                                                      opacity, ctx);
-                        }
-                    default:
-                        throw new Error(); // can't be reached
+                v = paintDef.item(1);
+                switch (v.getPrimitiveType()) {
+                case CSSPrimitiveValue.CSS_IDENT:
+                    return null; // none
+
+                case CSSPrimitiveValue.CSS_RGBCOLOR:
+                    if (paintDef.getLength() == 2) {
+                        return convertColor(v, opacity);
+                    } else {
+                        return convertRGBICCColor(paintedElement, v,
+                                                  paintDef.item(2),
+                                                  opacity, ctx);
                     }
+                default:
+                    throw new IllegalArgumentException
+                        ("Paint argument is not an appropriate CSS value");
                 }
+            }
             default:
-                throw new Error(); // can't be reached
+                // can't be reached
+                throw new IllegalArgumentException
+                    ("Paint argument is not an appropriate CSS value");
             }
         }
     }
@@ -370,8 +366,9 @@ public abstract class PaintServer
 
         Bridge bridge = ctx.getBridge(paintElement);
         if (bridge == null || !(bridge instanceof PaintBridge)) {
-            throw new BridgeException(paintedElement, ERR_CSS_URI_BAD_TARGET,
-                                      new Object[] {uri});
+            throw new BridgeException
+                (ctx, paintedElement, ERR_CSS_URI_BAD_TARGET,
+                 new Object[] {uri});
         }
         return ((PaintBridge)bridge).createPaint(ctx,
                                                  paintElement,
@@ -393,12 +390,20 @@ public abstract class PaintServer
      */
     public static Color convertRGBICCColor(Element paintedElement,
                                            Value colorDef,
-                                           ICCColor iccColor,
+                                           Value iccColor,
                                            float opacity,
                                            BridgeContext ctx) {
         Color color = null;
-        if (iccColor != null){
-            color = convertICCColor(paintedElement, iccColor, opacity, ctx);
+        if (iccColor != null) {
+            if (iccColor instanceof ICCColor) {
+                color = convertICCColor(paintedElement, (ICCColor)iccColor, opacity, ctx);
+            } else if (iccColor instanceof ICCNamedColor) {
+                color = convertICCNamedColor(paintedElement, (ICCNamedColor)iccColor, opacity, ctx);
+            } else if (iccColor instanceof CIELabColor) {
+                color = convertCIELabColor(paintedElement, (CIELabColor)iccColor, opacity, ctx);
+            } else if (iccColor instanceof DeviceColor) {
+                color = convertDeviceColor(paintedElement, colorDef, (DeviceColor)iccColor, opacity, ctx);
+            }
         }
         if (color == null){
             color = convertColor(colorDef, opacity);
@@ -411,7 +416,7 @@ public abstract class PaintServer
      * ICC color value or null if the related color profile could not
      * be used or loaded for any reason.
      *
-     * @param paintedElement the element using the color
+     * @param e the element using the color
      * @param c the ICC color definition
      * @param opacity the opacity
      * @param ctx the bridge context to use
@@ -419,7 +424,7 @@ public abstract class PaintServer
     public static Color convertICCColor(Element e,
                                         ICCColor c,
                                         float opacity,
-                                        BridgeContext ctx){
+                                        BridgeContext ctx) {
         // Get ICC Profile's name
         String iccProfileName = c.getColorProfile();
         if (iccProfileName == null){
@@ -433,8 +438,8 @@ public abstract class PaintServer
             return null; // no bridge for color profile
         }
 
-        ICCColorSpaceExt profileCS
-            = profileBridge.createICCColorSpaceExt(ctx, e, iccProfileName);
+        ICCColorSpaceWithIntent profileCS
+            = profileBridge.createICCColorSpaceWithIntent(ctx, e, iccProfileName);
         if (profileCS == null){
             return null; // no profile
         }
@@ -451,13 +456,132 @@ public abstract class PaintServer
 
         // Convert values to RGB
         float[] rgb = profileCS.intendedToRGB(colorValue);
+        //TODO Preserve original ICC color value!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
         return new Color(rgb[0], rgb[1], rgb[2], opacity);
     }
 
+    /**
+     * Returns a Color object that corresponds to the input Paint's
+     * ICC named color value or null if the related color profile could not
+     * be used or loaded for any reason.
+     *
+     * @param e the element using the color
+     * @param c the ICC named color definition
+     * @param opacity the opacity
+     * @param ctx the bridge context to use
+     */
+    public static Color convertICCNamedColor(Element e,
+                                        ICCNamedColor c,
+                                        float opacity,
+                                        BridgeContext ctx) {
+        // Get ICC Profile's name
+        String iccProfileName = c.getColorProfile();
+        if (iccProfileName == null){
+            return null;
+        }
+        // Ask the bridge to map the ICC profile name to an  ICC_Profile object
+        SVGColorProfileElementBridge profileBridge
+            = (SVGColorProfileElementBridge)
+            ctx.getBridge(SVG_NAMESPACE_URI, SVG_COLOR_PROFILE_TAG);
+        if (profileBridge == null){
+            return null; // no bridge for color profile
+        }
+
+        ICCColorSpaceWithIntent profileCS
+            = profileBridge.createICCColorSpaceWithIntent(ctx, e, iccProfileName);
+        if (profileCS == null){
+            return null; // no profile
+        }
+        ICC_Profile iccProfile = profileCS.getProfile();
+
+        String iccProfileSrc = null; //TODO Fill me!
+
+        if (NamedColorProfileParser.isNamedColorProfile(iccProfile)) {
+            NamedColorProfileParser parser = new NamedColorProfileParser();
+            NamedColorProfile ncp;
+            try {
+                ncp = parser.parseProfile(iccProfile, iccProfileName, iccProfileSrc);
+            } catch (IOException ioe) {
+                return null;
+            }
+            NamedColorSpace ncs = ncp.getNamedColor(c.getColorName());
+            if (ncs != null) {
+                Color specColor
+                    = new ColorWithAlternatives(ncs, new float[] {1.0f}, opacity, null);
+                return specColor;
+            } else {
+                /*
+                log.warn("Color '" + colorName
+                        + "' does not exist in named color profile: " + iccProfileSrc);
+                */
+            }
+        } else {
+            //log.warn("ICC profile is no named color profile: " + iccProfileSrc);
+        }
+
+        return null;
+    }
+
+    /**
+     * Returns a Color object that corresponds to the input Paint's
+     * CIE Lab color value.
+     *
+     * @param e the element using the color
+     * @param c the CIE Lab color definition
+     * @param opacity the opacity
+     * @param ctx the bridge context to use
+     */
+    public static Color convertCIELabColor(Element e,
+                                        CIELabColor c,
+                                        float opacity,
+                                        BridgeContext ctx) {
+        CIELabColorSpace cs = new CIELabColorSpace(c.getWhitePoint());
+        float[] lab = c.getColorValues();
+        Color specColor = cs.toColor(lab[0], lab[1], lab[2], opacity);
+        return specColor;
+    }
+
+    /**
+     * Returns a Color object that corresponds to the input Paint's
+     * device-specific color value.
+     *
+     * @param e the element using the color
+     * @param srgb the sRGB fallback color
+     * @param c the device-specific color definition
+     * @param opacity the opacity
+     * @param ctx the bridge context to use
+     */
+    public static Color convertDeviceColor(Element e,
+                                        Value srgb,
+                                        DeviceColor c,
+                                        float opacity,
+                                        BridgeContext ctx) {
+        int r = resolveColorComponent(srgb.getRed());
+        int g = resolveColorComponent(srgb.getGreen());
+        int b = resolveColorComponent(srgb.getBlue());
+        if (c.isNChannel()) {
+            return convertColor(srgb, opacity); //NYI
+        } else {
+            if (c.getNumberOfColors() == 4) {
+                DeviceCMYKColorSpace cmykCs = ColorSpaces.getDeviceCMYKColorSpace();
+                float[] comps = new float[4];
+                for (int i = 0; i < 4; i++) {
+                    comps[i] = c.getColor(i);
+                }
+                Color cmyk = new ColorWithAlternatives(cmykCs, comps, opacity, null);
+                Color specColor = new ColorWithAlternatives(r, g, b, Math.round(opacity * 255f),
+                        new Color[] {cmyk});
+                return specColor;
+            } else {
+                return convertColor(srgb, opacity); //NYI
+            }
+        }
+    }
+
     /**
      * Converts the given Value and opacity to a Color object.
      * @param c The CSS color to convert.
-     * @param o The opacity value (0 <= o <= 1).
+     * @param opacity The opacity value (0 &lt;= o &lt;= 1).
      */
     public static Color convertColor(Value c, float opacity) {
         int r = resolveColorComponent(c.getRed());
@@ -471,7 +595,7 @@ public abstract class PaintServer
     /////////////////////////////////////////////////////////////////////////
 
     /**
-     * Converts a <tt>Stroke</tt> object defined on the specified element.
+     * Converts a <code>Stroke</code> object defined on the specified element.
      *
      * @param e the element on which the stroke is specified
      */
@@ -539,7 +663,6 @@ public abstract class PaintServer
      * number in user units.
      *
      * @param v the CSS value describing the dasharray property
-     * @param uctx the unit processor context used to resolve units
      */
     public static float [] convertStrokeDasharray(Value v) {
         float [] dasharray = null;
@@ -567,7 +690,7 @@ public abstract class PaintServer
      */
     public static float convertStrokeMiterlimit(Value v) {
         float miterlimit = v.getFloatValue();
-        return (miterlimit < 1f) ? 1f : miterlimit;
+        return (miterlimit < 1.0f) ? 1.0f : miterlimit;
     }
 
     /**
@@ -584,7 +707,8 @@ public abstract class PaintServer
         case 's':
             return BasicStroke.CAP_SQUARE;
         default:
-            throw new Error(); // can't be reached
+            throw new IllegalArgumentException
+                ("Linecap argument is not an appropriate CSS value");
         }
     }
 
@@ -603,7 +727,8 @@ public abstract class PaintServer
         case 'b':
             return BasicStroke.JOIN_BEVEL;
         default:
-            throw new Error(); // can't be reached
+            throw new IllegalArgumentException
+                ("Linejoin argument is not an appropriate CSS value");
         }
     }
 
@@ -627,7 +752,8 @@ public abstract class PaintServer
             f = (f > 255f) ? 255f : (f < 0f) ? 0f : f;
             return Math.round(f);
         default:
-            throw new Error(); // can't be reached
+            throw new IllegalArgumentException
+                ("Color component argument is not an appropriate CSS value");
         }
     }
 
@@ -638,6 +764,6 @@ public abstract class PaintServer
      */
     public static float convertOpacity(Value v) {
         float r = v.getFloatValue();
-        return (r < 0f) ? 0f : (r > 1f) ? 1f : r;
+        return (r < 0f) ? 0f : (r > 1.0f) ? 1.0f : r;
     }
 }
