@@ -30,7 +30,10 @@ import java.util.Set;
 
 import org.apache.batik.anim.AnimationEngine;
 import org.apache.batik.anim.AnimationException;
-import org.apache.batik.dom.anim.AnimationTarget;
+import org.apache.batik.anim.dom.AnimationTarget;
+import org.apache.batik.anim.dom.SVGOMDocument;
+import org.apache.batik.anim.dom.SVGOMElement;
+import org.apache.batik.anim.dom.SVGStylableElement;
 import org.apache.batik.anim.timing.TimedDocumentRoot;
 import org.apache.batik.anim.timing.TimedElement;
 import org.apache.batik.anim.values.AnimatableAngleValue;
@@ -59,9 +62,6 @@ import org.apache.batik.css.engine.value.FloatValue;
 import org.apache.batik.css.engine.value.StringValue;
 import org.apache.batik.css.engine.value.Value;
 import org.apache.batik.css.engine.value.ValueManager;
-import org.apache.batik.dom.svg.SVGOMDocument;
-import org.apache.batik.dom.svg.SVGOMElement;
-import org.apache.batik.dom.svg.SVGStylableElement;
 import org.apache.batik.parser.DefaultPreserveAspectRatioHandler;
 import org.apache.batik.parser.FloatArrayProducer;
 import org.apache.batik.parser.DefaultLengthHandler;
@@ -94,7 +94,7 @@ import org.w3c.dom.svg.SVGPreserveAspectRatio;
  * An AnimationEngine for SVG documents.
  *
  * @author <a href="mailto:cam%40mcc%2eid%2eau">Cameron McCormack</a>
- * @version $Id: SVGAnimationEngine.java 579854 2007-09-27 00:07:53Z cam $
+ * @version $Id: SVGAnimationEngine.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class SVGAnimationEngine extends AnimationEngine {
 
@@ -120,6 +120,12 @@ public class SVGAnimationEngine extends AnimationEngine {
      */
     protected AnimationTickRunnable animationTickRunnable;
 
+    /**
+     * The initial time that will be seeked to when the animation engine starts,
+     * as set by {@link #setCurrentTime}.
+     */
+    protected float initialStartTime;
+
     /**
      * The factory for unparsed string values.
      */
@@ -392,18 +398,23 @@ public class SVGAnimationEngine extends AnimationEngine {
         if (p) {
             pause();
         }
-        return t;
+        return Float.isNaN(t) ? 0 : t;
     }
 
     /**
      * Sets the current document time.
      */
     public float setCurrentTime(float t) {
-        float ret = super.setCurrentTime(t);
-        if (animationTickRunnable != null) {
-            animationTickRunnable.resume();
+        if (started) {
+            float ret = super.setCurrentTime(t);
+            if (animationTickRunnable != null) {
+                animationTickRunnable.resume();
+            }
+            return ret;
+        } else {
+            initialStartTime = t;
+            return 0;
         }
-        return ret;
     }
 
     /**
@@ -438,6 +449,7 @@ public class SVGAnimationEngine extends AnimationEngine {
                         (SVGAnimationElementBridge) bridges[i];
                     bridge.initializeTimedElement();
                 }
+
                 // tick(0, false);
                 // animationThread = new AnimationThread();
                 // animationThread.start();
@@ -446,6 +458,9 @@ public class SVGAnimationEngine extends AnimationEngine {
                     RunnableQueue q = um.getUpdateRunnableQueue();
                     animationTickRunnable = new AnimationTickRunnable(q, this);
                     q.setIdleRunnable(animationTickRunnable);
+                    if (initialStartTime != 0) {
+                        setCurrentTime(initialStartTime);
+                    }
                 }
             } catch (AnimationException ex) {
                 throw new BridgeException(ctx, ex.getElement().getElement(),
