@@ -31,18 +31,18 @@ import java.io.IOException;
 import java.util.ArrayList;
 import java.util.List;
 
+import org.apache.batik.anim.dom.AbstractSVGAnimatedLength;
+import org.apache.batik.anim.dom.AnimatedLiveAttributeValue;
+import org.apache.batik.anim.dom.SVGOMAnimatedPreserveAspectRatio;
+import org.apache.batik.anim.dom.SVGOMDocument;
+import org.apache.batik.anim.dom.SVGOMElement;
 import org.apache.batik.css.engine.CSSEngine;
 import org.apache.batik.css.engine.SVGCSSEngine;
 import org.apache.batik.dom.AbstractNode;
 import org.apache.batik.dom.events.DOMMouseEvent;
 import org.apache.batik.dom.events.NodeEventTarget;
-import org.apache.batik.dom.svg.AbstractSVGAnimatedLength;
-import org.apache.batik.dom.svg.AnimatedLiveAttributeValue;
 import org.apache.batik.dom.svg.LiveAttributeException;
-import org.apache.batik.dom.svg.SVGOMAnimatedPreserveAspectRatio;
-import org.apache.batik.dom.svg.SVGOMDocument;
-import org.apache.batik.dom.svg.SVGOMElement;
-import org.apache.batik.ext.awt.color.ICCColorSpaceExt;
+import org.apache.batik.dom.util.DOMUtilities;
 import org.apache.batik.ext.awt.image.renderable.ClipRable8Bit;
 import org.apache.batik.ext.awt.image.renderable.Filter;
 import org.apache.batik.ext.awt.image.spi.BrokenLinkProvider;
@@ -58,6 +58,9 @@ import org.apache.batik.util.MimeTypeConstants;
 import org.apache.batik.util.ParsedURL;
 import org.apache.batik.util.XMLConstants;
 
+import org.apache.xmlgraphics.java2d.color.ICCColorSpaceWithIntent;
+import org.apache.xmlgraphics.java2d.color.RenderingIntent;
+
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.events.DocumentEvent;
@@ -72,7 +75,7 @@ import org.w3c.dom.svg.SVGSVGElement;
  * Bridge class for the &lt;image> element.
  *
  * @author <a href="mailto:tkormann@apache.org">Thierry Kormann</a>
- * @version $Id: SVGImageElementBridge.java 579487 2007-09-26 06:40:16Z cam $
+ * @version $Id: SVGImageElementBridge.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
 
@@ -204,7 +207,7 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
 
         DocumentLoader loader = ctx.getDocumentLoader();
         ImageTagRegistry reg = ImageTagRegistry.getRegistry();
-        ICCColorSpaceExt colorspace = extractColorSpace(e, ctx);
+        ICCColorSpaceWithIntent colorspace = extractColorSpace(e, ctx);
         {
             /**
              *  Before we open the URL we see if we have the
@@ -258,6 +261,11 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
             Filter img = reg.readURL(reference, purl, colorspace,
                                      false, false);
             if (img != null) {
+                try {
+                    reference.tie();
+                } catch (IOException ioe) {
+                    // This would be from a close,  Let it slide...
+                }
                 // It's a bouncing baby Raster...
                 return createRasterImageNode(ctx, e, img, purl);
             }
@@ -284,21 +292,26 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
              * Next see if it's an XML document.
              */
             Document doc = loader.loadDocument(purl.toString(), reference);
+            reference.release();
             imgDocument = (SVGDocument)doc;
             return createSVGImageNode(ctx, e, imgDocument);
         } catch (BridgeException ex) {
+            reference.release();
             throw ex;
         } catch (SecurityException secEx ) {
+            reference.release();
             throw new BridgeException(ctx, e, secEx, ERR_URI_UNSECURE,
                                       new Object[] {purl});
         } catch (InterruptedIOException iioe) {
+            reference.release();
             if (HaltingThread.hasBeenHalted())
                 throw new InterruptedBridgeException();
 
         } catch (InterruptedBridgeException ibe) {
+            reference.release();
             throw ibe;
         } catch (Exception ex) {
-            /* Nothing to do */
+            /* Do nothing drop out... */
             // ex.printStackTrace();
         }
 
@@ -352,34 +365,59 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
             throw new IOException("Reset unsupported");
         }
 
-        public void retry() throws IOException {
+        public synchronized void retry() throws IOException {
             super.reset();
+            wasClosed = false;
+            isTied = false;
         }
 
-        public void close() throws IOException {
-            /* do nothing */
+        public synchronized void close() throws IOException {
+            wasClosed = true;
+            if (isTied) {
+                super.close();
+                // System.err.println("Closing stream - from close");
+            }
         }
 
+        /**
+         * Let stream know that it is perminately tied to one
+         * image decoder.  This means that it can allow that
+         * decoder to close the stream.
+         */
+        public synchronized void tie() throws IOException {
+            isTied = true;
+            if (wasClosed) {
+                super.close();
+                // System.err.println("Closing stream - from tie");
+            }
+        }
+
+        /**
+         * Close the stream.
+         */
         public void release() {
             try {
                 super.close();
+                // System.err.println("Closing stream - from release");
             } catch (IOException ioe) {
                 // Like Duh! what would you do close it again?
             }
         }
+        boolean wasClosed = false;
+        boolean isTied = false;
     }
 
     protected ProtectedStream openStream(Element e, ParsedURL purl)
         throws IOException {
         List mimeTypes = new ArrayList
             (ImageTagRegistry.getRegistry().getRegisteredMimeTypes());
-        mimeTypes.add(MimeTypeConstants.MIME_TYPES_SVG);
+        mimeTypes.addAll(MimeTypeConstants.MIME_TYPES_SVG_LIST);
         InputStream reference = purl.openStream(mimeTypes.iterator());
         return new ProtectedStream(reference);
     }
 
     /**
-     * Creates an <tt>ImageNode</tt>.
+     * Creates an <code>ImageNode</code>.
      */
     protected GraphicsNode instantiateGraphicsNode() {
         return new ImageNode();
@@ -818,7 +856,7 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
         protected Element imgElement;
 
         /**
-         * Constructs a new <tt>ForwardEventListener</tt>
+         * Constructs a new <code>ForwardEventListener</code>
          */
         public ForwardEventListener(Element svgElement, Element imgElement) {
             this.svgElement = svgElement;
@@ -921,19 +959,19 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
      * @param element the element with the color-profile property
      * @param ctx the bridge context
      */
-    protected static ICCColorSpaceExt extractColorSpace(Element element,
+    protected static ICCColorSpaceWithIntent extractColorSpace(Element element,
                                                         BridgeContext ctx) {
 
         String colorProfileProperty = CSSUtilities.getComputedStyle
             (element, SVGCSSEngine.COLOR_PROFILE_INDEX).getStringValue();
 
         // The only cases that need special handling are 'sRGB' and 'name'
-        ICCColorSpaceExt colorSpace = null;
+        ICCColorSpaceWithIntent colorSpace = null;
         if (CSS_SRGB_VALUE.equalsIgnoreCase(colorProfileProperty)) {
 
-            colorSpace = new ICCColorSpaceExt
+            colorSpace = new ICCColorSpaceWithIntent
                 (ICC_Profile.getInstance(ColorSpace.CS_sRGB),
-                 ICCColorSpaceExt.AUTO);
+                 RenderingIntent.AUTO, "sRGB", null);
 
         } else if (!CSS_AUTO_VALUE.equalsIgnoreCase(colorProfileProperty)
                    && !"".equalsIgnoreCase(colorProfileProperty)){
@@ -943,7 +981,7 @@ public class SVGImageElementBridge extends AbstractGraphicsNodeBridge {
                 (SVGColorProfileElementBridge) ctx.getBridge
                 (SVG_NAMESPACE_URI, SVG_COLOR_PROFILE_TAG);
             if (profileBridge != null) {
-                colorSpace = profileBridge.createICCColorSpaceExt
+                colorSpace = profileBridge.createICCColorSpaceWithIntent
                     (ctx, element, colorProfileProperty);
 
             }
