@@ -1,74 +1,49 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.bridge;
 
 import java.awt.geom.AffineTransform;
 import java.util.StringTokenizer;
 
+import org.apache.batik.anim.dom.SVGOMAnimatedRect;
+import org.apache.batik.dom.svg.LiveAttributeException;
 import org.apache.batik.parser.AWTTransformProducer;
 import org.apache.batik.parser.FragmentIdentifierHandler;
 import org.apache.batik.parser.FragmentIdentifierParser;
 import org.apache.batik.parser.ParseException;
 import org.apache.batik.parser.PreserveAspectRatioParser;
+import org.apache.batik.dom.util.DOMUtilities;
 import org.apache.batik.util.SVGConstants;
+
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
+import org.w3c.dom.svg.SVGAnimatedPreserveAspectRatio;
+import org.w3c.dom.svg.SVGAnimatedRect;
 import org.w3c.dom.svg.SVGPreserveAspectRatio;
+import org.w3c.dom.svg.SVGRect;
 
 /**
  * This class provides convenient methods to handle viewport.
  *
  * @author <a href="mailto:tkormann@apache.org">Thierry Kormann</a>
- * @version $Id: ViewBox.java,v 1.11 2003/08/08 11:38:51 vhardy Exp $
+ * @version $Id: ViewBox.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public abstract class ViewBox implements SVGConstants, ErrorConstants {
 
@@ -86,17 +61,19 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
      * @param e the element interested in its view transform
      * @param w the width of the effective viewport
      * @param h The height of the effective viewport
+     * @param ctx The BridgeContext to use for error information
      * @exception BridgeException if an error occured while computing the
      *            preserveAspectRatio transform
      */
     public static AffineTransform getViewTransform(String ref,
                                                    Element e,
                                                    float w,
-                                                   float h) {
+                                                   float h,
+                                                   BridgeContext ctx) {
 
         // no reference has been specified, no extra viewBox is defined
         if (ref == null || ref.length() == 0) {
-            return getPreserveAspectRatioTransform(e, w, h);
+            return getPreserveAspectRatioTransform(e, w, h, ctx);
         }
 
         ViewHandler vh = new ViewHandler();
@@ -104,30 +81,37 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
         p.setFragmentIdentifierHandler(vh);
         p.parse(ref);
 
-        Element attrDefElement = e; // the element that defines the attributes
+        // Determine the 'view' element that ref refers to.
+        Element viewElement = e;
         if (vh.hasId) {
             Document document = e.getOwnerDocument();
-            attrDefElement = document.getElementById(vh.id);
+            viewElement = document.getElementById(vh.id);
         }
-        if (attrDefElement == null) {
-            throw new BridgeException(e, ERR_URI_MALFORMED,
+        if (viewElement == null) {
+            throw new BridgeException(ctx, e, ERR_URI_MALFORMED,
                                       new Object[] {ref});
         }
-        // if the referenced element is not a view, the attribute
-        // values to use are those defined on the enclosed svg element
-        if (!(attrDefElement.getNamespaceURI().equals(SVG_NAMESPACE_URI)
-              && attrDefElement.getLocalName().equals(SVG_VIEW_TAG))) {
-            attrDefElement = getClosestAncestorSVGElement(e);
+
+        Element ancestorSVG = getClosestAncestorSVGElement(e);
+        if (!(viewElement.getNamespaceURI().equals(SVG_NAMESPACE_URI)
+              && viewElement.getLocalName().equals(SVG_VIEW_TAG))) {
+            viewElement = ancestorSVG;
         }
 
         // 'viewBox'
-        float [] vb;
+        float[] vb;
         if (vh.hasViewBox) {
             vb = vh.viewBox;
         } else {
-            String viewBoxStr = attrDefElement.getAttributeNS
-                (null, SVG_VIEW_BOX_ATTRIBUTE);
-            vb = parseViewBoxAttribute(attrDefElement, viewBoxStr);
+            Element elt;
+            if (DOMUtilities.isAttributeSpecifiedNS
+                    (viewElement, null, SVG_VIEW_BOX_ATTRIBUTE)) {
+                elt = viewElement;
+            } else {
+                elt = ancestorSVG;
+            }
+            String viewBoxStr = elt.getAttributeNS(null, SVG_VIEW_BOX_ATTRIBUTE);
+            vb = parseViewBoxAttribute(elt, viewBoxStr, ctx);
         }
 
         // 'preserveAspectRatio'
@@ -137,18 +121,25 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
             align = vh.align;
             meet = vh.meet;
         } else {
-            String aspectRatio = attrDefElement.getAttributeNS
-                (null, SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE);
+            Element elt;
+            if (DOMUtilities.isAttributeSpecifiedNS
+                    (viewElement, null, SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE)) {
+                elt = viewElement;
+            } else {
+                elt = ancestorSVG;
+            }
+            String aspectRatio =
+                elt.getAttributeNS(null, SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE);
             PreserveAspectRatioParser pp = new PreserveAspectRatioParser();
             ViewHandler ph = new ViewHandler();
             pp.setPreserveAspectRatioHandler(ph);
             try {
                 pp.parse(aspectRatio);
-            } catch (ParseException ex) {
+            } catch (ParseException pEx) {
                 throw new BridgeException
-                    (attrDefElement, ERR_ATTRIBUTE_VALUE_MALFORMED,
+                    (ctx, elt, pEx, ERR_ATTRIBUTE_VALUE_MALFORMED,
                      new Object[] {SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE,
-                                       aspectRatio, ex});
+                                       aspectRatio, pEx });
             }
             align = ph.align;
             meet = ph.meet;
@@ -185,20 +176,38 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
      * Returns the transformation matrix to apply to initalize a viewport or
      * null if the specified viewBox disables the rendering of the element.
      *
+     * @deprecated Replaced by {@link
+     *     #getPreserveAspectRatioTransform(Element,float,float,BridgeContext)},
+     *     which has more accurate error reporting.
      * @param e the element with a viewbox
      * @param w the width of the effective viewport
-     * @param h The height of the effective viewport 
+     * @param h The height of the effective viewport
      */
     public static AffineTransform getPreserveAspectRatioTransform(Element e,
                                                                   float w,
                                                                   float h) {
+        return getPreserveAspectRatioTransform(e, w, h, null);
+    }
+
+    /**
+     * Returns the transformation matrix to apply to initalize a viewport or
+     * null if the specified viewBox disables the rendering of the element.
+     *
+     * @param e the element with a viewbox
+     * @param w the width of the effective viewport
+     * @param h The height of the effective viewport
+     * @param ctx The BridgeContext to use for error information
+     */
+    public static AffineTransform getPreserveAspectRatioTransform
+            (Element e, float w, float h, BridgeContext ctx) {
         String viewBox
             = e.getAttributeNS(null, SVG_VIEW_BOX_ATTRIBUTE);
 
         String aspectRatio
             = e.getAttributeNS(null, SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE);
 
-        return getPreserveAspectRatioTransform(e, viewBox, aspectRatio, w, h);
+        return getPreserveAspectRatioTransform(e, viewBox, aspectRatio, w, h,
+                                               ctx);
     }
 
     /**
@@ -208,20 +217,22 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
      * @param e the element with a viewbox
      * @param viewBox the viewBox definition
      * @param w the width of the effective viewport
-     * @param h The height of the effective viewport 
+     * @param h The height of the effective viewport
+     * @param ctx The BridgeContext to use for error information
      */
     public static
         AffineTransform getPreserveAspectRatioTransform(Element e,
                                                         String viewBox,
                                                         String aspectRatio,
                                                         float w,
-                                                        float h) {
+                                                        float h,
+                                                        BridgeContext ctx) {
 
         // no viewBox specified
         if (viewBox.length() == 0) {
             return new AffineTransform();
         }
-        float[] vb = parseViewBoxAttribute(e, viewBox);
+        float[] vb = parseViewBoxAttribute(e, viewBox, ctx);
 
         // 'preserveAspectRatio' attribute
         PreserveAspectRatioParser p = new PreserveAspectRatioParser();
@@ -229,11 +240,11 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
         p.setPreserveAspectRatioHandler(ph);
         try {
             p.parse(aspectRatio);
-        } catch (ParseException ex) {
+        } catch (ParseException pEx ) {
             throw new BridgeException
-                (e, ERR_ATTRIBUTE_VALUE_MALFORMED,
+                (ctx, e, pEx, ERR_ATTRIBUTE_VALUE_MALFORMED,
                  new Object[] {SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE,
-                                   aspectRatio, ex});
+                                   aspectRatio, pEx });
         }
 
         return getPreserveAspectRatioTransform(vb, ph.align, ph.meet, w, h);
@@ -246,13 +257,15 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
      * @param e the element with a viewbox
      * @param vb the viewBox definition as float
      * @param w the width of the effective viewport
-     * @param h The height of the effective viewport 
+     * @param h The height of the effective viewport
+     * @param ctx The BridgeContext to use for error information
      */
     public static
         AffineTransform getPreserveAspectRatioTransform(Element e,
-							float [] vb,
-							float w,
-							float h) {
+                                                        float[] vb,
+                                                        float w,
+                                                        float h,
+                                                        BridgeContext ctx) {
 
         String aspectRatio
             = e.getAttributeNS(null, SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE);
@@ -263,23 +276,80 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
         p.setPreserveAspectRatioHandler(ph);
         try {
             p.parse(aspectRatio);
-        } catch (ParseException ex) {
+        } catch (ParseException pEx ) {
             throw new BridgeException
-                (e, ERR_ATTRIBUTE_VALUE_MALFORMED,
+                (ctx, e, pEx, ERR_ATTRIBUTE_VALUE_MALFORMED,
                  new Object[] {SVG_PRESERVE_ASPECT_RATIO_ATTRIBUTE,
-                                   aspectRatio, ex});
+                                   aspectRatio, pEx });
         }
 
         return getPreserveAspectRatioTransform(vb, ph.align, ph.meet, w, h);
     }
 
+    /**
+     * Returns the transformation matrix to apply to initalize a viewport or
+     * null if the specified viewBox disables the rendering of the element.
+     *
+     * @param e the element with a viewbox
+     * @param vb the viewBox definition as float
+     * @param w the width of the effective viewport
+     * @param h The height of the effective viewport
+     * @param aPAR The animated preserveAspectRatio value
+     * @param ctx The BridgeContext to use for error information
+     */
+    public static AffineTransform getPreserveAspectRatioTransform
+            (Element e, float[] vb, float w, float h,
+             SVGAnimatedPreserveAspectRatio aPAR, BridgeContext ctx) {
+
+        // 'preserveAspectRatio' attribute
+        try {
+            SVGPreserveAspectRatio pAR = aPAR.getAnimVal();
+            short align = pAR.getAlign();
+            boolean meet = pAR.getMeetOrSlice() ==
+                SVGPreserveAspectRatio.SVG_MEETORSLICE_MEET;
+            return getPreserveAspectRatioTransform(vb, align, meet, w, h);
+        } catch (LiveAttributeException ex) {
+            throw new BridgeException(ctx, ex);
+        }
+    }
+
+    /**
+     * Returns the transformation matrix to apply to initialize a viewport or
+     * null if the specified viewBox disables the rendering of the element.
+     *
+     * @param e the element with a viewbox
+     * @param aViewBox the viewBox definition
+     * @param aPAR the preserveAspectRatio definition
+     * @param w the width of the effective viewport
+     * @param h the height of the effective viewport
+     * @param ctx the BridgeContext to use for error information
+     */
+    public static AffineTransform getPreserveAspectRatioTransform
+            (Element e, SVGAnimatedRect aViewBox,
+             SVGAnimatedPreserveAspectRatio aPAR,
+             float w, float h, BridgeContext ctx) {
+
+        if (!((SVGOMAnimatedRect) aViewBox).isSpecified()) {
+            // no viewBox specified
+            return new AffineTransform();
+        }
+        SVGRect viewBox = aViewBox.getAnimVal();
+        float[] vb = new float[] { viewBox.getX(), viewBox.getY(),
+                                   viewBox.getWidth(), viewBox.getHeight() };
+
+        return getPreserveAspectRatioTransform(e, vb, w, h, aPAR, ctx);
+    }
+
     /**
      * Parses a viewBox attribute.
      *
+     * @param e the element whose viewBox attribute value is being parsed
      * @param value the viewBox
+     * @param ctx the BridgeContext to use for error information
      * @return The 4 viewbox components or null.
      */
-    public static float[] parseViewBoxAttribute(Element e, String value) {
+    public static float[] parseViewBoxAttribute(Element e, String value,
+                                                BridgeContext ctx) {
         if (value.length() == 0) {
             return null;
         }
@@ -291,20 +361,20 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
                 vb[i] = Float.parseFloat(st.nextToken());
                 i++;
             }
-        } catch (NumberFormatException ex) {
+        } catch (NumberFormatException nfEx ) {
             throw new BridgeException
-                (e, ERR_ATTRIBUTE_VALUE_MALFORMED,
-                 new Object[] {SVG_VIEW_BOX_ATTRIBUTE, value, ex});
+                (ctx, e, nfEx, ERR_ATTRIBUTE_VALUE_MALFORMED,
+                 new Object[] {SVG_VIEW_BOX_ATTRIBUTE, value, nfEx });
         }
         if (i != 4) {
             throw new BridgeException
-                (e, ERR_ATTRIBUTE_VALUE_MALFORMED,
+                (ctx, e, ERR_ATTRIBUTE_VALUE_MALFORMED,
                  new Object[] {SVG_VIEW_BOX_ATTRIBUTE, value});
         }
         // A negative value for <width> or <height> is an error
         if (vb[2] < 0 || vb[3] < 0) {
             throw new BridgeException
-                (e, ERR_ATTRIBUTE_VALUE_MALFORMED,
+                (ctx, e, ERR_ATTRIBUTE_VALUE_MALFORMED,
                  new Object[] {SVG_VIEW_BOX_ATTRIBUTE, value});
         }
         // A value of zero for width or height disables rendering of the element
@@ -389,7 +459,7 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
 
 
         /**
-         * Constructs a new <tt>ViewHandler</tt> instance.
+         * Constructs a new <code>ViewHandler</code> instance.
          */
         protected ViewHandler() { }
 
@@ -438,8 +508,10 @@ public abstract class ViewBox implements SVGConstants, ErrorConstants {
 
         /**
          * Invoked when 'viewBox(x,y,width,height)' has been parsed.
-         * @param x&nbsp;y&nbsp;width&nbsp;height the coordinates of the
-         * viewbox.
+         * @param x the viewbox x coordinate
+         * @param y the viewbox y coordinate
+         * @param width the viewbox width
+         * @param height the viewbox height
          * @exception ParseException if an error occured while processing the
          *                           fragment identifier
          */
