@@ -1,58 +1,28 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.css.dom;
 
 import org.apache.batik.css.engine.value.FloatValue;
 import org.apache.batik.css.engine.value.Value;
 import org.apache.batik.css.engine.value.svg.ICCColor;
+import org.apache.batik.util.CSSConstants;
+
 import org.w3c.dom.DOMException;
 import org.w3c.dom.css.CSSPrimitiveValue;
 import org.w3c.dom.css.CSSValue;
@@ -62,10 +32,10 @@ import org.w3c.dom.svg.SVGPaint;
  * This class implements the {@link SVGPaint} interface.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: CSSOMSVGPaint.java,v 1.4 2003/08/08 11:38:54 vhardy Exp $
+ * @version $Id: CSSOMSVGPaint.java 476924 2006-11-19 21:13:26Z dvholten $
  */
 public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
-    
+
     /**
      * Creates a new CSSOMSVGPaint.
      */
@@ -100,12 +70,16 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
         switch (value.getCssValueType()) {
         case CSSValue.CSS_PRIMITIVE_VALUE:
             switch (value.getPrimitiveType()) {
-            case CSSPrimitiveValue.CSS_IDENT:
-                if (value.getStringValue().equalsIgnoreCase("none")) {
+            case CSSPrimitiveValue.CSS_IDENT: {
+                String str = value.getStringValue();
+                if (str.equalsIgnoreCase(CSSConstants.CSS_NONE_VALUE)) {
                     return SVG_PAINTTYPE_NONE;
+                } else if (str.equalsIgnoreCase
+                           (CSSConstants.CSS_CURRENTCOLOR_VALUE)) {
+                    return SVG_PAINTTYPE_CURRENTCOLOR;
                 }
-                return SVG_PAINTTYPE_CURRENTCOLOR;
-
+                return SVG_PAINTTYPE_RGBCOLOR;
+            }
             case CSSPrimitiveValue.CSS_RGBCOLOR:
                 return SVG_PAINTTYPE_RGBCOLOR;
 
@@ -118,26 +92,33 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
             Value v0 = value.item(0);
             Value v1 = value.item(1);
             switch (v0.getPrimitiveType()) {
+            case CSSPrimitiveValue.CSS_IDENT:
+                return SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR;
             case CSSPrimitiveValue.CSS_URI:
-                switch (v1.getPrimitiveType()) {
-                case CSSPrimitiveValue.CSS_IDENT:
-                    if (value.getStringValue().equalsIgnoreCase("none")) {
-                        return SVG_PAINTTYPE_URI_NONE;
-                    }
-                    return SVG_PAINTTYPE_URI_CURRENTCOLOR;
+                if (v1.getCssValueType() == CSSValue.CSS_VALUE_LIST)
+                    // Should probably check this more deeply...
+                    return SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR;
 
-                case CSSPrimitiveValue.CSS_RGBCOLOR:
-                    if (value.getLength() == 3) {
-                        return SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR;
+                switch (v1.getPrimitiveType()) {
+                case CSSPrimitiveValue.CSS_IDENT: {
+                    String str = v1.getStringValue();
+                    if (str.equalsIgnoreCase(CSSConstants.CSS_NONE_VALUE)) {
+                        return SVG_PAINTTYPE_URI_NONE;
+                    } else if (str.equalsIgnoreCase
+                               (CSSConstants.CSS_CURRENTCOLOR_VALUE)) {
+                        return SVG_PAINTTYPE_URI_CURRENTCOLOR;
                     }
                     return SVG_PAINTTYPE_URI_RGBCOLOR;
                 }
+                case CSSPrimitiveValue.CSS_RGBCOLOR:
+                    return SVG_PAINTTYPE_URI_RGBCOLOR;
+                }
 
             case CSSPrimitiveValue.CSS_RGBCOLOR:
                 return SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR;
             }
         }
-        throw new InternalError();
+        return SVG_PAINTTYPE_UNKNOWN;
     }
 
     /**
@@ -161,12 +142,12 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
      * <b>DOM</b>: Implements {@link org.w3c.dom.svg.SVGPaint#setUri(String)}.
      */
     public void setUri(String uri) {
-	if (handler == null) {
+        if (handler == null) {
             throw new DOMException
                 (DOMException.NO_MODIFICATION_ALLOWED_ERR, "");
-	} else {
+        } else {
             ((PaintModificationHandler)handler).uriChanged(uri);
-	}
+        }
     }
 
     /**
@@ -175,20 +156,20 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
      */
     public void setPaint(short paintType, String uri,
                          String rgbColor, String iccColor) {
-	if (handler == null) {
+        if (handler == null) {
             throw new DOMException
                 (DOMException.NO_MODIFICATION_ALLOWED_ERR, "");
-	} else {
+        } else {
             ((PaintModificationHandler)handler).paintChanged
                 (paintType, uri, rgbColor, iccColor);
-	}
+        }
     }
 
     /**
      * To manage the modifications on a SVGPaint value.
      */
     public interface PaintModificationHandler extends ModificationHandler {
-        
+
         /**
          * Called when the URI has been modified.
          */
@@ -216,11 +197,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
          */
         public void redTextChanged(String text) throws DOMException {
             switch (getPaintType()) {
-            case SVG_PAINTTYPE_RGBCOLOR: 
+            case SVG_PAINTTYPE_RGBCOLOR:
                 text = "rgb(" +
                     text + ", " +
                     getValue().getGreen().getCssText() + ", " +
-                    getValue().getBlue().getCssText() + ")";
+                    getValue().getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
@@ -236,7 +217,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                     " rgb(" +
                     text + ", " +
                     getValue().item(1).getGreen().getCssText() + ", " +
-                    getValue().item(1).getBlue().getCssText() + ")";
+                    getValue().item(1).getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
@@ -262,11 +243,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
             throws DOMException {
             String text;
             switch (getPaintType()) {
-            case SVG_PAINTTYPE_RGBCOLOR: 
+            case SVG_PAINTTYPE_RGBCOLOR:
                 text = "rgb(" +
                     FloatValue.getCssText(unit, value) + ", " +
                     getValue().getGreen().getCssText() + ", " +
-                    getValue().getBlue().getCssText() + ")";
+                    getValue().getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
@@ -282,7 +263,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                     " rgb(" +
                     FloatValue.getCssText(unit, value) + ", " +
                     getValue().item(1).getGreen().getCssText() + ", " +
-                    getValue().item(1).getBlue().getCssText() + ")";
+                    getValue().item(1).getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
@@ -306,11 +287,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
          */
         public void greenTextChanged(String text) throws DOMException {
             switch (getPaintType()) {
-            case SVG_PAINTTYPE_RGBCOLOR: 
+            case SVG_PAINTTYPE_RGBCOLOR:
                 text = "rgb(" +
                     getValue().getRed().getCssText() + ", " +
                     text + ", " +
-                    getValue().getBlue().getCssText() + ")";
+                    getValue().getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
@@ -326,7 +307,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                     " rgb(" +
                     getValue().item(1).getRed().getCssText() + ", " +
                     text + ", " +
-                    getValue().item(1).getBlue().getCssText() + ")";
+                    getValue().item(1).getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
@@ -352,11 +333,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
             throws DOMException {
             String text;
             switch (getPaintType()) {
-            case SVG_PAINTTYPE_RGBCOLOR: 
+            case SVG_PAINTTYPE_RGBCOLOR:
                 text = "rgb(" +
                     getValue().getRed().getCssText() + ", " +
                     FloatValue.getCssText(unit, value) + ", " +
-                    getValue().getBlue().getCssText() + ")";
+                    getValue().getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
@@ -372,7 +353,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                     " rgb(" +
                     getValue().item(1).getRed().getCssText() + ", " +
                     FloatValue.getCssText(unit, value) + ", " +
-                    getValue().item(1).getBlue().getCssText() + ")";
+                    getValue().item(1).getBlue().getCssText() + ')';
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
@@ -396,11 +377,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
          */
         public void blueTextChanged(String text) throws DOMException {
             switch (getPaintType()) {
-            case SVG_PAINTTYPE_RGBCOLOR: 
+            case SVG_PAINTTYPE_RGBCOLOR:
                 text = "rgb(" +
                     getValue().getRed().getCssText() + ", " +
                     getValue().getGreen().getCssText() + ", " +
-                    text + ")";
+                    text + ')';
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
@@ -442,11 +423,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
             throws DOMException {
             String text;
             switch (getPaintType()) {
-            case SVG_PAINTTYPE_RGBCOLOR: 
+            case SVG_PAINTTYPE_RGBCOLOR:
                 text = "rgb(" +
                     getValue().getRed().getCssText() + ", " +
                     getValue().getGreen().getCssText() + ", " +
-                    FloatValue.getCssText(unit, value) + ")";
+                    FloatValue.getCssText(unit, value) + ')';
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
@@ -462,7 +443,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                     " rgb(" +
                     getValue().item(1).getRed().getCssText() + ", " +
                     getValue().item(1).getGreen().getCssText() + ", " +
-                    FloatValue.getCssText(unit, value) + ")";
+                    FloatValue.getCssText(unit, value) + ')';
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
@@ -480,7 +461,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
             }
             textChanged(text);
         }
-        
+
         /**
          * Called when the RGBColor text has changed.
          */
@@ -494,11 +475,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR:
-                text = getValue().item(0).getCssText() + " " + text;
+                text = getValue().item(0).getCssText() + ' ' + text;
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
-                text = getValue().item(0).getCssText() + " " + text + " " +
+                text = getValue().item(0).getCssText() + ' ' + text + ' ' +
                     getValue().item(2).getCssText();
                 break;
 
@@ -516,12 +497,12 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
             throws DOMException {
             switch (getPaintType()) {
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
-                textChanged(rgb + " " + icc);
+                textChanged(rgb + ' ' + icc);
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
-                textChanged(getValue().item(0).getCssText() + " " +
-                            rgb + " " + icc);
+                textChanged(getValue().item(0).getCssText() + ' ' +
+                            rgb + ' ' + icc);
                 break;
 
             default:
@@ -545,7 +526,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
-                textChanged(rgb + " " + icc);
+                textChanged(rgb + ' ' + icc);
                 break;
 
             default:
@@ -565,25 +546,25 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 sb.append(cp);
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 for (int i = 0; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 sb.append(cp);
                 iccc = (ICCColor)getValue().item(1);
                 for (int i = 0; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -604,18 +585,18 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 sb.append(" icc-color(");
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -636,22 +617,22 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 sb.append(" icc-color(");
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -673,37 +654,37 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < idx; i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
                 for (int i = idx; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < idx; i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
                 for (int i = idx; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -725,37 +706,37 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < idx; i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
                 for (int i = idx + 1; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < idx; i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
                 for (int i = idx + 1; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -777,33 +758,33 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < idx; i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
                 for (int i = idx + 1; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < idx; i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
                 for (int i = idx + 1; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -825,29 +806,29 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 ICCColor iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
                 sb = new StringBuffer(getValue().item(0).getCssText());
-                sb.append(" ");
+                sb.append( ' ' );
                 sb.append(getValue().item(1).getCssText());
                 sb.append(" icc-color(");
                 iccc = (ICCColor)getValue().item(1);
                 sb.append(iccc.getColorProfile());
                 for (int i = 0; i < iccc.getLength(); i++) {
-                    sb.append(",");
+                    sb.append( ',' );
                     sb.append(iccc.getColor(i));
                 }
-                sb.append(",");
+                sb.append( ',' );
                 sb.append(f);
-                sb.append(")");
+                sb.append( ')' );
                 textChanged(sb.toString());
                 break;
 
@@ -883,11 +864,11 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 break;
 
             case SVG_PAINTTYPE_RGBCOLOR_ICCCOLOR:
-                textChanged(rgb + " " + icc);
+                textChanged(rgb + ' ' + icc);
                 break;
 
             case SVG_PAINTTYPE_URI:
-                textChanged("url(" + uri + ")");
+                textChanged("url(" + uri + ')' );
                 break;
 
             case SVG_PAINTTYPE_URI_NONE:
@@ -903,7 +884,7 @@ public class CSSOMSVGPaint extends CSSOMSVGColor implements SVGPaint {
                 break;
 
             case SVG_PAINTTYPE_URI_RGBCOLOR_ICCCOLOR:
-                textChanged("url(" + uri + ") " + rgb + " " + icc);
+                textChanged("url(" + uri + ") " + rgb + ' ' + icc);
             }
         }
     }
