@@ -83,7 +83,7 @@ import org.w3c.dom.xpath.XPathResult;
  * This class implements the {@link org.w3c.dom.Document} interface.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: AbstractDocument.java 588550 2007-10-26 07:52:41Z dvholten $
+ * @version $Id: AbstractDocument.java 1629149 2014-10-03 09:02:23Z vhennebert $
  */
 public abstract class AbstractDocument
     extends    AbstractParentNode
@@ -397,6 +397,14 @@ public abstract class AbstractDocument
             result = createDocumentFragment();
             break;
 
+        case DOCUMENT_TYPE_NODE:
+            DocumentType docType = (DocumentType) importedNode;
+            GenericDocumentType copy = new GenericDocumentType(docType.getName(),
+                    docType.getPublicId(), docType.getSystemId());
+            copy.ownerDocument = this;
+            result = copy;
+            break;
+
         default:
             throw createDOMException(DOMException.NOT_SUPPORTED_ERR,
                                      "import.node",
@@ -531,7 +539,7 @@ public abstract class AbstractDocument
     }
 
     /**
-     * Remove the mapping for <tt>element</tt> to <tt>id</tt>
+     * Remove the mapping for <code>element</code> to <code>id</code>
      */
     public void removeIdEntry(Element e, String id) {
         // Remove old Id mapping if we have one.
