@@ -32,7 +32,7 @@ import org.w3c.dom.svg.SVGNumberList;
  * This class is the implementation of {@link SVGNumberList}.
  *
  * @author <a href="mailto:tonny@kiyut.com">Tonny Kohar</a>
- * @version $Id: AbstractSVGNumberList.java 479349 2006-11-26 11:54:23Z cam $
+ * @version $Id: AbstractSVGNumberList.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public abstract class AbstractSVGNumberList
         extends AbstractSVGList
@@ -156,55 +156,6 @@ public abstract class AbstractSVGNumberList
         }
     }
 
-    /**
-     * Representation of the item SVGNumber.
-     */
-    protected class SVGNumberItem extends AbstractSVGNumber implements SVGItem {
-
-        /**
-         * The {@link SVGNumberList} this item belongs to.
-         */
-        protected AbstractSVGList parentList;
-
-        /**
-         * Creates a new SVGNumberItem.
-         */
-        public SVGNumberItem(float value) {
-            this.value = value;
-        }
-
-        /**
-         * Returns a String representation of the number.
-         */
-        public String getValueAsString() {
-            return Float.toString(value);
-        }
-
-        /**
-         * Associates this item to the given {@link SVGNumberList}.
-         */
-        public void setParent(AbstractSVGList list) {
-            parentList = list;
-        }
-
-        /**
-         * Returns the list the item belongs to.
-         */
-        public AbstractSVGList getParent() {
-            return parentList;
-        }
-
-        /**
-         * Notifies the parent {@link SVGNumberList} that this item's value
-         * has changed.
-         */
-        protected void reset() {
-            if (parentList != null) {
-                parentList.itemChanged();
-            }
-        }
-    }
-
     /**
      * Helper class to interface the {@link NumberListParser} and the
      * {@link NumberListHandler}.
