@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.dom.util;
 
 import java.io.IOException;
@@ -58,8 +26,13 @@ import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 
+import javax.xml.parsers.ParserConfigurationException;
+import javax.xml.parsers.SAXParser;
+import javax.xml.parsers.SAXParserFactory;
+
 import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
+import org.w3c.dom.DocumentType;
 import org.w3c.dom.Element;
 import org.w3c.dom.Node;
 import org.xml.sax.Attributes;
@@ -67,18 +40,23 @@ import org.xml.sax.ErrorHandler;
 import org.xml.sax.InputSource;
 import org.xml.sax.Locator;
 import org.xml.sax.SAXException;
+import org.xml.sax.SAXNotRecognizedException;
+import org.xml.sax.SAXNotSupportedException;
 import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
 import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.helpers.DefaultHandler;
 import org.xml.sax.helpers.XMLReaderFactory;
 
+import org.apache.batik.util.HaltingThread;
+import org.apache.batik.util.XMLConstants;
+
 /**
  * This class contains methods for creating Document instances
  * from an URI using SAX2.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: SAXDocumentFactory.java,v 1.21 2003/11/20 10:58:50 deweese Exp $
+ * @version $Id: SAXDocumentFactory.java 1664335 2015-03-05 12:53:44Z lbernardo $
  */
 public class SAXDocumentFactory
     extends    DefaultHandler
@@ -95,11 +73,16 @@ public class SAXDocumentFactory
      */
     protected String parserClassName;
 
+    /**
+     * The SAX2 parser object.
+     */
+    protected XMLReader parser;
+
     /**
      * The created document.
      */
     protected Document document;
-    
+
     /**
      * The created document descriptor.
      */
@@ -121,20 +104,52 @@ public class SAXDocumentFactory
     protected Locator locator;
 
     /**
-     * Whether the parser currently parses a CDATA section.
+     * Contains collected string data.  May be Text, CDATA or Comment.
      */
-    protected StringBuffer cdataBuffer;
+    protected StringBuffer stringBuffer = new StringBuffer();
 
     /**
-     * Whether the parser currently parses a DTD.
+     * The DTD to use when the document is created.
+     */
+    protected DocumentType doctype;
+
+    /**
+     * Indicates if stringBuffer has content, needed in case of
+     * zero sized "text" content.
+     */
+    protected boolean stringContent;
+
+    /**
+     * True if the parser is currently parsing a DTD.
      */
     protected boolean inDTD;
 
+    /**
+     * True if the parser is currently parsing a CDATA section.
+     */
+    protected boolean inCDATA;
+
+    /**
+     * Whether the parser still hasn't read the document element's
+     * opening tag.
+     */
+    protected boolean inProlog;
+
     /**
      * Whether the parser is in validating mode.
      */
     protected boolean isValidating;
 
+    /**
+     * Whether the document just parsed was standalone.
+     */
+    protected boolean isStandalone;
+
+    /**
+     * XML version of the document just parsed.
+     */
+    protected String xmlVersion;
+
     /**
      * The stack used to store the namespace URIs.
      */
@@ -146,7 +161,7 @@ public class SAXDocumentFactory
     protected ErrorHandler errorHandler;
 
     protected interface PreInfo {
-        public Node createNode(Document doc);
+        Node createNode(Document doc);
     }
 
     static class ProcessingInstructionInfo implements PreInfo {
@@ -155,7 +170,7 @@ public class SAXDocumentFactory
             this.target = target;
             this.data = data;
         }
-        public Node createNode(Document doc) { 
+        public Node createNode(Document doc) {
             return doc.createProcessingInstruction(target, data);
         }
     }
@@ -165,7 +180,7 @@ public class SAXDocumentFactory
         public CommentInfo(String comment) {
             this.comment = comment;
         }
-        public Node createNode(Document doc) { 
+        public Node createNode(Document doc) {
             return doc.createComment(comment);
         }
     }
@@ -175,7 +190,7 @@ public class SAXDocumentFactory
         public CDataInfo(String cdata) {
             this.cdata = cdata;
         }
-        public Node createNode(Document doc) { 
+        public Node createNode(Document doc) {
             return doc.createCDATASection(cdata);
         }
     }
@@ -185,7 +200,7 @@ public class SAXDocumentFactory
         public TextInfo(String text) {
             this.text = text;
         }
-        public Node createNode(Document doc) { 
+        public Node createNode(Document doc) {
             return doc.createTextNode(text);
         }
     }
@@ -204,8 +219,8 @@ public class SAXDocumentFactory
      */
     public SAXDocumentFactory(DOMImplementation impl,
                               String parser) {
-	implementation           = impl;
-	parserClassName          = parser;
+        implementation           = impl;
+        parserClassName          = parser;
     }
 
     /**
@@ -217,8 +232,8 @@ public class SAXDocumentFactory
     public SAXDocumentFactory(DOMImplementation impl,
                               String parser,
                               boolean dd) {
-	implementation           = impl;
-	parserClassName          = parser;
+        implementation           = impl;
+        parserClassName          = parser;
         createDocumentDescriptor = dd;
     }
 
@@ -252,8 +267,8 @@ public class SAXDocumentFactory
      * @param is The document input stream.
      * @exception IOException if an error occured while reading the document.
      */
-    public Document createDocument(String ns, String root, String uri, 
-				   InputStream is) throws IOException {
+    public Document createDocument(String ns, String root, String uri,
+                                   InputStream is) throws IOException {
         InputSource inp = new InputSource(is);
         inp.setSystemId(uri);
         return createDocument(ns, root, uri, inp);
@@ -265,7 +280,7 @@ public class SAXDocumentFactory
      * @param is The document input stream.
      * @exception IOException if an error occured while reading the document.
      */
-    public Document createDocument(String uri, InputStream is) 
+    public Document createDocument(String uri, InputStream is)
         throws IOException {
         InputSource inp = new InputSource(is);
         inp.setSystemId(uri);
@@ -307,11 +322,12 @@ public class SAXDocumentFactory
             if (ex != null && ex instanceof InterruptedIOException) {
                 throw (InterruptedIOException) ex;
             }
-            throw new IOException(e.getMessage());
+            throw new SAXIOException(e);
         }
         currentNode = null;
         Document ret = document;
         document = null;
+        doctype = null;
         return ret;
     }
 
@@ -337,7 +353,7 @@ public class SAXDocumentFactory
      */
     protected Document createDocument(String ns, String root, String uri,
                                       InputSource is)
-	throws IOException {
+        throws IOException {
         Document ret = createDocument(is);
         Element docElem = ret.getDocumentElement();
 
@@ -380,20 +396,41 @@ public class SAXDocumentFactory
         return ret;
     }
 
+    static SAXParserFactory saxFactory;
+    static {
+        saxFactory = SAXParserFactory.newInstance();
+        try {
+            saxFactory.setFeature("http://xml.org/sax/features/external-general-entities", false);
+            saxFactory.setFeature("http://xml.org/sax/features/external-parameter-entities", false);
+        } catch (SAXNotRecognizedException e) {
+            e.printStackTrace();
+        } catch (SAXNotSupportedException e) {
+            e.printStackTrace();
+        } catch (ParserConfigurationException e) {
+            e.printStackTrace();
+        }
+    }
 
     /**
      * Creates a Document.
-     * @param ns The namespace URI of the root element.
-     * @param root The name of the root element.
-     * @param uri The document URI.
      * @param is  The document input source.
      * @exception IOException if an error occured while reading the document.
      */
     protected Document createDocument(InputSource is)
-	throws IOException {
-	try {
-            XMLReader parser =
-                XMLReaderFactory.createXMLReader(parserClassName);
+        throws IOException {
+        try {
+            if (parserClassName != null) {
+                parser = XMLReaderFactory.createXMLReader(parserClassName);
+            } else {
+                SAXParser saxParser;
+                try {
+                    saxParser = saxFactory.newSAXParser();
+                } catch (ParserConfigurationException pce) {
+                    throw new IOException("Could not create SAXParser: "
+                            + pce.getMessage());
+                }
+                parser = saxParser.getXMLReader();
+            }
 
             parser.setContentHandler(this);
             parser.setDTDHandler(this);
@@ -401,27 +438,30 @@ public class SAXDocumentFactory
             parser.setErrorHandler((errorHandler == null) ?
                                    this : errorHandler);
 
-            parser.setFeature("http://xml.org/sax/features/namespaces", 
-			      true);
+            parser.setFeature("http://xml.org/sax/features/namespaces",
+                              true);
             parser.setFeature("http://xml.org/sax/features/namespace-prefixes",
                               true);
-	    parser.setFeature("http://xml.org/sax/features/validation",
-			      isValidating);
-	    parser.setProperty("http://xml.org/sax/properties/lexical-handler",
-			       this);
+            parser.setFeature("http://xml.org/sax/features/validation",
+                              isValidating);
+            parser.setProperty("http://xml.org/sax/properties/lexical-handler",
+                               this);
             parser.parse(is);
-	} catch (SAXException e) {
+        } catch (SAXException e) {
             Exception ex = e.getException();
             if (ex != null && ex instanceof InterruptedIOException) {
                 throw (InterruptedIOException)ex;
             }
-            throw new IOException(e.getMessage());
-	}
+            throw new SAXIOException(e);
+        }
 
-        currentNode = null;
+        currentNode  = null;
         Document ret = document;
-        document = null;
-	return ret;
+        document     = null;
+        doctype      = null;
+        locator      = null;
+        parser       = null;
+        return ret;
     }
 
     /**
@@ -446,10 +486,10 @@ public class SAXDocumentFactory
      * depending on the specified parameter.
      *
      * @param isValidating indicates that the XML parser will validate the XML
-     * document 
+     * document
      */
     public void setValidating(boolean isValidating) {
-	this.isValidating = isValidating;
+        this.isValidating = isValidating;
     }
 
     /**
@@ -457,7 +497,7 @@ public class SAXDocumentFactory
      * otherwise.
      */
     public boolean isValidating() {
-	return isValidating;
+        return isValidating;
     }
 
     /**
@@ -467,6 +507,33 @@ public class SAXDocumentFactory
         errorHandler = eh;
     }
 
+    public DOMImplementation getDOMImplementation(String ver) {
+        return implementation;
+    }
+
+    /**
+     * <b>SAX</b>: Implements {@link
+     * org.xml.sax.ErrorHandler#fatalError(SAXParseException)}.
+     */
+    public void fatalError(SAXParseException ex) throws SAXException {
+        throw ex;
+    }
+
+    /**
+     * <b>SAX</b>: Implements {@link
+     * org.xml.sax.ErrorHandler#error(SAXParseException)}.
+     */
+    public void error(SAXParseException ex) throws SAXException {
+        throw ex;
+    }
+
+    /**
+     * <b>SAX</b>: Implements {@link
+     * org.xml.sax.ErrorHandler#warning(SAXParseException)}.
+     */
+    public void warning(SAXParseException ex) throws SAXException {
+    }
+
     /**
      * <b>SAX</b>: Implements {@link
      * org.xml.sax.ContentHandler#startDocument()}.
@@ -474,13 +541,21 @@ public class SAXDocumentFactory
     public void startDocument() throws SAXException {
         preInfo    = new LinkedList();
         namespaces = new HashTableStack();
-	namespaces.put("xml", XMLSupport.XML_NAMESPACE_URI);
-	namespaces.put("xmlns", XMLSupport.XMLNS_NAMESPACE_URI);
-	namespaces.put("", null);
+        namespaces.put("xml", XMLSupport.XML_NAMESPACE_URI);
+        namespaces.put("xmlns", XMLSupport.XMLNS_NAMESPACE_URI);
+        namespaces.put("", null);
 
-        cdataBuffer = null;
-        inDTD = false;
-        currentNode = null;
+        inDTD        = false;
+        inCDATA      = false;
+        inProlog     = true;
+        currentNode  = null;
+        document     = null;
+        doctype      = null;
+        isStandalone = false;
+        xmlVersion   = XMLConstants.XML_VERSION_10;
+
+        stringBuffer.setLength(0);
+        stringContent = false;
 
         if (createDocumentDescriptor) {
             documentDescriptor = new DocumentDescriptor();
@@ -491,49 +566,75 @@ public class SAXDocumentFactory
 
     /**
      * <b>SAX</b>: Implements {@link
-     * ContentHandler#startElement(String,String,String,Attributes)}.
+     * org.xml.sax.ContentHandler#startElement(String,String,String,Attributes)}.
      */
     public void startElement(String     uri,
-			     String     localName,
-			     String     rawName,
-			     Attributes attributes) throws SAXException {
-        // Check for interruption.
-        if (Thread.currentThread().isInterrupted()) {
+                             String     localName,
+                             String     rawName,
+                             Attributes attributes) throws SAXException {
+        // Check If we should halt early.
+        if (HaltingThread.hasBeenHalted()) {
             throw new SAXException(new InterruptedIOException());
         }
 
-	// Namespaces resolution
-	int len = attributes.getLength();
-	namespaces.push();
-	for (int i = 0; i < len; i++) {
-	    String aname = attributes.getQName(i);
-	    if (aname.equals("xmlns")) {
+        if (inProlog) {
+            inProlog = false;
+            if (parser != null) {
+                try {
+                    isStandalone = parser.getFeature
+                        ("http://xml.org/sax/features/is-standalone");
+                } catch (SAXNotRecognizedException ex) {
+                }
+                try {
+                    xmlVersion = (String) parser.getProperty
+                        ("http://xml.org/sax/properties/document-xml-version");
+                } catch (SAXNotRecognizedException ex) {
+                }
+            }
+        }
+
+        // Namespaces resolution
+        int len = attributes.getLength();
+        namespaces.push();
+        String version = null;
+        for (int i = 0; i < len; i++) {
+            String aname = attributes.getQName(i);
+            int slen = aname.length();
+            if (slen < 5)
+                continue;
+            if (aname.equals("version")) {
+                version = attributes.getValue(i);
+                continue;
+            }
+            if (!aname.startsWith("xmlns"))
+                continue;
+            if (slen == 5) {
+                String ns = attributes.getValue(i);
+                if (ns.length() == 0)
+                    ns = null;
+                namespaces.put("", ns);
+            } else if (aname.charAt(5) == ':') {
                 String ns = attributes.getValue(i);
                 if (ns.length() == 0) {
                     ns = null;
                 }
-		namespaces.put("", ns);
-	    } else {
-		if (aname.startsWith("xmlns:")) {
-                    String ns = attributes.getValue(i);
-                    if (ns.length() == 0) {
-                        ns = null;
-                    }
-		    int idx = aname.indexOf(':');
-		    namespaces.put(aname.substring(idx + 1), ns);
-		}
-	    }
-	}
+                namespaces.put(aname.substring(6), ns);
+            }
+        }
 
-	// Element creation
-	Element e;
-	int idx = rawName.indexOf(':');
-	String nsp = (idx == -1 || idx == rawName.length()-1)
-	    ? ""
-	    : rawName.substring(0, idx);
-	String nsURI = namespaces.get(nsp);
+        // Add any collected String Data before element.
+        appendStringData();
+
+        // Element creation
+        Element e;
+        int idx = rawName.indexOf(':');
+        String nsp = (idx == -1 || idx == rawName.length()-1)
+            ? ""
+            : rawName.substring(0, idx);
+        String nsURI = namespaces.get(nsp);
         if (currentNode == null) {
-            document = implementation.createDocument(nsURI, rawName, null);
+            implementation = getDOMImplementation(version);
+            document = implementation.createDocument(nsURI, rawName, doctype);
             Iterator i = preInfo.iterator();
             currentNode = e = document.getDocumentElement();
             while (i.hasNext()) {
@@ -550,47 +651,26 @@ public class SAXDocumentFactory
 
         // Storage of the line number.
         if (createDocumentDescriptor && locator != null) {
-            documentDescriptor.setLocationLine(e, locator.getLineNumber());
+            documentDescriptor.setLocation(e,
+                                           locator.getLineNumber(),
+                                           locator.getColumnNumber());
         }
 
-	// Attributes creation
-	for (int i = 0; i < len; i++) {
-	    String aname = attributes.getQName(i);
-	    if (aname.equals("xmlns")) {
-		e.setAttributeNS(XMLSupport.XMLNS_NAMESPACE_URI,
-				 aname,
-				 attributes.getValue(i));
-	    } else {
-		idx = aname.indexOf(':');
-		nsURI = (idx == -1)
+        // Attributes creation
+        for (int i = 0; i < len; i++) {
+            String aname = attributes.getQName(i);
+            if (aname.equals("xmlns")) {
+                e.setAttributeNS(XMLSupport.XMLNS_NAMESPACE_URI,
+                                 aname,
+                                 attributes.getValue(i));
+            } else {
+                idx = aname.indexOf(':');
+                nsURI = (idx == -1)
                     ? null
                     : namespaces.get(aname.substring(0, idx));
-		e.setAttributeNS(nsURI, aname, attributes.getValue(i));
-	    }
-	}
-    }
-
-    /**
-     * <b>SAX</b>: Implements {@link
-     * org.xml.sax.ErrorHandler#fatalError(SAXParseException)}.
-     */
-    public void fatalError(SAXParseException ex) throws SAXException {
-        throw ex;
-    }
-
-    /**
-     * <b>SAX</b>: Implements {@link
-     * org.xml.sax.ErrorHandler#error(SAXParseException)}.
-     */
-    public void error(SAXParseException ex) throws SAXException {
-	throw ex;
-    }
-
-    /**
-     * <b>SAX</b>: Implements {@link
-     * org.xml.sax.ErrorHandler#warning(SAXParseException)}.
-     */
-    public void warning(SAXParseException ex) throws SAXException {
+                e.setAttributeNS(nsURI, aname, attributes.getValue(i));
+            }
+        }
     }
 
     /**
@@ -598,38 +678,65 @@ public class SAXDocumentFactory
      * org.xml.sax.ContentHandler#endElement(String,String,String)}.
      */
     public void endElement(String uri, String localName, String rawName)
-	throws SAXException {
+        throws SAXException {
+        appendStringData(); // add string data if any.
+
         if (currentNode != null)
             currentNode = currentNode.getParentNode();
-	namespaces.pop();
+        namespaces.pop();
     }
-    
+
+    public void appendStringData() {
+        if (!stringContent) return;
+
+        String str = stringBuffer.toString();
+        stringBuffer.setLength(0); // reuse buffer.
+        stringContent = false;
+        if (currentNode == null) {
+            if (inCDATA) preInfo.add(new CDataInfo(str));
+            else         preInfo.add(new TextInfo(str));
+        } else {
+            Node n;
+            if (inCDATA) n = document.createCDATASection(str);
+            else         n = document.createTextNode(str);
+            currentNode.appendChild(n);
+        }
+    }
+
     /**
      * <b>SAX</b>: Implements {@link
      * org.xml.sax.ContentHandler#characters(char[],int,int)}.
      */
-    public void characters(char ch[], int start, int length)
+    public void characters(char[] ch, int start, int length)
         throws SAXException {
-        if (cdataBuffer != null) 
-            cdataBuffer.append(ch, start, length);
-        else {
-            String data = new String(ch, start, length);
-            if (currentNode == null) {
-                preInfo.add(new TextInfo(data));
-            } else {
-                currentNode.appendChild(document.createTextNode(data));
-            }
-        }
+        stringBuffer.append(ch, start, length);
+        stringContent = true;
     }
-    
+
+
+    /**
+     * <b>SAX</b>: Implements {@link
+     * org.xml.sax.ContentHandler#ignorableWhitespace(char[],int,int)}.
+     */
+    public void ignorableWhitespace(char[] ch,
+                                    int start,
+                                    int length)
+        throws SAXException {
+        stringBuffer.append(ch, start, length);
+        stringContent = true;
+    }
+
     /**
      * <b>SAX</b>: Implements {@link
      * org.xml.sax.ContentHandler#processingInstruction(String,String)}.
      */
     public void processingInstruction(String target, String data)
         throws SAXException {
-	if (inDTD)
+        if (inDTD)
             return;
+
+        appendStringData(); // Add any collected String Data before PI
+
         if (currentNode == null)
             preInfo.add(new ProcessingInstructionInfo(target, data));
         else
@@ -644,15 +751,17 @@ public class SAXDocumentFactory
      * org.xml.sax.ext.LexicalHandler#startDTD(String,String,String)}.
      */
     public void startDTD(String name, String publicId, String systemId)
-	throws SAXException {
-	inDTD = true;
+        throws SAXException {
+        appendStringData(); // Add collected string data before entering DTD
+        doctype = implementation.createDocumentType(name, publicId, systemId);
+        inDTD = true;
     }
 
     /**
      * <b>SAX</b>: Implements {@link org.xml.sax.ext.LexicalHandler#endDTD()}.
      */
     public void endDTD() throws SAXException {
-	inDTD = false;
+        inDTD = false;
     }
 
     /**
@@ -674,7 +783,9 @@ public class SAXDocumentFactory
      * org.xml.sax.ext.LexicalHandler#startCDATA()}.
      */
     public void startCDATA() throws SAXException {
-        cdataBuffer = new StringBuffer();
+        appendStringData(); // Add any collected String Data before CData
+        inCDATA       = true;
+        stringContent = true; // always create CDATA even if empty.
     }
 
     /**
@@ -682,28 +793,23 @@ public class SAXDocumentFactory
      * org.xml.sax.ext.LexicalHandler#endCDATA()}.
      */
     public void endCDATA() throws SAXException {
-        String data = cdataBuffer.toString();
-        if (currentNode == null) {
-            preInfo.add(new CDataInfo(data));
-        } else {
-            currentNode.appendChild(document.createCDATASection(data));
-        }
-        cdataBuffer = null;
+        appendStringData(); // Add the CDATA section
+        inCDATA = false;
     }
 
     /**
      * <b>SAX</b>: Implements
      * {@link org.xml.sax.ext.LexicalHandler#comment(char[],int,int)}.
      */
-    public void comment(char ch[], int start, int length) throws SAXException {
-	if (inDTD) 
-            return;
+    public void comment(char[] ch, int start, int length) throws SAXException {
+        if (inDTD) return;
+        appendStringData();
+
         String str = new String(ch, start, length);
         if (currentNode == null) {
             preInfo.add(new CommentInfo(str));
         } else {
-            currentNode.appendChild
-                (document.createComment(str));
+            currentNode.appendChild(document.createComment(str));
         }
     }
 }
