@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt;
 
 import java.awt.Color;
@@ -62,50 +30,50 @@ import java.awt.image.ColorModel;
 /**
  * Provides the actual implementation for the LinearGradientPaint
  * This is where the pixel processing is done.
- * 
+ *
  * @author Nicholas Talian, Vincent Hardy, Jim Graham, Jerry Evans
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: LinearGradientPaintContext.java,v 1.10 2003/08/08 11:39:04 vhardy Exp $
+ * @version $Id: LinearGradientPaintContext.java 475685 2006-11-16 11:16:05Z cam $
  * @see java.awt.PaintContext
  * @see java.awt.Paint
  * @see java.awt.GradientPaint
  */
 final class LinearGradientPaintContext extends MultipleGradientPaintContext {
-    
+
     /**
-     * The following invariants are used to process the gradient value from 
+     * The following invariants are used to process the gradient value from
      * a device space coordinate, (X, Y):
      * g(X, Y) = dgdX*X + dgdY*Y + gc
      */
-    private float dgdX, dgdY, gc, pixSz;    
-           
+    private float dgdX, dgdY, gc, pixSz;
+
     private static final int DEFAULT_IMPL = 1;
     private static final int ANTI_ALIAS_IMPL  = 3;
 
     private int fillMethod;
 
-    /** 
+    /**
      * Constructor for LinearGradientPaintContext.
      *
      *  @param cm {@link ColorModel} that receives
      *  the <code>Paint</code> data. This is used only as a hint.
      *
-     *  @param deviceBounds the device space bounding box of the 
+     *  @param deviceBounds the device space bounding box of the
      *  graphics primitive being rendered
      *
-     *  @param userBounds the user space bounding box of the 
+     *  @param userBounds the user space bounding box of the
      *  graphics primitive being rendered
-     * 
+     *
      *  @param t the {@link AffineTransform} from user
-     *  space into device space (gradientTransform should be 
+     *  space into device space (gradientTransform should be
      *  concatenated with this)
      *
      *  @param hints the hints that the context object uses to choose
      *  between rendering alternatives
      *
-     *  @param start gradient start point, in user space
+     *  @param dStart gradient start point, in user space
      *
-     *  @param end gradient end point, in user space
+     *  @param dEnd gradient end point, in user space
      *
      *  @param fractions the fractions specifying the gradient distribution
      *
@@ -113,7 +81,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
      *
      *  @param cycleMethod either NO_CYCLE, REFLECT, or REPEAT
      *
-     *  @param colorSpace which colorspace to use for interpolation, 
+     *  @param colorSpace which colorspace to use for interpolation,
      *  either SRGB or LINEAR_RGB
      *
      */
@@ -125,22 +93,22 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                                       Point2D dStart,
                                       Point2D dEnd,
                                       float[] fractions,
-                                      Color[] colors, 
-                                      MultipleGradientPaint.CycleMethodEnum 
+                                      Color[] colors,
+                                      MultipleGradientPaint.CycleMethodEnum
                                       cycleMethod,
-                                      MultipleGradientPaint.ColorSpaceEnum 
+                                      MultipleGradientPaint.ColorSpaceEnum
                                       colorSpace)
         throws NoninvertibleTransformException
-    {	
-        super(cm, deviceBounds, userBounds, t, hints, fractions, 
+    {
+        super(cm, deviceBounds, userBounds, t, hints, fractions,
               colors, cycleMethod, colorSpace);
-        
+
         // Use single precision floating points
         Point2D.Float start = new Point2D.Float((float)dStart.getX(),
                                                 (float)dStart.getY());
         Point2D.Float end = new Point2D.Float((float)dEnd.getX(),
                                               (float)dEnd.getY());
-        
+
         // A given point in the raster should take on the same color as its
         // projection onto the gradient vector.
         // Thus, we want the projection of the current position vector
@@ -153,23 +121,23 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
         float dx = end.x - start.x; // change in x from start to end
         float dy = end.y - start.y; // change in y from start to end
         float dSq = dx*dx + dy*dy; // total distance squared
-	
+
         //avoid repeated calculations by doing these divides once.
         float constX = dx/dSq;
         float constY = dy/dSq;
-	
+
         //incremental change along gradient for +x
         dgdX = a00*constX + a10*constY;
         //incremental change along gradient for +y
         dgdY = a01*constX + a11*constY;
-        
+
         float dgdXAbs = Math.abs(dgdX);
         float dgdYAbs = Math.abs(dgdY);
         if (dgdXAbs > dgdYAbs)  pixSz = dgdXAbs;
         else                    pixSz = dgdYAbs;
 
         //constant, incorporates the translation components from the matrix
-        gc = (a02-start.x)*constX + (a12-start.y)*constY;	       	
+        gc = (a02-start.x)*constX + (a12-start.y)*constY;
 
         Object colorRend = hints.get(RenderingHints.KEY_COLOR_RENDERING);
         Object rend      = hints.get(RenderingHints.KEY_RENDERING);
@@ -185,10 +153,10 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                 fillMethod = DEFAULT_IMPL;
             else if (colorRend == RenderingHints.VALUE_COLOR_RENDER_QUALITY)
                 fillMethod = ANTI_ALIAS_IMPL;
-        } 
+        }
     }
 
-    protected void fillHardNoCycle(int[] pixels, int off, int adjust, 
+    protected void fillHardNoCycle(int[] pixels, int off, int adjust,
                               int x, int y, int w, int h) {
 
         //constant which can be pulled out of the inner loop
@@ -196,13 +164,13 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
 
         for(int i=0; i<h; i++) { //for every row
             //initialize current value to be start.
-            float g = initConst + dgdY*(y+i); 
+            float g = initConst + dgdY*(y+i);
             final int rowLimit = off+w;  // end of row iteration
 
             if (dgdX == 0) {
                 // System.out.println("In fillHard: " + g);
                 final int val;
-                if (g <= 0) 
+                if (g <= 0)
                     val = gradientUnderflow;
                 else if (g >= 1)
                     val = gradientOverflow;
@@ -228,25 +196,28 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                 int gradSteps;
                 int preGradSteps;
                 final int preVal, postVal;
+
+                float gradStepsF;
+                float preGradStepsF;
                 if (dgdX >= 0) {
-                    gradSteps    = (int)         ((1-g)/dgdX);
-                    preGradSteps = (int)Math.ceil((0-g)/dgdX);
+                    gradStepsF    =          ((1-g)/dgdX);
+                    preGradStepsF = (float)Math.ceil((0-g)/dgdX);
                     preVal  = gradientUnderflow;
                     postVal = gradientOverflow;
                 } else { // dgdX < 0
-                    gradSteps    = (int)         ((0-g)/dgdX);
-                    preGradSteps = (int)Math.ceil((1-g)/dgdX);
+                    gradStepsF    =          ((0-g)/dgdX);
+                    preGradStepsF = (float)Math.ceil((1-g)/dgdX);
                     preVal  = gradientOverflow;
                     postVal = gradientUnderflow;
                 }
 
-                if (gradSteps > w) 
-                    gradSteps = w;
+                if (gradStepsF > w)    gradSteps = w;
+                else                   gradSteps = (int)gradStepsF;
+                if (preGradStepsF > w) preGradSteps = w;
+                else                   preGradSteps = (int)preGradStepsF;
 
                 final int gradLimit    = off + gradSteps;
                 if (preGradSteps > 0) {
-                    if (preGradSteps > w)
-                        preGradSteps = w;
                     final int preGradLimit = off + preGradSteps;
 
                     while (off < preGradLimit) {
@@ -254,7 +225,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                     }
                     g += dgdX*preGradSteps;
                 }
-                        
+
                 if (dgdX > 0) {
                     // Could be a binary search...
                     int gradIdx = 0;
@@ -263,13 +234,16 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                             break;
                         gradIdx++;
                     }
-                    
+
                     while (off < gradLimit) {
                         float delta = (g-fractions[gradIdx]);
                         final int [] grad = gradients[gradIdx];
 
-                        int steps = 
-                            (int)Math.ceil((fractions[gradIdx+1]-g)/dgdX);
+                        double stepsD = Math.ceil
+                            ((fractions[gradIdx+1]-g)/dgdX);
+                        int steps;
+                        if (stepsD > w) steps = w;
+                        else            steps = (int)stepsD;
                         int subGradLimit = off + steps;
                         if (subGradLimit > gradLimit)
                             subGradLimit = gradLimit;
@@ -284,7 +258,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                             pixels[off++] = grad[idx>>16];
                             idx += step;
                         }
-                        g+=dgdX*steps;
+                        g+=dgdX*stepsD;
                         gradIdx++;
                     }
                 } else {
@@ -295,12 +269,15 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                             break;
                         gradIdx--;
                     }
-                    
+
                     while (off < gradLimit) {
                         float delta = (g-fractions[gradIdx]);
                         final int [] grad = gradients[gradIdx];
 
-                        int steps        = (int)Math.ceil(delta/-dgdX);
+                        double stepsD     = Math.ceil(delta/-dgdX);
+                        int    steps;
+                        if (stepsD > w) steps = w;
+                        else            steps = (int)stepsD;
                         int subGradLimit = off + steps;
                         if (subGradLimit > gradLimit)
                             subGradLimit = gradLimit;
@@ -315,7 +292,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                             pixels[off++] = grad[idx>>16];
                             idx += step;
                         }
-                        g+=dgdX*steps;
+                        g+=dgdX*stepsD;
                         gradIdx--;
                     }
                 }
@@ -328,7 +305,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
         }
     }
 
-    protected void fillSimpleNoCycle(int[] pixels, int off, int adjust, 
+    protected void fillSimpleNoCycle(int[] pixels, int off, int adjust,
                                 int x, int y, int w, int h) {
         //constant which can be pulled out of the inner loop
         final float initConst = (dgdX*x) + gc;
@@ -339,20 +316,22 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
 
         for(int i=0; i<h; i++){ //for every row
             //initialize current value to be start.
-            float g = initConst + dgdY*(y+i); 
+            float g = initConst + dgdY*(y+i);
             g *= fastGradientArraySize;
             g += 0.5; // rounding factor...
 
             final int rowLimit = off+w;  // end of row iteration
 
-            if (dgdX == 0) {
+            float check = dgdX*fastGradientArraySize*w;
+            if (check < 0) check = -check;
+            if (check < .3) {
                 // System.out.println("In fillSimpleNC: " + g);
                 final int val;
-                if (g<=0) 
+                if (g<=0)
                     val = gradientUnderflow;
-                else if (g>=fastGradientArraySize) 
+                else if (g>=fastGradientArraySize)
                     val = gradientOverflow;
-                else 
+                else
                     val = grad[(int)g];
                 while (off < rowLimit) {
                     pixels[off++] = val;
@@ -370,13 +349,13 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
 
                 } else { // dgdX < 0
                     gradSteps    = (int)((0-g)/step);
-                    preGradSteps = 
+                    preGradSteps =
                         (int)Math.ceil((fastGradientArraySize-g)/step);
                     preVal  = gradientOverflow;
                     postVal = gradientUnderflow;
                 }
 
-                if (gradSteps > w) 
+                if (gradSteps > w)
                     gradSteps = w;
                 final int gradLimit    = off + gradSteps;
 
@@ -390,13 +369,13 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                     }
                     g += step*preGradSteps;
                 }
-                        
+
                 int fpG = (int)(g*(1<<16));
                 while (off < gradLimit) {
                     pixels[off++] = grad[fpG>>16];
                     fpG += fpStep;
                 }
-                        
+
                 while (off < rowLimit) {
                     pixels[off++] = postVal;
                 }
@@ -404,8 +383,8 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
             off += adjust; //change in off from row to row
         }
     }
-    
-    protected void fillSimpleRepeat(int[] pixels, int off, int adjust, 
+
+    protected void fillSimpleRepeat(int[] pixels, int off, int adjust,
                                int x, int y, int w, int h) {
 
         final float initConst = (dgdX*x) + gc;
@@ -419,22 +398,22 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                 // Make it a Positive step (a small negative step is
                 // the same as a positive step slightly less than
                 // fastGradientArraySize.
-        if (step < 0) 
+        if (step < 0)
             step += fastGradientArraySize;
 
         final int [] grad = gradient;
 
         for(int i=0; i<h; i++) { //for every row
             //initialize current value to be start.
-            float g = initConst + dgdY*(y+i); 
+            float g = initConst + dgdY*(y+i);
 
             // now Limited between -1 and 1.
             g = g-(int)g;
             // put in the positive side.
             if (g < 0)
                 g += 1;
-                        
-            // scale for gradient array... 
+
+            // scale for gradient array...
             g *= fastGradientArraySize;
             g += 0.5; // rounding factor
             final int rowLimit = off+w;  // end of row iteration
@@ -442,7 +421,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                 int idx = (int)g;
                 if (idx >= fastGradientArraySize) {
                     g   -= fastGradientArraySize;
-                    idx -= fastGradientArraySize; 
+                    idx -= fastGradientArraySize;
                 }
                 pixels[off++] = grad[idx];
                 g += step;
@@ -452,8 +431,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
         }
     }
 
-
-    protected void fillSimpleReflect(int[] pixels, int off, int adjust, 
+    protected void fillSimpleReflect(int[] pixels, int off, int adjust,
                                 int x, int y, int w, int h) {
         final float initConst = (dgdX*x) + gc;
 
@@ -461,7 +439,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
 
         for (int i=0; i<h; i++) { //for every row
             //initialize current value to be start.
-            float g = initConst + dgdY*(y+i); 
+            float g = initConst + dgdY*(y+i);
 
             // now limited g to -2<->2
             g = g - 2*((int)(g/2.0f));
@@ -481,7 +459,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
             // all negative step values into the positive
             // side.
             step = step - 2*((int)step/2.0f);
-            if (step < 0) 
+            if (step < 0)
                 step += 2.0;
             final int reflectMax = 2*fastGradientArraySize;
 
@@ -507,19 +485,24 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
             off += adjust; //change in off from row to row
         }
     }
-        
+
     /**
      * Return a Raster containing the colors generated for the graphics
      * operation.  This is where the area is filled with colors distributed
      * linearly.
      *
-     * @param x,y,w,h The area in device space for which colors are
-     * generated.
-     *
+     * @param x The x coordinate of the area in device space for which colors
+     *          are generated.
+     * @param y The y coordinate of the area in device space for which colors
+     *          are generated.
+     * @param w The width of the area in device space for which colors
+     *          are generated.
+     * @param h The height of the area in device space for which colors
+     *          are generated.
      */
-    protected void fillRaster(int[] pixels, int off, int adjust, 
+    protected void fillRaster(int[] pixels, int off, int adjust,
                               int x, int y, int w, int h) {
-	
+
         //constant which can be pulled out of the inner loop
         final float initConst = (dgdX*x) + gc;
 
@@ -527,11 +510,11 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
             //initialize current value to be start.
             for(int i=0; i<h; i++){ //for every row
                 float g = initConst + dgdY*(y+i);
-                
+
                 final int rowLimit = off+w;  // end of row iteration
                 while(off < rowLimit){ //for every pixel in this row.
                     //get the color
-                    pixels[off++] = indexGradientAntiAlias(g, pixSz); 
+                    pixels[off++] = indexGradientAntiAlias(g, pixSz);
                     g += dgdX; //incremental change in g
                 }
                 off += adjust; //change in off from row to row
@@ -544,12 +527,12 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
             else {
                 //initialize current value to be start.
                 for(int i=0; i<h; i++){ //for every row
-                    float g = initConst + dgdY*(y+i); 
-                
+                    float g = initConst + dgdY*(y+i);
+
                     final int rowLimit = off+w;  // end of row iteration
                     while(off < rowLimit){ //for every pixel in this row.
                         //get the color
-                        pixels[off++] = indexIntoGradientsArrays(g); 
+                        pixels[off++] = indexIntoGradientsArrays(g);
                         g += dgdX; //incremental change in g
                     }
                     off += adjust; //change in off from row to row
@@ -557,7 +540,7 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
             }
         } else {
             // Simple implementations: just scale index by array size
-            
+
             if (cycleMethod == MultipleGradientPaint.NO_CYCLE)
                 fillSimpleNoCycle(pixels, off, adjust, x, y, w, h);
             else if (cycleMethod == MultipleGradientPaint.REPEAT)
@@ -566,6 +549,4 @@ final class LinearGradientPaintContext extends MultipleGradientPaintContext {
                 fillSimpleReflect(pixels, off, adjust, x, y, w, h);
         }
     }
-    
-    
 }
