@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt;
 
 import java.awt.Color;
@@ -67,71 +35,71 @@ import java.awt.image.ColorModel;
  *
  * @author Nicholas Talian, Vincent Hardy, Jim Graham, Jerry Evans
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: RadialGradientPaintContext.java,v 1.10 2003/08/09 16:58:43 deweese Exp $
+ * @version $Id: RadialGradientPaintContext.java 516969 2007-03-11 17:39:37Z dvholten $
  *
  */
-final class RadialGradientPaintContext extends MultipleGradientPaintContext {  
-    
+final class RadialGradientPaintContext extends MultipleGradientPaintContext {
+
     /** True when (focus == center)  */
     private boolean isSimpleFocus = false;
 
     /** True when (cycleMethod == NO_CYCLE) */
     private boolean isNonCyclic = false;
-       
+
     /** Radius of the outermost circle defining the 100% gradient stop. */
-    private float radius;   
-    
+    private float radius;
+
     /** Variables representing center and focus points. */
-    private float centerX, centerY, focusX, focusY;     
+    private float centerX, centerY, focusX, focusY;
 
     /** Radius of the gradient circle squared. */
-    private float radiusSq; 
-        
+    private float radiusSq;
+
     /** Constant part of X, Y user space coordinates. */
     private float constA, constB;
-       
+
     /** This value represents the solution when focusX == X.  It is called
      * trivial because it is easier to calculate than the general case.
      */
-    private float trivial;       
+    private float trivial;
 
     private static final int FIXED_POINT_IMPL = 1;
     private static final int DEFAULT_IMPL     = 2;
     private static final int ANTI_ALIAS_IMPL  = 3;
 
     private int fillMethod;
-    
+
     /** Amount for offset when clamping focus. */
-    private static final float SCALEBACK = .97f;
-    
-    /** 
+    private static final float SCALEBACK = 0.999f;
+
+    /**
      * Constructor for RadialGradientPaintContext.
      *
      *  @param cm {@link ColorModel} that receives
      *  the <code>Paint</code> data. This is used only as a hint.
      *
-     *  @param deviceBounds the device space bounding box of the 
+     *  @param deviceBounds the device space bounding box of the
      *  graphics primitive being rendered
      *
-     *  @param userBounds the user space bounding box of the 
+     *  @param userBounds the user space bounding box of the
      *  graphics primitive being rendered
-     * 
+     *
      *  @param t the {@link AffineTransform} from user
-     *  space into device space (gradientTransform should be 
-     *  concatenated with this)    
+     *  space into device space (gradientTransform should be
+     *  concatenated with this)
      *
      *  @param hints the hints that the context object uses to choose
      *  between rendering alternatives
-     *       
-     *  @param cx the center point in user space of the circle defining 
-     *  the gradient.  The last color of the gradient is mapped to the 
+     *
+     *  @param cx the center point in user space of the circle defining
+     *  the gradient.  The last color of the gradient is mapped to the
      *  perimeter of this circle X coordinate
      *
-     *  @param cy the center point in user space of the circle defining 
-     *  the gradient.  The last color of the gradient is mapped to the 
+     *  @param cy the center point in user space of the circle defining
+     *  the gradient.  The last color of the gradient is mapped to the
      *  perimeter of this circle Y coordinate
-     *     
-     *  @param r the radius of the circle defining the extents of the 
+     *
+     *  @param r the radius of the circle defining the extents of the
      *  color gradient
      *
      *  @param fx the point in user space to which the first color is mapped
@@ -146,7 +114,7 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
      *
      *  @param cycleMethod either NO_CYCLE, REFLECT, or REPEAT
      *
-     *  @param colorSpace which colorspace to use for interpolation, 
+     *  @param colorSpace which colorspace to use for interpolation,
      *  either SRGB or LINEAR_RGB
      *
      */
@@ -160,25 +128,25 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
                                       float fx, float fy,
                                       float[] fractions,
                                       Color[] colors,
-                                      MultipleGradientPaint.CycleMethodEnum 
+                                      MultipleGradientPaint.CycleMethodEnum
                                       cycleMethod,
-                                      MultipleGradientPaint.ColorSpaceEnum 
+                                      MultipleGradientPaint.ColorSpaceEnum
                                       colorSpace)
         throws NoninvertibleTransformException
-    {       	
-        super(cm, deviceBounds, userBounds, t, hints, fractions, colors, 
+    {
+        super(cm, deviceBounds, userBounds, t, hints, fractions, colors,
               cycleMethod, colorSpace);
 
         //copy some parameters.
         centerX = cx;
-        centerY = cy;	
+        centerY = cy;
         focusX = fx;
         focusY = fy;
         radius = r;
 
         this.isSimpleFocus = (focusX == centerX) && (focusY == centerY);
         this.isNonCyclic = (cycleMethod == RadialGradientPaint.NO_CYCLE);
-	
+
         //for use in the quadractic equation
         radiusSq = radius * radius;
 
@@ -190,10 +158,10 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
         //test if distance from focus to center is greater than the radius
         if (dist > radius* SCALEBACK) { //clamp focus to radius
           double angle = Math.atan2(dY, dX);
-			
+
           //x = r cos theta, y = r sin theta
           focusX = (float)(SCALEBACK * radius * Math.cos(angle)) + centerX;
-          
+
           focusY = (float)(SCALEBACK * radius * Math.sin(angle)) + centerY;
         }
 
@@ -202,7 +170,7 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
         dX = focusX - centerX;
         trivial = (float)Math.sqrt(radiusSq - (dX * dX));
 
-        // constant parts of X, Y user space coordinates 
+        // constant parts of X, Y user space coordinates
         constA = a02 - centerX;
         constB = a12 - centerY;
 
@@ -226,7 +194,7 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
         // We are in the 'default' case, no hint or hint set to
         // DEFAULT values...
         if (fillMethod == 0) {
-            // For now we will always use the 'default' impl if 
+            // For now we will always use the 'default' impl if
             // one is not specified.
             fillMethod = DEFAULT_IMPL;
 
@@ -248,19 +216,26 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
             fillMethod = FIXED_POINT_IMPL;
         }
     }
-    
+
     /**
      * Return a Raster containing the colors generated for the graphics
      * operation.
-     * @param x,y,w,h The area in device space for which colors are
-     * generated.
+     *
+     * @param x The x coordinate of the area in device space for which colors
+     *          are generated.
+     * @param y The y coordinate of the area in device space for which colors
+     *          are generated.
+     * @param w The width of the area in device space for which colors
+     *          are generated.
+     * @param h The height of the area in device space for which colors
+     *          are generated.
      */
-    protected void fillRaster(int pixels[], int off, int adjust,
+    protected void fillRaster(int[] pixels, int off, int adjust,
                               int x, int y, int w, int h) {
         switch(fillMethod) {
         case FIXED_POINT_IMPL:
             // System.out.println("Calling FP");
-            fixedPointSimplestCaseNonCyclicFillRaster(pixels, off, adjust, x, 
+            fixedPointSimplestCaseNonCyclicFillRaster(pixels, off, adjust, x,
                                                       y, w, h);
             break;
         case ANTI_ALIAS_IMPL:
@@ -272,21 +247,21 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
             // System.out.println("Calling Default");
             cyclicCircularGradientFillRaster(pixels, off, adjust, x, y, w, h);
         }
-    }    
-    
+    }
+
     /**
-     * This code works in the simplest of cases, where the focus == center 
-     * point, the gradient is noncyclic, and the gradient lookup method is 
+     * This code works in the simplest of cases, where the focus == center
+     * point, the gradient is noncyclic, and the gradient lookup method is
      * fast (single array index, no conversion necessary).
      *
-     */         
-    private void fixedPointSimplestCaseNonCyclicFillRaster(int pixels[], 
+     */
+    private void fixedPointSimplestCaseNonCyclicFillRaster(int[] pixels,
                                                            int off,
-                                                           int adjust, 
-                                                           int x, int y, 
+                                                           int adjust,
+                                                           int x, int y,
                                                            int w, int h) {
         float iSq=0;  // Square distance index
-        final float indexFactor = fastGradientArraySize / radius;      
+        final float indexFactor = fastGradientArraySize / radius;
 
         //constant part of X and Y coordinates for the entire raster
         final float constX = (a00*x) + (a01*y) + constA;
@@ -298,8 +273,8 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
             (fastGradientArraySize * fastGradientArraySize);
         float g, gDelta, gDeltaDelta, temp; //gradient square value
         int gIndex; // integer number used to index gradient array
-        int iSqInt; // Square distance index       		   
-	
+        int iSqInt; // Square distance index
+
         int end, j; //indexing variables
         int indexer = off;//used to index pixels array
 
@@ -313,7 +288,7 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
             final int val = gradientOverflow;
             for(j = 0; j < h; j++){ //for every row
                 //for every column (inner loop begins here)
-                for (end = indexer+w; indexer < end; indexer++) 
+                for (end = indexer+w; indexer < end; indexer++)
                     pixels[indexer] = val;
                 indexer += adjust;
             }
@@ -324,87 +299,87 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
         for(j = 0; j < h; j++){ //for every row
             //x and y (in user space) of the first pixel of this row
             dX = indexFactor * ((a01*j) + constX);
-            dY = indexFactor * ((a11*j) + constY);	   	   
+            dY = indexFactor * ((a11*j) + constY);
 
             // these values below here allow for an incremental calculation
-            // of dX^2 + dY^2 
+            // of dX^2 + dY^2
 
             //initialize to be equal to distance squared
             g = (((dY * dY) + (dX * dX)) );
-            gDelta =  (((((deltaY * dY) + (deltaX * dX))* 2) + 
-                        temp));	 
-	    
+            gDelta = (deltaY * dY + deltaX * dX) * 2 + temp;
+
             //for every column (inner loop begins here)
-            for (end = indexer+w; indexer < end; indexer++) {	       
+            for (end = indexer+w; indexer < end; indexer++) {
                 //determine the distance to the center
-		
+
                 //since this is a non cyclic fill raster, crop at "1" and 0
                 if (g >= fixedArraySizeSq) {
                     pixels[indexer] = gradientOverflow;
                 }
-		
+
                 // This should not happen as gIndex is a square
                 // quantity. Code commented out on purpose, can't underflow.
                 // else if (g < 0) {
-                //    gIndex = 0;		    
+                //    gIndex = 0;
                 // }
-		
+
                 else {
                     iSq = (g * invSqStepFloat);
-                    
+
                     iSqInt = (int)iSq; //chop off fractional part
-                    iSq -= iSqInt;		    
+                    iSq -= iSqInt;
                     gIndex = sqrtLutFixed[iSqInt];
                     gIndex += (int)(iSq * (sqrtLutFixed[iSqInt + 1]-gIndex));
-                    pixels[indexer] = gradient[gIndex]; 
+                    pixels[indexer] = gradient[gIndex];
                 }
-		
-				
+
+
                 //incremental calculation
                 g += gDelta;
-                gDelta += gDeltaDelta;		
-            }	  
+                gDelta += gDeltaDelta;
+            }
             indexer += adjust;
         }
     }
 
     /** Length of a square distance intervale in the lookup table */
-    private float invSqStepFloat; 
-    
+    private float invSqStepFloat;
+
     /** Used to limit the size of the square root lookup table */
-    private int MAX_PRECISION = 256;
-    
+    private static final int MAX_PRECISION = 256;
+
     /** Square root lookup table */
-    private int sqrtLutFixed[] = new int[MAX_PRECISION];
-    
+    private int[] sqrtLutFixed = new int[MAX_PRECISION];
+
     /**
      * Build square root lookup table
-     */       
-    private void calculateFixedPointSqrtLookupTable() {	      
+     */
+    private void calculateFixedPointSqrtLookupTable() {
         float sqStepFloat;
-        sqStepFloat = ((fastGradientArraySize  * fastGradientArraySize) 
-                       / (MAX_PRECISION - 2));
-	
-        // The last two values are the same so that linear square root 
-        // interpolation can happen on the maximum reachable element in the 
+        sqStepFloat = (fastGradientArraySize * fastGradientArraySize)
+                       / (MAX_PRECISION - 2.0f);
+
+        // The last two values are the same so that linear square root
+        // interpolation can happen on the maximum reachable element in the
         // lookup table (precision-2)
+        int[] workTbl = sqrtLutFixed;      // local is cheaper
         int i;
         for (i = 0; i < MAX_PRECISION - 1; i++) {
-            sqrtLutFixed[i] = (int)(Math.sqrt(i*sqStepFloat));
+            workTbl[i] = (int)Math.sqrt(i*sqStepFloat);
         }
-        sqrtLutFixed[i] = sqrtLutFixed[i-1];	
-        invSqStepFloat = 1/sqStepFloat;
+        workTbl[i] = workTbl[i-1];
+        invSqStepFloat = 1.0f/sqStepFloat;
     }
-    
+
     /** Fill the raster, cycling the gradient colors when a point falls outside
-     *  of the perimeter of the 100% stop circle.          
-     * 
+     *  of the perimeter of the 100% stop circle.
+     *
      *  This calculation first computes the intersection point of the line
      *  from the focus through the current point in the raster, and the
      *  perimeter of the gradient circle.
-     * 
+     *
      *  Then it determines the percentage distance of the current point along
-     *  that line (focus is 0%, perimeter is 100%). 
+     *  that line (focus is 0%, perimeter is 100%).
      *
      *  Equation of a circle centered at (a,b) with radius r:
      *  (x-a)^2 + (y-b)^2 = r^2
@@ -414,13 +389,13 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
      *  formula produces the following set of equations.  Constant factors have
      *  been extracted out of the inner loop.
      *
-     */   
-    private void cyclicCircularGradientFillRaster(int pixels[], int off, 
-                                                  int adjust, 
-                                                  int x, int y, 
+     */
+    private void cyclicCircularGradientFillRaster(int[] pixels, int off,
+                                                  int adjust,
+                                                  int x, int y,
                                                   int w, int h) {
         // Constant part of the C factor of the quadratic equation
-        final double constC = 
+        final double constC =
             -(radiusSq) + (centerX * centerX) + (centerY * centerY);
         double A; //coefficient of the quadratic equation (Ax^2 + Bx + C = 0)
         double B; //coefficient of the quadratic equation
@@ -428,12 +403,12 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
         double slope; //slope of the focus-perimeter line
         double yintcpt; //y-intercept of the focus-perimeter line
         double solutionX;//intersection with circle X coordinate
-        double solutionY;//intersection with circle Y coordinate       
-       	final float constX = (a00*x) + (a01*y) + a02;//const part of X coord
+        double solutionY;//intersection with circle Y coordinate
+               final float constX = (a00*x) + (a01*y) + a02;//const part of X coord
         final float constY = (a10*x) + (a11*y) + a12; //const part of Y coord
-       	final float precalc2 = 2 * centerY;//const in inner loop quad. formula
+               final float precalc2 = 2 * centerY;//const in inner loop quad. formula
         final float precalc3 =-2 * centerX;//const in inner loop quad. formula
-        float X; // User space point X coordinate 
+        float X; // User space point X coordinate
         float Y; // User space point Y coordinate
         float g;//value between 0 and 1 specifying position in the gradient
         float det; //determinant of quadratic formula (should always be >0)
@@ -446,53 +421,53 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
         int pixInc = w+adjust;//incremental index change for pixels array
 
         for (j = 0; j < h; j++) { //for every row
-	    
+
             X = (a01*j) + constX; //constants from column to column
             Y = (a11*j) + constY;
-	    
+
             //for every column (inner loop begins here)
-            for (i = 0; i < w; i++) {	       			
-	
+            for (i = 0; i < w; i++) {
+
                 // special case to avoid divide by zero or very near zero
-                if (((X-focusX)>-0.000001) &&
-                    ((X-focusX)< 0.000001)) {		   
+                if (((X-focusX)>-0.000001f) &&
+                    ((X-focusX)< 0.000001f)) {
                     solutionX = focusX;
-		    
+
                     solutionY = centerY;
-		    
+
                     solutionY += (Y > focusY)?trivial:-trivial;
                 }
-		
-                else {    
-		    
+
+                else {
+
                     //slope of the focus-current line
                     slope =   (Y - focusY) / (X - focusX);
-		    
+
                     yintcpt = Y - (slope * X); //y-intercept of that same line
-		    
+
                     //use the quadratic formula to calculate the intersection
-                    //point		  
-                    A = (slope * slope) + 1; 
-		    
+                    //point
+                    A = (slope * slope) + 1;
+
                     B =  precalc3 + (-2 * slope * (centerY - yintcpt));
-		    
+
                     C =  constC + (yintcpt* (yintcpt - precalc2));
-		    
+
                     det = (float)Math.sqrt((B * B) - ( 4 * A * C));
-		    
+
                     solutionX = -B;
-		    
+
                     //choose the positive or negative root depending
                     //on where the X coord lies with respect to the focus.
                     solutionX += (X < focusX)?-det:det;
-		    
-                    solutionX = solutionX / (2 * A);//divisor
-		    
-                    solutionY = (slope * solutionX) + yintcpt;
-                }	                    	
 
-                //calculate the square of the distance from the current point 
-                //to the focus and the square of the distance from the 
+                    solutionX = solutionX / (2 * A);//divisor
+
+                    solutionY = (slope * solutionX) + yintcpt;
+                }
+
+                //calculate the square of the distance from the current point
+                //to the focus and the square of the distance from the
                 //intersection point to the focus. Want the squares so we can
                 //do 1 square root after division instead of 2 before.
 
@@ -512,15 +487,15 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
 
                 currentToFocusSq = deltaXSq + deltaYSq;
 
-                //want the percentage (0-1) of the current point along the 
+                //want the percentage (0-1) of the current point along the
                 //focus-circumference line
                 g = (float)Math.sqrt(currentToFocusSq / intersectToFocusSq);
 
                 //Get the color at this point
                 pixels[indexer + i] = indexIntoGradientsArrays(g);
-		
+
                 X += a00; //incremental change in X, Y
-                Y += a10;	
+                Y += a10;
             } //end inner loop
             indexer += pixInc;
         } //end outer loop
@@ -534,9 +509,9 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
      *  This calculation first computes the intersection point of the line
      *  from the focus through the current point in the raster, and the
      *  perimeter of the gradient circle.
-     * 
+     *
      *  Then it determines the percentage distance of the current point along
-     *  that line (focus is 0%, perimeter is 100%). 
+     *  that line (focus is 0%, perimeter is 100%).
      *
      *  Equation of a circle centered at (a,b) with radius r:
      *  (x-a)^2 + (y-b)^2 = r^2
@@ -546,21 +521,21 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
      *  formula produces the following set of equations.  Constant factors have
      *  been extracted out of the inner loop.
      * */
-    private void antiAliasFillRaster(int pixels[], int off, 
-                                     int adjust, 
-                                     int x, int y, 
+    private void antiAliasFillRaster(int[] pixels, int off,
+                                     int adjust,
+                                     int x, int y,
                                      int w, int h) {
         // Constant part of the C factor of the quadratic equation
-        final double constC = 
+        final double constC =
             -(radiusSq) + (centerX * centerX) + (centerY * centerY);
         //coefficients of the quadratic equation (Ax^2 + Bx + C = 0)
-       	final float precalc2 = 2 * centerY;//const in inner loop quad. formula
+               final float precalc2 = 2 * centerY;//const in inner loop quad. formula
         final float precalc3 =-2 * centerX;//const in inner loop quad. formula
 
         //const part of X,Y coord (shifted to bottom left corner of pixel.
-       	final float constX = (a00*(x-.5f)) + (a01*(y+.5f)) + a02;
+               final float constX = (a00*(x-.5f)) + (a01*(y+.5f)) + a02;
         final float constY = (a10*(x-.5f)) + (a11*(y+.5f)) + a12;
-        float X; // User space point X coordinate 
+        float X; // User space point X coordinate
         float Y; // User space point Y coordinate
         int i, j; //indexing variables for FOR loops
         int indexer = off-1; //index variable for pixels array
@@ -578,147 +553,151 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
 
         // Calc top row of g's.
         for (i=0; i <= w; i++) {
+            final float dx = X - focusX;
+
             // special case to avoid divide by zero or very near zero
-            if (((X-focusX)>-0.000001) &&
-                ((X-focusX)< 0.000001)) {		   
+            if ( ( dx >-0.000001f ) &&
+                 ( dx < 0.000001f ))  {
                 solutionX = focusX;
                 solutionY = centerY;
                 solutionY += (Y > focusY)?trivial:-trivial;
             }
-            else { 
+            else {
                 // Formula for Circle: (X-Xc)^2 + (Y-Yc)^2 - R^2 = 0
                 // Formula line:        Y = Slope*x + Y0;
-                // 
+                //
                 // So you substitue line into Circle and apply
                 // Quadradic formula.
 
 
                 //slope of the focus-current line
                 slope =   (Y - focusY) / (X - focusX);
-		    
+
                 yintcpt = Y - (slope * X); //y-intercept of that same line
-		    
+
                 //use the quadratic formula to calculate the intersection
-                //point		  
-                A = (slope * slope) + 1; 
-		    
+                //point
+                A = (slope * slope) + 1;
+
                 B =  precalc3 + (-2 * slope * (centerY - yintcpt));
-		    
+
                 C =  constC + (yintcpt* (yintcpt - precalc2));
-		    
+
                 det = Math.sqrt((B * B) - ( 4 * A * C));
-		    
+
                 solutionX = -B;
-		    
+
                 //choose the positive or negative root depending
                 //on where the X coord lies with respect to the focus.
                 solutionX += (X < focusX)?-det:det;
-		    
-                solutionX = solutionX / (2 * A);//divisor
-		    
-                solutionY = (slope * solutionX) + yintcpt;
-            }	                    	
 
-            //calculate the square of the distance from the current point 
-            //to the focus and the square of the distance from the 
+                solutionX = solutionX / (2 * A);//divisor
+
+                solutionY = (slope * solutionX) + yintcpt;
+            }
+
+            //calculate the square of the distance from the current point
+            //to the focus and the square of the distance from the
             //intersection point to the focus. Want the squares so we can
             //do 1 square root after division instead of 2 before.
             deltaXSq = solutionX - focusX;
             deltaXSq = deltaXSq * deltaXSq;
-        
+
             deltaYSq = solutionY - focusY;
             deltaYSq = deltaYSq * deltaYSq;
-        
+
             intersectToFocusSq = deltaXSq + deltaYSq;
-        
+
             deltaXSq = X - focusX;
             deltaXSq = deltaXSq * deltaXSq;
-        
+
             deltaYSq = Y - focusY;
             deltaYSq = deltaYSq * deltaYSq;
-        
+
             currentToFocusSq = deltaXSq + deltaYSq;
-        
-            //want the percentage (0-1) of the current point along the 
+
+            //want the percentage (0-1) of the current point along the
             //focus-circumference line
             prevGs[i] = Math.sqrt(currentToFocusSq / intersectToFocusSq);
 
             X += a00; //incremental change in X, Y
-            Y += a10;	
+            Y += a10;
         }
 
         for (j = 0; j < h; j++) { //for every row
-	    
+
             // Set X,Y to bottom edge of pixel row.
             X = (a01*j) + constX; //constants from row to row
             Y = (a11*j) + constY;
 
             g10 = prevGs[0];
+
+            float dx = X - focusX;
             // special case to avoid divide by zero or very near zero
-            if (((X-focusX)>-0.000001) &&
-                ((X-focusX)< 0.000001)) {		   
+            if ( ( dx >-0.000001f ) &&
+                 ( dx < 0.000001f ))  {
                 solutionX = focusX;
                 solutionY = centerY;
                 solutionY += (Y > focusY)?trivial:-trivial;
             }
-            else { 
+            else {
                 // Formula for Circle: (X-Xc)^2 + (Y-Yc)^2 - R^2 = 0
                 // Formula line:        Y = Slope*x + Y0;
-                // 
+                //
                 // So you substitue line into Circle and apply
                 // Quadradic formula.
 
 
                 //slope of the focus-current line
                 slope =   (Y - focusY) / (X - focusX);
-		    
+
                 yintcpt = Y - (slope * X); //y-intercept of that same line
-		    
+
                 //use the quadratic formula to calculate the intersection
-                //point		  
-                A = (slope * slope) + 1; 
-		    
+                //point
+                A = (slope * slope) + 1;
+
                 B =  precalc3 + (-2 * slope * (centerY - yintcpt));
-		    
+
                 C =  constC + (yintcpt* (yintcpt - precalc2));
-		    
+
                 det = Math.sqrt((B * B) - ( 4 * A * C));
-		    
+
                 solutionX = -B;
-		    
+
                 //choose the positive or negative root depending
                 //on where the X coord lies with respect to the focus.
                 solutionX += (X < focusX)?-det:det;
-		    
-                solutionX = solutionX / (2 * A);//divisor
-		    
-                solutionY = (slope * solutionX) + yintcpt;
-            }	                    	
 
-            //calculate the square of the distance from the current point 
-            //to the focus and the square of the distance from the 
+                solutionX = solutionX / (2 * A);//divisor
+
+                solutionY = (slope * solutionX) + yintcpt;
+            }
+
+            //calculate the square of the distance from the current point
+            //to the focus and the square of the distance from the
             //intersection point to the focus. Want the squares so we can
             //do 1 square root after division instead of 2 before.
             deltaXSq = solutionX - focusX;
             deltaXSq = deltaXSq * deltaXSq;
-        
+
             deltaYSq = solutionY - focusY;
             deltaYSq = deltaYSq * deltaYSq;
-        
+
             intersectToFocusSq = deltaXSq + deltaYSq;
-        
+
             deltaXSq = X - focusX;
             deltaXSq = deltaXSq * deltaXSq;
-        
+
             deltaYSq = Y - focusY;
             deltaYSq = deltaYSq * deltaYSq;
-                
+
             currentToFocusSq = deltaXSq + deltaYSq;
             g11 = Math.sqrt(currentToFocusSq / intersectToFocusSq);
             prevGs[0] = g11;
-            
+
             X += a00; //incremental change in X, Y
-            Y += a10;	
+            Y += a10;
 
             //for every column (inner loop begins here)
             for (i=1; i <= w; i++) {
@@ -726,77 +705,78 @@ final class RadialGradientPaintContext extends MultipleGradientPaintContext {
                 g01 = g11;
                 g10 = prevGs[i];
 
+                dx = X - focusX;
                 // special case to avoid divide by zero or very near zero
-                if (((X-focusX)>-0.000001) &&
-                    ((X-focusX)< 0.000001)) {		   
+                if ( ( dx >-0.000001f ) &&
+                     ( dx < 0.000001f ))  {
                     solutionX = focusX;
                     solutionY = centerY;
                     solutionY += (Y > focusY)?trivial:-trivial;
                 }
-                else { 
+                else {
                     // Formula for Circle: (X-Xc)^2 + (Y-Yc)^2 - R^2 = 0
                     // Formula line:        Y = Slope*x + Y0;
-                    // 
+                    //
                     // So you substitue line into Circle and apply
                     // Quadradic formula.
 
 
                     //slope of the focus-current line
                     slope =   (Y - focusY) / (X - focusX);
-		    
+
                     yintcpt = Y - (slope * X); //y-intercept of that same line
-		    
+
                     //use the quadratic formula to calculate the intersection
-                    //point		  
-                    A = (slope * slope) + 1; 
-		    
+                    //point
+                    A = (slope * slope) + 1;
+
                     B =  precalc3 + (-2 * slope * (centerY - yintcpt));
-		    
+
                     C =  constC + (yintcpt* (yintcpt - precalc2));
-		    
+
                     det = Math.sqrt((B * B) - ( 4 * A * C));
-		    
+
                     solutionX = -B;
-		    
+
                     //choose the positive or negative root depending
                     //on where the X coord lies with respect to the focus.
                     solutionX += (X < focusX)?-det:det;
-		    
-                    solutionX = solutionX / (2 * A);//divisor
-		    
-                    solutionY = (slope * solutionX) + yintcpt;
-                }	                    	
 
-                //calculate the square of the distance from the current point 
-                //to the focus and the square of the distance from the 
+                    solutionX = solutionX / (2 * A);//divisor
+
+                    solutionY = (slope * solutionX) + yintcpt;
+                }
+
+                //calculate the square of the distance from the current point
+                //to the focus and the square of the distance from the
                 //intersection point to the focus. Want the squares so we can
                 //do 1 square root after division instead of 2 before.
                 deltaXSq = solutionX - focusX;
                 deltaXSq = deltaXSq * deltaXSq;
-        
+
                 deltaYSq = solutionY - focusY;
                 deltaYSq = deltaYSq * deltaYSq;
-        
+
                 intersectToFocusSq = deltaXSq + deltaYSq;
-        
+
                 deltaXSq = X - focusX;
                 deltaXSq = deltaXSq * deltaXSq;
-        
+
                 deltaYSq = Y - focusY;
                 deltaYSq = deltaYSq * deltaYSq;
-        
+
                 currentToFocusSq = deltaXSq + deltaYSq;
                 g11 = Math.sqrt(currentToFocusSq / intersectToFocusSq);
                 prevGs[i] = g11;
 
                 //Get the color at this point
                 pixels[indexer+i] = indexGradientAntiAlias
-                    ((float)((g00+g01+g10+g11)/4), 
+                    ((float)((g00+g01+g10+g11)/4),
                      (float)Math.max(Math.abs(g11-g00),
                                      Math.abs(g10-g01)));
 
                 X += a00; //incremental change in X, Y
-                Y += a10;	
+                Y += a10;
             } //end inner loop
             indexer += (w+adjust);
         } //end outer loop
