@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt.g2d;
 
 
@@ -67,15 +35,15 @@ import java.awt.image.renderable.RenderableImage;
 import java.text.AttributedCharacterIterator;
 
 /**
- * This concrete implementation of <tt>AbstractGraphics2D</tt> is a 
+ * This concrete implementation of <code>AbstractGraphics2D</code> is a 
  * simple help to programmers to get started with their own 
- * implementation of <tt>Graphics2D</tt>. 
- * <tt>DefaultGraphics2D</tt> implements all the abstract methods
- * is <tt>AbstractGraphics2D</tt> and makes it easy to start 
- * implementing a <tt>Graphic2D</tt> piece-meal.
+ * implementation of <code>Graphics2D</code>. 
+ * <code>DefaultGraphics2D</code> implements all the abstract methods
+ * is <code>AbstractGraphics2D</code> and makes it easy to start 
+ * implementing a <code>Graphic2D</code> piece-meal.
  *
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: DefaultGraphics2D.java,v 1.3 2003/08/08 11:39:04 vhardy Exp $
+ * @version $Id: DefaultGraphics2D.java 1372129 2012-08-12 15:31:50Z helder $
  * @see org.apache.batik.ext.awt.g2d.AbstractGraphics2D
  */
 public class DefaultGraphics2D extends AbstractGraphics2D {
@@ -197,7 +165,7 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * @see         java.awt.Component#paint
      * @see         java.awt.Component#update
      * @see         java.awt.Component#getGraphics
-     * @see         java.awt.Graphics#create
+     * @see         java.awt.Graphics#create()
      */
     public void dispose(){
         System.out.println("dispose");
@@ -210,14 +178,12 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * <code>Paint</code>, <code>Composite</code> and
      * <code>Stroke</code> attributes.
      * @param s the <code>Shape</code> to be rendered
-     * @see #setStroke
-     * @see #setPaint
+     * @see #setStroke(java.awt.Stroke)
+     * @see #setPaint(java.awt.Paint)
      * @see java.awt.Graphics#setColor
-     * @see #transform
-     * @see #setTransform
-     * @see #clip
-     * @see #setClip
-     * @see #setComposite
+     * @see #setTransform(AffineTransform)
+     * @see #setClip(Shape)
+     * @see #setComposite(java.awt.Composite)
      */
     public void draw(Shape s){
         System.out.println("draw(Shape)");
@@ -237,11 +203,9 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * noninvertible.
      * @param img the image to be rendered
      * @param xform the transformation from image space into user space
-     * @see #transform
-     * @see #setTransform
-     * @see #setComposite
-     * @see #clip
-     * @see #setClip
+     * @see #setTransform(AffineTransform)
+     * @see #setComposite(java.awt.Composite)
+     * @see #setClip(Shape)
      */
     public void drawRenderedImage(RenderedImage img,
                                   AffineTransform xform) {
@@ -272,11 +236,9 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      *{@link #drawRenderedImage(RenderedImage, AffineTransform) drawRenderedImage}.
      * @param img the image to be rendered
      * @param xform the transformation from image space into user space
-     * @see #transform
-     * @see #setTransform
-     * @see #setComposite
-     * @see #clip
-     * @see #setClip
+     * @see #setTransform(AffineTransform)
+     * @see #setComposite(java.awt.Composite)
+     * @see #setClip(Shape)
      * @see #drawRenderedImage
      */
      public void drawRenderableImage(RenderableImage img,
@@ -297,14 +259,16 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * left, in which case the coordinate supplied is the location of the
      * leftmost character on the baseline.
      * @param s the <code>String</code> to be rendered
-     * @param x,&nbsp;y the coordinates where the <code>String</code>
-     * should be rendered
-     * @see #setPaint
+     * @param x the x coordinate where the <code>String</code> should be
+     *          rendered
+     * @param y the y coordinate where the <code>String</code> should be
+     *          rendered
+     * @see #setPaint(java.awt.Paint)
      * @see java.awt.Graphics#setColor
      * @see java.awt.Graphics#setFont
-     * @see #setTransform
-     * @see #setComposite
-     * @see #setClip
+     * @see #setTransform(AffineTransform)
+     * @see #setComposite(java.awt.Composite)
+     * @see #setClip(Shape)
      */
     public void drawString(String s, float x, float y){
         System.out.println("drawString(String)");
@@ -325,13 +289,13 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * coordinate supplied is the location of the leftmost character
      * on the baseline.
      * @param iterator the iterator whose text is to be rendered
-     * @param x,&nbsp;y the coordinates where the iterator's text is to be
-     * rendered
-     * @see #setPaint
+     * @param x the x coordinate where the iterator's text is to be rendered
+     * @param y the y coordinate where the iterator's text is to be rendered
+     * @see #setPaint(java.awt.Paint)
      * @see java.awt.Graphics#setColor
-     * @see #setTransform
-     * @see #setComposite
-     * @see #setClip
+     * @see #setTransform(AffineTransform)
+     * @see #setComposite(java.awt.Composite)
+     * @see #setClip(Shape)
      */
      public void drawString(AttributedCharacterIterator iterator,
                             float x, float y) {
@@ -346,13 +310,11 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * include the <code>Clip</code>, <code>Transform</code>,
      * <code>Paint</code>, and <code>Composite</code>.
      * @param s the <code>Shape</code> to be filled
-     * @see #setPaint
+     * @see #setPaint(java.awt.Paint)
      * @see java.awt.Graphics#setColor
-     * @see #transform
-     * @see #setTransform
-     * @see #setComposite
-     * @see #clip
-     * @see #setClip
+     * @see #setTransform(AffineTransform)
+     * @see #setComposite(java.awt.Composite)
+     * @see #setClip(Shape)
      */
     public void fill(Shape s){
         System.err.println("fill");
@@ -363,7 +325,7 @@ public class DefaultGraphics2D extends AbstractGraphics2D {
      * <code>Graphics2D</code>.
      */
     public GraphicsConfiguration getDeviceConfiguration(){
-        System.out.println("getDeviceConviguration");
+        System.out.println("getDeviceConfiguration");
         return null;
     }
 
