@@ -1,62 +1,32 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt.geom;
 
 import java.awt.Shape;
+import java.awt.Rectangle;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.Arc2D;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.PathIterator;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
+import java.util.Arrays;
 
 /**
  * The <code>ExtendedGeneralPath</code> class represents a geometric
@@ -69,17 +39,19 @@ import java.awt.geom.Rectangle2D;
  * path segments. For futher details, see the SVG Appendix&nbsp;F.6
  *
  * @author <a href="mailto:Thierry.Kormann@sophia.inria.fr">Thierry Kormann</a>
- * @version $Id: ExtendedGeneralPath.java,v 1.6 2003/08/09 16:58:42 deweese Exp $
+ * @version $Id: ExtendedGeneralPath.java 1340064 2012-05-18 11:05:02Z deweese $
  */
 public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
 
     /** The enclosed general path. */
     protected GeneralPath path;
 
-    int       numVals = 0;
-    int       numSeg  = 0;
-    double [] values  = null;
-    int    [] types   = null;
+    int      numVals = 0;
+    int      numSeg  = 0;
+    float [] values  = null;
+    int   [] types   = null;
+
+    float    mx, my, cx, cy;
 
    /**
      * Constructs a new <code>ExtendedGeneralPath</code>.
@@ -121,9 +93,10 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
      * indicate if we increase or decrease the angles and the final
      * point of the arc.
      *
-     * @param rx,&nbsp;ry the radii of the ellipse
+     * @param rx the x radius of the ellipse
+     * @param ry the y radius of the ellipse
      *
-     * @param theta the angle from the x-axis of the current
+     * @param angle the angle from the x-axis of the current
      * coordinate system to the x-axis of the ellipse in degrees.
      *
      * @param largeArcFlag the large arc flag. If true the arc
@@ -134,32 +107,33 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
      * center to arc sweeps through decreasing angles otherwise it
      * sweeps through increasing angles
      *
-     * @param x,&nbsp;y the absolute coordinates of the final point of
-     * the arc.
+     * @param x the absolute x coordinate of the final point of the arc.
+     * @param y the absolute y coordinate of the final point of the arc.
      */
-    public synchronized void arcTo(double rx, double ry,
-                                   double angle,
+    public synchronized void arcTo(float rx, float ry,
+                                   float angle,
                                    boolean largeArcFlag,
                                    boolean sweepFlag,
-                                   double x, double y) {
+                                   float x, float y) {
 
         // Ensure radii are valid
         if (rx == 0 || ry == 0) {
-            lineTo((float) x, (float) y);
+            lineTo(x, y);
             return;
         }
 
+        checkMoveTo();  // check if prev command was moveto
+
         // Get the current (x, y) coordinates of the path
-        Point2D p2d = path.getCurrentPoint();
-        double x0 = p2d.getX();
-        double y0 = p2d.getY();
+        double x0 = cx;
+        double y0 = cy;
         if (x0 == x && y0 == y) {
             // If the endpoints (x, y) and (x0, y0) are identical, then this
             // is equivalent to omitting the elliptical arc segment entirely.
             return;
         }
 
-        Arc2D arc = computeArc(x0, y0, rx, ry, angle, 
+        Arc2D arc = computeArc(x0, y0, rx, ry, angle,
                                largeArcFlag, sweepFlag, x, y);
         if (arc == null) return;
 
@@ -175,16 +149,16 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
         values[numVals++] = angle;
         values[numVals++] = largeArcFlag?1:0;
         values[numVals++] = sweepFlag?1:0;
-        values[numVals++] = x;
-        values[numVals++] = y;
+        cx = values[numVals++] = x;
+        cy = values[numVals++] = y;
     }
 
 
-    /** 
-     * This constructs an unrotated Arc2D from the SVG specification of an 
+    /**
+     * This constructs an unrotated Arc2D from the SVG specification of an
      * Elliptical arc.  To get the final arc you need to apply a rotation
      * transform such as:
-     * 
+     *
      * AffineTransform.getRotateInstance
      *     (angle, arc.getX()+arc.getWidth()/2, arc.getY()+arc.getHeight()/2);
      */
@@ -220,9 +194,10 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
         double Py1 = y1 * y1;
         // check that radii are large enough
         double radiiCheck = Px1/Prx + Py1/Pry;
-        if (radiiCheck > 1) {
-            rx = Math.sqrt(radiiCheck) * rx;
-            ry = Math.sqrt(radiiCheck) * ry;
+        if (radiiCheck > 0.99999) {  // don't cut it too close
+            double radiiScale = Math.sqrt(radiiCheck) * 1.00001;
+            rx = radiiScale * rx;
+            ry = radiiScale * ry;
             Prx = rx * rx;
             Pry = ry * ry;
         }
@@ -256,13 +231,13 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
         // Compute the angle start
         n = Math.sqrt((ux * ux) + (uy * uy));
         p = ux; // (1 * ux) + (0 * uy)
-        sign = (uy < 0) ? -1d : 1d;
+        sign = (uy < 0) ? -1.0 : 1.0;
         double angleStart = Math.toDegrees(sign * Math.acos(p / n));
 
         // Compute the angle extent
         n = Math.sqrt((ux * ux + uy * uy) * (vx * vx + vy * vy));
         p = ux * vx + uy * vy;
-        sign = (ux * vy - uy * vx < 0) ? -1d : 1d;
+        sign = (ux * vy - uy * vx < 0) ? -1.0 : 1.0;
         double angleExtent = Math.toDegrees(sign * Math.acos(p / n));
         if(!sweepFlag && angleExtent > 0) {
             angleExtent -= 360f;
@@ -290,38 +265,40 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
     public synchronized void moveTo(float x, float y) {
-        path.moveTo(x, y);
-
+        // Don't add moveto to general path unless there is a reason.
         makeRoom(2);
         types [numSeg++]  = PathIterator.SEG_MOVETO;
-        values[numVals++] = x;
-        values[numVals++] = y;
+        cx = mx = values[numVals++] = x;
+        cy = my = values[numVals++] = y;
+
     }
 
     /**
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
     public synchronized void lineTo(float x, float y) {
+        checkMoveTo();  // check if prev command was moveto
         path.lineTo(x, y);
 
         makeRoom(2);
         types [numSeg++]  = PathIterator.SEG_LINETO;
-        values[numVals++] = x;
-        values[numVals++] = y;
+        cx = values[numVals++] = x;
+        cy = values[numVals++] = y;
     }
 
     /**
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
     public synchronized void quadTo(float x1, float y1, float x2, float y2) {
+        checkMoveTo();  // check if prev command was moveto
         path.quadTo(x1, y1, x2, y2);
 
         makeRoom(4);
         types [numSeg++]  = PathIterator.SEG_QUADTO;
         values[numVals++] = x1;
         values[numVals++] = y1;
-        values[numVals++] = x2;
-        values[numVals++] = y2;
+        cx = values[numVals++] = x2;
+        cy = values[numVals++] = y2;
     }
 
     /**
@@ -330,6 +307,7 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
     public synchronized void curveTo(float x1, float y1,
                                      float x2, float y2,
                                      float x3, float y3) {
+        checkMoveTo();   // check if prev command was moveto
         path.curveTo(x1, y1, x2, y2, x3, y3);
 
         makeRoom(6);
@@ -338,18 +316,50 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
         values[numVals++] = y1;
         values[numVals++] = x2;
         values[numVals++] = y2;
-        values[numVals++] = x3;
-        values[numVals++] = y3;
+        cx = values[numVals++] = x3;
+        cy = values[numVals++] = y3;
     }
 
     /**
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
     public synchronized void closePath() {
-        path.closePath();
+        // Don't double close path.
+        if ((numSeg != 0) && (types[numSeg-1] == PathIterator.SEG_CLOSE))
+            return;
+
+        // Only close path if the previous command wasn't a moveto
+        if ((numSeg != 0) && (types[numSeg-1] != PathIterator.SEG_MOVETO))
+            path.closePath();
 
         makeRoom(0);
         types [numSeg++]  = PathIterator.SEG_CLOSE;
+        cx = mx;
+        cy = my;
+    }
+
+    /**
+     * Checks if previous command was a moveto command,
+     * skipping a close command (if present).
+     */
+    protected void checkMoveTo() {
+        if (numSeg == 0) return;
+
+        switch(types[numSeg-1]) {
+
+        case PathIterator.SEG_MOVETO:
+            path.moveTo(values[numVals-2], values[numVals-1]);
+            break;
+
+        case PathIterator.SEG_CLOSE:
+            if (numSeg == 1) return;
+            if (types[numSeg-2] == PathIterator.SEG_MOVETO)
+                path.moveTo(values[numVals-2], values[numVals-1]);
+            break;
+
+        default:
+            break;
+        }
     }
 
     /**
@@ -363,17 +373,18 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
     public void append(PathIterator pi, boolean connect) {
-        
+        double [] vals = new double[6];
+
         while (!pi.isDone()) {
-            double [] vals = new double[6];
+            Arrays.fill( vals, 0 );
             int type = pi.currentSegment(vals);
             pi.next();
             if (connect && (numVals != 0)) {
                 if (type == PathIterator.SEG_MOVETO) {
                     double x = vals[0];
                     double y = vals[1];
-                    if ((x != values[numVals-2]) ||
-                        (y != values[numVals-1])) {
+                    if ((x != cx) ||
+                        (y != cy)) {
                         // Change MOVETO to LINETO.
                         type = PathIterator.SEG_LINETO;
                     } else {
@@ -388,15 +399,15 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
 
             switch(type) {
             case PathIterator.SEG_CLOSE:   closePath(); break;
-            case PathIterator.SEG_MOVETO:  
+            case PathIterator.SEG_MOVETO:
                 moveTo ((float)vals[0], (float)vals[1]); break;
-            case PathIterator.SEG_LINETO:  
+            case PathIterator.SEG_LINETO:
                 lineTo ((float)vals[0], (float)vals[1]); break;
-            case PathIterator.SEG_QUADTO:  
-                quadTo ((float)vals[0], (float)vals[1], 
+            case PathIterator.SEG_QUADTO:
+                quadTo ((float)vals[0], (float)vals[1],
                         (float)vals[2], (float)vals[3]); break;
-            case PathIterator.SEG_CUBICTO: 
-                curveTo((float)vals[0], (float)vals[1], 
+            case PathIterator.SEG_CUBICTO:
+                curveTo((float)vals[0], (float)vals[1],
                         (float)vals[2], (float)vals[3],
                         (float)vals[4], (float)vals[5]); break;
             }
@@ -407,20 +418,21 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
     public void append(ExtendedPathIterator epi, boolean connect) {
+        float[] vals = new float[ 7 ];
         while (!epi.isDone()) {
-            double [] vals = new double[7];
+            Arrays.fill( vals, 0 );
             int type = epi.currentSegment(vals);
             epi.next();
             if (connect && (numVals != 0)) {
                 if (type == PathIterator.SEG_MOVETO) {
-                    double x = vals[0];
-                    double y = vals[1];
-                    if ((x != values[numVals-2]) ||
-                        (y != values[numVals-1])) {
+                    float x = vals[0];
+                    float y = vals[1];
+                    if ((x != cx) ||
+                        (y != cy)) {
                         // Change MOVETO to LINETO.
                         type = PathIterator.SEG_LINETO;
                     } else {
-                        // Redundent segment (move to current loc) drop it...
+                        // Redundant segment (move to current loc) drop it...
                         if (epi.isDone()) break; // Nothing interesting
                         type = epi.currentSegment(vals);
                         epi.next();
@@ -431,20 +443,20 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
 
             switch(type) {
             case PathIterator.SEG_CLOSE:   closePath(); break;
-            case PathIterator.SEG_MOVETO:  
-                moveTo ((float)vals[0], (float)vals[1]); break;
-            case PathIterator.SEG_LINETO:  
-                lineTo ((float)vals[0], (float)vals[1]); break;
-            case PathIterator.SEG_QUADTO:  
-                quadTo ((float)vals[0], (float)vals[1], 
-                        (float)vals[2], (float)vals[3]); break;
-            case PathIterator.SEG_CUBICTO: 
-                curveTo((float)vals[0], (float)vals[1], 
-                        (float)vals[2], (float)vals[3],
-                        (float)vals[4], (float)vals[5]); break;
-            case ExtendedPathIterator.SEG_ARCTO:   
-                arcTo  (vals[0], vals[1], vals[2], 
-                        (vals[3]!=0), (vals[4]!=0), 
+            case PathIterator.SEG_MOVETO:
+                moveTo (vals[0], vals[1]); break;
+            case PathIterator.SEG_LINETO:
+                lineTo (vals[0], vals[1]); break;
+            case PathIterator.SEG_QUADTO:
+                quadTo (vals[0], vals[1],
+                        vals[2], vals[3]); break;
+            case PathIterator.SEG_CUBICTO:
+                curveTo(vals[0], vals[1],
+                        vals[2], vals[3],
+                        vals[4], vals[5]); break;
+            case ExtendedPathIterator.SEG_ARCTO:
+                arcTo  (vals[0], vals[1], vals[2],
+                        (vals[3]!=0), (vals[4]!=0),
                         vals[5], vals[6]); break;
             }
         }
@@ -465,10 +477,11 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
     }
 
     /**
-     * Delegates to the enclosed <code>GeneralPath</code>.
+     * get the current position or <code>null</code>.
      */
     public synchronized Point2D getCurrentPoint() {
-        return path.getCurrentPoint();
+        if (numVals == 0) return null;
+        return new Point2D.Double(cx, cy);
     }
 
     /**
@@ -479,6 +492,8 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
 
         numSeg = 0;
         numVals = 0;
+        values = null;
+        types = null;
     }
 
     /**
@@ -500,7 +515,7 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
     /**
      * Delegates to the enclosed <code>GeneralPath</code>.
      */
-    public java.awt.Rectangle getBounds() {
+    public synchronized Rectangle getBounds() {
         return path.getBounds();
     }
 
@@ -578,12 +593,16 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
         int segNum = 0;
         int valsIdx = 0;
 
+        public int currentSegment() {
+            return types[segNum];
+        }
+
         public int currentSegment(double[] coords) {
             int ret = types[segNum];
             switch (ret) {
             case SEG_CLOSE: break;
-            case SEG_MOVETO: 
-            case SEG_LINETO: 
+            case SEG_MOVETO:
+            case SEG_LINETO:
                 coords[0] = values[valsIdx];
                 coords[1] = values[valsIdx+1];
                 break;
@@ -611,7 +630,7 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
                 coords[6] = values[valsIdx+6];
                 break;
             }
-            // System.out.println("Seg: [" + segNum + "] type: " + ret + 
+            // System.out.println("Seg: [" + segNum + "] type: " + ret +
             //                    " vals: [" + coords[0] + ", " + coords[1] +
             //                    "]");
             return ret;
@@ -621,33 +640,19 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
             int ret = types[segNum];
             switch (ret) {
             case SEG_CLOSE: break;
-            case SEG_MOVETO: 
-            case SEG_LINETO: 
-                coords[0] = (float)values[valsIdx];
-                coords[1] = (float)values[valsIdx+1];
+            case SEG_MOVETO:
+            case SEG_LINETO:
+                coords[0] = values[valsIdx];
+                coords[1] = values[valsIdx+1];
                 break;
             case SEG_QUADTO:
-                coords[0] = (float)values[valsIdx];
-                coords[1] = (float)values[valsIdx+1];
-                coords[2] = (float)values[valsIdx+2];
-                coords[3] = (float)values[valsIdx+3];
+                System.arraycopy( values, valsIdx, coords, 0, 4 );
                 break;
             case SEG_CUBICTO:
-                coords[0] = (float)values[valsIdx];
-                coords[1] = (float)values[valsIdx+1];
-                coords[2] = (float)values[valsIdx+2];
-                coords[3] = (float)values[valsIdx+3];
-                coords[4] = (float)values[valsIdx+4];
-                coords[5] = (float)values[valsIdx+5];
+                System.arraycopy( values, valsIdx, coords, 0, 6 );
                 break;
             case SEG_ARCTO:
-                coords[0] = (float)values[valsIdx];
-                coords[1] = (float)values[valsIdx+1];
-                coords[2] = (float)values[valsIdx+2];
-                coords[3] = (float)values[valsIdx+3];
-                coords[4] = (float)values[valsIdx+4];
-                coords[5] = (float)values[valsIdx+5];
-                coords[6] = (float)values[valsIdx+6];
+                System.arraycopy( values, valsIdx, coords, 0, 7 );
                 break;
             }
             return ret;
@@ -663,7 +668,7 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
             int type = types[segNum++];
             switch (type) {
             case SEG_CLOSE: break;
-            case SEG_MOVETO: 
+            case SEG_MOVETO:                   // fallthrough is intended
             case SEG_LINETO: valsIdx+=2; break;
             case SEG_QUADTO: valsIdx+=4; break;
             case SEG_CUBICTO:valsIdx+=6; break;
@@ -680,12 +685,16 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
             ExtendedGeneralPath result = (ExtendedGeneralPath) super.clone();
             result.path = (GeneralPath) path.clone();
 
-            result.values = new double[values.length];
-            System.arraycopy(result.values, 0, values, 0, values.length);
+            if ( values != null ){
+                result.values = new float[values.length];
+                System.arraycopy(values, 0, result.values, 0, values.length);
+            }
             result.numVals = numVals;
 
-            result.types = new int[types.length];
-            System.arraycopy(result.types, 0, types, 0, types.length);
+            if ( types != null ){
+                result.types = new int[types.length];
+                System.arraycopy(types, 0, result.types, 0, types.length);
+            }
             result.numSeg = numSeg;
 
             return result;
@@ -693,21 +702,29 @@ public class ExtendedGeneralPath implements ExtendedShape, Cloneable {
         return null;
     }
 
+    /**
+     * Make sure, that the requested number of slots in vales[] are available.
+     * Must be called even for numValues = 0, because it is also
+     * used for initialization of those arrays.
+     *
+     * @param numValues number of requested coordinates
+     */
     private void makeRoom(int numValues) {
         if (values == null) {
-            values = new double[2*numValues];
+            values = new float[2*numValues];
             types  = new int[2];
             numVals = 0;
             numSeg  = 0;
             return;
         }
-        
-        if ((numVals + numValues) > values.length) {
+
+        int newSize = numVals + numValues;
+        if ( newSize > values.length) {
             int nlen = values.length*2;
-            if (nlen < (numVals + numValues))
-                nlen = numVals + numValues;
-        
-            double [] nvals = new double[nlen];
+            if ( nlen < newSize )
+                nlen = newSize;
+
+            float [] nvals = new float[nlen];
             System.arraycopy(values, 0, nvals, 0, numVals);
             values = nvals;
         }
