@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.ext.awt.image.rendered;
 
 import java.awt.Rectangle;
@@ -75,7 +43,7 @@ import org.apache.batik.ext.awt.image.GraphicsUtil;
  * of the SVG specification.
  *
  * @author <a href="mailto:sheng.pei@sun.com">Sheng Pei</a>
- * @version $Id: MorphologyOp.java,v 1.5 2003/08/09 16:58:42 deweese Exp $
+ * @version $Id: MorphologyOp.java 489226 2006-12-21 00:05:36Z cam $
  */
 public class MorphologyOp implements BufferedImageOp, RasterOp {
     /**
@@ -124,7 +92,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
      */
     public MorphologyOp (int radiusX, int radiusY, boolean doDilation){
         if (radiusX<=0 || radiusY<=0){
-            throw new IllegalArgumentException(new String("The radius of X-axis or Y-axis should not be Zero or Negatives."));
+            throw new IllegalArgumentException( "The radius of X-axis or Y-axis should not be Zero or Negatives." );
         }
         else {
             this.radiusX = radiusX;
@@ -181,7 +149,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
     }
 
     private boolean isCompatible(ColorModel colorModel,
-				 SampleModel sampleModel){
+                                 SampleModel sampleModel){
         ColorSpace cs = colorModel.getColorSpace();
         // Check that model is sRGB or linear RGB
         if((cs != ColorSpace.getInstance(ColorSpace.CS_sRGB))
@@ -214,7 +182,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
         // Check model is ok: should be SinglePixelPackedSampleModel
         if(!(model instanceof SinglePixelPackedSampleModel))
             throw new IllegalArgumentException
-                ("MorphologyOp only works with Rasters " + 
+                ("MorphologyOp only works with Rasters " +
                  "using SinglePixelPackedSampleModels");
         // Check number of bands
         int nBands = model.getNumBands();
@@ -227,7 +195,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
                 ("MorphologyOp only works with Rasters using DataBufferInt");
 
         // Check bit masks
-        int bitOffsets[]=((SinglePixelPackedSampleModel)model).getBitOffsets();
+        int[] bitOffsets=((SinglePixelPackedSampleModel)model).getBitOffsets();
         for(int i=0; i<bitOffsets.length; i++){
             if(bitOffsets[i]%8 != 0)
                 throw new IllegalArgumentException
@@ -246,7 +214,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
         return src.createCompatibleWritableRaster();
     }
 
-    public BufferedImage createCompatibleDestImage(BufferedImage src, 
+    public BufferedImage createCompatibleDestImage(BufferedImage src,
                                                    ColorModel destCM){
         BufferedImage dest = null;
         if(destCM==null)
@@ -257,7 +225,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
                                                    src.getHeight());
         checkCompatible(destCM, wr.getSampleModel());
 
-        dest = new BufferedImage(destCM, wr, 
+        dest = new BufferedImage(destCM, wr,
                                  destCM.isAlphaPremultiplied(), null);
         return dest;
     }
@@ -306,8 +274,8 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
         final int dstScanStride = ((SinglePixelPackedSampleModel)dest.getSampleModel()).getScanlineStride();
 
         // Access the pixel value array
-        final int srcPixels[] = srcDB.getBankData()[0];
-        final int destPixels[] = dstDB.getBankData()[0];
+        final int[] srcPixels = srcDB.getBankData()[0];
+        final int[] destPixels = dstDB.getBankData()[0];
 
         // The pointer of src and dest indicating where the pixel values are
         int sp, dp;
@@ -348,7 +316,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
                     r1 = currentPixel&0xff0000;
                     g1 = currentPixel&0xff00;
                     b1 = currentPixel&0xff;
-                    
+
                     if (isBetter(a1, a, doDilation)){
                         a = a1;
                     }
@@ -584,7 +552,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
         final int dstScanStride = ((SinglePixelPackedSampleModel)dest.getSampleModel()).getScanlineStride();
 
         // Access the pixel value array
-        final int destPixels[] = dstDB.getBankData()[0];
+        final int[] destPixels = dstDB.getBankData()[0];
 
         // The pointer of src and dest indicating where the pixel values are
         int dp, cp;
@@ -895,8 +863,8 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
         final int dstScanStride = ((SinglePixelPackedSampleModel)dest.getSampleModel()).getScanlineStride();
 
         // Access the pixel value array
-        final int srcPixels[] = srcDB.getBankData()[0];
-        final int destPixels[] = dstDB.getBankData()[0];
+        final int[] srcPixels = srcDB.getBankData()[0];
+        final int[] destPixels = dstDB.getBankData()[0];
 
         // The pointer of src and dest indicating where the pixel values are
         int sp, dp, cp;
@@ -1584,7 +1552,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
 
             src = new BufferedImage(srcCMPre, src.getRaster(),
                                     true, null);
-            
+
             GraphicsUtil.copyData(origSrc, src);
         }
 
@@ -1647,7 +1615,7 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
             srcCMUnpre = GraphicsUtil.coerceColorModel(srcCM, false);
             src = new BufferedImage(srcCMUnpre, src.getRaster(),
                                     false, null);
-            
+
             GraphicsUtil.copyData(origSrc, src);
         }
 
@@ -1682,13 +1650,13 @@ public class MorphologyOp implements BufferedImageOp, RasterOp {
             (dest.isAlphaPremultiplied() != finalDest.isAlphaPremultiplied())){
             // Coerce our source back the way it was...
             System.out.println("Dest: " + dest.isAlphaPremultiplied() +
-                               " finalDest: " + 
+                               " finalDest: " +
                                finalDest.isAlphaPremultiplied());
-            
+
             GraphicsUtil.copyData(dest, finalDest);
         }
         return finalDest;
-    } 
+    }
       */
 }
 
