@@ -26,15 +26,15 @@ import java.util.Iterator;
 import java.util.LinkedList;
 import java.util.List;
 
+import org.apache.batik.bridge.GlyphLayout;
 import org.apache.batik.gvt.font.GVTGlyphVector;
 import org.apache.batik.gvt.font.MultiGlyphVector;
-import org.apache.batik.gvt.text.GlyphLayout;
 
 /**
  * A GlyphLayout class for SVG 1.2 flowing text.
  *
  * @author <a href="mailto:deweese@apache.org">deweese</a>
- * @version $Id: FlowExtGlyphLayout.java 475477 2006-11-15 22:44:28Z cam $
+ * @version $Id: FlowExtGlyphLayout.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class FlowExtGlyphLayout extends GlyphLayout {
 
@@ -99,8 +99,8 @@ public class FlowExtGlyphLayout extends GlyphLayout {
     // display order not logical order).
 
     /**
-     * This will wrap the text associated with <tt>aci</tt> and
-     * <tt>layouts</tt>.
+     * This will wrap the text associated with <code>aci</code> and
+     * <code>layouts</code>.
      * @param acis An array of Attributed Charater Iterators containing the 
      *             text to wrap.  There is one aci per text chunk
      *             (which maps to flowPara elements. Used to access
