@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt;
 
 import java.awt.AlphaComposite;
@@ -74,14 +42,15 @@ import org.apache.batik.gvt.event.GraphicsNodeChangeListener;
 import org.apache.batik.gvt.filter.GraphicsNodeRable;
 import org.apache.batik.gvt.filter.GraphicsNodeRable8Bit;
 import org.apache.batik.gvt.filter.Mask;
+import org.apache.batik.util.HaltingThread;
 
 /**
- * A partial implementation of the <tt>GraphicsNode</tt> interface.
+ * A partial implementation of the <code>GraphicsNode</code> interface.
  *
  * @author <a href="mailto:Thierry.Kormann@sophia.inria.fr">Thierry Kormann</a>
  * @author <a href="mailto:etissandier@ilog.fr">Emmanuel Tissandier</a>
  * @author <a href="mailto:Thomas.DeWeeese@Kodak.com">Thomas DeWeese</a>
- * @version $Id: AbstractGraphicsNode.java,v 1.52 2003/10/31 22:39:41 deweese Exp $
+ * @version $Id: AbstractGraphicsNode.java 1372129 2012-08-12 15:31:50Z helder $
  */
 public abstract class AbstractGraphicsNode implements GraphicsNode {
 
@@ -224,7 +193,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
                 inverseTransform = transform.createInverse();
             }catch(NoninvertibleTransformException e){
                 // Should never happen.
-                throw new Error();
+                throw new Error( e.getMessage() );
             }
         } else {
             // The transform is not invertible. Use the same
@@ -251,7 +220,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     }
 
     /**
-     * Returns the concatenated transform of this node. i.e., this
+     * Returns the concatenated transform of this node. That is, this
      * node's transform preconcatenated with it's parent's transforms.
      */
     public AffineTransform getGlobalTransform(){
@@ -269,7 +238,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     /**
      * Sets the composite of this node.
      *
-     * @param composite the composite of this node
+     * @param newComposite the composite of this node
      */
     public void setComposite(Composite newComposite) {
         fireGraphicsNodeChangeStarted();
@@ -293,6 +262,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     public void setVisible(boolean isVisible) {
         fireGraphicsNodeChangeStarted();
         this.isVisible = isVisible;
+        invalidateGeometryCache();
         fireGraphicsNodeChangeCompleted();
     }
 
@@ -304,6 +274,9 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     }
 
     public void setClip(ClipRable newClipper) {
+        if ((newClipper == null) && (this.clip == null))
+            return; // No change still no clip.
+
         fireGraphicsNodeChangeStarted();
         invalidateGeometryCache();
         this.clip = newClipper;
@@ -358,7 +331,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
      */
     public void setRenderingHints(RenderingHints newHints) {
         fireGraphicsNodeChangeStarted();
-        this.hints = newHints;
+        hints = newHints;
         fireGraphicsNodeChangeCompleted();
     }
 
@@ -375,9 +348,12 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
      * @param newMask the new mask of this node
      */
     public void setMask(Mask newMask) {
+        if ((newMask == null) && (mask == null))
+            return; // No change still no mask.
+
         fireGraphicsNodeChangeStarted();
         invalidateGeometryCache();
-        this.mask = newMask;
+        mask = newMask;
         fireGraphicsNodeChangeCompleted();
     }
 
@@ -394,9 +370,12 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
      * @param newFilter the new filter of this node
      */
     public void setFilter(Filter newFilter) {
+        if ((newFilter == null) && (filter == null))
+            return; // No change still no filter.
+
         fireGraphicsNodeChangeStarted();
         invalidateGeometryCache();
-        this.filter = newFilter;
+        filter = newFilter;
         fireGraphicsNodeChangeCompleted();
     }
 
@@ -410,7 +389,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     /**
      * Returns the GraphicsNodeRable for this node.  This
      * GraphicsNodeRable is the Renderable (Filter) before any of the
-     * filter operations have been applied.  
+     * filter operations have been applied.
      */
     public Filter getGraphicsNodeRable(boolean createIfNeeded) {
         GraphicsNodeRable ret = null;
@@ -428,7 +407,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     /**
      * Returns the GraphicsNodeRable for this node.  This
      * GraphicsNodeRable is the Renderable (Filter) after all of the
-     * filter operations have been applied.  
+     * filter operations have been applied.
      */
     public Filter getEnableBackgroundGraphicsNodeRable
         (boolean createIfNeeded) {
@@ -455,66 +434,70 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
      * @param g2d the Graphics2D to use
      */
     public void paint(Graphics2D g2d){
-        // first, make sure we haven't been interrupted
-        if (Thread.currentThread().isInterrupted()) {
-            return;
-        }
-
         if ((composite != null) &&
             (composite instanceof AlphaComposite)) {
             AlphaComposite ac = (AlphaComposite)composite;
             if (ac.getAlpha() < 0.001)
                 return;         // No point in drawing
         }
+        Rectangle2D bounds = getBounds();
+        if (bounds == null) return;
 
-        // Set up graphic context. It is important to setup the transform first,
-        // because the clip is defined in this node's user space.
-        Shape defaultClip = g2d.getClip();
-        Composite defaultComposite = g2d.getComposite();
-        AffineTransform defaultTransform = g2d.getTransform();
-        RenderingHints defaultHints = null;
+        // Set up graphic context. It is important to setup the
+        // transform first, because the clip is defined in this node's
+        // user space.
+        Composite       defaultComposite = null;
+        AffineTransform defaultTransform = null;
+        RenderingHints  defaultHints     = null;
+        Graphics2D      baseG2d          = null;
 
-        if (hints != null) {
-            defaultHints = g2d.getRenderingHints();
-            g2d.addRenderingHints(hints);
-        }
-        if (transform != null) {
-            g2d.transform(transform);
-        }
-        if (composite != null) {
-            g2d.setComposite(composite);
-        }
-        if (clip != null){
+        if (clip != null)  {
+            baseG2d = g2d;
+            g2d = (Graphics2D)g2d.create();
+            if (hints != null)
+                g2d.addRenderingHints(hints);
+            if (transform != null)
+                g2d.transform(transform);
+            if (composite != null)
+                g2d.setComposite(composite);
             g2d.clip(clip.getClipPath());
+        } else {
+            if (hints != null) {
+                defaultHints = g2d.getRenderingHints();
+                g2d.addRenderingHints(hints);
+            }
+            if (transform != null) {
+                defaultTransform = g2d.getTransform();
+                g2d.transform(transform);
+            }
+            if (composite != null) {
+                defaultComposite = g2d.getComposite();
+                g2d.setComposite(composite);
+            }
         }
 
         Shape curClip = g2d.getClip();
-        g2d.setRenderingHint(RenderingHintsKeyExt.KEY_AREA_OF_INTEREST, 
+        g2d.setRenderingHint(RenderingHintsKeyExt.KEY_AREA_OF_INTEREST,
                              curClip);
 
         // Check if any painting is needed at all. Get the clip (in user space)
         // and see if it intersects with this node's bounds (in user space).
         boolean paintNeeded = true;
-        Rectangle2D bounds = getBounds();
         Shape g2dClip = curClip; //g2d.getClip();
         if (g2dClip != null) {
-            Rectangle2D clipBounds = g2dClip.getBounds2D();
-            if(bounds != null && !bounds.intersects(clipBounds.getX(),
-                                                    clipBounds.getY(),
-                                                    clipBounds.getWidth(),
-                                                    clipBounds.getHeight())){
+            Rectangle2D cb = g2dClip.getBounds2D();
+            if(!bounds.intersects(cb.getX(),     cb.getY(),
+                                  cb.getWidth(), cb.getHeight()))
                 paintNeeded = false;
-            }
         }
 
         // Only paint if needed.
         if (paintNeeded){
             boolean antialiasedClip = false;
-            if(clip != null){
-                antialiasedClip = false;
-                    /*                    isAntialiasedClip(g2d.getTransform(),
-                                      g2d.getRenderingHints(),
-                                      clip.getClipPath());*/
+            if ((clip != null) && clip.getUseAntialiasedClip()) {
+                antialiasedClip = isAntialiasedClip(g2d.getTransform(),
+                                                    g2d.getRenderingHints(),
+                                                    clip.getClipPath());
             }
 
             boolean useOffscreen = isOffscreenBufferNeeded();
@@ -549,6 +532,12 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
                     filteredImage = clip;
                 }
 
+                baseG2d = g2d;
+                // Only muck with the clip on a 'child'
+                // graphics 2D otherwise when we restore the
+                // clip it might 'wander' by a pixel.
+                g2d = (Graphics2D)g2d.create();
+
                 if(antialiasedClip){
                     // Remove hard edged clip
                     g2d.setClip(null);
@@ -559,17 +548,24 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
 
                 org.apache.batik.ext.awt.image.GraphicsUtil.drawImage
                     (g2d, filteredImage);
+
+                g2d.dispose();
+                g2d = baseG2d;
+                baseG2d = null;// Don't leave null we need g2d restored...
             }
         }
 
         // Restore default rendering attributes
-        if (defaultHints != null) {
-            g2d.setRenderingHints(defaultHints);
-        }
-        g2d.setTransform(defaultTransform);
-        g2d.setClip(defaultClip);
-        if (composite != null) {
-            g2d.setComposite(defaultComposite);
+        if (baseG2d != null) {
+            g2d.dispose();
+        } else {
+            if (defaultHints != null)
+                g2d.setRenderingHints(defaultHints);
+            if (defaultTransform != null)
+                g2d.setTransform(defaultTransform);
+            if (defaultComposite != null) {
+                g2d.setComposite(defaultComposite);
+            }
         }
     }
 
@@ -579,11 +575,11 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     private void traceFilter(Filter filter, String prefix){
         System.out.println(prefix + filter.getClass().getName());
         System.out.println(prefix + filter.getBounds2D());
-        java.util.Vector sources = filter.getSources();
+        List sources = filter.getSources();
         int nSources = sources != null ? sources.size() : 0;
         prefix += "\t";
         for(int i=0; i<nSources; i++){
-            Filter source = (Filter)sources.elementAt(i);
+            Filter source = (Filter)sources.get(i);
             traceFilter(source, prefix);
         }
 
@@ -607,9 +603,8 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     protected boolean isAntialiasedClip(AffineTransform usr2dev,
                                         RenderingHints hints,
                                         Shape clip){
-        //
         // Antialias clip if:
-        // + Antialiasing is on *or* rendering quality is on
+        // + The KEY_CLIP_ANTIALIASING is true.
         // *and*
         // + clip is not null
         // *and*
@@ -620,18 +615,19 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
         // of the current Graphics2D's clip and this node's clip)
         // is not a rectangle.
         //
-        boolean antialiased = false;
-        if((hints.get(RenderingHints.KEY_ANTIALIASING) ==
-            RenderingHints.VALUE_ANTIALIAS_ON) ||
-           (hints.get(RenderingHints.KEY_RENDERING) ==
-            RenderingHints.VALUE_RENDER_QUALITY)){
-            if(!(clip instanceof Rectangle2D &&
-                 usr2dev.getShearX() == 0 &&
-                 usr2dev.getShearY() == 0)){
-                antialiased = true;
-            }
-        }
-        return antialiased;
+        if (clip == null) return false;
+
+        Object val = hints.get(RenderingHintsKeyExt.KEY_TRANSCODING);
+        if ((val == RenderingHintsKeyExt.VALUE_TRANSCODING_PRINTING) ||
+            (val == RenderingHintsKeyExt.VALUE_TRANSCODING_VECTOR))
+            return false;
+
+        if(!(clip instanceof Rectangle2D &&
+             usr2dev.getShearX() == 0 &&
+             usr2dev.getShearY() == 0))
+            return true;
+
+        return false;
     }
 
     //
@@ -790,10 +786,10 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
 
             bounds = normalizeRectangle(bounds);
 
-            // Make sure we haven't been interrupted
-            if (Thread.currentThread().isInterrupted()) {
-                // The Thread has been interrupted. Invalidate
-                // any cached values and proceed.
+            // Check If we should halt early.
+            if (HaltingThread.hasBeenHalted()) {
+                // The Thread has been 'halted'.
+                // Invalidate any cached values and proceed.
                 invalidateGeometryCache();
             }
         }
@@ -870,7 +866,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
 
     /**
      * Returns the bounds of the area covered by this node, without
-     * taking any of its rendering attribute into accoun. i.e.,
+     * taking any of its rendering attribute into accoun. That is,
      * exclusive of any clipping, masking, filtering or stroking, for
      * example. The returned value is transformed by the concatenation
      * of the input transform and this node's transform.
@@ -972,7 +968,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
     /**
      * This method makes sure that neither the width nor height of the
      * rectangle is zero.  But it tries to make them very small
-     * relatively speaking.  
+     * relatively speaking.
      */
     protected Rectangle2D normalizeRectangle(Rectangle2D bounds) {
         if (bounds == null) return null;
@@ -988,7 +984,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
                 if (tmpW < bounds.getWidth())
                     tmpW = bounds.getWidth();
                 return new Rectangle2D.Double
-                    (bounds.getX(), bounds.getY(), 
+                    (bounds.getX(), bounds.getY(),
                      tmpW, bounds.getHeight());
             }
         } else if (bounds.getHeight() < EPSILON) {
@@ -996,7 +992,7 @@ public abstract class AbstractGraphicsNode implements GraphicsNode {
             if (tmpH < bounds.getHeight())
                 tmpH = bounds.getHeight();
             return new Rectangle2D.Double
-                (bounds.getX(), bounds.getY(), 
+                (bounds.getX(), bounds.getY(),
                  bounds.getWidth(), tmpH);
         }
         return bounds;
