@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt;
 
 import java.awt.Graphics2D;
@@ -64,16 +32,21 @@ import java.util.List;
 import java.util.ListIterator;
 import java.util.NoSuchElementException;
 
+import org.apache.batik.util.HaltingThread;
+
 /**
  * A CompositeGraphicsNode is a graphics node that can contain graphics nodes.
  *
+ * <br>Note: this class is a 'little bit aware of' other threads, but not really threadsafe.
+ *
  * @author <a href="mailto:Thierry.Kormann@sophia.inria.fr">Thierry Kormann</a>
- * @version $Id: CompositeGraphicsNode.java,v 1.36 2003/08/09 16:58:40 deweese Exp $
+ * @version $Id: CompositeGraphicsNode.java 1372129 2012-08-12 15:31:50Z helder $
  */
-public class CompositeGraphicsNode extends AbstractGraphicsNode 
+public class CompositeGraphicsNode extends AbstractGraphicsNode
     implements List {
 
-    public static final Rectangle2D VIEWPORT = new Rectangle(0, 0, 0, 0);
+    public static final Rectangle2D VIEWPORT  = new Rectangle();
+    public static final Rectangle2D NULL_RECT = new Rectangle();
 
     /**
      * The children of this composite graphics node.
@@ -83,12 +56,12 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
     /**
      * The number of children of this composite graphics node.
      */
-    protected int count;
+    protected volatile int count;
 
     /**
      * The number of times the children list has been structurally modified.
      */
-    protected int modCount;
+    protected volatile int modCount;
 
     /**
      * This flag indicates if this node has BackgroundEnable = 'new'.
@@ -100,17 +73,17 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * Internal Cache: Geometry bounds for this node, not taking into
      * account any of its children rendering attributes into account
      */
-    private Rectangle2D geometryBounds;
+    private volatile Rectangle2D geometryBounds;
 
     /**
      * Internal Cache: Primitive bounds.
      */
-    private Rectangle2D primitiveBounds;
+    private volatile Rectangle2D primitiveBounds;
 
     /**
      * Internal Cache: Sensitive bounds.
      */
-    private Rectangle2D sensitiveBounds;
+    private volatile Rectangle2D sensitiveBounds;
 
     /**
      * Internal Cache: the outline.
@@ -118,7 +91,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
     private Shape outline;
 
     /**
-     * Constructs a new empty <tt>CompositeGraphicsNode</tt>.
+     * Constructs a new empty <code>CompositeGraphicsNode</code>.
      */
     public CompositeGraphicsNode() {}
 
@@ -177,13 +150,20 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
             return;
         }
 
+        // Thread.currentThread() is potentially expensive, so reuse my instance in hasBeenHalted()
+        Thread currentThread = Thread.currentThread();
+
         // Paint children
         for (int i=0; i < count; ++i) {
+            if (HaltingThread.hasBeenHalted( currentThread ))
+                return;
+
             GraphicsNode node = children[i];
             if (node == null) {
                 continue;
             }
             node.paint(g2d);
+
         }
     }
 
@@ -213,61 +193,116 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * Returns the bounds of the area covered by this node's primitive paint.
      */
     public Rectangle2D getPrimitiveBounds() {
-        if (primitiveBounds == null) {
-            int i=0;
-            Rectangle2D bounds = null;
-            while ((bounds == null) && i < count) {
-                bounds = children[i++].getTransformedBounds(IDENTITY);
-            }
-            if (bounds == null) return null;
-            primitiveBounds = bounds;
+        if (primitiveBounds != null) {
+            if (primitiveBounds == NULL_RECT) return null;
+            return primitiveBounds;
+        }
 
-            Rectangle2D ctb = null;
-            while (i < count) {
-                ctb = children[i++].getTransformedBounds(IDENTITY);
-                if (ctb != null) {
-                    if (primitiveBounds == null) {
-                        // another thread has set the primitive bounds to null,
-                        // need to recall this function
-                        return null;
-                    } else {
-                        primitiveBounds.add(ctb);
-                    }
+        // Thread.currentThread() is potentially expensive, so reuse my instance in hasBeenHalted()
+        Thread currentThread = Thread.currentThread();
+
+        int i=0;
+        Rectangle2D bounds = null;
+        while ((bounds == null) && i < count) {
+            bounds = children[i++].getTransformedBounds(IDENTITY);
+            if (((i & 0x0F) == 0) && HaltingThread.hasBeenHalted( currentThread ))
+                break; // check every 16 children if we have been interrupted.
+        }
+        if (HaltingThread.hasBeenHalted( currentThread )) {
+            invalidateGeometryCache();
+            return null;
+        }
+
+        if (bounds == null) {
+            primitiveBounds = NULL_RECT;
+            return null;
+        }
+
+        primitiveBounds = bounds;
+
+        while (i < count) {
+            Rectangle2D ctb = children[i++].getTransformedBounds(IDENTITY);
+            if (ctb != null) {
+                if (primitiveBounds == null) {
+                    // another thread has set the primitive bounds to null,
+                    // need to recall this function
+                    return null;
+                } else {
+                    primitiveBounds.add(ctb);
                 }
             }
 
-            // Make sure we haven't been interrupted
-            if (Thread.currentThread().isInterrupted()) {
-                // The Thread has been interrupted.
-                // Invalidate any cached values and proceed.
-                invalidateGeometryCache();
-            }
+            if (((i & 0x0F) == 0) && HaltingThread.hasBeenHalted( currentThread ))
+                break; // check every 16 children if we have been interrupted.
+        }
+
+        // Check If we should halt early.
+        if (HaltingThread.hasBeenHalted( currentThread )) {
+            // The Thread has been halted.
+            // Invalidate any cached values and proceed.
+            invalidateGeometryCache();
         }
         return primitiveBounds;
     }
 
+    /**
+     * Transforms a Rectangle 2D by an affine transform.  It assumes the transform
+     * is only scale/translate so there is no loss of precision over transforming
+     * the source geometry.
+     */
+    public static Rectangle2D  getTransformedBBox(Rectangle2D r2d, AffineTransform t) {
+        if ((t  == null) || (r2d == null)) return r2d;
+
+        double x  = r2d.getX();
+        double w  = r2d.getWidth();
+        double y  = r2d.getY();
+        double h  = r2d.getHeight();
+
+        double sx = t.getScaleX();
+        double sy = t.getScaleY();
+        if (sx < 0) {
+            x = -(x + w);
+            sx = -sx;
+        }
+        if (sy < 0) {
+            y = -(y + h);
+            sy = -sy;
+        }
+
+        return new Rectangle2D.Float
+            ((float)(x*sx+t.getTranslateX()),
+             (float)(y*sy+t.getTranslateY()),
+             (float)(w*sx), (float)(h*sy));
+    }
+
     /**
      * Returns the bounds of this node's primitivePaint after applying
      * the input transform (if any), concatenated with this node's
      * transform (if any).
      *
      * @param txf the affine transform with which this node's transform should
-     *        be concatenated. Should not be null.  */
+     *        be concatenated. Should not be null.
+     */
     public Rectangle2D getTransformedPrimitiveBounds(AffineTransform txf) {
         AffineTransform t = txf;
         if (transform != null) {
             t = new AffineTransform(txf);
             t.concatenate(transform);
         }
+
+        if ((t == null) || ((t.getShearX() == 0) && (t.getShearY() == 0))) {
+            // No rotation it's safe to simply transform our bounding box.
+            return getTransformedBBox(getPrimitiveBounds(), t);
+        }
+
         int i = 0;
         Rectangle2D tpb = null;
         while (tpb == null && i < count) {
             tpb = children[i++].getTransformedBounds(t);
         }
 
-        Rectangle2D ctb = null;
         while (i < count) {
-            ctb = children[i++].getTransformedBounds(t);
+            Rectangle2D ctb = children[i++].getTransformedBounds(t);
             if(ctb != null){
                 tpb.add(ctb);
             }
@@ -278,22 +313,21 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * Returns the bounds of the area covered by this node, without
-     * taking any of its rendering attribute into account. i.e.,
+     * taking any of its rendering attributes into account. That is,
      * exclusive of any clipping, masking, filtering or stroking, for
      * example.
      */
     public Rectangle2D getGeometryBounds() {
         if (geometryBounds == null) {
-            // System.out.println("geometryBounds are null");
+            // System.err.println("geometryBounds are null");
             int i=0;
             while(geometryBounds == null && i < count){
-                geometryBounds = 
-		    children[i++].getTransformedGeometryBounds (IDENTITY);
+                geometryBounds =
+                children[i++].getTransformedGeometryBounds (IDENTITY);
             }
 
-            Rectangle2D cgb = null;
             while (i<count) {
-                cgb = children[i++].getTransformedGeometryBounds(IDENTITY);
+                Rectangle2D cgb = children[i++].getTransformedGeometryBounds(IDENTITY);
                 if (cgb != null) {
                     if (geometryBounds == null) {
                         // another thread has set the geometry bounds to null,
@@ -311,7 +345,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * Returns the bounds of the area covered by this node, without taking any
-     * of its rendering attribute into accoun. i.e., exclusive of any clipping,
+     * of its rendering attribute into accoun. That is, exclusive of any clipping,
      * masking, filtering or stroking, for example. The returned value is
      * transformed by the concatenation of the input transform and this node's
      * transform.
@@ -325,13 +359,18 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
             t = new AffineTransform(txf);
             t.concatenate(transform);
         }
-	
+
+        if ((t == null) || ((t.getShearX() == 0) && (t.getShearY() == 0))) {
+            // No rotation it's safe to simply transform our bounding box.
+            return getTransformedBBox(getGeometryBounds(), t);
+        }
+
         Rectangle2D gb = null;
         int i=0;
         while (gb == null && i < count) {
             gb = children[i++].getTransformedGeometryBounds(t);
         }
-	
+
         Rectangle2D cgb = null;
         while (i < count) {
             cgb = children[i++].getTransformedGeometryBounds(t);
@@ -355,19 +394,18 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
         // System.out.println("sensitiveBoundsBounds are null");
         int i=0;
         while(sensitiveBounds == null && i < count){
-            sensitiveBounds = 
+            sensitiveBounds =
                 children[i++].getTransformedSensitiveBounds(IDENTITY);
         }
 
-        Rectangle2D cgb = null;
         while (i<count) {
-            cgb = children[i++].getTransformedSensitiveBounds(IDENTITY);
+            Rectangle2D cgb = children[i++].getTransformedSensitiveBounds(IDENTITY);
             if (cgb != null) {
                 if (sensitiveBounds == null)
                     // another thread has set the geometry bounds to null,
                     // need to recall this function
                     return getSensitiveBounds();
-                
+
                 sensitiveBounds.add(cgb);
             }
         }
@@ -391,16 +429,20 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
             t = new AffineTransform(txf);
             t.concatenate(transform);
         }
-	
+
+        if ((t == null) || ((t.getShearX() == 0) && (t.getShearY() == 0))) {
+            // No rotation it's safe to simply transform our bounding box.
+            return getTransformedBBox(getSensitiveBounds(), t);
+        }
+
         Rectangle2D sb = null;
         int i=0;
         while (sb == null && i < count) {
             sb = children[i++].getTransformedSensitiveBounds(t);
         }
-	
-        Rectangle2D csb = null;
+
         while (i < count) {
-            csb = children[i++].getTransformedSensitiveBounds(t);
+            Rectangle2D csb = children[i++].getTransformedSensitiveBounds(t);
             if (csb != null) {
                 sb.add(csb);
             }
@@ -472,17 +514,18 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * Returns the outline of this node.
      */
     public Shape getOutline() {
-        if (outline == null) {
-            outline = new GeneralPath();
-            for (int i = 0; i < count; i++) {
-                Shape childOutline = children[i].getOutline();
-                if (childOutline != null) {
-                    AffineTransform tr = children[i].getTransform();
-                    if (tr != null) {
-                        ((GeneralPath)outline).append(tr.createTransformedShape(childOutline), false);
-                    } else {
-                        ((GeneralPath)outline).append(childOutline, false);
-                    }
+        if (outline != null)
+            return outline;
+
+        outline = new GeneralPath();
+        for (int i = 0; i < count; i++) {
+            Shape childOutline = children[i].getOutline();
+            if (childOutline != null) {
+                AffineTransform tr = children[i].getTransform();
+                if (tr != null) {
+                    ((GeneralPath)outline).append(tr.createTransformedShape(childOutline), false);
+                } else {
+                    ((GeneralPath)outline).append(childOutline, false);
                 }
             }
         }
@@ -541,24 +584,24 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
     }
 
     /**
-     * Returns an array containing all of the graphics node in the
-     * children list of this composite graphics node in the correct
-     * order. If the children list fits in the specified array, it is
-     * returned therein. Otherwise, a new array is allocated.
+     * Returns an array containing all of the graphics node in the children list
+     * of this composite graphics node in the correct order.
      */
     public Object [] toArray() {
         GraphicsNode [] result = new GraphicsNode[count];
-        for (int i=0; i < count; ++i) {
-            result[i] = children[i];
-        }
+
+        System.arraycopy( children, 0, result, 0, count );
+
         return result;
     }
 
     /**
-     * Returns an array containing all of the graphics node in the children list
-     * of this composite graphics node in the correct order.
+     * Returns an array containing all of the graphics node in the
+     * children list of this composite graphics node in the correct
+     * order. If the children list fits in the specified array, it is
+     * returned therein. Otherwise, a new array is allocated.
      *
-     * @param a the array to fit if possible 
+     * @param a the array to fit if possible
      */
     public Object[] toArray(Object [] a) {
         if (a.length < count) {
@@ -593,7 +636,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * @return the graphics node previously  at the specified position
      * @exception IndexOutOfBoundsException if the index is out of range
      * @exception IllegalArgumentException if the node is not an
-     * instance of GraphicsNode 
+     * instance of GraphicsNode
      */
     public Object set(int index, Object o) {
         // Check for correct arguments
@@ -677,7 +720,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * @param o the graphics node to be inserted.
      * @exception IndexOutOfBoundsException if the index is out of range
      * @exception IllegalArgumentException if the node is not an
-     * instance of GraphicsNode 
+     * instance of GraphicsNode
      */
     public void add(int index, Object o) {
         // Check for correct arguments
@@ -715,7 +758,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * <b>Not supported</b> -
-     * Throws <tt>UnsupportedOperationException</tt> exception.
+     * Throws <code>UnsupportedOperationException</code> exception.
      */
     public boolean addAll(Collection c) {
         throw new UnsupportedOperationException();
@@ -723,19 +766,20 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * <b>Not supported</b> -
-     * Throws <tt>UnsupportedOperationException</tt> exception.
+     * Throws <code>UnsupportedOperationException</code> exception.
      */
     public boolean addAll(int index, Collection c) {
         throw new UnsupportedOperationException();
     }
 
     /**
-     * Removes the specified graphics node from the children list.
+     * Removes the first instance of the specified graphics node from the children list.
      *
      * @param o the node the remove
      * @return true if the children list contains the specified graphics node
      * @exception IllegalArgumentException if the node is not an
      * instance of GraphicsNode
+     * @exception IndexOutOfBoundsException when o is not in children list
      */
     public boolean remove(Object o) {
         // Check for correct argument
@@ -748,7 +792,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
         }
         // Remove the node
         int index = 0;
-        for (; node != children[index]; index++);
+        for (; node != children[index]; index++);     // fires exception when node not found!
         remove(index);
         return true;
     }
@@ -760,7 +804,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      *
      * @param index the position of the graphics node to remove
      * @return the graphics node that was removed
-     * @exception IndexOutOfBoundsException if index out of range <tt> 
+     * @exception IndexOutOfBoundsException if index out of range <code>
      */
     public Object remove(int index) {
         // Check for correct argument
@@ -794,7 +838,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * <b>Not supported</b> -
-     * Throws <tt>UnsupportedOperationException</tt> exception.
+     * Throws <code>UnsupportedOperationException</code> exception.
      */
     public boolean removeAll(Collection c) {
         throw new UnsupportedOperationException();
@@ -802,7 +846,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * <b>Not supported</b> -
-     * Throws <tt>UnsupportedOperationException</tt> exception.
+     * Throws <code>UnsupportedOperationException</code> exception.
      */
     public boolean retainAll(Collection c) {
         throw new UnsupportedOperationException();
@@ -810,7 +854,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * <b>Not supported</b> -
-     * Throws <tt>UnsupportedOperationException</tt> exception.
+     * Throws <code>UnsupportedOperationException</code> exception.
      */
     public void clear() {
         throw new UnsupportedOperationException();
@@ -820,7 +864,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * Returns true if this composite graphics node contains all the graphics
      * node in the specified collection, false otherwise.
      *
-     * @param c the collection to be checked for containment 
+     * @param c the collection to be checked for containment
      */
     public boolean containsAll(Collection c) {
         Iterator i = c.iterator();
@@ -838,15 +882,17 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * Returns the index in the children list of the specified graphics node or
      * -1 if the children list does not contain this graphics node.
      *
-     * @param node the graphics node to search for 
+     * @param node the graphics node to search for
      */
     public int indexOf(Object node) {
         if (node == null || !(node instanceof GraphicsNode)) {
             return -1;
         }
         if (((GraphicsNode) node).getParent() == this) {
-            for (int i = 0; i < count; i++) {
-                if (node == children[i]) {
+            int iCount = count;                  // local is cheaper
+            GraphicsNode[] workList = children;  // local is cheaper
+            for (int i = 0; i < iCount; i++) {
+                if (node == workList[ i ]) {
                     return i;
                 }
             }
@@ -859,7 +905,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * specified graphics node, or -1 if the list does not contain this graphics
      * node.
      *
-     * @param node the graphics node to search for 
+     * @param node the graphics node to search for
      */
     public int lastIndexOf(Object node) {
         if (node == null || !(node instanceof GraphicsNode)) {
@@ -889,7 +935,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
      * the specified position in the children list.
      *
      * @param index the index of the first graphics node to return
-     * from the children list 
+     * from the children list
      */
     public ListIterator listIterator(int index) {
         if (index < 0 || index > count) {
@@ -902,7 +948,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
 
     /**
      * <b>Not supported</b> -
-     * Throws <tt>UnsupportedOperationException</tt> exception.
+     * Throws <code>UnsupportedOperationException</code> exception.
      */
     public List subList(int fromIndex, int toIndex) {
         throw new UnsupportedOperationException();
@@ -936,7 +982,7 @@ public class CompositeGraphicsNode extends AbstractGraphicsNode
         int oldCapacity = children.length;
         if (minCapacity > oldCapacity) {
             GraphicsNode [] oldData = children;
-            int newCapacity = (oldCapacity * 3)/2 + 1;
+            int newCapacity = oldCapacity + oldCapacity/2 + 1;
             if (newCapacity < minCapacity) {
                 newCapacity = minCapacity;
             }
