@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt;
 
 import java.awt.Graphics2D;
@@ -58,7 +26,8 @@ import java.awt.geom.PathIterator;
 import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.util.List;
-import java.util.Vector;
+import java.util.ArrayList;
+import java.util.Arrays;
 
 import org.apache.batik.ext.awt.geom.ExtendedGeneralPath;
 import org.apache.batik.ext.awt.geom.ExtendedPathIterator;
@@ -68,12 +37,12 @@ import org.apache.batik.ext.awt.geom.ShapeExtender;
 /**
  * A shape painter that can be used to paint markers on a shape.
  *
- * @author <a href="vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: MarkerShapePainter.java,v 1.15 2003/10/23 01:01:55 deweese Exp $
+ * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
+ * @version $Id: MarkerShapePainter.java 1372129 2012-08-12 15:31:50Z helder $
  */
 public class MarkerShapePainter implements ShapePainter {
 
-    /** 
+    /**
      * The Shape to be painted.
      */
     protected ExtendedShape extShape;
@@ -82,7 +51,7 @@ public class MarkerShapePainter implements ShapePainter {
      * Start Marker
      */
     protected Marker startMarker;
-    
+
     /**
      * Middle Marker
      */
@@ -101,7 +70,7 @@ public class MarkerShapePainter implements ShapePainter {
     /**
      * Middle marker proxy.
      */
-    private ProxyGraphicsNode middleMarkerProxies[];
+    private ProxyGraphicsNode[] middleMarkerProxies;
 
     /**
      * End marker proxy.
@@ -124,11 +93,11 @@ public class MarkerShapePainter implements ShapePainter {
     private Rectangle2D dGeometryBounds;
 
     /**
-     * Constructs a new <tt>MarkerShapePainter</tt> that can be used to markers
+     * Constructs a new <code>MarkerShapePainter</code> that can be used to markers
      * on top of a shape.
      *
      * @param shape Shape to be painted by this painter.
-     * Should not be null 
+     * Should not be null
      */
     public MarkerShapePainter(Shape shape) {
         if (shape == null) {
@@ -144,13 +113,12 @@ public class MarkerShapePainter implements ShapePainter {
     /**
      * Paints the specified shape using the specified Graphics2D.
      *
-     * @param shape the shape to paint
      * @param g2d the Graphics2D to use
      */
      public void paint(Graphics2D g2d) {
-	 if (markerGroup == null) {
-	     buildMarkerGroup();
-	 }
+         if (markerGroup == null) {
+             buildMarkerGroup();
+         }
          if (markerGroup.getChildren().size() > 0) {
              markerGroup.paint(g2d);
          }
@@ -160,9 +128,9 @@ public class MarkerShapePainter implements ShapePainter {
      * Returns the area painted by this shape painter.
      */
     public Shape getPaintedArea(){
-	 if (markerGroup == null) {
-	     buildMarkerGroup();
-	 }
+         if (markerGroup == null) {
+             buildMarkerGroup();
+         }
         return markerGroup.getOutline();
     }
 
@@ -170,9 +138,9 @@ public class MarkerShapePainter implements ShapePainter {
      * Returns the bounds of the area painted by this shape painter
      */
     public Rectangle2D getPaintedBounds2D(){
-	 if (markerGroup == null) {
-	     buildMarkerGroup();
-	 }
+         if (markerGroup == null) {
+             buildMarkerGroup();
+         }
          return markerGroup.getPrimitiveBounds();
     }
 
@@ -180,9 +148,9 @@ public class MarkerShapePainter implements ShapePainter {
      * Returns true if pt is in the area painted by this shape painter
      */
     public boolean inPaintedArea(Point2D pt){
-	 if (markerGroup == null) {
-	     buildMarkerGroup();
-	 }
+         if (markerGroup == null) {
+             buildMarkerGroup();
+         }
          GraphicsNode gn = markerGroup.nodeHitAt(pt);
          return (gn != null);
     }
@@ -225,7 +193,7 @@ public class MarkerShapePainter implements ShapePainter {
         this.startMarkerProxy = null;
         this.middleMarkerProxies = null;
         this.endMarkerProxy = null;
-	this.markerGroup = null;
+        this.markerGroup = null;
     }
 
     /**
@@ -258,12 +226,12 @@ public class MarkerShapePainter implements ShapePainter {
      * Sets the marker that shall be drawn at the first vertex of the given
      * shape.
      *
-     * @param startMarker the start marker 
+     * @param startMarker the start marker
      */
     public void setStartMarker(Marker startMarker){
         this.startMarker = startMarker;
         this.startMarkerProxy = null;
-	this.markerGroup = null;
+        this.markerGroup = null;
     }
 
     /**
@@ -283,7 +251,7 @@ public class MarkerShapePainter implements ShapePainter {
     public void setMiddleMarker(Marker middleMarker){
         this.middleMarker = middleMarker;
         this.middleMarkerProxies = null;
-	this.markerGroup = null;
+        this.markerGroup = null;
     }
 
     /**
@@ -298,12 +266,12 @@ public class MarkerShapePainter implements ShapePainter {
      * Sets the marker that shall be drawn at the last vertex of the given
      * shape.
      *
-     * @param endMarker the end marker 
+     * @param endMarker the end marker
      */
     public void setEndMarker(Marker endMarker){
         this.endMarker = endMarker;
         this.endMarkerProxy = null;
-	this.markerGroup = null;
+        this.markerGroup = null;
     }
 
     // ---------------------------------------------------------------------
@@ -331,13 +299,13 @@ public class MarkerShapePainter implements ShapePainter {
         if (startMarkerProxy != null) {
             children.add(startMarkerProxy);
         }
-	
+
         if (middleMarkerProxies != null) {
             for(int i=0; i<middleMarkerProxies.length; i++){
                 children.add(middleMarkerProxies[i]);
             }
         }
-	
+
         if (endMarkerProxy != null) {
             children.add(endMarkerProxy);
         }
@@ -346,15 +314,15 @@ public class MarkerShapePainter implements ShapePainter {
     }
 
     /**
-     * Builds a proxy <tt>GraphicsNode</tt> for the input <tt>Marker</tt> to be
-     * drawn at the start position 
+     * Builds a proxy <code>GraphicsNode</code> for the input <code>Marker</code> to be
+     * drawn at the start position
      */
     protected ProxyGraphicsNode buildStartMarkerProxy() {
 
         ExtendedPathIterator iter = getExtShape().getExtendedPathIterator();
 
         // Get initial point on the path
-        double coords[] = new double[7];
+        double[] coords = new double[7];
         int segType = 0;
 
         if (iter.isDone()) {
@@ -374,7 +342,7 @@ public class MarkerShapePainter implements ShapePainter {
         double rotation = startMarker.getOrient();
         if (Double.isNaN(rotation)) {
             if (!iter.isDone()) {
-                double next[] = new double[7];
+                double[] next = new double[7];
                 int nextSegType = 0;
                 nextSegType = iter.currentSegment(next);
                 if(nextSegType == PathIterator.SEG_CLOSE){
@@ -385,25 +353,25 @@ public class MarkerShapePainter implements ShapePainter {
                 rotation = computeRotation(null, 0,  // no previous seg.
                                            coords, segType,    // segment ending on start point
                                            next, nextSegType); // segment out of start point
-                
+
             }
         }
-        
+
         // Now, compute the marker's proxy transform
         AffineTransform markerTxf = computeMarkerTransform(startMarker,
-							   markerPosition,
-							   rotation);
-                                   
+                                                           markerPosition,
+                                                           rotation);
+
         ProxyGraphicsNode gn = new ProxyGraphicsNode();
 
         gn.setSource(startMarker.getMarkerNode());
         gn.setTransform(markerTxf);
-	
+
         return gn;
     }
 
     /**
-     * Builds a proxy <tt>GraphicsNode</tt> for the input <tt>Marker</tt> to be
+     * Builds a proxy <code>GraphicsNode</code> for the input <code>Marker</code> to be
      * drawn at the end position.
      */
     protected ProxyGraphicsNode buildEndMarkerProxy() {
@@ -417,9 +385,9 @@ public class MarkerShapePainter implements ShapePainter {
         if (iter.isDone()) {
             return null;
         }
-	
-        double coords[] = new double[7];
-        double moveTo[] = new double[2];
+
+        double[] coords = new double[7];
+        double[] moveTo = new double[2];
         int segType = 0;
         segType = iter.currentSegment(coords);
         if (segType != ExtendedPathIterator.SEG_MOVETO) {
@@ -430,7 +398,7 @@ public class MarkerShapePainter implements ShapePainter {
         moveTo[1] = coords[1];
 
         iter.next();
-        
+
         // Now, get the last two points on the path
         double[] lastButOne = new double[7];
         double[] last = {coords[0], coords[1], coords[2],
@@ -471,17 +439,17 @@ public class MarkerShapePainter implements ShapePainter {
         // the slope needs to be computed
         double rotation = endMarker.getOrient();
         if (Double.isNaN(rotation)) {
-            rotation = computeRotation(lastButOne, 
-                                       lastButOneSegType, 
+            rotation = computeRotation(lastButOne,
+                                       lastButOneSegType,
                                        last, lastSegType,
                                        null, 0);
         }
 
         // Now, compute the marker's proxy transform
         AffineTransform markerTxf = computeMarkerTransform(endMarker,
-							   markerPosition,
-							   rotation);
-                                   
+                                                           markerPosition,
+                                                           rotation);
+
         ProxyGraphicsNode gn = new ProxyGraphicsNode();
 
         gn.setSource(endMarker.getMarkerNode());
@@ -491,8 +459,8 @@ public class MarkerShapePainter implements ShapePainter {
     }
 
     /**
-     * Builds a proxy <tt>GraphicsNode</tt> for the input
-     * <tt>Marker</tt> to be drawn at the middle positions 
+     * Builds a proxy <code>GraphicsNode</code> for the input
+     * <code>Marker</code> to be drawn at the middle positions
      */
     protected ProxyGraphicsNode[] buildMiddleMarkerProxies() {
 
@@ -522,7 +490,7 @@ public class MarkerShapePainter implements ShapePainter {
         if (iter.isDone()) {
             return null;
         }
-	
+
         currSegType = iter.currentSegment(curr);
 
         if (currSegType == PathIterator.SEG_MOVETO) {
@@ -536,7 +504,7 @@ public class MarkerShapePainter implements ShapePainter {
 
         iter.next();
 
-        Vector proxies = new Vector();
+        List proxies = new ArrayList();
         while (!iter.isDone()) {
             nextSegType = iter.currentSegment(next);
 
@@ -548,23 +516,23 @@ public class MarkerShapePainter implements ShapePainter {
                 next[0] = moveTo[0];
                 next[1] = moveTo[1];
             }
-	    
-            proxies.addElement(createMiddleMarker(prev, prevSegType,
+
+            proxies.add(createMiddleMarker(prev, prevSegType,
                                                   curr, currSegType,
                                                   next, nextSegType));
-            
+
             tmp = prev;
             prev = curr;
             prevSegType = currSegType;
             curr = next;
             currSegType = nextSegType;
             next = tmp;
-            
+
             iter.next();
         }
 
         ProxyGraphicsNode [] gn = new ProxyGraphicsNode[proxies.size()];
-        proxies.copyInto(gn);
+        proxies.toArray( gn );
 
         return gn;
     }
@@ -588,12 +556,12 @@ public class MarkerShapePainter implements ShapePainter {
                                        curr, currSegType,
                                        next, nextSegType);
         }
-	
+
         // Now, compute the marker's proxy transform
         AffineTransform markerTxf = computeMarkerTransform(middleMarker,
-							   markerPosition,
-							   rotation);
-                                   
+                                                           markerPosition,
+                                                           rotation);
+
         ProxyGraphicsNode gn = new ProxyGraphicsNode();
 
         gn.setSource(middleMarker.getMarkerNode());
@@ -603,7 +571,7 @@ public class MarkerShapePainter implements ShapePainter {
     }
 
     /**
-     * Returns the rotation according to the specified parameters.
+     * Returns the rotation according to the specified parameters in degrees.
      */
     private double computeRotation(double[] prev, int prevSegType,
                                    double[] curr, int currSegType,
@@ -611,12 +579,12 @@ public class MarkerShapePainter implements ShapePainter {
 
         // Compute in slope, i.e., the slope of the segment
         // going into the current point
-        double[] inSlope = computeInSlope(prev, prevSegType, 
+        double[] inSlope = computeInSlope(prev, prevSegType,
                                           curr, currSegType);
 
         // Compute out slope, i.e., the slope of the segment
         // going out of the current point
-        double[] outSlope = computeOutSlope(curr, currSegType, 
+        double[] outSlope = computeOutSlope(curr, currSegType,
                                             next, nextSegType);
 
         if (inSlope == null) {
@@ -635,11 +603,11 @@ public class MarkerShapePainter implements ShapePainter {
         double dy = inSlope[1] + outSlope[1];
 
         if (dx == 0 && dy == 0) {
-            // The two vectors are exact opposites. There is no way to 
+            // The two vectors are exact opposites. There is no way to
             // know which direction to go (+90 or -90). Choose +90
-            return Math.atan2(inSlope[1], inSlope[0])*180./Math.PI + 90;
+            return Math.toDegrees( Math.atan2(inSlope[1], inSlope[0]) ) + 90;
         } else {
-            return Math.atan2(dy, dx)*180./Math.PI;
+            return Math.toDegrees( Math.atan2(dy, dx) );
         }
     }
 
@@ -659,7 +627,7 @@ public class MarkerShapePainter implements ShapePainter {
         case PathIterator.SEG_LINETO: {
             // This is equivalent to a line from the previous segment's
             // terminating point and the current end point.
-            Point2D prevEndPoint = 
+            Point2D prevEndPoint =
                 getSegmentTerminatingPoint(prev, prevSegType);
             dx = currEndPoint.getX() - prevEndPoint.getX();
             dy = currEndPoint.getY() - prevEndPoint.getY();
@@ -682,7 +650,7 @@ public class MarkerShapePainter implements ShapePainter {
         case ExtendedPathIterator.SEG_ARCTO: {
             // If the current segment is an ARCTO then we build the
             // arc and ask for it's end angle and get the tangent there.
-            Point2D prevEndPoint = 
+            Point2D prevEndPoint =
                 getSegmentTerminatingPoint(prev, prevSegType);
             boolean large   = (curr[3]!=0.);
             boolean goLeft = (curr[4]!=0.);
@@ -695,7 +663,7 @@ public class MarkerShapePainter implements ShapePainter {
             dx = -arc.getWidth()/2.0*Math.sin(theta);
             dy = arc.getHeight()/2.0*Math.cos(theta);
 
-            // System.out.println("In  Theta:  " + Math.toDegrees(theta) +  
+            // System.out.println("In  Theta:  " + Math.toDegrees(theta) +
             //                    " Dx/Dy: " + dx + "/" + dy);
             if (curr[2] != 0) {
                 double ang = Math.toRadians(-curr[2]);
@@ -706,20 +674,20 @@ public class MarkerShapePainter implements ShapePainter {
                 dx = tdx;
                 dy = tdy;
             }
-            // System.out.println("    Rotate: " + curr[2] + 
+            // System.out.println("    Rotate: " + curr[2] +
             //                    " Dx/Dy: " + dx + "/" + dy);
             if (goLeft) {
                 dx = -dx;
             } else {
                 dy = -dy;
             }
-            // System.out.println("    GoLeft? " + goLeft + 
+            // System.out.println("    GoLeft? " + goLeft +
             //                    " Dx/Dy: " + dx + "/" + dy);
         }
             break;
         case PathIterator.SEG_CLOSE:
             // Should not have any close at this point
-            throw new Error();
+            throw new Error("should not have SEG_CLOSE here");
         case PathIterator.SEG_MOVETO:
             // Cannot compute the slope
         default:
@@ -740,7 +708,7 @@ public class MarkerShapePainter implements ShapePainter {
                                      double[] next, int nextSegType){
 
         Point2D currEndPoint = getSegmentTerminatingPoint(curr, currSegType);
-        
+
         double dx = 0, dy = 0;
 
         switch(nextSegType){
@@ -770,7 +738,7 @@ public class MarkerShapePainter implements ShapePainter {
             theta = Math.toRadians(theta);
             dx = -arc.getWidth()/2.0*Math.sin(theta);
             dy = arc.getHeight()/2.0*Math.cos(theta);
-            // System.out.println("Out Theta:  " + Math.toDegrees(theta) +  
+            // System.out.println("Out Theta:  " + Math.toDegrees(theta) +
             //                    " Dx/Dy: " + dx + "/" + dy);
             if (next[2] != 0) {
                 double ang = Math.toRadians(-next[2]);
@@ -781,7 +749,7 @@ public class MarkerShapePainter implements ShapePainter {
                 dx = tdx;
                 dy = tdy;
             }
-            // System.out.println("    Rotate: " + next[2] + 
+            // System.out.println("    Rotate: " + next[2] +
             //                    " Dx/Dy: " + dx + "/" + dy);
 
             if (goLeft) {
@@ -789,7 +757,7 @@ public class MarkerShapePainter implements ShapePainter {
             } else {
                 dy = -dy;
             }
-            // System.out.println("    GoLeft? " + goLeft + 
+            // System.out.println("    GoLeft? " + goLeft +
             //                    " Dx/Dy: " + dx + "/" + dy);
         }
             break;
@@ -818,13 +786,13 @@ public class MarkerShapePainter implements ShapePainter {
 
     /**
      * Computes the transform for the input marker, so that it is positioned at
-     * the given position with the specified rotation 
+     * the given position with the specified rotation
      */
     private AffineTransform computeMarkerTransform(Marker marker,
-						   Point2D markerPosition,
-						   double rotation) {
+                                                   Point2D markerPosition,
+                                                   double rotation) {
         Point2D ref = marker.getRef();
-        /*AffineTransform txf = 
+        /*AffineTransform txf =
             AffineTransform.getTranslateInstance(markerPosition.getX()
                                                  - ref.getX(),
                                                  markerPosition.getY()
@@ -835,7 +803,7 @@ public class MarkerShapePainter implements ShapePainter {
                       markerPosition.getY() - ref.getY());
 
         if (!Double.isNaN(rotation)) {
-            txf.rotate(rotation*Math.PI/180., ref.getX(), ref.getY());
+            txf.rotate( Math.toRadians( rotation ), ref.getX(), ref.getY());
         }
 
         return txf;
@@ -844,7 +812,7 @@ public class MarkerShapePainter implements ShapePainter {
     /**
      * Extracts the terminating point, depending on the segment type.
      */
-    protected Point2D getSegmentTerminatingPoint(double coords[], int segType) {
+    protected Point2D getSegmentTerminatingPoint(double[] coords, int segType) {
         switch(segType){
         case PathIterator.SEG_CUBICTO:
             return new Point2D.Double(coords[4], coords[5]);
@@ -858,7 +826,7 @@ public class MarkerShapePainter implements ShapePainter {
             return new Point2D.Double(coords[5], coords[6]);
         case PathIterator.SEG_CLOSE:
         default:
-            throw new Error(); 
+            throw new Error( "invalid segmentType:" + segType );
             // Should never happen: close segments are replaced with lineTo
         }
     }
