@@ -1,63 +1,35 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt.event;
 
 import java.awt.Point;
+import java.awt.Toolkit;
 import java.awt.event.InputEvent;
 import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
+import java.awt.event.MouseWheelEvent;
+import java.awt.event.MouseWheelListener;
 import java.awt.event.MouseMotionListener;
 import java.awt.geom.AffineTransform;
+import java.awt.geom.NoninvertibleTransformException;
 import java.awt.geom.Point2D;
 import java.lang.reflect.Array;
 import java.util.EventListener;
@@ -68,7 +40,6 @@ import java.util.List;
 import javax.swing.event.EventListenerList;
 
 import org.apache.batik.gvt.GraphicsNode;
-import org.apache.batik.gvt.TextNode;
 
 /**
  * An EventDispatcher implementation based on AWT events.
@@ -78,15 +49,17 @@ import org.apache.batik.gvt.TextNode;
  * for containment are performed from the EventDispatcher's "root"
  * node.</p>
  *
- * @author <a href="bill.haneman@ireland.sun.com>Bill Haneman</a>
- * @author <a href="cjolif@ilog.fr>Christophe Jolif</a>
- * @author <a href="tkormann@ilog.fr>Thierry Kormann</a>
- * @version $Id: AWTEventDispatcher.java,v 1.20 2003/09/11 10:40:24 deweese Exp $
+ * @author <a href="mailto:bill.haneman@ireland.sun.com">Bill Haneman</a>
+ * @author <a href="mailto:cjolif@ilog.fr">Christophe Jolif</a>
+ * @author <a href="mailto:tkormann@ilog.fr">Thierry Kormann</a>
+ * @version $Id: AWTEventDispatcher.java 1372129 2012-08-12 15:31:50Z helder $
  */
-public class AWTEventDispatcher implements EventDispatcher,
-                                           MouseListener,
-                                           MouseMotionListener,
-                                           KeyListener {
+public class AWTEventDispatcher
+        implements EventDispatcher,
+                   MouseListener,
+                   MouseMotionListener,
+                   MouseWheelListener,
+                   KeyListener {
 
     /**
      * The root GraphicsNode as determined by setRootNode().
@@ -109,12 +82,6 @@ public class AWTEventDispatcher implements EventDispatcher,
      */
     protected GraphicsNode lastHit;
 
-    /**
-     * A dummy graphics node to dispacth "deselect" mouse events to. ie.
-     * when the mouse is clicked outside any nodes.
-     */
-    protected GraphicsNode dummyNode = new TextNode();
-
     /**
      * The current GraphicsNode targeted by an key events.
      */
@@ -130,7 +97,7 @@ public class AWTEventDispatcher implements EventDispatcher,
     /**
      * default max size of the event queue.
      */
-    final static int MAX_QUEUE_SIZE = 10;
+    static final int MAX_QUEUE_SIZE = 10;
 
     private int nodeIncrementEventID = KeyEvent.KEY_PRESSED;
     private int nodeIncrementEventCode = KeyEvent.VK_TAB;
@@ -174,7 +141,7 @@ public class AWTEventDispatcher implements EventDispatcher,
             ((baseTransform == null) || (!baseTransform.equals(t))))
             // new Display transform so events are not where user
             // thinks they were.
-            eventQueue.clear(); 
+            eventQueue.clear();
         baseTransform = t;
     }
 
@@ -253,6 +220,15 @@ public class AWTEventDispatcher implements EventDispatcher,
         dispatchEvent(evt);
     }
 
+    /**
+     * Dispatches the specified AWT mouse wheel event down to the GVT tree.
+     * The mouse wheel event is mutated to a GraphicsNodeMouseWheelEvent.
+     * @param evt the mouse event to propagate
+     */
+    public void mouseWheelMoved(MouseWheelEvent evt) {
+        dispatchEvent(evt);
+    }
+
     /**
      * Dispatches the specified AWT key event down to the GVT tree.
      * The mouse event is mutated to a GraphicsNodeKeyEvent.
@@ -307,6 +283,31 @@ public class AWTEventDispatcher implements EventDispatcher,
         }
     }
 
+    /**
+     * Adds the specified 'global' GraphicsNodeMouseWheelListener which is
+     * notified of all MouseWheelEvents dispatched.
+     * @param l the listener to add
+     */
+    public void addGraphicsNodeMouseWheelListener
+            (GraphicsNodeMouseWheelListener l) {
+        if (glisteners == null) {
+            glisteners = new EventListenerList();
+        }
+        glisteners.add(GraphicsNodeMouseWheelListener.class, l);
+    }
+
+    /**
+     * Removes the specified 'global' GraphicsNodeMouseWheelListener which is
+     * notified of all MouseWheelEvents dispatched.
+     * @param l the listener to remove
+     */
+    public void removeGraphicsNodeMouseWheelListener
+            (GraphicsNodeMouseWheelListener l) {
+        if (glisteners != null) {
+            glisteners.remove(GraphicsNodeMouseWheelListener.class, l);
+        }
+    }
+
     /**
      * Adds the specified 'global' GraphicsNodeKeyListener which is
      * notified of all KeyEvents dispatched.
@@ -336,6 +337,10 @@ public class AWTEventDispatcher implements EventDispatcher,
      * @param listenerType the type of the listeners to return
      */
     public EventListener [] getListeners(Class listenerType) {
+
+        // TODO the listeners should be cached per class in a map.
+        // this list is build again and again in mouse-event-handling...
+        // note: the cache must be flushed when the gListeners is modified
         Object array =
             Array.newInstance(listenerType,
                               glisteners.getListenerCount(listenerType));
@@ -366,6 +371,7 @@ public class AWTEventDispatcher implements EventDispatcher,
     public void setEventQueueMaxSize(int n) {
         eventQueueMaxSize = n;
         if (n == 0) eventQueue.clear();
+        // the current size is larger than the new size: shrink
         while(eventQueue.size() > eventQueueMaxSize)
             eventQueue.remove(0);
     }
@@ -383,11 +389,13 @@ public class AWTEventDispatcher implements EventDispatcher,
                 while (eventQueue.size() > eventQueueMaxSize)
                     // Limit how many events we queue - don't want
                     // user waiting forever for them to clear.
-                    eventQueue.remove(0); 
+                    eventQueue.remove(0);
             }
             return;
         }
-        if (evt instanceof MouseEvent) {
+        if (evt instanceof MouseWheelEvent) {
+            dispatchMouseWheelEvent((MouseWheelEvent) evt);
+        } else if (evt instanceof MouseEvent) {
             dispatchMouseEvent((MouseEvent) evt);
         } else if (evt instanceof KeyEvent) {
             InputEvent e = (InputEvent)evt;
@@ -401,23 +409,61 @@ public class AWTEventDispatcher implements EventDispatcher,
         }
     }
 
+    /**
+     * Returns a bitmask representing the state of the key locks.
+     */
+    protected int getCurrentLockState() {
+        Toolkit t = Toolkit.getDefaultToolkit();
+        int lockState = 0;
+        try {
+            if (t.getLockingKeyState(KeyEvent.VK_KANA_LOCK)) {
+                lockState++;
+            }
+        } catch (UnsupportedOperationException ex) {
+        }
+        lockState <<= 1;
+        try {
+            if (t.getLockingKeyState(KeyEvent.VK_SCROLL_LOCK)) {
+                lockState++;
+            }
+        } catch (UnsupportedOperationException ex) {
+        }
+        lockState <<= 1;
+        try {
+            if (t.getLockingKeyState(KeyEvent.VK_NUM_LOCK)) {
+                lockState++;
+            }
+        } catch (UnsupportedOperationException ex) {
+        }
+        lockState <<= 1;
+        try {
+            if (t.getLockingKeyState(KeyEvent.VK_CAPS_LOCK)) {
+                lockState++;
+            }
+        } catch (UnsupportedOperationException ex) {
+        }
+        return lockState;
+    }
+
     /**
      * Dispatches the specified AWT key event.
      * @param evt the key event to dispatch
      */
     protected void dispatchKeyEvent(KeyEvent evt) {
         currentKeyEventTarget = lastHit;
-        if (currentKeyEventTarget != null) {
-            processKeyEvent
-                (new GraphicsNodeKeyEvent(currentKeyEventTarget,
-                                          evt.getID(),
-                                          evt.getWhen(),
-                                          evt.getModifiers(),
-                                          evt.getKeyCode(),
-                                          evt.getKeyChar()));
-        }
+        GraphicsNode target =
+            currentKeyEventTarget == null ? root : currentKeyEventTarget;
+        processKeyEvent
+            (new GraphicsNodeKeyEvent(target,
+                                      evt.getID(),
+                                      evt.getWhen(),
+                                      evt.getModifiersEx(),
+                                      getCurrentLockState(),
+                                      evt.getKeyCode(),
+                                      evt.getKeyChar(),
+                                      evt.getKeyLocation()));
     }
-    
+
     /**
      * Dispatches the specified AWT mouse event.
      * @param evt the mouse event to dispatch
@@ -431,6 +477,12 @@ public class AWTEventDispatcher implements EventDispatcher,
         }
 
         GraphicsNode node = root.nodeHitAt(gnp);
+        if (node != null) {
+            try {
+                node.getGlobalTransform().createInverse().transform(gnp, gnp);
+            } catch (NoninvertibleTransformException ex) {
+            }
+        }
         
         // If the receiving node has changed, send a notification
         // check if we enter a new node
@@ -443,18 +495,22 @@ public class AWTEventDispatcher implements EventDispatcher,
             screenPos.y += evt.getY();
         }
 
+        int currentLockState = getCurrentLockState();
+
         if (lastHit != node) {
-            // post an MOUSE_EXITED
+            // post a MOUSE_EXITED event
             if (lastHit != null) {
                 gvtevt = new GraphicsNodeMouseEvent(lastHit,
                                                     MouseEvent.
                                                     MOUSE_EXITED,
                                                     evt.getWhen(),
-                                                    evt.getModifiers(),
+                                                    evt.getModifiersEx(),
+                                                    currentLockState,
+                                                    evt.getButton(),
                                                     (float)gnp.getX(),
                                                     (float)gnp.getY(),
-                                                    (int)Math.floor(p.getX()),
-                                                    (int)Math.floor(p.getY()),
+                                                    (int)Math.floor(p.getX()),  // evt.getX() ??
+                                                    (int)Math.floor(p.getY()),  // evt.getY() ??
                                                     screenPos.x,
                                                     screenPos.y,
                                                     evt.getClickCount(),
@@ -462,14 +518,15 @@ public class AWTEventDispatcher implements EventDispatcher,
                 processMouseEvent(gvtevt);
                 // lastHit.processMouseEvent(gvtevt);
             }
-            // post an MOUSE_ENTERED
+            // post a MOUSE_ENTERED event
             if (node != null) {
                 gvtevt = new GraphicsNodeMouseEvent(node,
                                                     MouseEvent.
                                                     MOUSE_ENTERED,
                                                     evt.getWhen(),
-                                                    evt.
-                                                    getModifiers(),
+                                                    evt.getModifiersEx(),
+                                                    currentLockState,
+                                                    evt.getButton(),
                                                     (float)gnp.getX(),
                                                     (float)gnp.getY(),
                                                     (int)Math.floor(p.getX()),
@@ -487,7 +544,9 @@ public class AWTEventDispatcher implements EventDispatcher,
             gvtevt = new GraphicsNodeMouseEvent(node,
                                                 evt.getID(),
                                                 evt.getWhen(),
-                                                evt.getModifiers(),
+                                                evt.getModifiersEx(),
+                                                currentLockState,
+                                                evt.getButton(),
                                                 (float)gnp.getX(),
                                                 (float)gnp.getY(),
                                                 (int)Math.floor(p.getX()),
@@ -500,13 +559,14 @@ public class AWTEventDispatcher implements EventDispatcher,
             // node.processMouseEvent(gvtevt);
             processMouseEvent(gvtevt);
 
-        } else if (node == null && evt.getID() == MouseEvent.MOUSE_CLICKED
-              && evt.getClickCount() == 1) {
-            // this is a deselect event, dispatch it to dummy node
-            gvtevt = new GraphicsNodeMouseEvent(dummyNode,
+        } else {
+            // this is a deselect event, dispatch it to the root
+            gvtevt = new GraphicsNodeMouseEvent(root,
                                                 evt.getID(),
                                                 evt.getWhen(),
-                                                evt.getModifiers(),
+                                                evt.getModifiersEx(),
+                                                currentLockState,
+                                                evt.getButton(),
                                                 (float)gnp.getX(),
                                                 (float)gnp.getY(),
                                                 (int)Math.floor(p.getX()),
@@ -521,6 +581,22 @@ public class AWTEventDispatcher implements EventDispatcher,
         lastHit = node;
     }
 
+    /**
+     * Dispatches the specified AWT mouse wheel event.
+     * @param evt the mouse wheel event to dispatch
+     */
+    protected void dispatchMouseWheelEvent(MouseWheelEvent evt) {
+        if (lastHit != null) {
+            processMouseWheelEvent
+                (new GraphicsNodeMouseWheelEvent(lastHit,
+                                                 evt.getID(),
+                                                 evt.getWhen(),
+                                                 evt.getModifiersEx(),
+                                                 getCurrentLockState(),
+                                                 evt.getWheelRotation()));
+        }
+    }
+
     /**
      * Processes the specified event by firing the 'global' listeners
      * attached to this event dispatcher.
@@ -568,7 +644,23 @@ public class AWTEventDispatcher implements EventDispatcher,
                 }
                 break;
             default:
-                throw new Error("Unknown Mouse Event type: "+evt.getID());
+                throw new IllegalArgumentException("Unknown Mouse Event type: "+evt.getID());
+            }
+        }
+    }
+
+    /**
+     * Processes the specified event by firing the 'global' listeners
+     * attached to this event dispatcher.
+     * @param evt the event to process
+     */
+    protected void processMouseWheelEvent(GraphicsNodeMouseWheelEvent evt) {
+        if (glisteners != null) {
+            GraphicsNodeMouseWheelListener[] listeners =
+                (GraphicsNodeMouseWheelListener[])
+                getListeners(GraphicsNodeMouseWheelListener.class);
+            for (int i = 0; i < listeners.length; i++) {
+                listeners[i].mouseWheelMoved(evt);
             }
         }
     }
@@ -602,25 +694,24 @@ public class AWTEventDispatcher implements EventDispatcher,
                 }
                 break;
             default:
-                throw new Error("Unknown Key Event type: "+evt.getID());
+                throw new IllegalArgumentException("Unknown Key Event type: "+evt.getID());
             }
         }
         evt.consume();
     }
 
-
     private void incrementKeyTarget() {
         // <!> FIXME TODO: Not implemented.
-        throw new Error("Increment not implemented.");
+        throw new UnsupportedOperationException("Increment not implemented.");
     }
 
     private void decrementKeyTarget() {
         // <!> FIXME TODO: Not implemented.
-        throw new Error("Decrement not implemented.");
+        throw new UnsupportedOperationException("Decrement not implemented.");
     }
 
     /**
-     * Associates all InputEvents of type <tt>e.getID()</tt>
+     * Associates all InputEvents of type <code>e.getID()</code>
      * with "incrementing" of the currently selected GraphicsNode.
      */
     public void setNodeIncrementEvent(InputEvent e) {
@@ -632,7 +723,7 @@ public class AWTEventDispatcher implements EventDispatcher,
     }
 
     /**
-     * Associates all InputEvents of type <tt>e.getID()</tt>
+     * Associates all InputEvents of type <code>e.getID()</code>
      * with "decrementing" of the currently selected GraphicsNode.
      * The notion of "currently selected" GraphicsNode is used
      * for dispatching KeyEvents.
@@ -651,11 +742,26 @@ public class AWTEventDispatcher implements EventDispatcher,
      * @param e the input event
      */
     protected boolean isNodeIncrementEvent(InputEvent e) {
-        // TODO: Improve code readability!
-        return ((e.getID() == nodeIncrementEventID) &&
-                ((e instanceof KeyEvent) ?
-                     (((KeyEvent) e).getKeyCode() == nodeIncrementEventCode) : true) &&
-                ((e.getModifiers() & nodeIncrementEventModifiers) != 0));
+        if (e.getID() != nodeIncrementEventID) {
+            // not an incrementEvent: false
+            return false;
+        }
+
+        if (e instanceof KeyEvent) {
+            if (((KeyEvent) e).getKeyCode() != nodeIncrementEventCode) {
+                // it was a KeyEvent, but not a nodeIncrementEventCode : false
+                return false;
+            }
+        }
+
+        // here: it was not a KeyEvent at all OR a KeyEvent with nodeIncrementEventCode
+        if ((e.getModifiers() & nodeIncrementEventModifiers) == 0) {
+            // no nodeIncrementEventModifiers were set: false
+            return false;
+        }
+
+        // this is the only path to success...
+        return true;
     }
 
     /**
@@ -663,13 +769,33 @@ public class AWTEventDispatcher implements EventDispatcher,
      * false otherwise.
      */
     protected boolean isNodeDecrementEvent(InputEvent e) {
-        // TODO: Improve code readability!
-        return ((e.getID() == nodeDecrementEventID) &&
-                ((e instanceof KeyEvent) ?
-                     ( ((KeyEvent) e).getKeyCode() == nodeDecrementEventCode) : true) &&
-                ((e.getModifiers() & nodeDecrementEventModifiers) != 0  ));
+        if (e.getID() != nodeDecrementEventID) {
+            // not an nodeDecrementEvent: false
+            return false;
+        }
 
+        if (e instanceof KeyEvent) {
+            if (((KeyEvent) e).getKeyCode() != nodeDecrementEventCode) {
+                // it was a KeyEvent, but not a nodeDecrementEventCode : false
+                return false;
+            }
+        }
+
+        // here: it was not a KeyEvent at all OR a KeyEvent with nodeIncrementEventCode
+        if ((e.getModifiers() & nodeDecrementEventModifiers) == 0) {
+            // no nodeDecrementEventModifiers were set: false
+            return false;
+        }
+
+        // this is the only path to success...
+        return true;
+    }
+
+    /**
+     * Returns whether the meta key is down according to the given modifiers
+     * bitfield.
+     */
+    protected static boolean isMetaDown(int modifiers) {
+        return (modifiers & GraphicsNodeInputEvent.META_MASK) != 0;
     }
 }
-
-
