@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt.font;
 
 import java.awt.Color;
@@ -60,7 +28,6 @@ import java.awt.font.FontRenderContext;
 import java.awt.font.GlyphJustificationInfo;
 import java.awt.font.GlyphMetrics;
 import java.awt.font.GlyphVector;
-import java.awt.font.TextAttribute;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Point2D;
@@ -71,16 +38,17 @@ import java.text.CharacterIterator;
 import org.apache.batik.gvt.text.ArabicTextHandler;
 import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
 import org.apache.batik.gvt.text.TextPaintInfo;
+import org.apache.batik.util.Platform;
 
 /**
  * This is a wrapper class for a java.awt.font.GlyphVector instance.
  *
  * @author <a href="mailto:bella.robinson@cmis.csiro.au">Bella Robinson</a>
- * @version $Id: AWTGVTGlyphVector.java,v 1.30 2003/09/30 00:30:54 deweese Exp $
+ * @version $Id: AWTGVTGlyphVector.java 1619958 2014-08-23 03:38:55Z gadams $
  */
 public class AWTGVTGlyphVector implements GVTGlyphVector {
 
-    public static final AttributedCharacterIterator.Attribute PAINT_INFO 
+    public static final AttributedCharacterIterator.Attribute PAINT_INFO
         = GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO;
 
     private GlyphVector awtGlyphVector;
@@ -105,7 +73,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
     private Rectangle2D visualBounds;
     private Rectangle2D logicalBounds;
     private Rectangle2D bounds2D;
-    private float scaleFactor;
+    private double scaleFactor;
     private float ascent;
     private float descent;
     private TextPaintInfo cacheTPI;
@@ -125,7 +93,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
      */
     public AWTGVTGlyphVector(GlyphVector glyphVector,
                              AWTGVTFont font,
-                             float scaleFactor,
+                             double scaleFactor,
                              CharacterIterator ci) {
 
         this.awtGlyphVector = glyphVector;
@@ -196,7 +164,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
     }
 
     /**
-     * Returns a tight bounds on the GylphVector including stroking.
+     * Returns a tight bounds on the GlyphVector including stroking.
      */
     public Rectangle2D getBounds2D(AttributedCharacterIterator aci) {
         aci.first();
@@ -207,6 +175,8 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
 
         if (tpi == null)
             return null;
+        if (!tpi.visible)
+            return null;
 
         cacheTPI = new TextPaintInfo(tpi);
         Shape outline = null;
@@ -214,20 +184,21 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
             outline = getOutline();
             bounds2D = outline.getBounds2D();
         }
-        
-        // check if we need to include the 
+
+        // check if we need to include the
         // outline of this glyph
         Stroke stroke = tpi.strokeStroke;
         Paint  paint  = tpi.strokePaint;
         if ((stroke != null) && (paint != null)) {
             if (outline == null)
                 outline = getOutline();
-            Rectangle2D strokeBounds 
+            Rectangle2D strokeBounds
                 = stroke.createStrokedShape(outline).getBounds2D();
             if (bounds2D == null)
                 bounds2D = strokeBounds;
-            else 
-                bounds2D = bounds2D.createUnion(strokeBounds);
+            else
+                // bounds2D = bounds2D.createUnion(strokeBounds);
+                bounds2D.add(strokeBounds);
         }
         if (bounds2D == null)
             return null;
@@ -240,7 +211,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
     }
 
     /**
-     *  Returns the logical bounds of this GlyphVector.
+     * Returns the logical bounds of this GlyphVector.
      * This is a bound useful for hit detection and highlighting.
      */
     public Rectangle2D getLogicalBounds() {
@@ -252,8 +223,8 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
     }
 
     /**
-     *  Returns the logical bounds of the specified glyph within this
-     *  GlyphVector.
+     * Returns the logical bounds of the specified glyph within this
+     * GlyphVector.
      */
     public Shape getGlyphLogicalBounds(int glyphIndex) {
         if (glyphLogicalBounds[glyphIndex] == null &&
@@ -274,9 +245,8 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
         Shape[] tempLogicalBounds = new Shape[getNumGlyphs()];
         boolean[] rotated  = new boolean[getNumGlyphs()];
 
-        double maxWidth = -1;
-        double maxHeight = -1;
-
+        double maxWidth = -1.0;
+        double maxHeight = -1.0;
         for (int i = 0; i < getNumGlyphs(); i++) {
 
             if (!glyphVisible[i]) {
@@ -288,13 +258,12 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
             AffineTransform glyphTransform = getGlyphTransform(i);
             GVTGlyphMetrics glyphMetrics   = getGlyphMetrics(i);
 
-            float glyphX      = 0;
-            float glyphY      = -ascent/scaleFactor;
-            float glyphWidth  = (glyphMetrics.getHorizontalAdvance()/
+            float glyphX      = 0.0f;
+            float glyphY      = (float)(-ascent/scaleFactor);
+            float glyphWidth  = (float)(glyphMetrics.getHorizontalAdvance()/
                                  scaleFactor);
-            float glyphHeight = (glyphMetrics.getVerticalAdvance()/
+            float glyphHeight = (float)(glyphMetrics.getVerticalAdvance()/
                                  scaleFactor);
-
             Rectangle2D glyphBounds = new Rectangle2D.Double(glyphX,
                                                              glyphY,
                                                              glyphWidth,
@@ -318,9 +287,9 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
                 Point2D p3 = new Point2D.Double(glyphBounds.getMinX(),
                                                 glyphBounds.getMaxY());
 
+                Point2D gpos = getGlyphPosition(i);
                 AffineTransform tr = AffineTransform.getTranslateInstance
-                    (getGlyphPosition(i).getX(),
-                     getGlyphPosition(i).getY());
+                    (gpos.getX(), gpos.getY());
 
                 if (glyphTransform != null)
                     tr.concatenate(glyphTransform);
@@ -376,28 +345,26 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
 
                 Rectangle2D glyphBounds = tempLogicalBounds[i].getBounds2D();
 
-                double x = glyphBounds.getMinX();
+                double x     = glyphBounds.getMinX();
                 double width = glyphBounds.getWidth();
 
                 if ((i < getNumGlyphs()-1) &&
                     (tempLogicalBounds[i+1] != null)) {
                     // make this glyph extend to the start of the next one
-                    Rectangle2D nextGlyphBounds =
-                        tempLogicalBounds[i+1].getBounds2D();
+                    Rectangle2D ngb = tempLogicalBounds[i+1].getBounds2D();
 
-                    if (nextGlyphBounds.getX() > x) {
-                        // going left to right (this is pretty hoky)
-                        width = nextGlyphBounds.getX() - x;
-                    } else {
-                        double newGlyphX = (nextGlyphBounds.getX() +
-                                            nextGlyphBounds.getWidth());
-                        width += (x - newGlyphX);
-                        x = newGlyphX;
+                    if (ngb.getX() > x) {
+                        double nw = ngb.getX() - x;
+                        if ((nw < width*1.15) && (nw > width*.85)) {
+                            double delta = (nw-width)*.5;
+                            width += delta;
+                            ngb.setRect(ngb.getX()-delta, ngb.getY(),
+                                        ngb.getWidth()+delta, ngb.getHeight());
+                        }
                     }
                 }
-
                 tempLogicalBounds[i] = new Rectangle2D.Double
-                    (x, logicalBounds.getMinY(),
+                    (x,     logicalBounds.getMinY(),
                      width, logicalBounds.getHeight());
             }
         } else if (logicalBounds.getWidth() < maxWidth*1.5) {
@@ -415,27 +382,24 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
                 if ((i < getNumGlyphs()-1) &&
                     (tempLogicalBounds[i+1] != null)) {
                     // make this glyph extend to the start of the next one
-                    Rectangle2D nextGlyphBounds =
-                        tempLogicalBounds[i+1].getBounds2D();
-                    if (nextGlyphBounds.getY() > y) { // going top to bottom
-                        height = nextGlyphBounds.getY() - y;
-                    } else {
-                        double newGlyphY = (nextGlyphBounds.getY() +
-                                            nextGlyphBounds.getHeight());
-                        height += (y - newGlyphY);
-                        y = newGlyphY;
+                    Rectangle2D ngb = tempLogicalBounds[i+1].getBounds2D();
+                    if (ngb.getY() > y) { // going top to bottom
+                        double nh = ngb.getY() - y;
+                        if ((nh < height*1.15) && (nh > height*.85)) {
+                            double delta = (nh-height)*.5;
+                            height += delta;
+                            ngb.setRect(ngb.getX(), ngb.getY()-delta,
+                                        ngb.getWidth(), ngb.getHeight()+delta);
+                        }
                     }
                 }
-
                 tempLogicalBounds[i] = new Rectangle2D.Double
                     (logicalBounds.getMinX(),  y,
                      logicalBounds.getWidth(), height);
             }
         }
 
-        for (int i = 0; i < getNumGlyphs(); i++) {
-            glyphLogicalBounds[i] = tempLogicalBounds[i];
-        }
+        System.arraycopy( tempLogicalBounds, 0, glyphLogicalBounds, 0, getNumGlyphs() );
     }
 
     /**
@@ -443,33 +407,31 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
      * GVTGlyphVector.
      */
     public GVTGlyphMetrics getGlyphMetrics(int glyphIndex) {
-        if (glyphMetrics[glyphIndex] == null) {
-/*
-            GlyphMetrics gm = awtGlyphVector.getGlyphMetrics(glyphIndex);
-            Rectangle2D gmB = gm.getBounds2D();
-*/
-            // -- start glyph cache code --
-            Point2D glyphPos = defaultGlyphPositions[glyphIndex];
-            char c = ci.setIndex(ci.getBeginIndex()+glyphIndex);
-            ci.setIndex(ci.getBeginIndex());
-            AWTGlyphGeometryCache.Value v = AWTGVTFont.getGlyphGeometry
-                (gvtFont, c, awtGlyphVector, glyphIndex, glyphPos);
-            Rectangle2D gmB = v.getBounds2D();
-           // -- end glyph cache code --
+        if (glyphMetrics[glyphIndex] != null)
+            return glyphMetrics[glyphIndex];
 
-            Rectangle2D bounds = new Rectangle2D.Double
-                (gmB.getX()     * scaleFactor, gmB.getY()      * scaleFactor,
-                 gmB.getWidth() * scaleFactor, gmB.getHeight() * scaleFactor);
+        // -- start glyph cache code --
+        Point2D glyphPos = defaultGlyphPositions[glyphIndex];
+        char c = ci.setIndex(ci.getBeginIndex()+glyphIndex);
+        ci.setIndex(ci.getBeginIndex());
+        AWTGlyphGeometryCache.Value v = AWTGVTFont.getGlyphGeometry
+            (gvtFont, c, awtGlyphVector, glyphIndex, glyphPos);
+        Rectangle2D gmB = v.getBounds2D();
+        // -- end glyph cache code --
+
+        Rectangle2D bounds = new Rectangle2D.Double
+            (gmB.getX()     * scaleFactor, gmB.getY()      * scaleFactor,
+             gmB.getWidth() * scaleFactor, gmB.getHeight() * scaleFactor);
+
+        // defaultGlyphPositions has one more entry than glyphs
+        // the last entry stores the total advance for the
+        // glyphVector.
+        float adv = (float)(defaultGlyphPositions[glyphIndex+1].getX()-
+                            defaultGlyphPositions[glyphIndex]  .getX());
+        glyphMetrics[glyphIndex] =  new GVTGlyphMetrics
+            ((float)(adv*scaleFactor), (ascent+descent),
+             bounds, GlyphMetrics.STANDARD);
 
-            // defaultGlyphPositions has one more entry than glyphs
-            // the last entry stores the total advance for the
-            // glyphVector.
-            float adv = (float)(defaultGlyphPositions[glyphIndex+1].getX()-
-                                defaultGlyphPositions[glyphIndex]  .getX());
-            glyphMetrics[glyphIndex] =  new GVTGlyphMetrics
-                (adv*scaleFactor, (ascent+descent),
-                 bounds, GlyphMetrics.STANDARD);
-        }
         return glyphMetrics[glyphIndex];
     }
 
@@ -541,10 +503,11 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
     static {
         String s = System.getProperty("java.specification.version");
         if ("1.4".compareTo(s) <= 0) {
+            // TODO Java 5
             outlinesPositioned = true;
             drawGlyphVectorWorks = true;
             glyphVectorTransformWorks = true;
-        } else if ("Mac OS X".equals(System.getProperty("os.name"))) {
+        } else if (Platform.isOSX) {
             outlinesPositioned = true;
             drawGlyphVectorWorks = false;
             glyphVectorTransformWorks = false;
@@ -561,6 +524,15 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
         return outlinesPositioned;
     }
 
+    /**
+     * Returns the bounding box of the specified glyph, considering only the
+     * glyph's metrics (ascent, descent, advance) rather than the actual glyph
+     * shape.
+     */
+    public Rectangle2D getGlyphCellBounds(int glyphIndex) {
+        return getGlyphLogicalBounds(glyphIndex).getBounds2D();
+    }
+
     /**
      * Returns the position of the specified glyph within this GlyphVector.
      */
@@ -641,9 +613,9 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
      * of this GlyphVector.
      */
     public Shape getOutline() {
-        if (outline != null) 
+        if (outline != null)
             return outline;
-        
+
         outline = new GeneralPath();
         for (int i = 0; i < getNumGlyphs(); i++) {
             if (glyphVisible[i]) {
@@ -704,19 +676,50 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
             Point2D glyphPos = defaultGlyphPositions[i];
             float x = (float)((glyphPos.getX() * scaleFactor)-shiftLeft);
             float y = (float) (glyphPos.getY() * scaleFactor);
-            if (glyphPositions[i] == null) {
-                glyphPositions[i] = new Point2D.Float(x,y);
-            } else {
-                glyphPositions[i].x = x;
-                glyphPositions[i].y = y;
-            }
 
             // if c is a transparent arabic char then need to shift the
             // following glyphs left so that the current glyph is overwritten
-            char c = ci.setIndex(i + ci.getBeginIndex());
+            /*char c =*/ ci.setIndex(i + ci.getBeginIndex());
+            /*
             if (ArabicTextHandler.arabicCharTransparent(c)) {
+                int j;
                 shiftLeft += getGlyphMetrics(i).getHorizontalAdvance();
-            }
+                for (j=i+1; j<getNumGlyphs(); j++) {
+                    char c2 = ci.setIndex(j+ci.getBeginIndex());
+                    if (!ArabicTextHandler.arabicCharTransparent(c2)) break;
+                    shiftLeft += getGlyphMetrics(j).getHorizontalAdvance();
+                }
+                if (j != getNumGlyphs()) {
+                    Point2D glyphPosBase = defaultGlyphPositions[j];
+                    double rEdge = glyphPosBase.getX()+getGlyphMetrics(j).getHorizontalAdvance();
+                    rEdge -= shiftLeft;
+                    for (int k=i; k<j; k++) {
+                        glyphTransforms   [k] = null;
+                        glyphVisualBounds [k] = null;
+                        glyphLogicalBounds[k] = null;
+                        glyphOutlines     [k] = null;
+                        glyphMetrics      [k] = null;
+                        x = (float)rEdge-getGlyphMetrics(k).getHorizontalAdvance();
+                        y = (float) (defaultGlyphPositions[k].getY() * scaleFactor);
+                        if (glyphPositions[k] == null) {
+                            glyphPositions[k] = new Point2D.Float(x,y);
+                        } else {
+                            glyphPositions[k].x = x;
+                            glyphPositions[k].y = y;
+                        }
+                    }
+                    i = j-1;
+                }
+            } else {
+            */
+                if (glyphPositions[i] == null) {
+                    glyphPositions[i] = new Point2D.Float(x,y);
+                } else {
+                    glyphPositions[i].x = x;
+                    glyphPositions[i].y = y;
+                }
+                // }
+
         }
 
         // Need glyph pos for point after last char...
@@ -765,7 +768,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
      * Tells the glyph vector whether or not to draw the specified glyph.
      */
     public void setGlyphVisible(int glyphIndex, boolean visible) {
-        if (visible == glyphVisible[glyphIndex]) 
+        if (visible == glyphVisible[glyphIndex])
             return;
         glyphVisible[glyphIndex] = visible;
         outline       = null;
@@ -806,16 +809,21 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
         int end   = endGlyphIndex+ci.getBeginIndex();
 
         for (char c = ci.setIndex(start); ci.getIndex() <= end; c=ci.next()) {
-            if (ArabicTextHandler.isLigature(c)) {
-                charCount += ArabicTextHandler.getNumChars(c);
-            } else {
-                charCount++;
-            }
+            charCount += ArabicTextHandler.getNumChars(c);
         }
 
         return charCount;
     }
 
+    @Override
+    public boolean isReversed() {
+        return false;
+    }
+
+    @Override
+    public void maybeReverse(boolean mirror) {
+    }
+
     /**
      * Draws this glyph vector.
      */
@@ -827,6 +835,8 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
         TextPaintInfo tpi = (TextPaintInfo)aci.getAttribute
             (GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO);
         if (tpi == null) return;
+        if (!tpi.visible) return;
+
         Paint  fillPaint   = tpi.fillPaint;
         Stroke stroke      = tpi.strokeStroke;
         Paint  strokePaint = tpi.strokePaint;
@@ -840,7 +850,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
             // Can't stroke with drawGlyphVector.
             useHinting = false;
 
-        if (useHinting && 
+        if (useHinting &&
             (fillPaint != null) && !(fillPaint instanceof Color))
             // The coordinate system is different for drawGlyphVector.
             // So complex paints aren't positioned properly.
@@ -868,7 +878,7 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
             if (((type & typeGTrans) != 0) || ((type & typeGRot)  != 0))
                 useHinting = false;
         }
-            
+
         if (useHinting) {
             for (int i=0; i<numGlyphs; i++) {
                 if (!glyphVisible[i]) {
@@ -924,8 +934,8 @@ public class AWTGVTGlyphVector implements GVTGlyphVector {
             }
             graphics2D.scale(sf, sf);
             graphics2D.setPaint(fillPaint);
-            graphics2D.drawGlyphVector(awtGlyphVector, 0, 0);
-            graphics2D.scale(1/sf, 1/sf);
+            graphics2D.drawGlyphVector(awtGlyphVector, 0.0f, 0.0f);
+            graphics2D.scale(1.0/sf, 1.0/sf);
 
             for (int i=0; i< numGlyphs; i++) {
                 Point2D         pos = defaultGlyphPositions[i];
