@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt.font;
 
 import java.awt.Graphics2D;
@@ -60,6 +28,7 @@ import java.awt.geom.Point2D;
 import java.awt.geom.Rectangle2D;
 import java.text.AttributedCharacterIterator;
 
+import org.apache.batik.gvt.text.ArabicTextHandler;
 import org.apache.batik.gvt.text.GVTAttributedCharacterIterator;
 import org.apache.batik.gvt.text.TextPaintInfo;
 
@@ -67,11 +36,11 @@ import org.apache.batik.gvt.text.TextPaintInfo;
  * A GVTGlyphVector class for SVG fonts.
  *
  * @author <a href="mailto:bella.robinson@cmis.csiro.au">Bella Robinson</a>
- * @version $Id: SVGGVTGlyphVector.java,v 1.17 2003/09/07 22:25:28 deweese Exp $
+ * @version $Id: SVGGVTGlyphVector.java 1619958 2014-08-23 03:38:55Z gadams $
  */
 public final class SVGGVTGlyphVector implements GVTGlyphVector {
 
-    public static final AttributedCharacterIterator.Attribute PAINT_INFO 
+    public static final AttributedCharacterIterator.Attribute PAINT_INFO
         = GVTAttributedCharacterIterator.TextAttribute.PAINT_INFO;
 
     private GVTFont           font;
@@ -93,7 +62,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
      * glyph vector.
      * @param frc The current font render context.
      */
-    public SVGGVTGlyphVector(GVTFont font, Glyph[] glyphs, 
+    public SVGGVTGlyphVector(GVTFont font, Glyph[] glyphs,
                              FontRenderContext frc) {
         this.font = font;
         this.glyphs = glyphs;
@@ -226,8 +195,6 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
 
         Shape[] tempLogicalBounds = new Shape[getNumGlyphs()];
         boolean[] rotated = new boolean[getNumGlyphs()];
-        boolean[] flippedH = new boolean[getNumGlyphs()];
-        boolean[] flippedV = new boolean[getNumGlyphs()];
 
         double maxWidth = -1;
         double maxHeight = -1;
@@ -243,7 +210,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
             AffineTransform glyphTransform = getGlyphTransform(i);
             GVTGlyphMetrics glyphMetrics   = getGlyphMetrics(i);
             Rectangle2D glyphBounds = new Rectangle2D.Double
-                (0, -ascent, glyphMetrics.getHorizontalAdvance(), 
+                (0, -ascent, glyphMetrics.getHorizontalAdvance(),
                  ascent+descent);
 
             if (glyphBounds.isEmpty()) {
@@ -251,33 +218,28 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
                 // the same as the previous glyph, if we have one...
                 if (i > 0) {
                     rotated[i] = rotated[i-1];
-                    flippedH[i] = flippedH[i-1];
-                    flippedV[i] = flippedV[i-1];
                 } else {
                     rotated [i] = true;
-                    flippedH[i] = false;
-                    flippedV[i] = false;
                 }
             } else {
                 // get three corner points so we can determine
                 // whether the glyph is rotated
-                Point2D p1 = new Point2D.Double(glyphBounds.getMinX(), 
+                Point2D p1 = new Point2D.Double(glyphBounds.getMinX(),
                                                 glyphBounds.getMinY());
-                Point2D p2 = new Point2D.Double(glyphBounds.getMaxX(), 
+                Point2D p2 = new Point2D.Double(glyphBounds.getMaxX(),
                                                 glyphBounds.getMinY());
-                Point2D p3 = new Point2D.Double(glyphBounds.getMinX(), 
+                Point2D p3 = new Point2D.Double(glyphBounds.getMinX(),
                                                 glyphBounds.getMaxY());
-                    
+                Point2D gpos = getGlyphPosition(i);
                 AffineTransform tr = AffineTransform.getTranslateInstance
-                    (getGlyphPosition(i).getX(),
-                     getGlyphPosition(i).getY());
-                    
+                    (gpos.getX(), gpos.getY());
+
                 if (glyphTransform != null)
                     tr.concatenate(glyphTransform);
 
-                tempLogicalBounds[i] = 
+                tempLogicalBounds[i] =
                     tr.createTransformedShape(glyphBounds);
-                    
+
                 Point2D tp1 = new Point2D.Double();
                 Point2D tp2 = new Point2D.Double();
                 Point2D tp3 = new Point2D.Double();
@@ -291,32 +253,21 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
 
                 if ((Math.abs(tdx12) < 0.001) &&
                     (Math.abs(tdy13) < 0.001)) {
-                    // If these are both zero then it is axially aligned 
+                    // If these are both zero then it is axially aligned
                     // on it's "side"...
                     rotated[i] = false;
-                    double dx13 = p1.getX()-p3.getX();
-                    double dy12 = p1.getY()-p2.getY();
-                    if (Math.abs(tdx13+dx13) < 0.001) flippedH[i] = true;
-                    if (Math.abs(tdy12+dy12) < 0.001) flippedV[i] = true;
-                        
                 } else if ((Math.abs(tdx13) < 0.001) &&
                            (Math.abs(tdy12) < 0.001)) {
-                    // If these are both zero then it is axially aligned 
+                    // If these are both zero then it is axially aligned
                     // vertically.
                     rotated[i] = false;
-                    double dx12 = p1.getX()-p2.getX();
-                    double dy13 = p1.getY()-p3.getY();
-                    if (Math.abs(tdx12+dx12) < 0.001) flippedH[i] = true;
-                    if (Math.abs(tdy13+dy13) < 0.001) flippedV[i] = true;
                 } else {
-                    rotated [i] = true;
-                    flippedH[i] = false;
-                    flippedV[i] = false;
+                    rotated[i] = true;
                 }
-                    
+
                 Rectangle2D rectBounds;
                 rectBounds = tempLogicalBounds[i].getBounds2D();
-                if (rectBounds.getWidth() > maxWidth) 
+                if (rectBounds.getWidth() > maxWidth)
                     maxWidth = rectBounds.getWidth();
                 if (rectBounds.getHeight() > maxHeight)
                     maxHeight = rectBounds.getHeight();
@@ -345,63 +296,24 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
                 double x = glyphBounds.getMinX();
                 double width = glyphBounds.getWidth();
 
-                if ((i < getNumGlyphs()-1) && 
+                if ((i < getNumGlyphs()-1) &&
                     (tempLogicalBounds[i+1] != null)) {
                     // make this glyph extend to the start of the next one
-                    Rectangle2D nextGlyphBounds = 
-                        tempLogicalBounds[i+1].getBounds2D();
+                    Rectangle2D ngb = tempLogicalBounds[i+1].getBounds2D();
 
-                    if (nextGlyphBounds.getX() > x) { 
-                        // going left to right (this is pretty hoky)
-                        width = nextGlyphBounds.getX() - x;
-                    } else {
-                        double newGlyphX = (nextGlyphBounds.getX() + 
-                                            nextGlyphBounds.getWidth());
-                        width += (x - newGlyphX);
-                        x = newGlyphX;
-                    }
-                }
-
-                float x0 = (float)x;
-                float x1 = (float)(x0+width);
-                float y0 = (float)fullBounds.getMinY();
-                float y1 = (float)(y0+fullBounds.getHeight());
-                // Build the bounds rect the way things expect to see it...
-                if (flippedH[i]) {
-                    if (flippedV[i]) {
-                        GeneralPath gp = new GeneralPath();
-                        gp.moveTo(x1,y1);
-                        gp.lineTo(x0,y1);
-                        gp.lineTo(x0,y0);
-                        gp.lineTo(x1,y0);
-                        gp.lineTo(x1,y1);
-                        gp.closePath();
-                        tempLogicalBounds[i] = gp;
-                    } else {
-                        GeneralPath gp = new GeneralPath();
-                        gp.moveTo(x1,y0);
-                        gp.lineTo(x0,y0);
-                        gp.lineTo(x0,y1);
-                        gp.lineTo(x1,y1);
-                        gp.lineTo(x1,y0);
-                        gp.closePath();
-                        tempLogicalBounds[i] = gp;
-                    }
-                } else {
-                    if (flippedV[i]) {
-                        GeneralPath gp = new GeneralPath();
-                        gp.moveTo(x0,y1);
-                        gp.lineTo(x1,y1);
-                        gp.lineTo(x1,y0);
-                        gp.lineTo(x0,y0);
-                        gp.lineTo(x0,y1);
-                        gp.closePath();
-                        tempLogicalBounds[i] = gp;
-                    } else {
-                        tempLogicalBounds[i] = new Rectangle2D.Double
-                            (x0, y0, x1-x0, y1-y0);
+                    if (ngb.getX() > x) {
+                        double nw = ngb.getX() - x;
+                        if ((nw < width*1.15) && (nw > width*.85)) {
+                            double delta = (nw-width)*.5;
+                            width += delta;
+                            ngb.setRect(ngb.getX()-delta, ngb.getY(),
+                                        ngb.getWidth()+delta, ngb.getHeight());
+                        }
                     }
                 }
+                tempLogicalBounds[i] = new Rectangle2D.Double
+                    (x,     fullBounds.getMinY(),
+                     width, fullBounds.getHeight());
             }
         } else if (fullBounds.getWidth() < maxWidth*1.5) {
             // make all glyphs left and right edges the same as the full bounds
@@ -415,98 +327,57 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
                 double      y           = glyphBounds.getMinY();
                 double      height      = glyphBounds.getHeight();
 
-                if ((i < getNumGlyphs()-1) && 
+                if ((i < getNumGlyphs()-1) &&
                     (tempLogicalBounds[i+1] != null)) {
                     // make this glyph extend to the start of the next one
-                    Rectangle2D nextGlyphBounds = 
-                        tempLogicalBounds[i+1].getBounds2D();
-                    if (nextGlyphBounds.getY() > y) { // going top to bottom
-                        height = nextGlyphBounds.getY() - y;
-                    } else {
-                        double newGlyphY = (nextGlyphBounds.getY() + 
-                                            nextGlyphBounds.getHeight());
-                        height += (y - newGlyphY);
-                        y = newGlyphY;
-                    }
-                }
-
-                float x0 = (float)fullBounds.getMinX();
-                float x1 = (float)(x0+fullBounds.getWidth());
-                float y0 = (float)y;
-                float y1 = (float)(y0+height);
-                // Build the rect the way things expect to see it...
-                if (flippedH[i]) {
-                    if (flippedV[i]) {
-                        GeneralPath gp = new GeneralPath();
-                        gp.moveTo(x1,y1);
-                        gp.lineTo(x0,y1);
-                        gp.lineTo(x0,y0);
-                        gp.lineTo(x1,y0);
-                        gp.lineTo(x1,y1);
-                        gp.closePath();
-                        tempLogicalBounds[i] = gp;
-                    } else {
-                        GeneralPath gp = new GeneralPath();
-                        gp.moveTo(x1,y0);
-                        gp.lineTo(x0,y0);
-                        gp.lineTo(x0,y1);
-                        gp.lineTo(x1,y1);
-                        gp.lineTo(x1,y0);
-                        gp.closePath();
-                        tempLogicalBounds[i] = gp;
-                    }
-                } else {
-                    if (flippedV[i]) {
-                        GeneralPath gp = new GeneralPath();
-                        gp.moveTo(x0,y1);
-                        gp.lineTo(x1,y1);
-                        gp.lineTo(x1,y0);
-                        gp.lineTo(x0,y0);
-                        gp.lineTo(x0,y1);
-                        gp.closePath();
-                        tempLogicalBounds[i] = gp;
-                    } else {
-                        tempLogicalBounds[i] = new Rectangle2D.Double
-                            (x0, y0, x1-x0, y1-y0);
+                    Rectangle2D ngb = tempLogicalBounds[i+1].getBounds2D();
+                    if (ngb.getY() > y) { // going top to bottom
+                        double nh = ngb.getY() - y;
+                        if ((nh < height*1.15) && (nh > height*.85)) {
+                            double delta = (nh-height)*.5;
+                            height += delta;
+                            ngb.setRect(ngb.getX(), ngb.getY()-delta,
+                                        ngb.getWidth(), ngb.getHeight()+delta);
+                        }
                     }
                 }
+                tempLogicalBounds[i] = new Rectangle2D.Double
+                    (fullBounds.getMinX(),  y,
+                     fullBounds.getWidth(), height);
             }
         }
 
-        for (int i = 0; i < getNumGlyphs(); i++) {
-            glyphLogicalBounds[i] = tempLogicalBounds[i];
-        }
+        System.arraycopy( tempLogicalBounds, 0, glyphLogicalBounds, 0, getNumGlyphs() );
     }
 
     /**
      * Returns the metrics of the glyph at the specified index into this
      * GlyphVector.
      */
-    public GVTGlyphMetrics getGlyphMetrics(int glyphIndex) {
+    public GVTGlyphMetrics getGlyphMetrics(int idx) {
 
-        if (glyphIndex < 0 || (glyphIndex > glyphs.length-1)) {
-            throw new IndexOutOfBoundsException("glyphIndex: " + glyphIndex
-            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + ".");
-        }
+        if (idx < 0 || (idx > glyphs.length-1))
+            throw new IndexOutOfBoundsException
+                ("idx: " + idx + ", is out of bounds. Should be between 0 and "
+                 + (glyphs.length-1) + '.' );
 
         // check to see if we should kern this glyph
         // I return the kerning information in the glyph metrics
         // as a first pass at implementation (I don't want to
         // fiddle with layout too much right now).
-
-        if (glyphIndex < glyphs.length - 1) {
+        if (idx < glyphs.length - 1) {
             // check for kerning
             if (font != null) {
-                float hkern = font.getHKern(glyphs[glyphIndex].getGlyphCode(),
-                                            glyphs[glyphIndex+1].getGlyphCode());
-                float vkern = font.getVKern(glyphs[glyphIndex].getGlyphCode(),
-                                            glyphs[glyphIndex+1].getGlyphCode());
-                return glyphs[glyphIndex].getGlyphMetrics(hkern, vkern);
+                float hkern = font.getHKern(glyphs[idx].getGlyphCode(),
+                                            glyphs[idx+1].getGlyphCode());
+                float vkern = font.getVKern(glyphs[idx].getGlyphCode(),
+                                            glyphs[idx+1].getGlyphCode());
+                return glyphs[idx].getGlyphMetrics(hkern, vkern);
             }
         }
 
         // get a normal metrics
-        return glyphs[glyphIndex].getGlyphMetrics();
+        return glyphs[idx].getGlyphMetrics();
     }
 
     /**
@@ -521,6 +392,15 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
         return glyphs[glyphIndex].getOutline();
     }
 
+    /**
+     * Returns the bounding box of the specified glyph, considering only the
+     * glyph's metrics (ascent, descent, advance) rather than the actual glyph
+     * shape.
+     */
+    public Rectangle2D getGlyphCellBounds(int glyphIndex) {
+        return getGlyphLogicalBounds(glyphIndex).getBounds2D();
+    }
+
     /**
      * Returns the position of the specified glyph within this GlyphVector.
      */
@@ -530,7 +410,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
 
         if (glyphIndex < 0 || (glyphIndex > glyphs.length-1)) {
             throw new IndexOutOfBoundsException("glyphIndex: " + glyphIndex
-            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + ".");
+            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + '.' );
         }
         return glyphs[glyphIndex].getPosition();
     }
@@ -552,7 +432,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
         }
         if ((beginGlyphIndex+numEntries) > glyphs.length+1) {
              throw new IndexOutOfBoundsException("beginGlyphIndex + numEntries ("
-                       + beginGlyphIndex + "+" + numEntries
+                       + beginGlyphIndex + '+' + numEntries
                        + ") exceeds the number of glpyhs in this GlyphVector");
         }
         if (positionReturn == null) {
@@ -578,7 +458,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
     public AffineTransform getGlyphTransform(int glyphIndex) {
         if (glyphIndex < 0 || (glyphIndex > glyphs.length-1)) {
             throw new IndexOutOfBoundsException("glyphIndex: " + glyphIndex
-            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + ".");
+            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + '.' );
         }
         return glyphs[glyphIndex].getTransform();
     }
@@ -589,7 +469,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
     public Shape getGlyphVisualBounds(int glyphIndex) {
         if (glyphIndex < 0 || (glyphIndex > glyphs.length-1)) {
             throw new IndexOutOfBoundsException("glyphIndex: " + glyphIndex
-            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + ".");
+            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + '.' );
         }
         return glyphs[glyphIndex].getOutline();
     }
@@ -606,21 +486,24 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
             return bounds2D;
 
         Rectangle2D b=null;
-        for (int i = 0; i < getNumGlyphs(); i++) {
-            if (!glyphVisible[i])  continue;
+        if (tpi.visible) {
+            for (int i = 0; i < getNumGlyphs(); i++) {
+                if (!glyphVisible[i])  continue;
 
-            Rectangle2D glyphBounds = glyphs[i].getBounds2D();
-            // System.out.println("GB["+i+"]: " + glyphBounds);
-            if (glyphBounds == null) continue;
-            if (b == null) b=glyphBounds;
-            else b = glyphBounds.createUnion(b);
+                Rectangle2D glyphBounds = glyphs[i].getBounds2D();
+                // System.out.println("GB["+i+"]: " + glyphBounds);
+                if (glyphBounds == null) continue;
+                if (b == null) b=glyphBounds;
+                //else b = glyphBounds.createUnion(b);
+                else b.add( glyphBounds );
+            }
         }
 
         bounds2D = b;
         if ( bounds2D == null ){
             bounds2D = new Rectangle2D.Float();
         }
-        cacheTPI = tpi;
+        cacheTPI = new TextPaintInfo(tpi);
         return bounds2D;
     }
 
@@ -696,16 +579,45 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
      * layout is horizontal.
      */
     public void performDefaultLayout() {
+        logicalBounds = null;
+        outline       = null;
+        bounds2D      = null;
+
         float currentX = 0;
         float currentY = 0;
         for (int i = 0; i < glyphs.length; i++) {
-            glyphs[i].setPosition(new Point2D.Float(currentX, currentY));
-            glyphs[i].setTransform(null);
+            Glyph g = glyphs[i];
+            g.setTransform(null);
             glyphLogicalBounds[i] = null;
-            currentX += glyphs[i].getHorizAdvX();
-            logicalBounds = null;
-            outline = null;
-            bounds2D = null;
+
+            String uni = g.getUnicode();
+            if ((uni != null)  && (uni.length() != 0) &&
+                ArabicTextHandler.arabicCharTransparent(uni.charAt(0))) {
+                int j;
+                for (j=i+1; j<glyphs.length; j++) {
+                    uni = glyphs[j].getUnicode();
+                    if ((uni == null)  || (uni.length() == 0)) break;
+                    char ch = uni.charAt(0);
+                    if (!ArabicTextHandler.arabicCharTransparent(ch))
+                        break;
+                }
+                if (j != glyphs.length) {
+                    Glyph bg = glyphs[j];
+                    float rEdge = currentX + bg.getHorizAdvX();
+                    for (int k=i; k<j; k++) {
+                        g = glyphs[k];
+                        g.setTransform(null);
+                        glyphLogicalBounds[i] = null;
+                        g.setPosition(new Point2D.Float(rEdge-g.getHorizAdvX(),
+                                                        currentY));
+                    }
+                    i = j;
+                    g = bg;
+                }
+            }
+
+            g.setPosition(new Point2D.Float(currentX, currentY));
+            currentX += g.getHorizAdvX();
         }
         endPos = new Point2D.Float(currentX, currentY);
     }
@@ -722,7 +634,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
 
         if (glyphIndex < 0 || (glyphIndex > glyphs.length-1)) {
             throw new IndexOutOfBoundsException("glyphIndex: " + glyphIndex
-            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + ".");
+            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + '.' );
         }
         glyphs[glyphIndex].setPosition(newPos);
         glyphLogicalBounds[glyphIndex] = null;
@@ -737,7 +649,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
     public void setGlyphTransform(int glyphIndex, AffineTransform newTX) {
         if (glyphIndex < 0 || (glyphIndex > glyphs.length-1)) {
             throw new IndexOutOfBoundsException("glyphIndex: " + glyphIndex
-            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + ".");
+            + ", is out of bounds. Should be between 0 and " + (glyphs.length-1) + '.' );
         }
         glyphs[glyphIndex].setTransform(newTX);
         glyphLogicalBounds[glyphIndex] = null;
@@ -750,7 +662,7 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
      * Tells the glyph vector whether or not to draw the specified glyph.
      */
     public void setGlyphVisible(int glyphIndex, boolean visible) {
-        if (visible == glyphVisible[glyphIndex]) 
+        if (visible == glyphVisible[glyphIndex])
             return;
 
         glyphVisible[glyphIndex] = visible;
@@ -795,11 +707,24 @@ public final class SVGGVTGlyphVector implements GVTGlyphVector {
         return numChars;
     }
 
+    @Override
+    public boolean isReversed() {
+        return false;
+    }
+
+    @Override
+    public void maybeReverse(boolean mirror) {
+    }
+
     /**
      * Draws this glyph vector.
      */
-    public void draw(Graphics2D graphics2D, 
+    public void draw(Graphics2D graphics2D,
                      AttributedCharacterIterator aci) {
+        aci.first();
+        TextPaintInfo tpi = (TextPaintInfo)aci.getAttribute(PAINT_INFO);
+        if (!tpi.visible) return;
+
         for (int i = 0; i < glyphs.length; i++) {
             if (glyphVisible[i]) {
                 glyphs[i].draw(graphics2D);
