@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt.renderer;
 
 import java.awt.AlphaComposite;
@@ -65,10 +33,10 @@ import java.awt.image.SampleModel;
 import java.awt.image.WritableRaster;
 import java.awt.image.renderable.RenderContext;
 import java.lang.ref.SoftReference;
-import java.util.ArrayList;
-import java.util.Iterator;
 import java.util.Collection;
+import java.util.Iterator;
 
+import org.apache.batik.ext.awt.geom.RectListManager;
 import org.apache.batik.ext.awt.image.GraphicsUtil;
 import org.apache.batik.ext.awt.image.PadMode;
 import org.apache.batik.ext.awt.image.renderable.Filter;
@@ -76,24 +44,17 @@ import org.apache.batik.ext.awt.image.rendered.CachableRed;
 import org.apache.batik.ext.awt.image.rendered.PadRed;
 import org.apache.batik.ext.awt.image.rendered.TileCacheRed;
 import org.apache.batik.ext.awt.image.rendered.TranslateRed;
-import org.apache.batik.ext.awt.geom.RectListManager;
 import org.apache.batik.gvt.GraphicsNode;
+import org.apache.batik.util.HaltingThread;
 
 /**
  * Simple implementation of the Renderer that simply does static
  * rendering in an offscreen buffer image.
  *
- * @author <a href="mailto:vincent.hardy@eng.sun.com>Vincent Hardy</a>
- * @version $Id: StaticRenderer.java,v 1.28 2003/10/31 22:39:42 deweese Exp $
+ * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
+ * @version $Id: StaticRenderer.java 1372129 2012-08-12 15:31:50Z helder $
  */
 public class StaticRenderer implements ImageRenderer {
-    /**
-     * Error messages
-     */
-    private static final String ILLEGAL_ARGUMENT_NULL_OFFSCREEN =
-        "offScreen should not be null";
-    private static final String ILLEGAL_ARGUMENT_ZERO_WIDTH_OR_HEIGHT =
-        "offScreen should have positive width/height";
 
     /**
      * Tree this Renderer paints.
@@ -145,15 +106,17 @@ public class StaticRenderer implements ImageRenderer {
      */
     public StaticRenderer(RenderingHints rh,
                           AffineTransform at){
-        renderingHints = new RenderingHints(rh);
+        renderingHints = new RenderingHints(null);
+        renderingHints.add(rh);
         usr2dev = new AffineTransform(at);
     }
 
     /**
-     * @param offScreen image where the Renderer should do its rendering
+     * Creates a new StaticRenderer object.
      */
     public StaticRenderer(){
-        renderingHints = new RenderingHints(defaultRenderingHints);
+        renderingHints = new RenderingHints(null);
+        renderingHints.add(defaultRenderingHints);
         usr2dev = new AffineTransform();
     }
 
@@ -165,7 +128,7 @@ public class StaticRenderer implements ImageRenderer {
         rootGN     = null;
         rootFilter = null;
         rootCR     = null;
-        
+
         workingOffScreen = null;
         workingBaseRaster = null;
         workingRaster = null;
@@ -175,6 +138,8 @@ public class StaticRenderer implements ImageRenderer {
         currentRaster = null;
 
         renderingHints = null;
+        lastCache = null;
+        lastCR = null;
     }
 
     /**
@@ -207,7 +172,8 @@ public class StaticRenderer implements ImageRenderer {
      * @param rh Set of rendering hints to use for future renderings
      */
     public void setRenderingHints(RenderingHints rh) {
-        renderingHints = new RenderingHints(rh);
+        renderingHints = new RenderingHints(null);
+        renderingHints.add(rh);
 
         rootFilter = null;
         rootCR     = null;
@@ -221,7 +187,7 @@ public class StaticRenderer implements ImageRenderer {
 
     /**
      * @return the RenderingHints which the Renderer is using for its
-     *         rendering 
+     *         rendering
      */
     public RenderingHints getRenderingHints() {
         return renderingHints;
@@ -247,9 +213,8 @@ public class StaticRenderer implements ImageRenderer {
     }
 
     /**
-     * Returns a copy of the transform from the current user space (as
-     * defined by the top node of the GVT tree) to the device space (1
-     * unit = 1/72nd of an inch / 1 pixel, roughly speaking
+     * Returns the transform from the current user space (as defined
+     * by the top node of the GVT tree) to the device space.
      */
     public AffineTransform getTransform(){
         return usr2dev;
@@ -258,7 +223,7 @@ public class StaticRenderer implements ImageRenderer {
     /**
      * Returns true if the Renderer is currently doubleBuffering is
      * rendering requests.  If it is then getOffscreen will only
-     * return completed renderings (or null if nothing is available).  
+     * return completed renderings (or null if nothing is available).
      */
     public boolean isDoubleBuffered(){
         return isDoubleBuffered;
@@ -295,7 +260,7 @@ public class StaticRenderer implements ImageRenderer {
      * Note that this change will not be reflected by calls to
      * getOffscreen until either clearOffScreen has completed (when
      * isDoubleBuffered is false) or reapint has completed (when
-     * isDoubleBuffered is true).  
+     * isDoubleBuffered is true).
      *
      */
     public void updateOffScreen(int width, int height) {
@@ -305,7 +270,7 @@ public class StaticRenderer implements ImageRenderer {
 
     /**
      * Returns the current offscreen image.
-     * 
+     *
      * The exact symantics of this vary base on the value of
      * isDoubleBuffered.  If isDoubleBuffered is false this will
      * return the image currently being worked on as soon as it is
@@ -330,20 +295,20 @@ public class StaticRenderer implements ImageRenderer {
      *
      * When double buffering this call can effectively be skipped,
      * since getOffscreen will only refect the new rendering after
-     * repaint completes.  
+     * repaint completes.
      */
     public void clearOffScreen() {
 
         // No need to clear in double buffer case people will
         // only see it when it is done...
-        if (isDoubleBuffered) 
+        if (isDoubleBuffered)
             return;
 
         updateWorkingBuffers();
         if ((rootCR == null)           ||
             (workingBaseRaster == null))
             return;
-        
+
         ColorModel     cm         = rootCR.getColorModel();
         WritableRaster syncRaster = workingBaseRaster;
 
@@ -360,8 +325,8 @@ public class StaticRenderer implements ImageRenderer {
 
 
     /**
-     * Repaints the associated GVT tree under <tt>area</tt>.
-     * 
+     * Repaints the associated GVT tree under <code>area</code>.
+     *
      * If double buffered is true and this method completes cleanly it
      * will set the result of the repaint as the image returned by
      * getOffscreen otherwise the old image will still be returned.
@@ -370,7 +335,7 @@ public class StaticRenderer implements ImageRenderer {
      * by getOffscreen.
      *
      * @param area region to be repainted, in the current user space
-     * coordinate system.  
+     * coordinate system.
      */
     public void repaint(Shape area) {
         if (area == null) return;
@@ -380,8 +345,8 @@ public class StaticRenderer implements ImageRenderer {
     }
 
     /**
-     * Repaints the associated GVT tree under the list of <tt>areas</tt>.
-     * 
+     * Repaints the associated GVT tree under the list of <code>areas</code>.
+     *
      * If double buffered is true and this method completes cleanly it
      * will set the result of the repaint as the image returned by
      * getOffscreen otherwise the old image will still be returned.
@@ -390,7 +355,7 @@ public class StaticRenderer implements ImageRenderer {
      * by getOffscreen.
      *
      * @param areas a List of regions to be repainted, in the current
-     * user space coordinate system.  
+     * user space coordinate system.
      */
     public void repaint(RectListManager areas) {
 
@@ -429,7 +394,7 @@ public class StaticRenderer implements ImageRenderer {
             cr.copyData(copyRaster);
         }
 
-        if (!Thread.currentThread().isInterrupted()) {
+        if (!HaltingThread.hasBeenHalted()) {
             // Swap the buffers if the rendering completed cleanly.
             BufferedImage tmpBI = workingOffScreen;
 
@@ -452,7 +417,7 @@ public class StaticRenderer implements ImageRenderer {
         if (lastCache == null) return;
         Object o = lastCache.get();
         if (o == null) return;
-        
+
         TileCacheRed tcr = (TileCacheRed)o;
         tcr.flushCache(tcr.getBounds());
     }
@@ -512,7 +477,7 @@ public class StaticRenderer implements ImageRenderer {
                                    0, 0);
 
         RenderContext rc = new RenderContext(rcAT, null, renderingHints);
-            
+
         RenderedImage ri = rootFilter.createRendering(rc);
         if (ri == null)
             return null;
@@ -520,7 +485,7 @@ public class StaticRenderer implements ImageRenderer {
         CachableRed ret;
         ret = GraphicsUtil.wrap(ri);
         ret = setupCache(ret);
-        
+
         int dx = Math.round((float)at.getTranslateX());
         int dy = Math.round((float)at.getTranslateY());
         ret = new TranslateRed(ret, ret.getMinX()+dx, ret.getMinY()+dy);
@@ -532,7 +497,7 @@ public class StaticRenderer implements ImageRenderer {
 
     /**
      * Internal method used to synchronize local state in response to
-     * various set methods.  
+     * various set methods.
      */
     protected void updateWorkingBuffers() {
         if (rootFilter == null) {
@@ -546,7 +511,7 @@ public class StaticRenderer implements ImageRenderer {
             workingRaster = null;
             workingOffScreen = null;
             workingBaseRaster = null;
-            
+
             currentOffScreen = null;
             currentBaseRaster = null;
             currentRaster = null;
@@ -567,8 +532,8 @@ public class StaticRenderer implements ImageRenderer {
             (workingBaseRaster.getHeight() < h)) {
 
             sm = sm.createCompatibleSampleModel(w, h);
-            
-            workingBaseRaster 
+
+            workingBaseRaster
                 = Raster.createWritableRaster(sm, new Point(0,0));
         }
 
@@ -582,9 +547,9 @@ public class StaticRenderer implements ImageRenderer {
 
         int xloc = xt*tw - tgx;
         int yloc = yt*th - tgy;
-        
-        // System.out.println("Info: [" + 
-        //                    xloc + "," + yloc + "] [" + 
+
+        // System.out.println("Info: [" +
+        //                    xloc + "," + yloc + "] [" +
         //                    tgx  + "," + tgy  + "] [" +
         //                    xt   + "," + yt   + "] [" +
         //                    tw   + "," + th   + "]");
@@ -593,7 +558,7 @@ public class StaticRenderer implements ImageRenderer {
           (0, 0, w, h, xloc, yloc, null);
 
         workingOffScreen =  new BufferedImage
-          (rootCR.getColorModel(), 
+          (rootCR.getColorModel(),
            workingRaster.createWritableChild (0, 0, offScreenWidth,
                                            offScreenHeight, 0, 0, null),
            rootCR.getColorModel().isAlphaPremultiplied(), null);
