@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt.text;
 
 import java.awt.font.FontRenderContext;
@@ -67,7 +35,7 @@ import java.util.Set;
  * in a text run will all have the same bidi level.
  *
  * @author <a href="mailto:bella.robinson@cmis.csiro.au">Bella Robinson</a>
- * @version $Id: BidiAttributedCharacterIterator.java,v 1.10 2003/10/31 22:39:42 deweese Exp $
+ * @version $Id: BidiAttributedCharacterIterator.java 522271 2007-03-25 14:42:45Z dvholten $
  */
 public class BidiAttributedCharacterIterator implements AttributedCharacterIterator {
 
@@ -75,8 +43,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
     private FontRenderContext frc;
     private int chunkStart;
     private int [] newCharOrder;
-
-    private final static Map mirroredGlyphs = new HashMap(50);
+    private static final Float FLOAT_NAN = new Float(Float.NaN);
 
 
     protected BidiAttributedCharacterIterator
@@ -89,7 +56,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
         this.chunkStart = chunkStart;
         this.newCharOrder = newCharOrder;
     }
-                                              
+
 
     /**
      * Constructs a character iterator that represents the visual display order
@@ -99,7 +66,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
      * order.
      * @param frc The current font render context
      */
-    public BidiAttributedCharacterIterator(AttributedCharacterIterator aci, 
+    public BidiAttributedCharacterIterator(AttributedCharacterIterator aci,
                                            FontRenderContext           frc,
                                            int chunkStart) {
 
@@ -113,11 +80,11 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
          // attributes to decide if we really need to do bidi or not.
         if (false) {
             // Believe it or not this is much slower than the else case
-            // but the two are exactly equivilent (including the stripping
+            // but the two are exactly equivalent (including the stripping
             // of null keys/values).
             as = new AttributedString(aci);
         } else {
-            StringBuffer strB = new StringBuffer();
+            StringBuffer strB = new StringBuffer( numChars );
             char c = aci.first();
             for (int i = 0; i < numChars; i++) {
                 strB.append(c);
@@ -132,13 +99,14 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
                 Map attrMap = aci.getAttributes();
                 int extent  = aci.getRunLimit();
                 Map destMap = new HashMap(attrMap.size());
-                Iterator i  = attrMap.keySet().iterator();
-                while (i.hasNext()) {
+                Iterator it  = attrMap.entrySet().iterator();
+                while (it.hasNext()) {
                     // Font doesn't like getting attribute sets with
                     // null keys or values so we strip them here.
-                    Object key = i.next();
+                    Map.Entry e = (Map.Entry)it.next();
+                    Object key = e.getKey();
                     if (key == null) continue;
-                    Object value = attrMap.get(key);
+                    Object value = e.getValue();
                     if (value == null) continue;
                     destMap.put(key, value);
                 }
@@ -194,17 +162,19 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
         }
 
         //  work out the new character order
-        newCharOrder = doBidiReorder(charIndices, charLevels, 
-                                           numChars, maxBiDi);
+        newCharOrder = doBidiReorder(charIndices, charLevels,
+                                     numChars, maxBiDi);
 
         // construct the string in the new order
-        StringBuffer reorderedString = new StringBuffer();
-        char c;
+        StringBuffer reorderedString = new StringBuffer( numChars );
+        int reorderedFirstChar = 0;
         for (int i = 0; i < numChars; i++) {
-            c = aci.setIndex(newCharOrder[i]);
+            int srcIdx = newCharOrder[i];
+            char c = aci.setIndex(srcIdx);
+            if (srcIdx == 0) reorderedFirstChar = i;
 
             // check for mirrored char
-            int bidiLevel = tl.getCharacterLevel(newCharOrder[i]);
+            int bidiLevel = tl.getCharacterLevel(srcIdx);
             if ((bidiLevel & 0x01) != 0) {
                 // bidi level is odd so writing dir is right to left
                 // So get the mirror version of the char if there
@@ -216,7 +186,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
         }
 
         // construct the reordered ACI
-        AttributedString reorderedAS 
+        AttributedString reorderedAS
             = new AttributedString(reorderedString.toString());
         Map [] attrs = new Map[numChars];
         int start=aci.getBeginIndex();
@@ -248,24 +218,44 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
         aci.first();
         Float x = (Float) aci.getAttribute
             (GVTAttributedCharacterIterator.TextAttribute.X);
-        Float y = (Float) aci.getAttribute
-            (GVTAttributedCharacterIterator.TextAttribute.Y);
-
         if (x != null && !x.isNaN()) {
             reorderedAS.addAttribute
                 (GVTAttributedCharacterIterator.TextAttribute.X,
-                 new Float(Float.NaN), newCharOrder[0], newCharOrder[0]+1);
+                 FLOAT_NAN, reorderedFirstChar, reorderedFirstChar+1);
             reorderedAS.addAttribute
                 (GVTAttributedCharacterIterator.TextAttribute.X, x, 0, 1);
         }
+
+        Float y = (Float) aci.getAttribute
+            (GVTAttributedCharacterIterator.TextAttribute.Y);
         if (y != null && !y.isNaN()) {
             reorderedAS.addAttribute
                 (GVTAttributedCharacterIterator.TextAttribute.Y,
-                 new Float(Float.NaN), newCharOrder[0], newCharOrder[0]+1);
+                 FLOAT_NAN, reorderedFirstChar, reorderedFirstChar+1);
             reorderedAS.addAttribute
                 (GVTAttributedCharacterIterator.TextAttribute.Y, y, 0, 1);
         }
 
+
+        Float dx = (Float) aci.getAttribute
+            (GVTAttributedCharacterIterator.TextAttribute.DX);
+        if (dx != null && !dx.isNaN()) {
+            reorderedAS.addAttribute
+                (GVTAttributedCharacterIterator.TextAttribute.DX,
+                 FLOAT_NAN, reorderedFirstChar, reorderedFirstChar+1);
+            reorderedAS.addAttribute
+                (GVTAttributedCharacterIterator.TextAttribute.DX, dx, 0, 1);
+        }
+        Float dy = (Float) aci.getAttribute
+            (GVTAttributedCharacterIterator.TextAttribute.DY);
+        if (dy != null && !dy.isNaN()) {
+            reorderedAS.addAttribute
+                (GVTAttributedCharacterIterator.TextAttribute.DY,
+                 FLOAT_NAN, reorderedFirstChar, reorderedFirstChar+1);
+            reorderedAS.addAttribute
+                (GVTAttributedCharacterIterator.TextAttribute.DY, dy, 0, 1);
+        }
+
         // assign arabic form attributes to any arabic chars in the string
         reorderedAS = ArabicTextHandler.assignArabicForms(reorderedAS);
 
@@ -290,7 +280,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
      *
      * @return An array contianing the reordered character indices.
      */
-    private int[] doBidiReorder(int[] charIndices, int[] charLevels, 
+    private int[] doBidiReorder(int[] charIndices, int[] charLevels,
                                 int numChars, int highestLevel) {
         if (highestLevel == 0) return charIndices;
 
@@ -300,7 +290,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
         while (currentIndex < numChars) {
 
             // find the first char at the highest index
-            while ((currentIndex < numChars) && 
+            while ((currentIndex < numChars) &&
                    (charLevels[currentIndex] < highestLevel)) {
                 currentIndex++;
             }
@@ -312,7 +302,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
 
             currentIndex++;
             // now find the index where the run at the highestLevel end
-            while ((currentIndex < numChars) && 
+            while ((currentIndex < numChars) &&
                    (charLevels[currentIndex] == highestLevel)) {
                 currentIndex++;
             }
@@ -321,12 +311,11 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
             // now reverse the chars between startIndex and endIndex
 
             // Calculate the middle of the swap region, we include
-            // the middle char when region is an odd number of 
-            // chars wide so we properly decriment it's charLevel.
+            // the middle char when region is an odd number of
+            // chars wide so we properly decrement it's charLevel.
             int middle = ((endIndex-startIndex)>>1)+1;
-            int tmp;
             for (int i = 0; i<middle; i++) {
-                tmp = charIndices[startIndex+i];
+                int tmp = charIndices[startIndex+i];
                 charIndices[startIndex+i] = charIndices[endIndex-i];
                 charIndices[endIndex  -i] = tmp;
 
@@ -421,7 +410,7 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
      */
     public Object clone() {
         return new BidiAttributedCharacterIterator
-            ((AttributedCharacterIterator)reorderedACI.clone(), 
+            ((AttributedCharacterIterator)reorderedACI.clone(),
              frc, chunkStart, (int [])newCharOrder.clone());
     }
 
@@ -491,8 +480,16 @@ public class BidiAttributedCharacterIterator implements AttributedCharacterItera
        return reorderedACI.setIndex(position);
     }
 
-
+    /**
+     * @param c the character to 'mirror'
+     * @return either the 'mirror'-character for c or c itself
+     */
     public static int mirrorChar(int c) {
+
+        // note: the switch-statement is compiled to a tableswitch,
+        // which is evaluated by doing a binary search through the sorted case-list.
+        // the ca 130 cases are searched with max 8 compares
+
         switch(c) {
             // set up the mirrored glyph case statement;
         case 0x0028: return 0x0029;  //LEFT PARENTHESIS
