@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.gvt.text;
 
 import java.text.AttributedCharacterIterator;
@@ -65,7 +33,7 @@ import java.util.Set;
  * SVGAttributedCharacterIterator.TextAttributes.
  *
  * @author <a href="mailto:bill.haneman@ireland.sun.com">Bill Haneman</a>
- * @version $Id: GVTAttributedCharacterIterator.java,v 1.23 2003/09/07 22:25:28 deweese Exp $
+ * @version $Id: GVTAttributedCharacterIterator.java 1615279 2014-08-02 02:26:11Z gadams $
  */
 
 public interface GVTAttributedCharacterIterator extends AttributedCharacterIterator {
@@ -73,27 +41,27 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
     /**
      * Sets this iterator's contents to an unattributed copy of String s.
      */
-    public void setString(String s);
+    void setString(String s);
 
     /**
      * Assigns this iterator's contents to be equivalent to AttributedString s.
      */
-    public void setString(AttributedString s);
+    void setString(AttributedString s);
 
     /**
      * Sets values of a per-character attribute associated with the content
      *     string.
-     * Characters from <tt>beginIndex</tt> to <tt>endIndex</tt>
-     *     (zero-offset) are assigned values for attribute key <tt>attr</tt>
-     *     from the array <tt>attValues.</tt>
+     * Characters from <code>beginIndex</code> to <code>endIndex</code>
+     *     (zero-offset) are assigned values for attribute key <code>attr</code>
+     *     from the array <code>attValues.</code>
      * If the length of attValues is less than character span
-     *     <tt>(endIndex-beginIndex)</tt> the last value is duplicated;
+     *     <code>(endIndex-beginIndex)</code> the last value is duplicated;
      *     if attValues is longer than the character span
      *     the extra values are ignored.
      * Note that if either beginIndex or endIndex are outside the bounds
      *     of the current character array they are clipped accordingly.
      */
-    public void setAttributeArray(TextAttribute attr,
+    void setAttributeArray(TextAttribute attr,
                         Object[] attValues, int beginIndex, int endIndex);
 
     //From java.text.AttributedCharacterIterator
@@ -101,46 +69,46 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
     /**
      * Get the keys of all attributes defined on the iterator's text range.
      */
-    public Set getAllAttributeKeys();
+    Set getAllAttributeKeys();
 
     /**
      * Get the value of the named attribute for the current
      *     character.
      */
-    public Object getAttribute(AttributedCharacterIterator.Attribute attribute);
+    Object getAttribute(AttributedCharacterIterator.Attribute attribute);
 
     /**
      * Returns a map with the attributes defined on the current
      * character.
      */
-    public Map getAttributes();
+    Map getAttributes();
 
     /**
      * Get the index of the first character following the
      *     run with respect to all attributes containing the current
      *     character.
      */
-    public int getRunLimit();
+    int getRunLimit();
 
     /**
      * Get the index of the first character following the
      *      run with respect to the given attribute containing the current
      *      character.
      */
-    public int getRunLimit(AttributedCharacterIterator.Attribute attribute);
+    int getRunLimit(AttributedCharacterIterator.Attribute attribute);
 
     /**
      * Get the index of the first character following the
      *     run with respect to the given attributes containing the current
      *     character.
      */
-    public int getRunLimit(Set attributes);
+    int getRunLimit(Set attributes);
 
     /**
      * Get the index of the first character of the run with
      *    respect to all attributes containing the current character.
      */
-    public int getRunStart();
+    int getRunStart();
 
     /**
      * Get the index of the first character of the run with
@@ -148,74 +116,74 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
      * @param attribute The attribute for whose appearance the first offset
      *      is requested.
      */
-    public int getRunStart(AttributedCharacterIterator.Attribute attribute);
+    int getRunStart(AttributedCharacterIterator.Attribute attribute);
 
     /**
      * Get the index of the first character of the run with
      *      respect to the given attributes containing the current character.
      * @param attributes the Set of attributes which begins at the returned index.
      */
-    public int getRunStart(Set attributes);
+    int getRunStart(Set attributes);
 
     //From CharacterIterator
 
     /**
      * Create a copy of this iterator
      */
-    public Object clone();
+    Object clone();
 
     /**
      * Get the character at the current position (as returned
      *      by getIndex()).
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public char current();
+    char current();
 
     /**
      * Sets the position to getBeginIndex().
      * @return the character at the start index of the text.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public char first();
+    char first();
 
     /**
      * Get the start index of the text.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public int getBeginIndex();
+    int getBeginIndex();
 
     /**
      * Get the end index of the text.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public int getEndIndex();
+    int getEndIndex();
 
     /**
      * Get the current index.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public int getIndex();
+    int getIndex();
 
     /**
      * Sets the position to getEndIndex()-1 (getEndIndex() if
      * the text is empty) and returns the character at that position.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public char last();
+    char last();
 
     /**
      * Increments the iterator's index by one, returning the next character.
      * @return the character at the new index.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public char next();
+    char next();
 
     /**
      * Decrements the iterator's index by one and returns
      * the character at the new index.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public char previous();
+    char previous();
 
     /**
      * Sets the position to the specified position in the text.
@@ -223,7 +191,7 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
      * @return the character at new index <em>position</em>.
      * <br><b>Specified by:</b> java.text.CharacterIterator.
      */
-    public char setIndex(int position);
+    char setIndex(int position);
 
     //Inner classes:
 
@@ -234,183 +202,200 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
      * We duplicate the features of java.awt.font.TextAttribute rather than
      * subclassing because java.awt.font.TextAttribute is <em>final</em>.
      */
-    public static class TextAttribute extends AttributedCharacterIterator.Attribute {
+    class TextAttribute extends AttributedCharacterIterator.Attribute {
 
         /** Construct a TextAttribute key with name s */
         public TextAttribute(String s) {
             super(s);
         }
 
-        public final static TextAttribute FLOW_PARAGRAPH =
+        public static final TextAttribute FLOW_PARAGRAPH =
             new TextAttribute("FLOW_PARAGRAPH");
 
-        public final static TextAttribute FLOW_EMPTY_PARAGRAPH =
+        public static final TextAttribute FLOW_EMPTY_PARAGRAPH =
             new TextAttribute("FLOW_EMPTY_PARAGRAPH");
 
-        public final static TextAttribute FLOW_LINE_BREAK =
+        public static final TextAttribute FLOW_LINE_BREAK =
             new TextAttribute("FLOW_LINE_BREAK");
 
-        public final static TextAttribute FLOW_REGIONS =
+        public static final TextAttribute FLOW_REGIONS =
             new TextAttribute("FLOW_REGIONS");
 
-        public final static TextAttribute PREFORMATTED =
+        public static final TextAttribute LINE_HEIGHT =
+            new TextAttribute("LINE_HEIGHT");
+
+        public static final TextAttribute PREFORMATTED =
             new TextAttribute("PREFORMATTED");
 
         /** Attribute span delimiter - new tspan, tref, or textelement.*/
-        public final static TextAttribute TEXT_COMPOUND_DELIMITER =
+        public static final TextAttribute TEXT_COMPOUND_DELIMITER =
                               new TextAttribute("TEXT_COMPOUND_DELIMITER");
 
+        /** Element identifier all chars from same element will share an
+         *  ID. */
+        public static final TextAttribute TEXT_COMPOUND_ID =
+                              new TextAttribute("TEXT_COMPOUND_ID");
+
         /** Anchor type.*/
-        public final static TextAttribute ANCHOR_TYPE =
+        public static final TextAttribute ANCHOR_TYPE =
                               new TextAttribute("ANCHOR_TYPE");
 
         /** Marker attribute indicating explicit glyph layout.*/
-        public final static TextAttribute EXPLICIT_LAYOUT =
+        public static final TextAttribute EXPLICIT_LAYOUT =
                               new TextAttribute("EXPLICIT_LAYOUT");
 
         /** User-space X coordinate for character.*/
-        public final static TextAttribute X = new TextAttribute("X");
+        public static final TextAttribute X = new TextAttribute("X");
 
         /** User-space Y coordinate for character.*/
-        public final static TextAttribute Y = new TextAttribute("Y");
+        public static final TextAttribute Y = new TextAttribute("Y");
 
         /** User-space relative X coordinate for character.*/
-        public final static TextAttribute DX = new TextAttribute("DX");
+        public static final TextAttribute DX = new TextAttribute("DX");
 
         /** User-space relative Y coordinate for character.*/
-        public final static TextAttribute DY = new TextAttribute("DY");
+        public static final TextAttribute DY = new TextAttribute("DY");
 
         /** Rotation for character, in degrees.*/
-        public final static TextAttribute ROTATION =
+        public static final TextAttribute ROTATION =
                                           new TextAttribute("ROTATION");
 
         /** All the paint attributes for the text.*/
-        public final static TextAttribute PAINT_INFO =
+        public static final TextAttribute PAINT_INFO =
                                           new TextAttribute("PAINT_INFO");
 
         /** Author-expected width for bounding box containing
          *  all text string glyphs.
          */
-        public final static TextAttribute BBOX_WIDTH =
+        public static final TextAttribute BBOX_WIDTH =
                                           new TextAttribute("BBOX_WIDTH");
 
         /** Method specified for adjusting text element layout size.
          */
-        public final static TextAttribute LENGTH_ADJUST =
+        public static final TextAttribute LENGTH_ADJUST =
                                           new TextAttribute("LENGTH_ADJUST");
 
         /** Convenience flag indicating that non-default glyph spacing is needed.
          */
-        public final static TextAttribute CUSTOM_SPACING =
+        public static final TextAttribute CUSTOM_SPACING =
                                           new TextAttribute("CUSTOM_SPACING");
 
         /** User-specified inter-glyph kerning value.
          */
-        public final static TextAttribute KERNING =
+        public static final TextAttribute KERNING =
                                           new TextAttribute("KERNING");
 
         /** User-specified inter-glyph spacing value.
          */
-        public final static TextAttribute LETTER_SPACING =
+        public static final TextAttribute LETTER_SPACING =
                                           new TextAttribute("LETTER_SPACING");
 
         /** User-specified width for whitespace characters.
          */
-        public final static TextAttribute WORD_SPACING =
+        public static final TextAttribute WORD_SPACING =
                                           new TextAttribute("WORD_SPACING");
 
         /** Path along which text is to be laid out */
-        public final static TextAttribute TEXTPATH =
+        public static final TextAttribute TEXTPATH =
                                           new TextAttribute("TEXTPATH");
 
         /** Font variant to be used for this character span.
          * @see org.apache.batik.gvt.text.GVTAttributedCharacterIterator.TextAttribute#SMALL_CAPS
          */
-        public final static TextAttribute FONT_VARIANT =
+        public static final TextAttribute FONT_VARIANT =
                                           new TextAttribute("FONT_VARIANT");
 
         /** Baseline adjustment to be applied to this character span.
          */
-        public final static TextAttribute BASELINE_SHIFT =
+        public static final TextAttribute BASELINE_SHIFT =
                                           new TextAttribute("BASELINE_SHIFT");
 
         /** Directional writing mode applied to this character span.
          */
-        public final static TextAttribute WRITING_MODE =
+        public static final TextAttribute WRITING_MODE =
                                           new TextAttribute("WRITING_MODE");
 
-        public final static TextAttribute VERTICAL_ORIENTATION =
+        public static final TextAttribute VERTICAL_ORIENTATION =
                                           new TextAttribute("VERTICAL_ORIENTATION");
 
-        public final static TextAttribute VERTICAL_ORIENTATION_ANGLE =
+        public static final TextAttribute VERTICAL_ORIENTATION_ANGLE =
                                           new TextAttribute("VERTICAL_ORIENTATION_ANGLE");
 
-        public final static TextAttribute HORIZONTAL_ORIENTATION_ANGLE =
+        public static final TextAttribute HORIZONTAL_ORIENTATION_ANGLE =
                                           new TextAttribute("HORIZONTAL_ORIENTATION_ANGLE");
 
-        public final static TextAttribute GVT_FONT_FAMILIES =
-                                          new TextAttribute("GVT_FONT_FAMILIES");
+        public static final TextAttribute GVT_FONT_FAMILIES =
+                       new TextAttribute("GVT_FONT_FAMILIES");
 
-        public final static TextAttribute GVT_FONT =
+        public static final TextAttribute GVT_FONTS =
+                                          new TextAttribute("GVT_FONTS");
+
+        public static final TextAttribute GVT_FONT =
                                           new TextAttribute("GVT_FONT");
 
-        public final static TextAttribute ALT_GLYPH_HANDLER =
+        public static final TextAttribute ALT_GLYPH_HANDLER =
                                           new TextAttribute("ALT_GLYPH_HANDLER");
 
-        public final static TextAttribute BIDI_LEVEL =
+        public static final TextAttribute BIDI_LEVEL =
                                           new TextAttribute("BIDI_LEVEL");
 
-        public final static TextAttribute CHAR_INDEX =
+        public static final TextAttribute CHAR_INDEX =
                                           new TextAttribute("CHAR_INDEX");
 
-        public final static TextAttribute ARABIC_FORM =
+        public static final TextAttribute ARABIC_FORM =
                                           new TextAttribute("ARABIC_FORM");
 
+        public static final TextAttribute SCRIPT =
+                                          new TextAttribute("SCRIPT");
+
+        public static final TextAttribute LANGUAGE =
+                                          new TextAttribute("LANGUAGE");
+
         // VALUES
 
         /** Value for WRITING_MODE indicating left-to-right */
-        public final static Integer WRITING_MODE_LTR = new Integer(0x1);
+        public static final Integer WRITING_MODE_LTR = new Integer(0x1);
 
         /** Value for WRITING_MODE indicating right-to-left */
-        public final static Integer WRITING_MODE_RTL = new Integer(0x2);
+        public static final Integer WRITING_MODE_RTL = new Integer(0x2);
 
         /** Value for WRITING_MODE indicating top-to-botton */
-        public final static Integer WRITING_MODE_TTB = new Integer(0x3);
+        public static final Integer WRITING_MODE_TTB = new Integer(0x3);
 
         /** Value for VERTICAL_ORIENTATION indicating an angle */
-        public final static Integer ORIENTATION_ANGLE = new Integer(0x1);
+        public static final Integer ORIENTATION_ANGLE = new Integer(0x1);
 
         /** Value for VERTICAL_ORIENTATION indicating auto */
-        public final static Integer ORIENTATION_AUTO = new Integer(0x2);
+        public static final Integer ORIENTATION_AUTO = new Integer(0x2);
 
         /** Value for FONT_VARIANT specifying small caps */
-        public final static Integer SMALL_CAPS = new Integer(0x10);
+        public static final Integer SMALL_CAPS = new Integer(0x10);
 
         /** Value for UNDERLINE specifying underlining-on */
-        public final static Integer UNDERLINE_ON =
+        public static final Integer UNDERLINE_ON =
                             java.awt.font.TextAttribute.UNDERLINE_ON;
 
         /** Value for OVERLINE specifying overlining-on */
-        public final static Boolean OVERLINE_ON = new Boolean(true);
+        public static final Boolean OVERLINE_ON = Boolean.TRUE;
 
         /** Value for STRIKETHROUGH specifying strikethrough-on */
-        public final static Boolean STRIKETHROUGH_ON =
+        public static final Boolean STRIKETHROUGH_ON =
                             java.awt.font.TextAttribute.STRIKETHROUGH_ON;
 
         /** Value for LENGTH_ADJUST specifying adjustment to inter-glyph spacing */
-        public final static Integer ADJUST_SPACING =
+        public static final Integer ADJUST_SPACING =
                             new Integer(0x0);
 
         /** Value for LENGTH_ADJUST specifying overall scaling of layout outlines */
-        public final static Integer ADJUST_ALL =
+        public static final Integer ADJUST_ALL =
                             new Integer(0x01);
 
         // constant values for the arabic glyph forms
-        public final static Integer ARABIC_NONE = new Integer(0x0);
-        public final static Integer ARABIC_ISOLATED = new Integer(0x1);
-        public final static Integer ARABIC_TERMINAL = new Integer(0x2);
-        public final static Integer ARABIC_INITIAL = new Integer(0x3);
-        public final static Integer ARABIC_MEDIAL = new Integer(0x4);
+        public static final Integer ARABIC_NONE = new Integer(0x0);
+        public static final Integer ARABIC_ISOLATED = new Integer(0x1);
+        public static final Integer ARABIC_TERMINAL = new Integer(0x2);
+        public static final Integer ARABIC_INITIAL = new Integer(0x3);
+        public static final Integer ARABIC_MEDIAL = new Integer(0x4);
 
     }
 
@@ -421,7 +406,7 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
      * TextAttribute.TRANSFORM attributes, or convert between implementations
      * of AttributedCharacterIterator.Attribute.
      */
-    public interface AttributeFilter {
+    interface AttributeFilter {
 
         /**
          * Modify an AttributedCharacterIterator's attributes systematically.
@@ -433,7 +418,7 @@ public interface GVTAttributedCharacterIterator extends AttributedCharacterItera
          * @return an instance of AttributedCharacterIterator with mutated
          *     attributes.
          */
-        public AttributedCharacterIterator
+        AttributedCharacterIterator
             mutateAttributes(AttributedCharacterIterator aci);
 
     }
