@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.svggen;
 
 import java.awt.Dimension;
@@ -69,11 +37,11 @@ import org.w3c.dom.Element;
  * for handlers implementing a caching strategy.
  *
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: DefaultCachedImageHandler.java,v 1.7 2003/08/09 16:58:39 deweese Exp $
+ * @version $Id: DefaultCachedImageHandler.java 1372129 2012-08-12 15:31:50Z helder $
  * @see             org.apache.batik.svggen.SVGGraphics2D
  */
-public abstract class DefaultCachedImageHandler 
-    implements CachedImageHandler, 
+public abstract class DefaultCachedImageHandler
+    implements CachedImageHandler,
                SVGSyntax,
                ErrorConstants {
 
@@ -87,7 +55,7 @@ public abstract class DefaultCachedImageHandler
     // for createGraphics method.
     private static Method createGraphics = null;
     private static boolean initDone = false;
-    private final static Class[] paramc = new Class[] {BufferedImage.class};
+    private static final Class[] paramc = new Class[] {BufferedImage.class};
     private static Object[] paramo = null;
 
     protected ImageCacher imageCacher;
@@ -117,7 +85,7 @@ public abstract class DefaultCachedImageHandler
     }
 
     /**
-     * This <tt>GenericImageHandler</tt> implementation does not
+     * This <code>GenericImageHandler</code> implementation does not
      * need to interact with the DOMTreeManager.
      */
     public void setDOMTreeManager(DOMTreeManager domTreeManager){
@@ -159,7 +127,7 @@ public abstract class DefaultCachedImageHandler
     /**
      * Creates an Element which can refer to an image.
      * Note that no assumptions should be made by the caller about the
-     * corresponding SVG tag. By default, an &lt;image&gt; tag is 
+     * corresponding SVG tag. By default, an &lt;image&gt; tag is
      * used, but the {@link CachedImageHandlerBase64Encoder}, for
      * example, overrides this method to use a different tag.
      */
@@ -328,10 +296,10 @@ public abstract class DefaultCachedImageHandler
                                     generatorContext.doubleString(dstHeight));
         return null;
     }
-              
+
     protected void handleEmptyImage(Element imageElement) {
         imageElement.setAttributeNS(XLINK_NAMESPACE_URI,
-                                    ATTR_XLINK_HREF, "");
+                                    XLINK_HREF_QNAME, "");
         imageElement.setAttributeNS(null, SVG_WIDTH_ATTRIBUTE, "0");
         imageElement.setAttributeNS(null, SVG_HEIGHT_ATTRIBUTE, "0");
     }
@@ -385,16 +353,16 @@ public abstract class DefaultCachedImageHandler
         // Create an buffered image if necessary
         //
         BufferedImage buf = null;
-        if (image instanceof BufferedImage 
+        if (image instanceof BufferedImage
             &&
             ((BufferedImage)image).getType() == getBufferedImageType()){
             buf = (BufferedImage)image;
         } else {
             Dimension size = new Dimension(image.getWidth(), image.getHeight());
             buf = buildBufferedImage(size);
-            
+
             Graphics2D g = createGraphics(buf);
-            
+
             g.drawRenderedImage(image, IDENTITY);
             g.dispose();
         }
@@ -406,8 +374,8 @@ public abstract class DefaultCachedImageHandler
     }
 
     /**
-     * This method will delegate to the <tt>handleHREF</tt> which
-     * uses a <tt>RenderedImage</tt>
+     * This method will delegate to the <code>handleHREF</code> which
+     * uses a <code>RenderedImage</code>
      */
     protected void handleHREF(RenderableImage image, Element imageElement,
                               SVGGeneratorContext generatorContext)
@@ -429,12 +397,12 @@ public abstract class DefaultCachedImageHandler
                                       BufferedImage buf,
                                       SVGGeneratorContext generatorContext)
         throws SVGGraphics2DIOException {
-        
+
         ByteArrayOutputStream os;
-        
+
         if (generatorContext == null)
             throw new SVGGraphics2DRuntimeException(ERR_CONTEXT_NULL);
-        
+
         try {
             os = new ByteArrayOutputStream();
             // encode the image in memory
@@ -445,19 +413,19 @@ public abstract class DefaultCachedImageHandler
             // should not happen since we do in-memory processing
             throw new SVGGraphics2DIOException(ERR_UNEXPECTED, e);
         }
-        
+
         // ask the cacher for a reference
         String ref = imageCacher.lookup(os,
                                                   buf.getWidth(),
                                                   buf.getHeight(),
                                                   generatorContext);
-        
+
         // set the URL
         imageElement.setAttributeNS(XLINK_NAMESPACE_URI,
-                                    ATTR_XLINK_HREF,
+                                    XLINK_HREF_QNAME,
                                     getRefPrefix() + ref);
-    }                
- 
+    }
+
     /**
      * Should return the prefix with wich the image reference
      * should be pre-concatenated.
@@ -470,11 +438,11 @@ public abstract class DefaultCachedImageHandler
      */
     public abstract void encodeImage(BufferedImage buf, OutputStream os)
         throws IOException;
- 
+
     /**
-     * This template method should be overridden by derived classes to 
+     * This template method should be overridden by derived classes to
      * declare the image type they need for saving to file.
      */
     public abstract int getBufferedImageType();
-  
+
 }
