@@ -1,50 +1,19 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
 */
 
@@ -73,14 +42,18 @@ import org.w3c.dom.Text;
  * for Sun's Project X by David Brownell.
  *
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: XmlWriter.java,v 1.9 2003/08/09 16:58:39 deweese Exp $
+ * @version $Id: XmlWriter.java 763473 2009-04-09 00:22:25Z cam $
  */
 class XmlWriter implements SVGConstants {
 
-    static private String EOL;
-    static private final String TAG_END = " />";
-    static private final String TAG_START = "</";
-    static private final String SPACE = " ";
+    private static String EOL;
+    private static final String TAG_END = "/>";
+    private static final String TAG_START = "</";
+
+    private static final char[] SPACES =
+    { ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',
+      ' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ',' ' };
+    private static final int    SPACES_LEN = SPACES.length;
 
     static {
         String  temp;
@@ -90,12 +63,14 @@ class XmlWriter implements SVGConstants {
     }
 
     static class IndentWriter extends Writer {
-        private Writer proxied;
-        private int indentLevel;
+        protected Writer proxied;
+        protected int    indentLevel;
+        protected int    column;
 
         public IndentWriter(Writer proxied){
             if (proxied == null)
-                throw new SVGGraphics2DRuntimeException(ErrorConstants.ERR_PROXY);
+                throw new SVGGraphics2DRuntimeException
+                    (ErrorConstants.ERR_PROXY);
 
             this.proxied = proxied;
         }
@@ -112,32 +87,45 @@ class XmlWriter implements SVGConstants {
             proxied.write(EOL);
             int temp = indentLevel;
             while(temp > 0){
-                proxied.write(' ');
-                temp--;
+                if (temp > SPACES_LEN) {
+                    proxied.write(SPACES, 0, SPACES_LEN);
+                    temp -= SPACES_LEN;
+                } else {
+                    proxied.write(SPACES, 0, temp);
+                    break;
+                }
             }
+            column = indentLevel;
         }
 
         public Writer getProxied(){
             return proxied;
         }
 
+        public int getColumn() { return column; }
+
         public void write(int c) throws IOException {
+            column++;
             proxied.write(c);
         }
 
-        public void write(char cbuf[]) throws IOException {
+        public void write(char[] cbuf) throws IOException {
+            column+=cbuf.length;
             proxied.write(cbuf);
         }
 
-        public void write(char cbuf[], int off, int len) throws IOException{
+        public void write(char[] cbuf, int off, int len) throws IOException{
+            column+=len;
             proxied.write(cbuf, off, len);
         }
 
         public void write(String str) throws IOException {
+            column+=str.length();
             proxied.write(str);
         }
 
         public void write(String str, int off, int len) throws IOException {
+            column+=len;
             proxied.write(str, off, len);
         }
 
@@ -146,35 +134,67 @@ class XmlWriter implements SVGConstants {
         }
 
         public void close() throws IOException{
+            column = -1;
             proxied.close();
         }
     }
 
-    private static void writeXml(Attr attr, IndentWriter out) throws IOException{
+    private static void writeXml(Attr attr, IndentWriter out,
+                                 boolean escaped)
+        throws IOException{
         String name = attr.getName();
         out.write (name);
         out.write ("=\"");
-        writeChildrenXml(attr, out);
+        writeChildrenXml(attr, out, escaped);
         out.write ('"');
     }
 
     /**
      * Writes the attribute's value.
      */
-    private static void writeChildrenXml(Attr attr, IndentWriter out)
+    private static void writeChildrenXml(Attr attr, IndentWriter out,
+                                         boolean escaped)
         throws IOException {
-        String value = attr.getValue();
-        for (int i = 0; i < value.length (); i++) {
-            int c = value.charAt (i);
+        char[] data = attr.getValue().toCharArray();
+        if (data == null) return;
+
+        int         length = data.length;
+        int         start=0, last=0;
+        while (last < length) {
+            char c = data[last];
             switch (c) {
-            case '<':  out.write ("&lt;"); continue;
-            case '>':  out.write ("&gt;"); continue;
-            case '&':  out.write ("&amp;"); continue;
-            case '\'': out.write ("&apos;"); continue;
-            case '"':  out.write ("&quot;"); continue;
-            default:   out.write (c); continue;
+            case '<':
+                out.write (data, start, last - start);
+                start = last + 1;
+                out.write ("&lt;");
+                break;
+            case '>':
+                out.write (data, start, last - start);
+                start = last + 1;
+                out.write ("&gt;");
+                break;
+            case '&':
+                out.write (data, start, last - start);
+                start = last + 1;
+                out.write ("&amp;");
+                break;
+            case '"':
+                out.write (data, start, last - start);
+                start = last + 1;
+                out.write ("&quot;");
+                break;
+            default: // to be able to escape characters if allowed
+                if (escaped && (c > 0x007F)) {
+                    out.write (data, start, last - start);
+                    String hex = "0000"+Integer.toHexString(c);
+                    out.write("&#x"+hex.substring(hex.length()-4)+";");
+                    start = last + 1;
+                }
+                break;
             }
+            last++;
         }
+        out.write (data, start, last - start);
     }
 
     /**
@@ -182,47 +202,74 @@ class XmlWriter implements SVGConstants {
      * prevent illegal comments:  between consecutive dashes ("--")
      * or if the last character of the comment is a dash.
      */
-    private static void writeXml(Comment comment, IndentWriter out)
+    private static void writeXml(Comment comment, IndentWriter out,
+                                 boolean escaped)
         throws IOException {
-        char data[] = comment.getData().toCharArray();
-        out.write ("<!--");
-        if (data != null) {
-            boolean     sawDash = false;
-            int         length = data.length;
 
-            // "--" illegal in comments, expand it
-            for (int i = 0; i < length; i++) {
-                if (data [i] == '-') {
-                    if (sawDash)
-                        out.write (' ');
-                    else {
-                        sawDash = true;
-                        out.write ('-');
-                        continue;
-                    }
-                }
-                sawDash = false;
-                out.write (data [i]);
-            }
-            if (data [data.length - 1] == '-')
-                out.write (' ');
+        char[] data = comment.getData().toCharArray();
+
+        if (data == null) {
+            out.write("<!---->");
+            return;
         }
+
+        out.write ("<!--");
+        boolean     sawDash = false;
+        int         length = data.length;
+        int         start=0, last=0;
+
+        // "--" illegal in comments, insert a space.
+        while (last < length) {
+            char c = data[last];
+            if (c == '-') {
+                if (sawDash) {
+                    out.write (data, start, last - start);
+                    start = last;
+                    out.write (' ');
+                }
+                sawDash = true;
+            } else {
+                sawDash = false;
+            }
+            last++;
+        }
+        out.write (data, start, last - start);
+        if (sawDash)
+            out.write (' ');
         out.write ("-->");
     }
 
-    private static void writeXml(Text text, IndentWriter out)
+    private static void writeXml(Text text, IndentWriter out, boolean escaped)
         throws IOException {
-        char data[] = text.getData().toCharArray();
-        int     start = 0, last = 0;
+        writeXml(text, out, false, escaped);
+    }
+
+    private static void writeXml(Text text, IndentWriter out, boolean trimWS,
+                                 boolean escaped)
+        throws IOException {
+        char[] data = text.getData().toCharArray();
 
         // XXX saw this once -- being paranoid
         if (data == null)
             { System.err.println ("Null text data??"); return; }
 
-        while (last < data.length) {
+        int length = data.length;
+        int start = 0, last = 0;
+        if (trimWS) {
+            while (last < length) {
+                char c = data[last];
+                switch (c) {
+                case ' ': case '\t': case '\n': case '\r': last++; continue;
+                default: break;
+                }
+                break;
+            }
+            start = last;
+        }
+
+        while (last < length) {
             char c = data [last];
 
-            //
             // escape markup delimiters only ... and do bulk
             // writes wherever possible, for best performance
             //
@@ -231,47 +278,90 @@ class XmlWriter implements SVGConstants {
             // doing it very generally helps simple parsers
             // that may not be quite correct.
             //
-            if (c == '<') {                     // not legal in char data
+            switch(c) {
+            case ' ': case '\t': case '\n': case '\r':
+                if (trimWS) {
+                    int wsStart = last; last++;
+                    while (last < length) {
+                        switch(data[last]) {
+                        case ' ': case '\t': case '\n': case '\r':
+                            last++; continue;
+                        default: break;
+                        }
+                        break;
+                    }
+                    if (last == length) {
+                        out.write(data, start, wsStart-start);
+                        return;
+                    } else {
+                        continue;
+                    }
+                }
+                break;
+            case '<':                     // not legal in char data
                 out.write (data, start, last - start);
                 start = last + 1;
                 out.write ("&lt;");
-            } else if (c == '>') {              // see above
+                break;
+            case '>':                     // see above
                 out.write (data, start, last - start);
                 start = last + 1;
                 out.write ("&gt;");
-            } else if (c == '&') {              // not legal in char data
+                break;
+            case '&':                    // not legal in char data
                 out.write (data, start, last - start);
                 start = last + 1;
                 out.write ("&amp;");
+                break;
+            default: // to be able to escape characters if allowed
+                if (escaped && (c > 0x007F)) {
+                    out.write (data, start, last - start);
+                    String hex = "0000"+Integer.toHexString(c);
+                    out.write("&#x"+hex.substring(hex.length()-4)+";");
+                    start = last + 1;
+                }
+                break;
             }
             last++;
         }
         out.write (data, start, last - start);
     }
 
-    private static void writeXml(CDATASection cdataSection, IndentWriter out)
+    private static void writeXml(CDATASection cdataSection, IndentWriter out,
+                                 boolean escaped)
         throws IOException {
         char[] data = cdataSection.getData().toCharArray();
+        if (data == null) {
+            out.write ("<![CDATA[]]>");
+            return;
+        }
+
         out.write ("<![CDATA[");
-        for (int i = 0; i < data.length; i++) {
-            char c = data [i];
+        int length = data.length;
+        int  start = 0, last = 0;
+        while (last < length) {
+            char c = data [last];
 
             // embedded "]]>" needs to be split into adjacent
             // CDATA blocks ... can be split at either point
             if (c == ']') {
-                if ((i + 2) < data.length
-                    && data [i + 1] == ']'
-                    && data [i + 2] == '>') {
+                if (((last + 2) < data.length) &&
+                    (data [last + 1] == ']')   &&
+                    (data [last + 2] == '>')) {
+                    out.write (data, start, last - start);
+                    start = last + 1;
                     out.write ("]]]]><![CDATA[>");
                     continue;
                 }
             }
-            out.write (c);
+            last++;
         }
+        out.write (data, start, last - start);
         out.write ("]]>");
     }
 
-    private static void writeXml(Element element, IndentWriter out)
+    private static void writeXml(Element element, IndentWriter out,
+                                 boolean escaped)
         throws IOException, SVGGraphics2DIOException {
         out.write (TAG_START, 0, 1);    // "<"
         out.write (element.getTagName());
@@ -282,48 +372,49 @@ class XmlWriter implements SVGConstants {
             for(int i=0; i<nAttr; i++){
                 Attr attr = (Attr)attributes.item(i);
                 out.write(' ');
-                writeXml(attr, out);
+                writeXml(attr, out, escaped);
             }
         }
 
+        boolean lastElem = (element.getParentNode().getLastChild()==element);
+
         //
         // Write empty nodes as "<EMPTY />" to make sure version 3
         // and 4 web browsers can read empty tag output as HTML.
         // XML allows "<EMPTY/>" too, of course.
         //
-        if (!element.hasChildNodes())
-            out.write(TAG_END, 0, 3);   // " />"
-        else  {
-            out.write(TAG_END, 2, 1);   // ">"
-            writeChildrenXml(element, out);
-            out.write (TAG_START, 0, 2);        // "</"
-            out.write (element.getTagName());
-            out.write (TAG_END, 2, 1);  // ">"
+        if (!element.hasChildNodes()) {
+            if (lastElem)
+                out.setIndentLevel(out.getIndentLevel()-2);
+            out.printIndent ();
+            out.write(TAG_END, 0, 2);   // "/>"
+            return;
         }
+        Node child = element.getFirstChild();
+        out.printIndent ();
+        out.write(TAG_END, 1, 1);   // ">"
+        if ((child.getNodeType() != Node.TEXT_NODE) ||
+            (element.getLastChild() != child)) { // one text node child..
+            out.setIndentLevel(out.getIndentLevel()+2);
+        }
+
+        writeChildrenXml(element, out, escaped);
+
+        out.write (TAG_START, 0, 2);        // "</"
+        out.write (element.getTagName());
+        if (lastElem)
+            out.setIndentLevel(out.getIndentLevel()-2);
+        out.printIndent ();
+        out.write (TAG_END, 1, 1);  // ">"
     }
 
-    private static void writeChildrenXml(Element element, IndentWriter out)
+    private static void writeChildrenXml(Element element, IndentWriter out,
+                                         boolean escaped)
         throws IOException, SVGGraphics2DIOException {
-        NodeList children = element.getChildNodes();
-        if (children == null)
-            return;
-
-        int length = children.getLength();
-        int     oldIndent = 0;
-        oldIndent = out.getIndentLevel();
-        try {
-            out.setIndentLevel(oldIndent + 2);
-            for(int i = 0; i < length; i++) {
-                if(children.item(i).getNodeType () != Node.TEXT_NODE) {
-                    out.printIndent ();
-                }
-                writeXml(children.item(i), out);
-            }
-        } finally {
-            out.setIndentLevel(oldIndent);
-            if (length > 0 && children.item(length-1).getNodeType() != Node.TEXT_NODE){
-                out.printIndent();          // for ETag
-            }
+        Node child = element.getFirstChild();
+        while (child != null) {
+            writeXml(child, out, escaped);
+            child = child.getNextSibling();
         }
     }
 
@@ -331,9 +422,10 @@ class XmlWriter implements SVGConstants {
         throws IOException {
         String  encoding = null;
 
-        if (out.getProxied() instanceof OutputStreamWriter)
-            encoding =
-                java2std(((OutputStreamWriter)out.getProxied()).getEncoding());
+        if (out.getProxied() instanceof OutputStreamWriter) {
+            OutputStreamWriter osw = (OutputStreamWriter)out.getProxied();
+            encoding = java2std(osw.getEncoding());
+        }
 
         out.write ("<?xml version=\"1.0\"");
         if (encoding != null) {
@@ -343,27 +435,27 @@ class XmlWriter implements SVGConstants {
         }
         out.write ("?>");
         out.write (EOL);
-        out.write (EOL);
 
         // Write DOCTYPE declaration here. Skip until specification is released.
         out.write ("<!DOCTYPE svg PUBLIC '");
         out.write (SVG_PUBLIC_ID);
-        out.write ("' '");
-        out.write (SVG_SYSTEM_ID);
-        out.write ("'");
+        out.write ("'"); out.write (EOL);
 
-        out.write (">");
-        out.write (EOL);
+        out.write ("          '");
+        out.write (SVG_SYSTEM_ID);
+        out.write ("'"); out.write (">"); out.write (EOL);
     }
 
-    private static void writeXml(Document document, IndentWriter out)
+    private static void writeXml(Document document, IndentWriter out,
+                                 boolean escaped)
         throws IOException, SVGGraphics2DIOException {
         writeDocumentHeader(out);
         NodeList childList = document.getChildNodes();
-        writeXml(childList, out);
+        writeXml(childList, out, escaped);
     }
 
-    private static void writeXml(NodeList childList, IndentWriter out)
+    private static void writeXml(NodeList childList, IndentWriter out,
+                                 boolean escaped)
         throws IOException, SVGGraphics2DIOException {
         int     length = childList.getLength ();
 
@@ -371,7 +463,7 @@ class XmlWriter implements SVGConstants {
             return;
         for (int i = 0; i < length; i++) {
             Node child = childList.item(i);
-            writeXml(child, out);
+            writeXml(child, out, escaped);
             out.write (EOL);
         }
     }
@@ -416,11 +508,11 @@ class XmlWriter implements SVGConstants {
         if ("EUCJIS".equalsIgnoreCase (encodingName))
             return "EUC-JP";
 
-        // else we can't really do anything
-        return encodingName;
+        // else we force UTF-8 encoding, better than nothing...
+        return "UTF-8";
     }
 
-    public static void writeXml(Node node, Writer writer)
+    public static void writeXml(Node node, Writer writer, boolean escaped)
         throws SVGGraphics2DIOException {
         try {
             IndentWriter out = null;
@@ -431,33 +523,31 @@ class XmlWriter implements SVGConstants {
 
             switch (node.getNodeType()) {
             case Node.ATTRIBUTE_NODE:
-                writeXml((Attr)node, out);
+                writeXml((Attr)node, out, escaped);
                 break;
             case Node.COMMENT_NODE:
-                writeXml((Comment)node, out);
+                writeXml((Comment)node, out, escaped);
                 break;
             case Node.TEXT_NODE:
-                writeXml((Text)node, out);
+                writeXml((Text)node, out, escaped);
                 break;
             case Node.CDATA_SECTION_NODE:
-                writeXml((CDATASection)node, out);
+                writeXml((CDATASection)node, out, escaped);
                 break;
             case Node.DOCUMENT_NODE:
-                writeXml((Document)node, out);
+                writeXml((Document)node, out, escaped);
                 break;
             case Node.DOCUMENT_FRAGMENT_NODE:
                 writeDocumentHeader(out);
                 NodeList childList = node.getChildNodes();
-                writeXml(childList, out);
+                writeXml(childList, out, escaped);
                 break;
             case Node.ELEMENT_NODE:
-                writeXml((Element)node, out);
+                writeXml((Element)node, out, escaped);
                 break;
             default:
-                throw
-                    new SVGGraphics2DRuntimeException(ErrorConstants.INVALID_NODE+
-                                                      node.getClass().
-                                                      getName());
+                throw new SVGGraphics2DRuntimeException
+                    (ErrorConstants.INVALID_NODE+node.getClass().getName());
             }
         } catch (IOException io) {
             throw new SVGGraphics2DIOException(io);
