@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.swing.gvt;
 
 import java.awt.AlphaComposite;
@@ -58,7 +26,10 @@ import java.awt.Graphics;
 import java.awt.Graphics2D;
 import java.awt.Rectangle;
 import java.awt.RenderingHints;
+import java.awt.Toolkit;
 import java.awt.Shape;
+import java.awt.datatransfer.Clipboard;
+import java.awt.datatransfer.StringSelection;
 import java.awt.event.ComponentAdapter;
 import java.awt.event.ComponentEvent;
 import java.awt.event.InputEvent;
@@ -66,10 +37,13 @@ import java.awt.event.KeyEvent;
 import java.awt.event.KeyListener;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
+import java.awt.event.MouseWheelEvent;
+import java.awt.event.MouseWheelListener;
 import java.awt.event.MouseMotionListener;
 import java.awt.geom.AffineTransform;
 import java.awt.geom.NoninvertibleTransformException;
 import java.awt.image.BufferedImage;
+import java.text.CharacterIterator;
 import java.util.Collections;
 import java.util.Iterator;
 import java.util.LinkedList;
@@ -77,18 +51,23 @@ import java.util.List;
 
 import javax.swing.JComponent;
 
+import org.apache.batik.bridge.Mark;
 import org.apache.batik.gvt.GraphicsNode;
 import org.apache.batik.gvt.event.AWTEventDispatcher;
+import org.apache.batik.gvt.event.EventDispatcher;
+import org.apache.batik.gvt.event.SelectionAdapter;
+import org.apache.batik.gvt.event.SelectionEvent;
 import org.apache.batik.gvt.renderer.ConcreteImageRendererFactory;
 import org.apache.batik.gvt.renderer.ImageRenderer;
 import org.apache.batik.gvt.renderer.ImageRendererFactory;
-import org.apache.batik.gvt.text.Mark;
+import org.apache.batik.util.HaltingThread;
+import org.apache.batik.util.Platform;
 
 /**
  * This class represents a component which can display a GVT tree.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: JGVTComponent.java,v 1.43 2003/11/21 13:36:25 deweese Exp $
+ * @version $Id: JGVTComponent.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class JGVTComponent extends JComponent {
 
@@ -137,7 +116,7 @@ public class JGVTComponent extends JComponent {
     /**
      * The progressive paint thread.
      */
-    protected Thread progressivePaintThread;
+    protected HaltingThread progressivePaintThread;
 
     /**
      * The image to paint.
@@ -205,6 +184,14 @@ public class JGVTComponent extends JComponent {
      */
     protected boolean selectableText;
 
+    /**
+     * Whether the JGVTComponent should adhere to 'Unix' text
+     * selection semantics where as soon as text is selected it
+     * is copied to the clipboard.  If users want Mac/Windows
+     * behaviour they need to handle selections them selves.
+     */
+    protected boolean useUnixTextSelection = true;
+
     /**
      * Whether to suspend interactions.
      */
@@ -224,22 +211,22 @@ public class JGVTComponent extends JComponent {
 
     /**
      * Creates a new JGVTComponent.
-     * @param eventEnabled Whether the GVT tree should be reactive
+     * @param eventsEnabled Whether the GVT tree should be reactive
      *        to mouse and key events.
      * @param selectableText Whether the text should be selectable.
      *        if eventEnabled is false, this flag is ignored.
      */
-    public JGVTComponent(boolean eventsEnabled, boolean selectableText) {
+    public JGVTComponent(boolean eventsEnabled,
+                         boolean selectableText) {
         setBackground(Color.white);
+        // setDoubleBuffered(false);
 
         this.eventsEnabled = eventsEnabled;
         this.selectableText = selectableText;
 
         listener = createListener();
 
-        addKeyListener(listener);
-        addMouseListener(listener);
-        addMouseMotionListener(listener);
+        addAWTListeners();
 
         addGVTTreeRendererListener(listener);
 
@@ -252,14 +239,58 @@ public class JGVTComponent extends JComponent {
 
     }
 
+    /**
+     * Creates an instance of Listener.
+     */
+    protected Listener createListener() {
+        return new Listener();
+    }
+
+    /**
+     * Adds the AWT listeners.
+     */
+    protected void addAWTListeners() {
+        addKeyListener(listener);
+        addMouseListener(listener);
+        addMouseMotionListener(listener);
+        addMouseWheelListener(listener);
+    }
+
+    /**
+     * Turn off all 'interactor' objects (pan, zoom, etc) if
+     * 'b' is true, turn them on if 'b' is false.
+     */
     public void setDisableInteractions(boolean b) {
         disableInteractions = b;
     }
 
+    /**
+     * Returns true if all 'interactor' objects
+     * (pan, zoom, etc) are disabled.
+     */
     public boolean getDisableInteractions() {
         return disableInteractions;
     }
 
+    /**
+     * If 'b' is true text selections will copied to
+     * the clipboard immediately.  If 'b' is false
+     * then nothing will be done when selections are
+     * made (the application is responsable for copying
+     * the selection in response to user actions).
+     */
+    public void setUseUnixTextSelection(boolean b) {
+        useUnixTextSelection = b;
+    }
+
+    /**
+     * Returns true if the canvas will copy selections
+     * to the clipboard when they are completed.
+     */
+    public void getUseUnixTextSelection(boolean b) {
+        useUnixTextSelection = b;
+    }
+
     /**
      * Returns the interactor list.
      */
@@ -306,8 +337,8 @@ public class JGVTComponent extends JComponent {
     public void stopProcessing() {
         if (gvtTreeRenderer != null) {
             needRender = false;
-            gvtTreeRenderer.interrupt();
-            interruptProgressivePaintThread();
+            gvtTreeRenderer.halt();
+            haltProgressivePaintThread();
         }
     }
 
@@ -346,18 +377,39 @@ public class JGVTComponent extends JComponent {
      */
     protected void initializeEventHandling() {
         if (eventsEnabled) {
-            eventDispatcher = new AWTEventDispatcher();
+            eventDispatcher = createEventDispatcher();
             if (selectableText) {
-                textSelectionManager =
-                    new TextSelectionManager(this, eventDispatcher);
+                textSelectionManager = createTextSelectionManager
+                    (eventDispatcher);
+                textSelectionManager.addSelectionListener
+                     (new UnixTextSelectionListener());
             }
         }
     }
 
+    protected AWTEventDispatcher createEventDispatcher() {
+        return new AWTEventDispatcher();
+    }
+
+
     ////////////////////////////////////////////////////////////////////////
     // Selection methods
     ////////////////////////////////////////////////////////////////////////
 
+    protected TextSelectionManager 
+        createTextSelectionManager(EventDispatcher ed) {
+        return new TextSelectionManager(this, ed);
+    }
+
+    /**
+     * Returns the current Text selection manager for the Component.
+     * Users can register with this to be notifed of changes in
+     * the text selection.
+     */
+    public TextSelectionManager getTextSelectionManager() {
+        return textSelectionManager;
+    }
+
     /**
      * Sets the color of the selection overlay to the specified color.
      *
@@ -458,7 +510,7 @@ public class JGVTComponent extends JComponent {
     public void setProgressivePaint(boolean b) {
         if (progressivePaint != b) {
             progressivePaint = b;
-            interruptProgressivePaintThread();
+            haltProgressivePaintThread();
         }
     }
 
@@ -481,10 +533,10 @@ public class JGVTComponent extends JComponent {
         if (EventQueue.isDispatchThread()) {
             Rectangle visRect = getRenderRect();
             if (doubleBufferedRendering)
-                repaint(visRect.x,     visRect.y, 
+                repaint(visRect.x,     visRect.y,
                         visRect.width, visRect.height);
             else
-                paintImmediately(visRect.x,     visRect.y, 
+                paintImmediately(visRect.x,     visRect.y,
                                  visRect.width, visRect.height);
         } else {
             try {
@@ -492,10 +544,10 @@ public class JGVTComponent extends JComponent {
                         public void run() {
                             Rectangle visRect = getRenderRect();
                             if (doubleBufferedRendering)
-                                repaint(visRect.x,     visRect.y, 
+                                repaint(visRect.x,     visRect.y,
                                         visRect.width, visRect.height);
                             else
-                                paintImmediately(visRect.x,    visRect.y, 
+                                paintImmediately(visRect.x,    visRect.y,
                                                  visRect.width,visRect.height);
                         }
                     });
@@ -515,7 +567,7 @@ public class JGVTComponent extends JComponent {
         Rectangle visRect = getRenderRect();
         g2d.setComposite(AlphaComposite.SrcOver);
         g2d.setPaint(getBackground());
-        g2d.fillRect(visRect.x,     visRect.y, 
+        g2d.fillRect(visRect.x,     visRect.y,
                      visRect.width, visRect.height);
 
         if (image != null) {
@@ -557,9 +609,9 @@ public class JGVTComponent extends JComponent {
         setRenderingTransform(at, true);
     }
 
-    public void setRenderingTransform(AffineTransform at, 
+    public void setRenderingTransform(AffineTransform at,
                                       boolean performRedraw) {
-        renderingTransform = at;
+        renderingTransform = new AffineTransform(at);
         suspendInteractions = true;
         if (eventDispatcher != null) {
             try {
@@ -587,14 +639,14 @@ public class JGVTComponent extends JComponent {
      * Returns the initial transform.
      */
     public AffineTransform getInitialTransform() {
-        return initialTransform;
+        return new AffineTransform(initialTransform);
     }
 
     /**
      * Returns the current rendering transform.
      */
     public AffineTransform getRenderingTransform() {
-        return renderingTransform;
+        return new AffineTransform(renderingTransform);
     }
 
     /**
@@ -671,7 +723,7 @@ public class JGVTComponent extends JComponent {
         try {
             inv = renderingTransform.createInverse();
         } catch (NoninvertibleTransformException e) {
-            throw new InternalError(e.getMessage());
+            throw new IllegalStateException( "NoninvertibleTransformEx:" + e.getMessage() );
         }
         Shape s = inv.createTransformedShape(visRect);
 
@@ -702,7 +754,7 @@ public class JGVTComponent extends JComponent {
      */
     protected boolean computeRenderingTransform() {
         initialTransform = new AffineTransform();
-        if (initialTransform != renderingTransform) {
+        if (!initialTransform.equals(renderingTransform)) {
             setRenderingTransform(initialTransform, false);
             return true;
         }
@@ -745,26 +797,19 @@ public class JGVTComponent extends JComponent {
     protected void scheduleGVTRendering() {
         if (gvtTreeRenderer != null) {
             needRender = true;
-            gvtTreeRenderer.interrupt();
+            gvtTreeRenderer.halt();
         } else {
             renderGVTTree();
         }
     }
 
-    private void interruptProgressivePaintThread() {
+    private void haltProgressivePaintThread() {
         if (progressivePaintThread != null) {
-            progressivePaintThread.interrupt();
+            progressivePaintThread.halt();
             progressivePaintThread = null;
         }
     }
 
-    /**
-     * Creates an instance of Listener.
-     */
-    protected Listener createListener() {
-        return new Listener();
-    }
-
     /**
      * To hide the listener methods.
      */
@@ -772,11 +817,12 @@ public class JGVTComponent extends JComponent {
         implements GVTTreeRendererListener,
                    KeyListener,
                    MouseListener,
-                   MouseMotionListener {
+                   MouseMotionListener,
+                   MouseWheelListener {
         boolean checkClick = false;
         boolean hadDrag = false;
         int startX, startY;
-        long startTime;
+        long startTime, fakeClickTime;
         int MAX_DISP = 4*4;
         long CLICK_TIME = 200;
 
@@ -795,7 +841,6 @@ public class JGVTComponent extends JComponent {
             suspendInteractions = true;
             if (!progressivePaint && !doubleBufferedRendering) {
                 image = null;
-                immediateRepaint();
             }
         }
 
@@ -803,20 +848,19 @@ public class JGVTComponent extends JComponent {
          * Called when a rendering started.
          */
         public void gvtRenderingStarted(GVTTreeRendererEvent e) {
-            paintingTransform = null;
             if (progressivePaint && !doubleBufferedRendering) {
                 image = e.getImage();
-                progressivePaintThread = new Thread() {
+                progressivePaintThread = new HaltingThread() {
                     public void run() {
                         final Thread thisThread = this;
                         try {
-                            while (!isInterrupted()) {
+                            while (!hasBeenHalted()) {
                                 EventQueue.invokeLater(new Runnable() {
                                     public void run() {
                                         if (progressivePaintThread ==
                                             thisThread) {
                                             Rectangle vRect = getRenderRect();
-                                            repaint(vRect.x,     vRect.y, 
+                                            repaint(vRect.x,     vRect.y,
                                                     vRect.width, vRect.height);
                                         }
                                     }
@@ -835,6 +879,7 @@ public class JGVTComponent extends JComponent {
                 progressivePaintThread.start();
             }
             if (!doubleBufferedRendering) {
+                paintingTransform = null;
                 suspendInteractions = false;
             }
         }
@@ -843,9 +888,10 @@ public class JGVTComponent extends JComponent {
          * Called when a rendering was completed.
          */
         public void gvtRenderingCompleted(GVTTreeRendererEvent e) {
-            interruptProgressivePaintThread();
+            haltProgressivePaintThread();
 
             if (doubleBufferedRendering) {
+                paintingTransform = null;
                 suspendInteractions = false;
             }
 
@@ -881,7 +927,7 @@ public class JGVTComponent extends JComponent {
          * gvtRenderingFailed().
          */
         private void renderingStopped() {
-            interruptProgressivePaintThread();
+            haltProgressivePaintThread();
 
             if (doubleBufferedRendering) {
                 suspendInteractions = false;
@@ -969,6 +1015,13 @@ public class JGVTComponent extends JComponent {
          * Invoked when the mouse has been clicked on a component.
          */
         public void mouseClicked(MouseEvent e) {
+            // Supress mouse click if we generated a
+            // fake click with the same time stamp.
+            if (fakeClickTime != e.getWhen())
+                handleMouseClicked(e);
+        }
+
+        public void handleMouseClicked(MouseEvent e) {
             selectInteractor(e);
             if (interactor != null) {
                 interactor.mouseClicked(e);
@@ -991,7 +1044,8 @@ public class JGVTComponent extends JComponent {
         public void mousePressed(MouseEvent e) {
             startX = e.getX();
             startY = e.getY();
-            startTime = System.currentTimeMillis();
+            startTime = e.getWhen();
+
             checkClick = true;
 
             selectInteractor(e);
@@ -1017,7 +1071,7 @@ public class JGVTComponent extends JComponent {
             if ((checkClick) && hadDrag) {
                 int dx = startX-e.getX();
                 int dy = startY-e.getY();
-                long cTime = System.currentTimeMillis();
+                long cTime = e.getWhen();
                 if ((dx*dx+dy*dy < MAX_DISP) &&
                     (cTime-startTime) < CLICK_TIME) {
                     // our drag was short! dispatch a CLICK event.
@@ -1026,13 +1080,14 @@ public class JGVTComponent extends JComponent {
                         (e.getComponent(),
                          MouseEvent.MOUSE_CLICKED,
                          e.getWhen(),
-                         e.getModifiers(),		// modifiers
+                         e.getModifiers(),
                          e.getX(),
                          e.getY(),
                          e.getClickCount(),
                          e.isPopupTrigger());
-							
-                    mouseClicked(click);
+
+                    fakeClickTime = click.getWhen();
+                    handleMouseClicked(click);
                 }
             }
             checkClick = false;
@@ -1134,7 +1189,12 @@ public class JGVTComponent extends JComponent {
         public void mouseMoved(MouseEvent e) {
             selectInteractor(e);
             if (interactor != null) {
-                interactor.mouseMoved(e);
+            	// because the mouseDragged event doesn't seem to be generated on OSX when ctrl is held down
+            	if (Platform.isOSX &&
+            		interactor instanceof AbstractZoomInteractor)
+            		mouseDragged(e);
+            	else
+            		interactor.mouseMoved(e);
                 deselectInteractor();
             } else if (eventDispatcher != null) {
                 dispatchMouseMoved(e);
@@ -1148,6 +1208,28 @@ public class JGVTComponent extends JComponent {
             eventDispatcher.mouseMoved(e);
         }
 
+        // MouseWheelListener ///////////////////////////////////////////////
+
+        /**
+         * Invoked when the mouse wheel has been scrolled.
+         */
+        public void mouseWheelMoved(MouseWheelEvent e) {
+            /*selectInteractor(e);
+            if (interactor != null) {
+                interactor.mouseWheelMoved(e);
+                deselectInteractor();
+            } else*/ if (eventDispatcher != null) {
+                dispatchMouseWheelMoved(e);
+            }
+        }
+
+        /**
+         * Dispatches the mouse event to the GVT tree.
+         */
+        protected void dispatchMouseWheelMoved(MouseWheelEvent e) {
+            eventDispatcher.mouseWheelMoved(e);
+        }
+
         /**
          * Selects an interactor, given an input event.
          */
@@ -1176,4 +1258,52 @@ public class JGVTComponent extends JComponent {
             }
         }
     }
+
+    protected class UnixTextSelectionListener
+        extends SelectionAdapter {
+
+        public void selectionDone(SelectionEvent evt) {
+            if (!useUnixTextSelection) return;
+
+            Object o = evt.getSelection();
+            if (!(o instanceof CharacterIterator))
+                return;
+            CharacterIterator iter = (CharacterIterator) o;
+
+            // first see if we can access the clipboard
+            SecurityManager securityManager;
+            securityManager = System.getSecurityManager();
+            if (securityManager != null) {
+                try {
+                    securityManager.checkSystemClipboardAccess();
+                } catch (SecurityException e) {
+                    return; // Can't access clipboard.
+                }
+            }
+
+            int sz = iter.getEndIndex()-iter.getBeginIndex();
+            if (sz == 0) return;
+
+            char[] cbuff = new char[sz];
+            cbuff[0] = iter.first();
+            for (int i=1; i<cbuff.length;++i) {
+                cbuff[i] = iter.next();
+            }
+            final String strSel = new String(cbuff);
+            // HACK: getSystemClipboard sometimes deadlocks on
+            // linux when called from the AWT Thread. The Thread
+            // creation prevents that.
+            new Thread() {
+                public void run() {
+                    Clipboard cb;
+                    cb = Toolkit.getDefaultToolkit().getSystemClipboard();
+                    StringSelection sel;
+                    sel = new StringSelection(strSel);
+                    cb.setContents(sel, sel);
+                }
+            }.start();
+        }
+    }
+
+
 }
