@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
 */
-
 package org.apache.batik.transcoder.print;
 
 import java.awt.Graphics;
@@ -63,8 +31,10 @@ import java.awt.print.PrinterException;
 import java.awt.print.PrinterJob;
 import java.io.File;
 import java.util.StringTokenizer;
-import java.util.Vector;
+import java.util.List;
+import java.util.ArrayList;
 
+import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.ext.awt.RenderingHintsKeyExt;
 import org.apache.batik.transcoder.SVGAbstractTranscoder;
 import org.apache.batik.transcoder.Transcoder;
@@ -76,33 +46,35 @@ import org.apache.batik.transcoder.keys.BooleanKey;
 import org.apache.batik.transcoder.keys.LengthKey;
 import org.apache.batik.transcoder.keys.StringKey;
 
+import org.w3c.dom.Document;
+
 /**
- * This class is a <tt>Transcoder</tt> that prints SVG images.
+ * This class is a <code>Transcoder</code> that prints SVG images.
  * This class works as follows: any-time the transcode method
  * is invoked, the corresponding input is cached and nothing
  * else happens. <br />
- * However, the <tt>PrintTranscoder</tt> is also a Printable. If used
+ * However, the <code>PrintTranscoder</code> is also a Printable. If used
  * in a print operation, it will print each of the input
  * it cached, one input per page.
  * <br />
- * The <tt>PrintTranscoder</tt> uses several different hints that
+ * The <code>PrintTranscoder</code> uses several different hints that
  * guide its printing:<br />
  * <ul>
- *   <li><tt>KEY_LANGUAGE, KEY_USER_STYLESHEET_URI, KEY_PIXEL_TO_MM,
- *       KEY_XML_PARSER_CLASSNAME</tt> can be used to set the defaults for
+ *   <li><code>KEY_LANGUAGE, KEY_USER_STYLESHEET_URI, KEY_PIXEL_TO_MM,
+ *       KEY_XML_PARSER_CLASSNAME</code> can be used to set the defaults for
  *       the various SVG properties.</li>
- *   <li><tt>KEY_PAGE_WIDTH, KEY_PAGE_HEIGHT, KEY_MARGIN_TOP, KEY_MARGIN_BOTTOM,
- *       KEY_MARGIN_LEFT, KEY_MARGIN_RIGHT</tt> and <tt>KEY_PAGE_ORIENTATION</tt>
+ *   <li><code>KEY_PAGE_WIDTH, KEY_PAGE_HEIGHT, KEY_MARGIN_TOP, KEY_MARGIN_BOTTOM,
+ *       KEY_MARGIN_LEFT, KEY_MARGIN_RIGHT</code> and <code>KEY_PAGE_ORIENTATION</code>
  *       can be used to specify the printing page characteristics.</li>
- *   <li><tt>KEY_WIDTH, KEY_HEIGHT</tt> can be used to specify how to scale the
+ *   <li><code>KEY_WIDTH, KEY_HEIGHT</code> can be used to specify how to scale the
  *       SVG image</li>
- *   <li><tt>KEY_SCALE_TO_PAGE</tt> can be used to specify whether or not the
+ *   <li><code>KEY_SCALE_TO_PAGE</code> can be used to specify whether or not the
  *       SVG image should be scaled uniformly to fit into the printed page or
  *       if it should just be centered into the printed page.</li>
  * </ul>
  *
  * @author <a href="mailto:vincent.hardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: PrintTranscoder.java,v 1.28 2003/08/09 16:58:45 deweese Exp $
+ * @version $Id: PrintTranscoder.java 1636249 2014-11-03 04:35:18Z gadams $
  */
 public class PrintTranscoder extends SVGAbstractTranscoder
     implements Printable {
@@ -131,21 +103,30 @@ public class PrintTranscoder extends SVGAbstractTranscoder
 
     /**
      * Set of inputs this transcoder has been requested to
-     * transcode so far
+     * transcode so far.
+     * Purpose is not really clear: some data is added, and it is copied into
+     * printedInputs. But it is never read or cleared...
      */
-    private Vector inputs = new Vector();
+    private List inputs = new ArrayList();
 
     /**
      * Currently printing set of pages. This vector is
      * created as a clone of inputs when the first page is printed.
      */
-    private Vector printedInputs = null;
+    private List printedInputs = null;
 
     /**
      * Index of the page corresponding to root
      */
     private int curIndex = -1;
 
+    /**
+     * Place to cache BridgeContext so we can dispose of it when
+     * it is appropriate.  The Baseclass would dispose of it too
+     * soon.
+     */
+    private BridgeContext theCtx;
+
     /**
      * Constructs a new transcoder that prints images.
      */
@@ -159,10 +140,30 @@ public class PrintTranscoder extends SVGAbstractTranscoder
     public void transcode(TranscoderInput in,
                           TranscoderOutput out){
         if(in != null){
-            inputs.addElement(in);
+            inputs.add(in);
         }
     }
 
+    /**
+     * Transcodes the specified Document as an image in the specified output.
+     *
+     * @param document the document to transcode
+     * @param uri the uri of the document or null if any
+     * @param output the ouput where to transcode
+     * @exception TranscoderException if an error occured while transcoding
+     */
+    protected void transcode(Document document,
+                             String uri,
+                             TranscoderOutput output)
+            throws TranscoderException {
+        super.transcode(document, uri, output);
+
+        // We do this to hide 'ctx' from the SVGAbstractTranscoder
+        // otherwise it will dispose of the context before we can
+        // print the document.
+        theCtx = ctx;
+        ctx = null;
+    }
     /**
      * Convenience method
      */
@@ -171,11 +172,8 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         // Now, request the transcoder to actually perform the
         // printing job.
         //
-        PrinterJob printerJob =
-            PrinterJob.getPrinterJob();
-
-        PageFormat pageFormat =
-            printerJob.defaultPage();
+        PrinterJob printerJob = PrinterJob.getPrinterJob();
+        PageFormat pageFormat = printerJob.defaultPage();
 
         //
         // Set the page parameters from the hints
@@ -194,19 +192,20 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         }
 
         float x=0, y=0;
-        float width=(float)paper.getWidth(), height=(float)paper.getHeight();
+        float width =(float)paper.getWidth();
+        float height=(float)paper.getHeight();
 
-        Float leftMargin = (Float)hints.get(KEY_MARGIN_LEFT);
-        Float topMargin = (Float)hints.get(KEY_MARGIN_TOP);
-        Float rightMargin = (Float)hints.get(KEY_MARGIN_RIGHT);
+        Float leftMargin   = (Float)hints.get(KEY_MARGIN_LEFT);
+        Float topMargin    = (Float)hints.get(KEY_MARGIN_TOP);
+        Float rightMargin  = (Float)hints.get(KEY_MARGIN_RIGHT);
         Float bottomMargin = (Float)hints.get(KEY_MARGIN_BOTTOM);
 
         if(leftMargin != null){
-            x = leftMargin.floatValue();
+            x      = leftMargin.floatValue();
             width -= leftMargin.floatValue();
         }
         if(topMargin != null){
-            y = topMargin.floatValue();
+            y       = topMargin.floatValue();
             height -= topMargin.floatValue();
         }
         if(rightMargin != null){
@@ -236,7 +235,7 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         // If required, pop up a dialog to adjust the page format
         //
         Boolean showPageFormat = (Boolean)hints.get(KEY_SHOW_PAGE_DIALOG);
-        if(showPageFormat != null && showPageFormat.booleanValue()){
+        if ((showPageFormat != null) && (showPageFormat.booleanValue())) {
             PageFormat tmpPageFormat = printerJob.pageDialog(pageFormat);
             if(tmpPageFormat == pageFormat){
                 // Dialog was cancelled, meaning that the print process should
@@ -247,10 +246,15 @@ public class PrintTranscoder extends SVGAbstractTranscoder
             pageFormat = tmpPageFormat;
         }
 
+        // Set printable before showing printer dialog so
+        // it can update the pageFormat if it wishes...
+        printerJob.setPrintable(this, pageFormat);
+
         //
         // If required, pop up a dialog to select the printer
         //
-        Boolean showPrinterDialog = (Boolean)hints.get(KEY_SHOW_PRINTER_DIALOG);
+        Boolean showPrinterDialog;
+        showPrinterDialog = (Boolean)hints.get(KEY_SHOW_PRINTER_DIALOG);
         if(showPrinterDialog != null && showPrinterDialog.booleanValue()){
             if(!printerJob.printDialog()){
                 // Dialog was cancelled, meaning that the print process
@@ -260,7 +264,6 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         }
 
         // Print now
-        printerJob.setPrintable(this, pageFormat);
         printerJob.print();
 
     }
@@ -273,8 +276,8 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         // On the first page, take a snapshot of the vector of
         // TranscodeInputs.
         //
-        if(pageIndex == 0){
-            printedInputs = (Vector)inputs.clone();
+        if(printedInputs == null){
+            printedInputs = new ArrayList( inputs );
         }
 
         //
@@ -282,7 +285,9 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         //
         if(pageIndex >= printedInputs.size()){
             curIndex = -1;
-            System.out.println("Done");
+            if (theCtx != null)
+                theCtx.dispose();
+            userAgent.displayMessage("Done");
             return NO_SUCH_PAGE;
         }
 
@@ -290,12 +295,17 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         // Load a new document now if we are printing a new page
         //
         if(curIndex != pageIndex){
+            if (theCtx != null)
+                theCtx.dispose();
+
             // The following call will invoke this class' transcode
             // method which takes a document as an input. That method
             // builds the GVT root tree.{
             try{
-                super.transcode((TranscoderInput)printedInputs.elementAt(pageIndex),
-                                null);
+                width  = (int)pageFormat.getImageableWidth();
+                height = (int)pageFormat.getImageableHeight();
+                super.transcode
+                    ((TranscoderInput)printedInputs.get(pageIndex),null);
                 curIndex = pageIndex;
             }catch(TranscoderException e){
                 drawError(_g, e);
@@ -318,38 +328,19 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         AffineTransform t = g.getTransform();
         Shape clip = g.getClip();
 
-        Rectangle2D bounds = curAOI;
-
-        double scaleX = pageFormat.getImageableWidth() / bounds.getWidth();
-        double scaleY = pageFormat.getImageableHeight() / bounds.getHeight();
-        double scale = scaleX < scaleY ? scaleX : scaleY;
-
-        // Check hint to know if scaling is really needed
-        Boolean scaleToPage = (Boolean)hints.get(KEY_SCALE_TO_PAGE);
-        if(scaleToPage != null && !scaleToPage.booleanValue()) {
-            // Printing Graphics is always set up for 72dpi, so scale
-            // according to what user agent thinks it should be.
-            double pixSzMM = userAgent.getPixelUnitToMillimeter();
-            double pixSzInch = (25.4/pixSzMM);
-            scale = 72/pixSzInch;
-        }
-
-        double xMargin = (pageFormat.getImageableWidth() - 
-                          bounds.getWidth()*scale)/2;
-        double yMargin = (pageFormat.getImageableHeight() - 
-                          bounds.getHeight()*scale)/2;
-        g.translate(pageFormat.getImageableX() + xMargin,
-                    pageFormat.getImageableY() + yMargin);
-        g.scale(scale, scale);
-
+        // System.err.println("X/Y: " + pageFormat.getImageableX() + ", " +
+        //                    pageFormat.getImageableY());
+        // System.err.println("W/H: " + width + ", " + height);
+        // System.err.println("Clip: " + clip.getBounds2D());
 
+        // Offset 0,0 to the start of the imageable Area.
+        g.translate(pageFormat.getImageableX(),
+                    pageFormat.getImageableY());
         //
         // Append transform to selected area
         //
         g.transform(curTxf);
 
-        g.clip(curAOI);
-
         //
         // Delegate rendering to painter
         //
@@ -377,11 +368,34 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         return PAGE_EXISTS;
     }
 
+    /**
+     * Sets document size according to the hints.
+     * Global variables width and height are modified.
+     *
+     * @param docWidth Width of the document.
+     * @param docHeight Height of the document.
+     */
+    protected void setImageSize(float docWidth, float docHeight) {
+        // Check hint to know if scaling is really needed
+        Boolean scaleToPage = (Boolean)hints.get(KEY_SCALE_TO_PAGE);
+        if(scaleToPage != null && !scaleToPage.booleanValue()) {
+            float w = docWidth;
+            float h = docHeight;
+            if (hints.containsKey(KEY_AOI)) {
+                Rectangle2D aoi = (Rectangle2D)hints.get(KEY_AOI);
+                w = (float)aoi.getWidth();
+                h = (float)aoi.getHeight();
+            }
+            super.setImageSize(w, h);
+        }
+    }
+
     /**
      * Prints an error on the output page
      */
     private void drawError(Graphics g, Exception e){
-        // Do nothing now.
+        userAgent.displayError(e);
+        // Should also probably draw exception on page.
     }
 
     // --------------------------------------------------------------------
@@ -390,236 +404,290 @@ public class PrintTranscoder extends SVGAbstractTranscoder
 
     /**
      * The showPageDialog key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_SHOW_PAGE_DIALOG</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Boolean</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">false</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">Specifies whether or not the transcoder
-     *                  should pop up a dialog box for selecting
-     *                  the page format.</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_SHOW_PAGE_DIALOG</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Boolean</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">false</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">Specifies whether or not the transcoder
+     *       should pop up a dialog box for selecting
+     *       the page format.</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_SHOW_PAGE_DIALOG
         = new BooleanKey();
 
     /**
      * The showPrinterDialog key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_SHOW_PAGE_DIALOG</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Boolean</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">false</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">Specifies whether or not the transcoder
-     *                  should pop up a dialog box for selecting
-     *                  the printer. If the dialog box is not
-     *                  shown, the transcoder will use the default
-     *                  printer.</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_SHOW_PAGE_DIALOG</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Boolean</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">false</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">Specifies whether or not the transcoder
+     *       should pop up a dialog box for selecting
+     *       the printer. If the dialog box is not
+     *       shown, the transcoder will use the default
+     *       printer.</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_SHOW_PRINTER_DIALOG
         = new BooleanKey();
 
 
     /**
      * The pageWidth key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_PAGE_WIDTH</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Length</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">None</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The width of the print page</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_PAGE_WIDTH</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Length</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">None</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">The width of the print page</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_PAGE_WIDTH
         = new LengthKey();
 
     /**
      * The pageHeight key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_PAGE_HEIGHT</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Length</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">None</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The height of the print page</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_PAGE_HEIGHT</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Length</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">none</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">The height of the print page</td>
+     *   </tr>
+     * </table> 
+     */
     public static final TranscodingHints.Key KEY_PAGE_HEIGHT
         = new LengthKey();
 
     /**
      * The marginTop key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_MARGIN_TOP</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Length</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">None</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The print page top margin</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_MARGIN_TOP</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Length</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">None</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">The print page top margin</td>
+     *   </tr>
+     * </table> */
     public static final TranscodingHints.Key KEY_MARGIN_TOP
         = new LengthKey();
 
     /**
      * The marginRight key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_MARGIN_RIGHT</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Length</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">None</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The print page right margin</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_MARGIN_RIGHT</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Length</TD>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">None</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">The print page right margin</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_MARGIN_RIGHT
         = new LengthKey();
 
     /**
      * The marginBottom key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_MARGIN_BOTTOM</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Length</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">None</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The print page bottom margin</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_MARGIN_BOTTOM</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Length</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">None</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">The print page bottom margin</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_MARGIN_BOTTOM
         = new LengthKey();
 
     /**
      * The marginLeft key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_MARGIN_LEFT</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Length</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">None</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The print page left margin</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     * <tr>
+     * <th valign="top" align="right">Key:</th>
+     * <td valign="top">KEY_MARGIN_LEFT</td></tr>
+     * <tr>
+     * <th valign="top" align="right">Value:</th>
+     * <td valign="top">Length</td></tr>
+     * <tr>
+     * <th valign="top" align="right">Default:</th>
+     * <td valign="top">None</td></tr>
+     * <tr>
+     * <th valign="top" align="right">Required:</th>
+     * <td valign="top">No</td></tr>
+     * <tr>
+     * <th valign="top" align="right">Description:</th>
+     * <td valign="top">The print page left margin</td></tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_MARGIN_LEFT
         = new LengthKey();
 
     /**
      * The pageOrientation key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_PAGE_ORIENTATION</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">String</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">VALUE_PAGE_ORIENTATION_PORTRAIT</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">The print page's orientation</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_PAGE_ORIENTATION</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">String</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">VALUE_PAGE_ORIENTATION_PORTRAIT</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">The print page's orientation</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_PAGE_ORIENTATION
         = new StringKey();
 
 
     /**
      * The scaleToPage key.
-     * <TABLE BORDER="0" CELLSPACING="0" CELLPADDING="1">
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Key: </TH>
-     * <TD VALIGN="TOP">KEY_SCALE_TO_PAGE</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Value: </TH>
-     * <TD VALIGN="TOP">Boolean</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Default: </TH>
-     * <TD VALIGN="TOP">true</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Required: </TH>
-     * <TD VALIGN="TOP">No</TD></TR>
-     * <TR>
-     * <TH VALIGN="TOP" ALIGN="RIGHT"><P ALIGN="RIGHT">Description: </TH>
-     * <TD VALIGN="TOP">Specifies whether or not the SVG images are scaled to
-     *                  fit into the printed page</TD></TR>
-     * </TABLE> */
+     * <table border="0" cellspacing="0" cellpadding="1">
+     *   <tr>
+     *     <th valign="top" align="right">Key:</th>
+     *     <td valign="top">KEY_SCALE_TO_PAGE</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Value:</th>
+     *     <td valign="top">Boolean</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Default:</th>
+     *     <td valign="top">true</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Required:</th>
+     *     <td valign="top">No</td>
+     *   </tr>
+     *   <tr>
+     *     <th valign="top" align="right">Description:</th>
+     *     <td valign="top">Specifies whether or not the SVG images are scaled to
+     *       fit into the printed page</td>
+     *   </tr>
+     * </table>
+     */
     public static final TranscodingHints.Key KEY_SCALE_TO_PAGE
         = new BooleanKey();
 
     public static final String USAGE = "java org.apache.batik.transcoder.print.PrintTranscoder <svgFileToPrint>";
 
-    public static void main(String args[]) throws Exception{
+    public static void main(String[] args) throws Exception{
         if(args.length < 1){
             System.err.println(USAGE);
             System.exit(0);
@@ -714,7 +782,7 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         // each of the input files
         //
         for(int i=0; i<args.length; i++){
-            transcoder.transcode(new TranscoderInput(new File(args[i]).toURL().toString()),
+            transcoder.transcode(new TranscoderInput(new File(args[i]).toURI().toURL().toString()),
                                  null);
         }
 
@@ -722,6 +790,8 @@ public class PrintTranscoder extends SVGAbstractTranscoder
         // Now, print...
         //
         transcoder.print();
+
+        System.exit(0);
     }
 
     public static void setTranscoderFloatHint(Transcoder transcoder,
@@ -769,7 +839,7 @@ public class PrintTranscoder extends SVGAbstractTranscoder
                                                 TranscodingHints.Key key){
         String str = System.getProperty(property);
         if(str != null){
-            Boolean value = new Boolean("true".equalsIgnoreCase(str));
+            Boolean value = "true".equalsIgnoreCase(str) ? Boolean.TRUE : Boolean.FALSE;
             transcoder.addTranscodingHint(key, value);
         }
     }
