@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.transcoder.svg2svg;
 
 import java.io.IOException;
@@ -62,7 +30,7 @@ import org.apache.batik.xml.XMLUtilities;
  * This class is responsible of the output of XML constructs.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: OutputManager.java,v 1.7 2003/08/08 11:39:26 vhardy Exp $
+ * @version $Id: OutputManager.java 475477 2006-11-15 22:44:28Z cam $
  */
 public class OutputManager {
 
@@ -121,7 +89,8 @@ public class OutputManager {
 
     /**
      * Creates a new output manager.
-     * @param s The scanner used for input tokenization.
+     * @param pp The PrettyPrinter used for formatting the output.
+     * @param w The Writer to write the output to.
      */
     public OutputManager(PrettyPrinter pp, Writer w) {
         prettyPrinter = pp;
@@ -188,7 +157,6 @@ public class OutputManager {
     /**
      * Prints top level white spaces.
      * @param text The space text.
-     * @param opt whether the space is optional.
      */
     public void printTopSpaces(char[] text) throws IOException {
         if (prettyPrinter.getFormat()) {
@@ -215,7 +183,7 @@ public class OutputManager {
             if (column + text.length + 3 < prettyPrinter.getDocumentWidth()) {
                 printCharacters(text);
             } else {
-                formatText(text, margin.toString());
+                formatText(text, margin.toString(), false);
                 printCharacter(' ');
             }
             if (column + 3 > prettyPrinter.getDocumentWidth()) {
@@ -436,7 +404,14 @@ public class OutputManager {
     /**
      * Prints an entity reference.
      */
-    public void printEntityReference(char[] name) throws IOException {
+    public void printEntityReference(char[] name, 
+                                     boolean first) throws IOException {
+        if ((prettyPrinter.getFormat()) &&
+            (xmlSpace.get(0) != Boolean.TRUE) &&
+            first) {
+            printNewline();
+            printString(margin.toString());
+        }
         printCharacter('&');
         printCharacters(name);
         printCharacter(';');
@@ -445,7 +420,25 @@ public class OutputManager {
     /**
      * Prints a character entity reference.
      */
-    public void printCharacterEntityReference(char[] code) throws IOException {
+    public void printCharacterEntityReference
+        (char[] code, boolean first, boolean preceedingSpace) 
+        throws IOException {
+        if ((prettyPrinter.getFormat()) &&
+            (xmlSpace.get(0) != Boolean.TRUE)) {
+
+            if (first) {
+                printNewline();
+                printString(margin.toString());
+            } else if (preceedingSpace) {
+                int endCol = column + code.length + 3;
+                if (endCol > prettyPrinter.getDocumentWidth()){
+                    printNewline();
+                    printString(margin.toString());
+                } else {
+                    printCharacter(' ');
+                }
+            }
+        }
         printString("&#");
         printCharacters(code);
         printCharacter(';');
@@ -584,27 +577,35 @@ public class OutputManager {
     /**
      * Prints the character data of an element content.
      */
-    public void printCharacterData(char[] data) throws IOException {
-        if (prettyPrinter.getFormat()) {
-            canIndent = true;
-            if (isWhiteSpace(data)) {
-                int nl = newlines(data);
-                for (int i = 0; i < nl - 1; i++) {
-                    printNewline();
-                }
-            } else {
-                if (xmlSpace.get(0) == Boolean.TRUE) {
-                    printCharacters(data);
-                    canIndent = false;
-                } else {
-                    printNewline();
-                    printString(margin.toString());
-                    formatText(data, margin.toString());
-                }
-            }
-        } else {
+    public boolean printCharacterData(char[] data, 
+                                      boolean first,
+                                      boolean preceedingSpace) 
+        throws IOException {
+        if (!prettyPrinter.getFormat()) {
             printCharacters(data);
+            return false;
         }
+
+        canIndent = true;
+        if (isWhiteSpace(data)) {
+            int nl = newlines(data);
+            for (int i = 0; i < nl - 1; i++) {
+                printNewline();
+            }
+            return true;
+        }
+
+        if (xmlSpace.get(0) == Boolean.TRUE) {
+            printCharacters(data);
+            canIndent = false;
+            return false;
+        }
+
+        if (first) {
+            printNewline();
+            printString(margin.toString());
+        }
+        return formatText(data, margin.toString(), preceedingSpace);
     }
 
     /**
@@ -747,9 +748,10 @@ public class OutputManager {
     /**
      * Formats the given text.
      */
-    protected void formatText(char[] text, String margin) throws IOException {
+    protected boolean formatText(char[] text, String margin,
+                                 boolean preceedingSpace) throws IOException {
         int i = 0;
-
+        boolean startsWithSpace = preceedingSpace;
         loop: while (i < text.length) {
             for (;;) {
                 if (i >= text.length) {
@@ -758,6 +760,7 @@ public class OutputManager {
                 if (!XMLUtilities.isXMLSpace(text[i])) {
                     break;
                 }
+                startsWithSpace = true;
                 i++;
             }
             StringBuffer sb = new StringBuffer();
@@ -768,28 +771,26 @@ public class OutputManager {
                 sb.append(text[i++]);
             }
             if (sb.length() == 0) {
-                break;
+                return startsWithSpace;
             }
-            if (column + sb.length() >= prettyPrinter.getDocumentWidth() - 1) {
-                if (margin.length() + sb.length() <
-                    prettyPrinter.getDocumentWidth() - 1 ||
-                    margin.length() < column) {
+            if (startsWithSpace) {
+                // Consider reformatting ws so things look nicer.
+                int endCol = column + sb.length();
+                if ((endCol >= prettyPrinter.getDocumentWidth() - 1) &&
+                    ((margin.length() + sb.length() <
+                      prettyPrinter.getDocumentWidth() - 1) ||
+                     (margin.length() < column))) {
                     printNewline();
                     printString(margin);
-                    printString(sb.toString());
-                } else {
-                    if (column >= margin.length()) {
-                        printCharacter(' ');
-                    }
-                    printString(sb.toString());
-                }
-            } else {
-                if (column > margin.length()) {
+                } else if (column > margin.length()) {
+                    // Don't print space at start of new line.
                     printCharacter(' ');
                 }
-                printString(sb.toString());
             }
+            printString(sb.toString());
+            startsWithSpace = false;
         }
+        return startsWithSpace;
     }
 
     /**
