@@ -1,82 +1,55 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+    contributor license agreements.  See the NOTICE file distributed with
+    this work for additional information regarding copyright ownership.
+    The ASF licenses this file to You under the Apache License, Version 2.0
+    (the "License"); you may not use this file except in compliance with
+    the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.util;
 
 import java.util.Iterator;
+import java.util.NoSuchElementException;
 
 /**
  * This class represents an object which queues Runnable objects for
  * invocation in a single thread.
  *
  * @author <a href="mailto:stephane@hillion.org">Stephane Hillion</a>
- * @version $Id: RunnableQueue.java,v 1.15 2003/11/05 22:21:10 deweese Exp $
+ * @version $Id: RunnableQueue.java 1372129 2012-08-12 15:31:50Z helder $
  */
 public class RunnableQueue implements Runnable {
 
     /**
      * Type-safe enumeration of queue states.
      */
-    public static class RunnableQueueState extends Object {
-        final String value;
+    public static final class RunnableQueueState {
+
+        private final String value;
+
         private RunnableQueueState(String value) {
-            this.value = value.intern(); }
+            this.value = value; }
+
         public String getValue() { return value; }
-        public String toString() { 
-            return "[RunnableQueueState: " + value + "]"; }
+
+        public String toString() {
+            return "[RunnableQueueState: " + value + ']'; }
     }
 
     /**
-     * The queue is in the processes of running tasks.
+     * The queue is in the process of running tasks.
      */
-    public static final RunnableQueueState RUNNING 
+    public static final RunnableQueueState RUNNING
         = new RunnableQueueState("Running");
 
     /**
@@ -96,22 +69,31 @@ public class RunnableQueue implements Runnable {
     /**
      * The Suspension state of this thread.
      */
-    protected RunnableQueueState state;
+    protected volatile RunnableQueueState state;
 
     /**
      * Object to synchronize/wait/notify for suspension
      * issues.
      */
-    protected Object stateLock = new Object();
-
+    protected final Object stateLock = new Object();
 
     /**
-     * The Runnable objects list, also used as synchoronization point
+     * Used to indicate if the queue was resumed while
+     * still running, so a 'resumed' event can be sent.
+     */
+    protected boolean wasResumed;
+
+    /**
+     * The Runnable objects list, also used as synchronization point
      * for pushing/poping runables.
      */
-    protected DoublyLinkedList list = new DoublyLinkedList();
+    private final DoublyLinkedList list = new DoublyLinkedList();
 
-    protected int preemptCount = 0;
+    /**
+     * Count of preempt entries in queue, so preempt entries
+     * can be kept properly ordered.
+     */
+    protected int preemptCount;
 
     /**
      * The object which handle run events.
@@ -121,21 +103,35 @@ public class RunnableQueue implements Runnable {
     /**
      * The current thread.
      */
-    protected Thread runnableQueueThread;
+    protected volatile HaltingThread runnableQueueThread;
+
+    /**
+     * The {@link IdleRunnable} to run if the queue is empty.
+     */
+    private IdleRunnable idleRunnable;
+
+    /**
+     * The time (in milliseconds) that the idle runnable should be run next.
+     */
+    private long idleRunnableWaitTime;
 
     /**
      * Creates a new RunnableQueue started in a new thread.
-     * @return a RunnableQueue which is garanteed to have entered its
-     *         <tt>run()</tt> method.
+     * @return a RunnableQueue which is guaranteed to have entered its
+     *         <code>run()</code> method.
      */
     public static RunnableQueue createRunnableQueue() {
         RunnableQueue result = new RunnableQueue();
         synchronized (result) {
-            Thread t = new Thread(result, "RunnableQueue-" + threadCount++);
-            t.setDaemon(true);
-            t.start();
+            // Sync on the new object, so we can wait until the new
+            // thread is ready to go.
+
+            HaltingThread ht = new HaltingThread
+                (result, "RunnableQueue-" + threadCount++);
+            ht.setDaemon(true);
+            ht.start();
             while (result.getThread() == null) {
-                try { 
+                try {
                     result.wait();
                 } catch (InterruptedException ie) {
                 }
@@ -143,14 +139,15 @@ public class RunnableQueue implements Runnable {
         }
         return result;
     }
-    private static int threadCount;
-    
+
+    private static volatile int threadCount;
+
     /**
      * Runs this queue.
      */
     public void run() {
         synchronized (this) {
-            runnableQueueThread = Thread.currentThread();
+            runnableQueueThread = (HaltingThread)Thread.currentThread();
             // Wake the create method so it knows we are in
             // our run and ready to go.
             notify();
@@ -159,32 +156,45 @@ public class RunnableQueue implements Runnable {
         Link l;
         Runnable rable;
         try {
-            while (!Thread.currentThread().isInterrupted()) {
-
-                // Mutex for suspention work.
+            while (!HaltingThread.hasBeenHalted()) {
+                boolean callSuspended = false;
+                boolean callResumed   = false;
+                // Mutex for suspension work.
                 synchronized (stateLock) {
                     if (state != RUNNING) {
                         state = SUSPENDED;
+                        callSuspended = true;
+                    }
+                }
+                if (callSuspended)
+                    executionSuspended();
+
+                synchronized (stateLock) {
+                    while (state != RUNNING) {
+                        state = SUSPENDED;
 
                         // notify suspendExecution in case it is
                         // waiting til we shut down.
                         stateLock.notifyAll();
 
-                        executionSuspended();
-
-                        while (state != RUNNING) {
-                            state = SUSPENDED;
-                            // Wait until resumeExecution called.
+                        // Wait until resumeExecution called.
+                        try {
                             stateLock.wait();
-                        }
+                        } catch(InterruptedException ie) { }
+                    }
 
-                        executionResumed();
+                    if (wasResumed) {
+                        wasResumed = false;
+                        callResumed = true;
                     }
                 }
 
+                if (callResumed)
+                    executionResumed();
+
                 // The following seriously stress tests the class
                 // for stuff happening between the two sync blocks.
-                // 
+                //
                 // try {
                 //     Thread.sleep(1);
                 // } catch (InterruptedException ie) { }
@@ -195,15 +205,38 @@ public class RunnableQueue implements Runnable {
                     l = (Link)list.pop();
                     if (preemptCount != 0) preemptCount--;
                     if (l == null) {
-                        // No item to run, wait till there is one.
-                        list.wait();
-                        continue; // start loop over again...
+                        // No item to run, see if there is an idle runnable
+                        // to run instead.
+                        if (idleRunnable != null &&
+                                (idleRunnableWaitTime = idleRunnable.getWaitTime())
+                                    < System.currentTimeMillis()) {
+                            rable = idleRunnable;
+                        } else {
+                            // Wait for a runnable.
+                            try {
+                                if (idleRunnable != null && idleRunnableWaitTime
+                                        != Long.MAX_VALUE) {
+                                    long t = idleRunnableWaitTime
+                                        - System.currentTimeMillis();
+                                    if (t > 0) {
+                                        list.wait(t);
+                                    }
+                                } else {
+                                    list.wait();
+                                }
+                            } catch (InterruptedException ie) {
+                                // just loop again.
+                            }
+                            continue; // start loop over again...
+                        }
+                    } else {
+                        rable = l.runnable;
                     }
-
-                    rable = l.runnable;
                 }
 
                 try {
+                    runnableStart(rable);
+
                     rable.run();
                 } catch (ThreadDeath td) {
                     // Let it kill us...
@@ -213,11 +246,36 @@ public class RunnableQueue implements Runnable {
                     // But this is more or less what Swing does.
                     t.printStackTrace();
                 }
-                l.unlock();
-                runnableInvoked(rable);
+                // Notify something waiting on the runnable just completed,
+                // if we just ran one from the queue.
+                if (l != null) {
+                    l.unlock();
+                }
+
+                try {
+                    runnableInvoked(rable);
+                } catch (ThreadDeath td) {
+                    // Let it kill us...
+                    throw td;
+                } catch (Throwable t) {
+                    // Might be nice to notify someone directly.
+                    // But this is more or less what Swing does.
+                    t.printStackTrace();
+                }
             }
-        } catch (InterruptedException e) {
         } finally {
+            do {
+                // Empty the list of pending runnables and unlock them (so
+                // invokeAndWait will return).
+                // It's up to the runnables to check if the runnable actually
+                // ran, if that is important.
+                synchronized (list) {
+                    l = (Link)list.pop();
+                }
+                if (l == null) break;
+                else           l.unlock();
+            } while (true);
+
             synchronized (this) {
                 runnableQueueThread = null;
             }
@@ -227,9 +285,9 @@ public class RunnableQueue implements Runnable {
     /**
      * Returns the thread in which the RunnableQueue is currently running.
      * @return null if the RunnableQueue has not entered his
-     *         <tt>run()</tt> method.
+     *         <code>run()</code> method.
      */
-    public Thread getThread() {
+    public HaltingThread getThread() {
         return runnableQueueThread;
     }
 
@@ -251,10 +309,10 @@ public class RunnableQueue implements Runnable {
     }
 
     /**
-     * Waits until the given Runnable's <tt>run()</tt> has returned.
-     * <em>Note: <tt>invokeAndWait()</tt> must not be called from the
-     * current thread (for example from the <tt>run()</tt> method of the
-     * argument).
+     * Waits until the given Runnable's <code>run()</code> has returned.
+     * <em>Note: <code>invokeAndWait()</code> must not be called from the
+     * current thread (for example from the <code>run()</code> method of the
+     * argument).</em>
      * @throws IllegalStateException if getThread() is null or if the
      *         thread returned by getThread() is the current one.
      */
@@ -273,7 +331,7 @@ public class RunnableQueue implements Runnable {
             list.push(l);
             list.notify();
         }
-        l.lock();
+        l.lock();           // todo: the 'other side' of list may retrieve the l before it is locked...
     }
 
 
@@ -282,8 +340,8 @@ public class RunnableQueue implements Runnable {
      * returns. The given runnable preempts any runnable that is not
      * currently executing (ie the next runnable started will be the
      * one given).  An exception is thrown if the RunnableQueue was
-     * not started.  
-     * @throws IllegalStateException if getThread() is  null.  
+     * not started.
+     * @throws IllegalStateException if getThread() is  null.
      */
     public void preemptLater(Runnable r) {
         if (runnableQueueThread == null) {
@@ -298,12 +356,12 @@ public class RunnableQueue implements Runnable {
     }
 
     /**
-     * Waits until the given Runnable's <tt>run()</tt> has returned.
+     * Waits until the given Runnable's <code>run()</code> has returned.
      * The given runnable preempts any runnable that is not currently
      * executing (ie the next runnable started will be the one given).
-     * <em>Note: <tt>preemptAndWait()</tt> must not be called from the
-     * current thread (for example from the <tt>run()</tt> method of the
-     * argument).
+     * <em>Note: <code>preemptAndWait()</code> must not be called from the
+     * current thread (for example from the <code>run()</code> method of the
+     * argument).</em>
      * @throws IllegalStateException if getThread() is null or if the
      *         thread returned by getThread() is the current one.
      */
@@ -323,12 +381,12 @@ public class RunnableQueue implements Runnable {
             preemptCount++;
             list.notify();
         }
-        l.lock();
+        l.lock();               // todo: the 'other side' of list may retrieve the l before it is locked...
     }
 
-    public RunnableQueueState getQueueState() { 
+    public RunnableQueueState getQueueState() {
         synchronized (stateLock) {
-            return state; 
+            return state;
         }
     }
 
@@ -341,16 +399,23 @@ public class RunnableQueue implements Runnable {
      *        called while waiting will simply return (this really
      *        indicates a race condition in your code).  This may
      *        return before an associated RunHandler is notified.
-     * @throws IllegalStateException if getThread() is null.  */
+     * @throws IllegalStateException if getThread() is null.
+     */
     public void suspendExecution(boolean waitTillSuspended) {
         if (runnableQueueThread == null) {
             throw new IllegalStateException
                 ("RunnableQueue not started or has exited");
         }
+        // System.err.println("Suspend Called");
         synchronized (stateLock) {
-            if (state == SUSPENDED) 
-                // already suspended...
+            wasResumed = false;
+
+            if (state == SUSPENDED) {
+                // already suspended, notify stateLock so an event is
+                // generated.
+                stateLock.notifyAll();
                 return;
+            }
 
             if (state == RUNNING) {
                 state = SUSPENDING;
@@ -362,10 +427,13 @@ public class RunnableQueue implements Runnable {
                 }
             }
 
-            if (waitTillSuspended)
-                try {
-                    stateLock.wait();
-                } catch(InterruptedException ie) { }
+            if (waitTillSuspended) {
+                while (state == SUSPENDING) {
+                    try {
+                        stateLock.wait();
+                    } catch(InterruptedException ie) { }
+                }
+            }
         }
     }
 
@@ -374,12 +442,15 @@ public class RunnableQueue implements Runnable {
      * @throws IllegalStateException if getThread() is null.
      */
     public void resumeExecution() {
+        // System.err.println("Resume Called");
         if (runnableQueueThread == null) {
             throw new IllegalStateException
                 ("RunnableQueue not started or has exited");
         }
 
         synchronized (stateLock) {
+            wasResumed = true;
+
             if (state != RUNNING) {
                 state = RUNNING;
                 stateLock.notifyAll(); // wake it up.
@@ -413,7 +484,7 @@ public class RunnableQueue implements Runnable {
                 }
                 public Object next() {
                     if (head == null || head == link) {
-                        throw new java.util.NoSuchElementException();
+                        throw new NoSuchElementException();
                     }
                     if (link == null) {
                         link = (Link)head.getNext();
@@ -443,11 +514,23 @@ public class RunnableQueue implements Runnable {
         return runHandler;
     }
 
+    /**
+     * Sets a Runnable to be run whenever the queue is empty.
+     */
+    public void setIdleRunnable(IdleRunnable r) {
+        synchronized (list) {
+            idleRunnable = r;
+            idleRunnableWaitTime = 0;
+            list.notify();
+        }
+    }
+
     /**
      * Called when execution is being suspended.
      * Currently just notifies runHandler
      */
     protected synchronized void executionSuspended() {
+        // System.err.println("Suspend Sent");
         if (runHandler != null) {
             runHandler.executionSuspended(this);
         }
@@ -458,11 +541,23 @@ public class RunnableQueue implements Runnable {
      * Currently just notifies runHandler
      */
     protected synchronized void executionResumed() {
+        // System.err.println("Resumed Sent");
         if (runHandler != null) {
             runHandler.executionResumed(this);
         }
     }
-        
+
+    /**
+     * Called just prior to executing a Runnable.
+     * Currently just notifies runHandler
+     * @param rable The runnable that is about to start
+     */
+    protected synchronized void runnableStart(Runnable rable ) {
+        if (runHandler != null) {
+            runHandler.runnableStart(this, rable);
+        }
+    }
+
     /**
      * Called when a Runnable completes.
      * Currently just notifies runHandler
@@ -474,12 +569,34 @@ public class RunnableQueue implements Runnable {
         }
     }
 
+    /**
+     * A {@link Runnable} that can also inform the caller how long it should
+     * be until it is run again.
+     */
+    public interface IdleRunnable extends Runnable {
+
+        /**
+         * Returns the system time that can be safely waited until before this
+         * {@link Runnable} is run again.
+         *
+         * @return time to wait until, <code>0</code> if no waiting can
+         *         be done, or {@link Long#MAX_VALUE} if the {@link Runnable}
+         *         should not be run again at this time
+         */
+        long getWaitTime();
+    }
+
     /**
      * This interface must be implemented by an object which wants to
      * be notified of run events.
      */
     public interface RunHandler {
 
+        /**
+         * Called just prior to invoking the runnable
+         */
+        void runnableStart(RunnableQueue rq, Runnable r);
+
         /**
          * Called when the given Runnable has just been invoked and
          * has returned.
@@ -497,15 +614,43 @@ public class RunnableQueue implements Runnable {
         void executionResumed(RunnableQueue rq);
     }
 
+    /**
+     * This is an adapter class that implements the RunHandler interface.
+     * It simply does nothing in response to the calls.
+     */
+    public static class RunHandlerAdapter implements RunHandler {
+
+        /**
+         * Called just prior to invoking the runnable
+         */
+        public void runnableStart(RunnableQueue rq, Runnable r) { }
+
+        /**
+         * Called when the given Runnable has just been invoked and
+         * has returned.
+         */
+        public void runnableInvoked(RunnableQueue rq, Runnable r) { }
+
+        /**
+         * Called when the execution of the queue has been suspended.
+         */
+        public void executionSuspended(RunnableQueue rq) { }
+
+        /**
+         * Called when the execution of the queue has been resumed.
+         */
+        public void executionResumed(RunnableQueue rq) { }
+    }
+
     /**
      * To store a Runnable.
      */
     protected static class Link extends DoublyLinkedList.Node {
-        
+
         /**
          * The Runnable.
          */
-        public Runnable runnable;
+        private final Runnable runnable;
 
         /**
          * Creates a new link.
@@ -515,10 +660,10 @@ public class RunnableQueue implements Runnable {
         }
 
         /**
-         * unlock link and notify locker.  
+         * unlock link and notify locker.
          * Basic implementation does nothing.
          */
-        public void unlock() throws InterruptedException { return; }
+        public void unlock() { return; }
     }
 
     /**
@@ -529,7 +674,7 @@ public class RunnableQueue implements Runnable {
         /**
          * Whether this link is actually locked.
          */
-        protected boolean locked;
+        private volatile boolean locked;
 
         /**
          * Creates a new link.
@@ -557,11 +702,15 @@ public class RunnableQueue implements Runnable {
         /**
          * unlocks this link.
          */
-        public synchronized void unlock() throws InterruptedException {
+        public synchronized void unlock() {
             while (!locked) {
-                // Wait until lock is called...
-                wait();
+                try {
+                    wait(); // Wait until lock is called...
+                } catch (InterruptedException ie) {
+                    // Loop again...
+                }
             }
+            locked = false;
             // Wake the locking thread...
             notify();
         }
