@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2002  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -17,15 +18,13 @@
  */package org.apache.batik.bridge;
 
 import org.w3c.dom.*;
-import org.apache.batik.script.ScriptHandler;
-import org.apache.batik.script.Window;
 
 /**
  * If this script is loaded by jarCheckLoad.svg, it will mark
  * the test result as passed.
  *
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: IWasLoadedToo.java,v 1.3 2004/08/18 07:16:02 vhardy Exp $
+ * @version $Id: IWasLoadedToo.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class IWasLoadedToo implements ScriptHandler {
     public void run(final Document document, final Window win){
