@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2002-2003  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -23,7 +24,6 @@ import java.io.SerializablePermission;
 import java.lang.reflect.ReflectPermission;
 import java.net.NetPermission;
 import java.net.SocketPermission;
-import java.net.URL;
 import java.security.AllPermission;
 import java.security.Permission;
 import java.sql.SQLPermission;
@@ -38,13 +38,12 @@ import org.w3c.dom.events.Event;
 import org.w3c.dom.events.EventListener;
 import org.w3c.dom.events.EventTarget;
 
-import org.apache.batik.dom.svg.SVGOMDocument;
-import org.apache.batik.script.ScriptHandler;
-import org.apache.batik.script.Window;
+import org.apache.batik.anim.dom.SVGOMDocument;
+import org.apache.batik.util.ParsedURL;
 
 /**
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: JarCheckPermissionsGranted.java,v 1.4 2004/08/18 07:16:02 vhardy Exp $
+ * @version $Id: JarCheckPermissionsGranted.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class JarCheckPermissionsGranted implements ScriptHandler {
     public static final String svgNS = "http://www.w3.org/2000/svg";
@@ -139,14 +138,9 @@ public class JarCheckPermissionsGranted implements ScriptHandler {
      */
     private Object[][] permissions;
 
-    /**
-     * Reference to the rectangles which show the test status
-     */
-    private Element[] statusRects;
-
     /**
      * Runs this handler.  
-     * @param doc The current document.
+     * @param document The current document.
      * @param win An object which represents the current viewer.
      */
     public void run(final Document document, final Window win){
@@ -154,8 +148,9 @@ public class JarCheckPermissionsGranted implements ScriptHandler {
         // If the document is loaded over the network, check that the
         // class has permission to access the server
         //
-        URL docURL = ((SVGOMDocument)document).getURLObject();
-        if (docURL != null && docURL.getHost() != null && !"".equals(docURL.getHost())) {
+        ParsedURL docURL = ((SVGOMDocument)document).getParsedURL();
+        if (docURL != null && docURL.getHost() != null &&
+                !"".equals(docURL.getHost())) {
             permissions = new Object[basePermissions.length + 3][2];
             System.arraycopy(basePermissions, 0, 
                              permissions, 3, basePermissions.length);
