@@ -1,52 +1,19 @@
 /*
-
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
-
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
-
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ */
 
 package org.apache.batik.apps.rasterizer;
 
@@ -60,14 +27,15 @@ import org.apache.batik.transcoder.image.PNGTranscoder;
 import java.awt.*;
 import java.io.*;
 import java.util.*;
+import java.util.List;
 
 /**
- * Validates the operation of the <tt>SVGRasterizer</tt>.
+ * Validates the operation of the <code>SVGRasterizer</code>.
  * It validates the option setting and the manipulation
  * of source and destination sources.
  *
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: SVGConverterTest.java,v 1.19 2003/08/08 11:39:37 vhardy Exp $
+ * @version $Id: SVGConverterTest.java 1629155 2014-10-03 09:09:09Z vhennebert $
  */
 public class SVGConverterTest extends DefaultTestSuite {
     public SVGConverterTest(){
@@ -105,16 +73,16 @@ public class SVGConverterTest extends DefaultTestSuite {
         //
         // Checks that the proper hints are used
         //
-        t = new HintsConfigTest(new Object[][]{ 
+        t = new HintsConfigTest(new Object[][]{
             {ImageTranscoder.KEY_AOI, new Rectangle(40, 50, 40, 80)}}){
                 protected void deltaConfigure(SVGConverter c){
                     c.setArea(new Rectangle(40, 50, 40, 80));
                 }
             };
-        
+
         addTest(t);
         t.setId("HintsConfigTest.KEY_AOI");
-        
+
         t = new HintsConfigTest(new Object[][]{
             {JPEGTranscoder.KEY_QUALITY, new Float(.5)}}){
                 protected void deltaConfigure(SVGConverter c){
@@ -212,7 +180,8 @@ public class SVGConverterTest extends DefaultTestSuite {
         t.setId("HintsConfigTest.KEY_ALTERNATE_STYLESHEET");
 
         t = new HintsConfigTest(new Object[][]{
-            {ImageTranscoder.KEY_USER_STYLESHEET_URI, "userStylesheet.css"}}){
+            {ImageTranscoder.KEY_USER_STYLESHEET_URI,
+                new File(System.getProperty("user.dir"), "userStylesheet.css").toURI().toString()}}){
                 protected void deltaConfigure(SVGConverter c){
                     c.setUserStylesheet("userStylesheet.css");
                 }
@@ -239,7 +208,7 @@ public class SVGConverterTest extends DefaultTestSuite {
         t.setId("HintsConfigTest.KEY_PIXEL_UNIT_TO_MILLIMETER");
 
         t = new HintsConfigTest(new Object[][]{
-            {ImageTranscoder.KEY_XML_PARSER_VALIDATING, new Boolean(true)}}){
+            {ImageTranscoder.KEY_XML_PARSER_VALIDATING, Boolean.TRUE }}){
                 protected void deltaConfigure(SVGConverter c){
                     c.setValidate(true);
                 }
@@ -256,10 +225,10 @@ public class SVGConverterTest extends DefaultTestSuite {
                     c.setSources(new String[] {"samples/anne.svg", "samples/batikFX.svg", "samples/tests/spec/styling/smiley.svg"});
                 }
             };
-            
+
         addTest(t);
         t.setId("SourcesConfigTest.SimpleList");
-        
+
 
         //
         // Check destination
@@ -304,7 +273,7 @@ public class SVGConverterTest extends DefaultTestSuite {
             };
         addTest(t);
         t.setId("Operationtest.PDFTranscoding");
-        
+
         ///////////////////////////////////////////////////////////////////////
         // Add configuration error test. These tests check that the expected
         // error gets reported for a given mis-configuration
@@ -331,8 +300,8 @@ public class SVGConverterTest extends DefaultTestSuite {
                     File dummy = null;
                     try {
                         dummy = File.createTempFile("dummyPNG", ".png");
-                    } catch(IOException e){
-                        throw new Error();
+                    } catch(IOException ioEx ){
+                        throw new Error( ioEx.getMessage() );
                     }
                     c.setSources(new String[]{"samples/anne.svg", "samples/batikFX.svg"});
                     c.setDst(dummy);
@@ -380,17 +349,17 @@ public class SVGConverterTest extends DefaultTestSuite {
 
                 public boolean proceedWithComputedTask(Transcoder transcoder,
                                                        Map hints,
-                                                       Vector sources,
-                                                       Vector dest){
+                                                       List sources,
+                                                       List dest){
                     System.out.println("==================> Hacked Starting to process Task <=========================");
-                    SVGConverterFileSource source = (SVGConverterFileSource)sources.elementAt(0);
+                    SVGConverterFileSource source = (SVGConverterFileSource)sources.get(0);
                     source = new SVGConverterFileSource(source.file){
                             public InputStream openStream() throws FileNotFoundException {
                                 throw new FileNotFoundException("Simulated FileNotFoundException");
                             }
                         };
 
-                    sources.setElementAt(source, 0);
+                    sources.set(0, source );
                     return true;
                 }
 
@@ -408,10 +377,10 @@ public class SVGConverterTest extends DefaultTestSuite {
             };
         addTest(t);
         t.setId("ConfigErrorTest.ERROR_OUTPUT_NOT_WRITEABLE");
-                   
+
         t = new ConfigErrorTest(SVGConverter.ERROR_UNABLE_TO_CREATE_OUTPUT_DIR){
                 protected void configure(SVGConverter c){
-                    c.setDst(new File("ZYZ::/cannotCreate"));
+                    c.setDst(new File("ZYZ::/cannotCreate\000"));
                 }
             };
         addTest(t);
@@ -424,9 +393,9 @@ public class SVGConverterTest extends DefaultTestSuite {
             };
         addTest(t);
         t.setId("ConfigErrorTest(SVGConverter.ERROR_WHILE_RASTERIZING_FILE");
-        
+
         //
-        // Test that files are created as expected and are producing the 
+        // Test that files are created as expected and are producing the
         // expected result.
         //
 
@@ -450,7 +419,7 @@ public class SVGConverterTest extends DefaultTestSuite {
 /**
  * A ConfigTest builds an SVGConverter, configures it,
  * sets itself as the SVGConverterController and checks that
- * the computed task is as expected (i.e., right set of 
+ * the computed task is as expected (i.e., right set of
  * hints).
  */
 abstract class AbstractConfigTest extends AbstractTest implements SVGConverterController {
@@ -557,9 +526,10 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
     static class Config {
         Class transcoderClass;
         HashMap hints;
-        Vector sources, dest;
+        List sources;
+        List dest;
     }
-            
+
     protected Config expectedConfig;
     protected Config computedConfig;
 
@@ -572,23 +542,23 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
 
     protected abstract void configure(SVGConverter c);
 
-    protected String makeSourceList(Vector v){
+    protected String makeSourceList(List v){
         int n = v.size();
-        StringBuffer sb = new StringBuffer();
+        StringBuffer sb = new StringBuffer( n * 8 );
         for (int i=0; i<n; i++){
-            sb.append(v.elementAt(i).toString());
+            sb.append( v.get(i).toString() );
         }
 
         return sb.toString();
     }
 
-    protected String makeHintsString(HashMap map){
+    protected String makeHintsString( Map map){
         Iterator iter = map.keySet().iterator();
         StringBuffer sb = new StringBuffer();
         while (iter.hasNext()){
             Object key = iter.next();
             sb.append(key.toString());
-            sb.append("(");
+            sb.append( '(' );
             sb.append(map.get(key).toString());
             sb.append(") -- ");
         }
@@ -607,7 +577,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
         c.execute();
 
         //
-        // Now, check that the expectedConfig and the 
+        // Now, check that the expectedConfig and the
         // computedConfig are identical
         //
         if (computedConfig == null){
@@ -628,7 +598,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
         // Compare sources
         int en = expectedConfig.sources.size();
         int cn = computedConfig.sources.size();
-        
+
         if (en != cn){
             TestReport report = reportError(ERROR_UNEXPECTED_SOURCES_LIST);
             report.addDescriptionEntry(ENTRY_KEY_EXPECTED_NUMBER_OF_SOURCES,
@@ -645,8 +615,8 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
         }
 
         for (int i=0; i<en; i++){
-            Object es = expectedConfig.sources.elementAt(i);
-            Object cs = computedConfig.sources.elementAt(i);
+            Object es = expectedConfig.sources.get(i);
+            Object cs = computedConfig.sources.get(i);
             if (!computedConfig.sources.contains(es)){
                 TestReport report = reportError(ERROR_UNEXPECTED_SOURCES_LIST);
                 report.addDescriptionEntry(ENTRY_KEY_EXPECTED_SOURCE_AT_INDEX,
@@ -657,7 +627,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
                                 makeSourceList(expectedConfig.sources));
                 report.addDescriptionEntry(ENTRY_KEY_COMPUTED_SOURCES_LIST,
                                 makeSourceList(computedConfig.sources));
-                
+
                 return report;
             }
         }
@@ -682,8 +652,8 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
         }
 
         for (int i=0; i<en; i++){
-            Object es = expectedConfig.dest.elementAt(i);
-            Object cs = computedConfig.dest.elementAt(i);
+            Object es = expectedConfig.dest.get(i);
+            Object cs = computedConfig.dest.get(i);
             if (!computedConfig.dest.contains(cs)){
                 TestReport report = reportError(ERROR_UNEXPECTED_DEST_LIST);
                 report.addDescriptionEntry(ENTRY_KEY_EXPECTED_DEST_AT_INDEX,
@@ -694,7 +664,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
                                 makeSourceList(expectedConfig.dest));
                 report.addDescriptionEntry(ENTRY_KEY_COMPUTED_DEST_LIST,
                                 makeSourceList(computedConfig.dest));
-                
+
                 return report;
             }
         }
@@ -704,7 +674,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
         //
         en = expectedConfig.hints.size();
         cn = computedConfig.hints.size();
-        
+
         if (en != cn){
             TestReport report = reportError(ERROR_UNEXPECTED_NUMBER_OF_HINTS);
             report.addDescriptionEntry(ENTRY_KEY_EXPECTED_NUMBER_OF_HINTS,
@@ -716,7 +686,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
                                        makeHintsString(expectedConfig.hints));
             report.addDescriptionEntry(ENTRY_KEY_COMPUTED_HINTS,
                                        makeHintsString(computedConfig.hints));
-            
+
             return report;
         }
 
@@ -724,9 +694,9 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
         while (iter.hasNext()){
             Object hintKey = iter.next();
             Object expectedHintValue = expectedConfig.hints.get(hintKey);
-            
+
             Object computedHintValue = computedConfig.hints.get(hintKey);
-            
+
             if (!expectedHintValue.equals(computedHintValue)){
                 TestReport report = reportError(ERROR_UNEXPECTED_TRANSCODING_HINT);
                 report.addDescriptionEntry(ENTRY_KEY_EXPECTED_HINT_KEY,
@@ -743,19 +713,19 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
                 return report;
             }
         }
-                                            
+
         return reportSuccess();
-    
+
     }
 
     public boolean proceedWithComputedTask(Transcoder transcoder,
                                            Map hints,
-                                           Vector sources,
-                                           Vector dest){
+                                           List sources,
+                                           List dest){
         computedConfig = new Config();
         computedConfig.transcoderClass = transcoder.getClass();
-        computedConfig.sources = (Vector)sources.clone();
-        computedConfig.dest = (Vector)dest.clone();
+        computedConfig.sources = new ArrayList( sources );
+        computedConfig.dest = new ArrayList( dest );
         computedConfig.hints = new HashMap(hints);
         return false; // Do not proceed with the convertion process,
         // we are only checking the config in this test.
@@ -765,7 +735,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
                                                 File dest) {
         return true;
     }
-        
+
     public boolean proceedOnSourceTranscodingFailure(SVGConverterSource source,
                                                      File dest,
                                                      String errorCode){
@@ -774,7 +744,7 @@ abstract class AbstractConfigTest extends AbstractTest implements SVGConverterCo
 
     public void onSourceTranscodingSuccess(SVGConverterSource source,
                                            File dest){
-    }    
+    }
 }
 
 /**
@@ -794,8 +764,8 @@ class OperationTest extends AbstractTest{
 }
 
 /**
- * Provides a simple string constructor which allows the user to 
- * create a given test to check that a specific transcoder class is 
+ * Provides a simple string constructor which allows the user to
+ * create a given test to check that a specific transcoder class is
  * used for a given mime type.
  */
 class TranscoderConfigTest extends AbstractConfigTest {
@@ -814,23 +784,23 @@ class TranscoderConfigTest extends AbstractConfigTest {
 
         Config config = new Config();
         config.transcoderClass = expectedTranscoderClass;
-        
-        Vector sources = new Vector();
-        sources.addElement(new SVGConverterFileSource(new File(SOURCE_FILE)));
+
+        List sources = new ArrayList();
+        sources.add(new SVGConverterFileSource(new File(SOURCE_FILE)));
         config.sources = sources;
 
-        Vector dest = new Vector();
-        dest.addElement(new File(DEST_FILE_NAME + dstType.getExtension()));
+        List dest = new ArrayList();
+        dest.add(new File(DEST_FILE_NAME + dstType.getExtension()));
         config.dest = dest;
 
         HashMap hints = new HashMap();
         config.hints = hints;
-                  
+
         setExpectedConfig(config);
     }
 
     /**
-     * Configures the test with the given mime type 
+     * Configures the test with the given mime type
      */
     public void configure(SVGConverter c){
         c.setSources(new String[] { SOURCE_FILE });
@@ -841,7 +811,7 @@ class TranscoderConfigTest extends AbstractConfigTest {
 
 
 /**
- * Provides a simple string array constructor which allows the user to 
+ * Provides a simple string array constructor which allows the user to
  * create a test checking for a specific hint configuration.
  */
 class HintsConfigTest extends AbstractConfigTest {
@@ -855,13 +825,13 @@ class HintsConfigTest extends AbstractConfigTest {
     public HintsConfigTest(Object[][] hintsMap){
         Config config = new Config();
         config.transcoderClass = EXPECTED_TRANSCODER_CLASS;
-        
-        Vector sources = new Vector();
-        sources.addElement(new SVGConverterFileSource(new File(SOURCE_FILE)));
+
+        List sources = new ArrayList();
+        sources.add(new SVGConverterFileSource(new File(SOURCE_FILE)));
         config.sources = sources;
 
-        Vector dest = new Vector();
-        dest.addElement(new File(DEST_FILE_NAME + DST_TYPE.getExtension()));
+        List dest = new ArrayList();
+        dest.add(new File(DEST_FILE_NAME + DST_TYPE.getExtension()));
         config.dest = dest;
 
         HashMap hints = new HashMap();
@@ -874,12 +844,12 @@ class HintsConfigTest extends AbstractConfigTest {
             hints.put(hintsMap[i][0], hintsMap[i][1]);
         }
         config.hints = hints;
-                  
+
         setExpectedConfig(config);
     }
 
     /**
-     * Configures the test with the given mime type 
+     * Configures the test with the given mime type
      */
     public void configure(SVGConverter c){
         c.setSources(new String[] { SOURCE_FILE });
@@ -893,11 +863,11 @@ class HintsConfigTest extends AbstractConfigTest {
 }
 
 /**
- * Provides a simple string array constructor which allows the user to 
+ * Provides a simple string array constructor which allows the user to
  * create a test checking for a specific source configuration.
- * The constructor argument takes the list of expected files and the 
- * deltaConfigure method should set the sources which is expected to 
- * produce that list of sources. The sources should be file names 
+ * The constructor argument takes the list of expected files and the
+ * deltaConfigure method should set the sources which is expected to
+ * produce that list of sources. The sources should be file names
  * which ommit the ".svg" extension.
  */
 class SourcesConfigTest extends AbstractConfigTest {
@@ -910,24 +880,24 @@ class SourcesConfigTest extends AbstractConfigTest {
     public SourcesConfigTest(Object[] expectedSources){
         Config config = new Config();
         config.transcoderClass = EXPECTED_TRANSCODER_CLASS;
-        
-        Vector sources = new Vector();
-        Vector dest = new Vector();
+
+        List sources = new ArrayList();
+        List dest = new ArrayList();
         for (int i=0; i<expectedSources.length; i++){
-            sources.addElement(new SVGConverterFileSource(new File(expectedSources[i] + SVG_EXTENSION)));
-            dest.addElement(new File(expectedSources[i] + DST_TYPE.getExtension()));
+            sources.add(new SVGConverterFileSource(new File(expectedSources[i] + SVG_EXTENSION)));
+            dest.add(new File(expectedSources[i] + DST_TYPE.getExtension()));
         }
         config.sources = sources;
         config.dest = dest;
 
         HashMap hints = new HashMap();
         config.hints = hints;
-                  
+
         setExpectedConfig(config);
     }
 
     /**
-     * Configures the test with the given mime type 
+     * Configures the test with the given mime type
      */
     public void configure(SVGConverter c){
         c.setDestinationType(DST_TYPE);
@@ -939,7 +909,7 @@ class SourcesConfigTest extends AbstractConfigTest {
 }
 
 /**
- * Provides a simple string array constructor which allows the user to 
+ * Provides a simple string array constructor which allows the user to
  * create a test checking for a specific destination configuration.
  * The constructor argument takes the list of sources and the list of
  * expected configuration which is influenced by the 'setDestination'
@@ -956,15 +926,15 @@ class DestConfigTest extends AbstractConfigTest {
         this.sourcesStrings = sourcesStrings;
         Config config = new Config();
         config.transcoderClass = EXPECTED_TRANSCODER_CLASS;
-        
-        Vector sources = new Vector();
-        Vector dest = new Vector();
+
+        List sources = new ArrayList();
+        List dest = new ArrayList();
         for (int i=0; i<sourcesStrings.length; i++){
-            sources.addElement(new SVGConverterFileSource(new File(sourcesStrings[i])));
+            sources.add(new SVGConverterFileSource(new File(sourcesStrings[i])));
         }
 
         for (int i=0; i<expectedDest.length; i++){
-            dest.addElement(new File(expectedDest[i]));
+            dest.add(new File(expectedDest[i]));
         }
 
         config.sources = sources;
@@ -972,12 +942,12 @@ class DestConfigTest extends AbstractConfigTest {
 
         HashMap hints = new HashMap();
         config.hints = hints;
-                  
+
         setExpectedConfig(config);
     }
 
     /**
-     * Configures the test with the given mime type 
+     * Configures the test with the given mime type
      */
     public void configure(SVGConverter c){
         c.setDestinationType(DST_TYPE);
@@ -1059,18 +1029,18 @@ class ConfigErrorTest extends AbstractTest implements SVGConverterController{
 
     public boolean proceedWithComputedTask(Transcoder transcoder,
                                            Map hints,
-                                           Vector sources,
-                                           Vector dest){
+                                           List sources,
+                                           List dest){
         System.out.println("==================> Starting to process Task <=========================");
         return true;
     }
-    
+
     public boolean proceedWithSourceTranscoding(SVGConverterSource source,
                                                 File dest) {
         System.out.print("Transcoding " + source + " to " + dest + " ... ");
         return true;
     }
-    
+
     public boolean proceedOnSourceTranscodingFailure(SVGConverterSource source,
                                                      File dest,
                                                      String errorCode){
@@ -1109,7 +1079,7 @@ class ConverterOutputTest extends AbstractTest {
         c.setDst(new File(pngDest));
 
         c.setDestinationType(DestinationType.PNG);
-        
+
         c.execute();
 
         ImageCompareTest t = new ImageCompareTest(pngDest,
