@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2003  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -28,7 +29,7 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
 
-import org.apache.batik.dom.svg.SVGDOMImplementation;
+import org.apache.batik.anim.dom.SVGDOMImplementation;
 import org.apache.batik.test.AbstractTest;
 import org.apache.batik.test.TestReport;
 
@@ -37,7 +38,7 @@ import org.apache.batik.test.TestReport;
  * as shown by bug report 21259.
  *
  * @author <a href="mailto:vincent.hardy@sun.com">Vincent Hardy</a>
- * @version $Id: Bug21259.java,v 1.4 2004/08/18 07:16:44 vhardy Exp $
+ * @version $Id: Bug21259.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class Bug21259 extends AbstractTest{
     public TestReport runImpl() throws Exception {
