@@ -28,7 +28,7 @@ import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
-import org.apache.batik.dom.svg.SVGDOMImplementation;
+import org.apache.batik.anim.dom.SVGDOMImplementation;
 import org.apache.batik.svggen.SVGGeneratorContext.GraphicContextDefaults;
 import org.apache.batik.util.SVGConstants;
 
@@ -37,7 +37,7 @@ import org.apache.batik.util.SVGConstants;
  * SVG Fonts.
  *
  * @author <a href="mailto:cjolif@ilog.fr">Christophe Jolif</a>
- * @version $Id: GeneratorContext.java 475477 2006-11-15 22:44:28Z cam $
+ * @version $Id: GeneratorContext.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class GeneratorContext extends SVGAccuracyTest implements SVGConstants {
     public static class TestIDGenerator extends SVGIDGenerator {
