@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2001,2003  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -30,7 +31,7 @@ import javax.swing.JPanel;
 import org.w3c.dom.DOMImplementation;
 import org.w3c.dom.Document;
 
-import org.apache.batik.dom.svg.SVGDOMImplementation;
+import org.apache.batik.anim.dom.SVGDOMImplementation;
 import org.apache.batik.swing.JSVGCanvas;
 import org.apache.batik.swing.svg.SVGDocumentLoaderAdapter;
 import org.apache.batik.swing.svg.SVGDocumentLoaderEvent;
@@ -38,13 +39,13 @@ import org.apache.batik.util.SVGConstants;
 
 /**
  * Simple component which displays, side by side, the drawing
- * created by a <tt>Painter</tt>, rendered in a 
- * <tt>JPainterComponent</tt> on the left, and in a 
- * <tt>JSVGCanvas</tt> on the right, where the SVG
- * displayed is the one created by the <tt>SVGGraphics2D</tt>
+ * created by a <code>Painter</code>, rendered in a
+ * <code>JPainterComponent</code> on the left, and in a
+ * <code>JSVGCanvas</code> on the right, where the SVG
+ * displayed is the one created by the <code>SVGGraphics2D</code>
  *
  * @author <a href="mailto:vincent.hardy@sun.com">Vincent Hardy</a>
- * @version $Id: JPainterCompare.java,v 1.5 2004/08/18 07:16:45 vhardy Exp $
+ * @version $Id: JPainterCompare.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class JPainterCompare extends JPanel implements SVGConstants{
     /**
@@ -53,7 +54,7 @@ public class JPainterCompare extends JPanel implements SVGConstants{
     public static final Dimension CANVAS_SIZE
         = new Dimension(300, 400);
 
-    public static String MESSAGES_USAGE 
+    public static String MESSAGES_USAGE
         = "JPainterCompare.messages.usage";
 
     public static String MESSAGES_LOADING_CLASS
@@ -74,7 +75,7 @@ public class JPainterCompare extends JPanel implements SVGConstants{
     public static String ERROR_CLASS_NOT_PAINTER
         = "JPainterCompare.error.class.not.painter";
 
-    public static String ERROR_COULD_NOT_TRANSCODE_TO_SVG 
+    public static String ERROR_COULD_NOT_TRANSCODE_TO_SVG
         = "JPainterCompare.error.could.not.transcode.to.svg";
 
     public static String ERROR_COULD_NOT_CONVERT_FILE_PATH_TO_URL
@@ -87,7 +88,7 @@ public class JPainterCompare extends JPanel implements SVGConstants{
         = "JPainterCompare.config.tmp.file.prefix";
 
     /**
-     * Builds an <tt>SVGGraphics2D</tt> with a default
+     * Builds an <code>SVGGraphics2D</code> with a default
      * configuration.
      */
     protected SVGGraphics2D buildSVGGraphics2D() {
@@ -112,7 +113,7 @@ public class JPainterCompare extends JPanel implements SVGConstants{
                 sem.notifyAll();
             }
         }
-        
+
         public void documentLoadingCompleted(SVGDocumentLoaderEvent e){
             success = true;
             synchronized(sem){
@@ -140,7 +141,7 @@ public class JPainterCompare extends JPanel implements SVGConstants{
         try{
             tmpFile = File.createTempFile(CONFIG_TMP_FILE_PREFIX,
                                           ".svg");
-            
+
             OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream(tmpFile), "UTF-8");
 
             painter.paint(g2d);
@@ -152,7 +153,7 @@ public class JPainterCompare extends JPanel implements SVGConstants{
                 (Messages.formatMessage(ERROR_COULD_NOT_TRANSCODE_TO_SVG,
                                         new Object[]{e.getClass().getName()}));
         }
-        
+
         //
         // Now, transcode SVG to a BufferedImage
         //
@@ -189,13 +190,13 @@ public class JPainterCompare extends JPanel implements SVGConstants{
     }
 
     /*
-     * Debug application: shows the image creatd by a <tt>Painter</tt>
-     * on the left and the image created by a <tt>JSVGComponent</tt>
-     * from the SVG generated by <tt>SVGGraphics2D</tt> from the same
-     * <tt>Painter</tt> on the right.
+     * Debug application: shows the image creatd by a <code>Painter</code>
+     * on the left and the image created by a <code>JSVGComponent</code>
+     * from the SVG generated by <code>SVGGraphics2D</code> from the same
+     * <code>Painter</code> on the right.
      *
      */
-    public static void main(String args[]){
+    public static void main(String[] args){
         if(args.length <= 0){
             System.out.println(Messages.formatMessage
                                (MESSAGES_USAGE, null));
