@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2001  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -25,8 +26,8 @@ import java.awt.Graphics2D;
  * compare it to a reference.
  *
  * @author <a href="mailto:vhardy@eng.sun.com">Vincent Hardy</a>
- * @version $Id: Painter.java,v 1.4 2005/03/27 08:58:37 cam Exp $
+ * @version $Id: Painter.java 482118 2006-12-04 09:52:54Z dvholten $
  */
 public interface Painter {
-    public void paint(Graphics2D g);
+    void paint(Graphics2D g);
 }
