@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2001-2003  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -23,24 +24,24 @@ import java.io.PrintWriter;
 /**
  * Base class containing convenience methods for writing tests. <br />
  * There are at least three approaches to write new tests derived from
- * <tt>AbstractTest</tt>:<br /><ul>
- * <li>You can simply override the <tt>runImplBase</tt> method and 
+ * <code>AbstractTest</code>:<br /><ul>
+ * <li>You can simply override the <code>runImplBasic</code> method and 
  * return true or false depending on whether or not the test fails.</li>
  * <li>You can choose to report more complex test failure conditions 
- * by overriding the <tt>runImpl</tt> method which returns a <tt>TestReport</tt>.
- * In that case, you can use the convenience methods such as <tt>reportFailure</tt>
- * <tt>reportSuccess</tt> or <tt>reportException</tt> to help build a <tt>TestReport</tt>,
- * and use the <tt>TestReport</tt>'s <tt>addDescriptionEntry</tt> to populate
+ * by overriding the <code>runImpl</code> method which returns a <code>TestReport</code>.
+ * In that case, you can use the convenience methods such as <code>reportFailure</code>
+ * <code>reportSuccess</code> or <code>reportException</code> to help build a <code>TestReport</code>,
+ * and use the <code>TestReport</code>'s <code>addDescriptionEntry</code> to populate
  * the report with relevant error description.</li>
- * <li>You can choose to use the various assertion methods such as <tt>assertNull</tt>,
- * <tt>assertEquals</tt> or <tt>assertTrue</tt>. These methods throw exceptions which
- * will be turned in <tt>TestReports</tt> by the <tt>AbstractTest</tt>.</li>
+ * <li>You can choose to use the various assertion methods such as <code>assertNull</code>,
+ * <code>assertEquals</code> or <code>assertTrue</code>. These methods throw exceptions which
+ * will be turned in <code>TestReports</code> by the <code>AbstractTest</code>.</li>
  * </ul>
  * 
  * Here are some examples:
  * <code>
  * public class MyTestA extends AbstractTest {
- * public boolean runImplBase() {
+ * public boolean runImplBasic() {
  *    if(someConditionFails){
  *       return false;
  *    }
@@ -79,7 +80,7 @@ import java.io.PrintWriter;
  * </code>
  *
  * @author <a href="mailto:vhardy@apache.lorg">Vincent Hardy</a>
- * @version $Id: AbstractTest.java,v 1.12 2004/08/18 07:16:55 vhardy Exp $
+ * @version $Id: AbstractTest.java 1372129 2012-08-12 15:31:50Z helder $
  */
 public abstract class AbstractTest implements Test {
     /**
@@ -110,7 +111,7 @@ public abstract class AbstractTest implements Test {
             };
     
     /**
-     * Returns this <tt>Test</tt>'s name. 
+     * Returns this <code>Test</code>'s name. 
      */
     public String getName(){
         if(name == null){
@@ -132,14 +133,14 @@ public abstract class AbstractTest implements Test {
     }
 
     /**
-     * Return this <tt>Test</tt>'s id.
+     * Return this <code>Test</code>'s id.
      */
     public String getId(){
         return id;
     }
 
     /**
-     * Return this <tt>Test</tt>'s qualified id.
+     * Return this <code>Test</code>'s qualified id.
      */
     public String getQualifiedId(){
         if(parent == null){
@@ -149,7 +150,7 @@ public abstract class AbstractTest implements Test {
     }
 
     /**
-     * Set this <tt>Test</tt>'s id. Null is not allowed.
+     * Set this <code>Test</code>'s id. Null is not allowed.
      */
     public void setId(String id){
         if(id == null){
@@ -170,11 +171,11 @@ public abstract class AbstractTest implements Test {
     /**
      * This default implementation of the run method
      * catches any Exception thrown from the 
-     * runImpl method and creates a <tt>TestReport</tt>
-     * indicating an internal <tt>Test</tt> failure
+     * runImpl method and creates a <code>TestReport</code>
+     * indicating an internal <code>Test</code> failure
      * when that happens. Otherwise, this method
-     * simply returns the <tt>TestReport</tt> generated
-     * by the <tt>runImpl</tt> method.
+     * simply returns the <code>TestReport</code> generated
+     * by the <code>runImpl</code> method.
      */
     public TestReport run(){
         try{
@@ -206,17 +207,15 @@ public abstract class AbstractTest implements Test {
 
             }catch(Exception ex){
                 ex.printStackTrace();
-            }finally {
-                //
-                // In case we are in severe trouble, even filling in the 
-                // TestReport may fail. Because the TestReport instance
-                // was created up-front, this ensures we can return 
-                // the report, even though it may be incomplete.
-                e.printStackTrace();
-                System.out.println("SERIOUS ERROR");
-                return report;
             }
-                
+
+            // In case we are in severe trouble, even filling in the 
+            // TestReport may fail. Because the TestReport instance
+            // was created up-front, this ensures we can return 
+            // the report, even though it may be incomplete.
+            e.printStackTrace();
+            System.out.println("SERIOUS ERROR");
+            return report;
         }
     }
 
@@ -314,9 +313,9 @@ public abstract class AbstractTest implements Test {
 
     /**
      * Convenience method to help implementations report errors.
-     * An <tt>AbstractTest</tt> extension will typically catch 
+     * An <code>AbstractTest</code> extension will typically catch 
      * exceptions for specific error conditions it wants to point 
-     * out. For example:<tt>
+     * out. For example:<code>
      * public TestReport runImpl() throws Exception { <br />
      *   try{ <br />
      *      .... something .... <br />
@@ -326,11 +325,11 @@ public abstract class AbstractTest implements Test {
      * <br />
      * public static final String MY_SPECIAL_ERROR_CODE = "myNonQualifiedClassName.my.error.code" <br />
      * <br />
-     * </tt> <br />
+     * </code> <br />
      * Note that the implementor will also need to add an entry
      * in its Messages.properties file. That file is expected to be 
-     * in a resource file called <tt>Messages</tt> having the same package 
-     * name as the <tt>Test</tt> class, appended with "<tt>.resources</tt>".
+     * in a resource file called <code>Messages</code> having the same package 
+     * name as the <code>Test</code> class, appended with "<code>.resources</code>".
      */
     public TestReport reportException(String errorCode,
                                       Exception e){
