@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2003-2004  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -35,7 +36,7 @@ import org.apache.batik.util.CleanerThread;
  * Complete Class Desc
  *
  * @author <a href="mailto:deweese@apache.org">l449433</a>
- * @version $Id: MemoryLeakTest.java,v 1.9 2005/03/27 08:58:37 cam Exp $
+ * @version $Id: MemoryLeakTest.java 502489 2007-02-02 04:57:56Z cam $
  */
 public abstract class MemoryLeakTest  extends AbstractTest {
 
@@ -45,21 +46,23 @@ public abstract class MemoryLeakTest  extends AbstractTest {
     // (why so large I don't know) - it will bail if the all
     // the objects of interest are collected sooner so the runtime
     // is really only a concern for failures.
-    final static int NUM_GC=60;
+    static final int NUM_GC=10;
+    static final int MIN_MEMORY=200000; // 200KB
+    static final int ALLOC_SZ=1000; // 100KB
 
-    final static String ERROR_OBJS_NOT_CLEARED = 
+    static final String ERROR_OBJS_NOT_CLEARED =
         "MemoryLeakTest.message.error.objs.not.cleared";
 
-    final static String ERROR_DESCRIPTION = 
+    static final String ERROR_DESCRIPTION =
         "TestReport.entry.key.error.description";
-        
+
     public static String fmt(String key, Object []args) {
         return Messages.formatMessage(key, args);
     }
 
     public MemoryLeakTest() {
     }
-    
+
     Map objs = new HashMap();
     List entries = new ArrayList();
 
@@ -74,7 +77,7 @@ public abstract class MemoryLeakTest  extends AbstractTest {
             objs.put(desc, new WeakRef(o, desc));
         }
     }
-    
+
     public boolean checkObject(String desc) {
         String [] strs = new String[1];
         strs[0] = desc;
@@ -82,48 +85,70 @@ public abstract class MemoryLeakTest  extends AbstractTest {
     }
 
     public boolean checkObjects(String [] descs) {
-        for (int i=0; i<NUM_GC; i++) {
-            System.gc();
-            boolean passed = true;
-            for (int j=0; j< descs.length; j++) {
-                String desc = descs[j];
-                WeakRef wr = (WeakRef)objs.get(desc);
-                if ((wr != null) && (wr.get() != null)) {
-                    passed = false;
-                    break;
+        Runtime rt = Runtime.getRuntime();
+        List l = new ArrayList();
+        int nBlock = (int)(rt.totalMemory()/(ALLOC_SZ*NUM_GC));
+        try {
+            while (true) {
+                boolean passed = true;
+                synchronized (objs) {
+                    // System.err.println("FreeMemory: " + rt.freeMemory() +
+                    //                    " of " +rt.totalMemory());
+
+                    for (int i=0; i<descs.length; i++) {
+                        String desc = descs[i];
+                        WeakRef wr = (WeakRef)objs.get(desc);
+                        if ((wr == null) || (wr.get() == null)) continue;
+                        passed = false;
+                        break;
+                    }
                 }
+                if (passed) return true;
+
+                List l2 = new ArrayList();
+                for (int i=0; i<nBlock; i++) {
+                    l2.add(new byte[ALLOC_SZ]);
+                }
+                l.add(l2);
             }
-            if (passed) return true;
-            Thread.yield();
+        } catch (OutOfMemoryError oom) {
+        } finally {
+            l = null;
         }
 
+        for (int i=0; i<NUM_GC; i++)
+            rt.gc();
+
         StringBuffer sb = new StringBuffer();
+        boolean passed = true;
         synchronized (objs) {
-            boolean passed = true;
-            for (int j=0; j< descs.length; j++) {
-                String desc = descs[j];
+            for (int i=0; i<descs.length; i++) {
+                String desc = descs[i];
                 WeakRef wr = (WeakRef)objs.get(desc);
-                if ((wr == null) || (wr.get() == null)) continue;
+                if (wr == null) continue;
+                Object o = wr.get();
+                if (o == null) continue;
                 if (!passed)
-                    sb.append(","); // Already put one obj out
+                    sb.append(",");
                 passed = false;
                 sb.append("'");
                 sb.append(wr.getDesc());
                 sb.append("'");
             }
+            if (passed) return true;
         }
-        
+
         String objStr = sb.toString();
         TestReport.Entry entry = new TestReport.Entry
             (fmt(ERROR_DESCRIPTION, null),
              fmt(ERROR_OBJS_NOT_CLEARED, new Object[]{objStr}));
         entries.add(entry);
 
-        if (objStr.length() > 40) 
+        if (objStr.length() > 40)
             objStr = objStr.substring(0,40) + "..." ;
         System.err.println(">>>>> Objects not cleared: " + objStr);
-        // System.err.println("Waiting for heap dump...");
-        // try { Thread.sleep(60000); } catch (InterruptedException ie) { }
+        // System.err.println("Waiting 5 second for heap dump...");
+        // try { Thread.sleep(5000); } catch (InterruptedException ie) { }
         return false;
     }
 
@@ -134,23 +159,42 @@ public abstract class MemoryLeakTest  extends AbstractTest {
     }
 
     public boolean checkAllObjects() {
-        for (int i=0; i<NUM_GC; i++) {
-            System.gc();
-            synchronized (objs) {
+        Runtime rt = Runtime.getRuntime();
+        List l = new ArrayList();
+        int nBlock = (int)(rt.totalMemory()/(ALLOC_SZ*NUM_GC));
+        try {
+            while (true) {
+                // System.err.println("FreeMemory: " + rt.freeMemory() +
+                //                    " of " +rt.totalMemory());
+
                 boolean passed = true;
-                Iterator iter = objs.values().iterator();
-                while (iter.hasNext()) {
-                    WeakRef wr = (WeakRef)iter.next();
-                    Object o = wr.get();
-                    if (o != null) {
-                        passed = false;
-                        break;
+                synchronized (objs) {
+                    Iterator iter = objs.values().iterator();
+                    while (iter.hasNext()) {
+                        WeakRef wr = (WeakRef)iter.next();
+                        if ((wr != null) && (wr.get() != null)) {
+                            passed = false;
+                            break;
+                        }
                     }
                 }
                 if (passed) return true;
+
+                List l2 = new ArrayList();
+                for (int i=0; i<nBlock; i++) {
+                    l2.add(new byte[ALLOC_SZ]);
+                }
+                l.add(l2);
             }
+        } catch (OutOfMemoryError oom) {
+        } finally {
+            l = null;
         }
-        
+
+        for (int i=0; i<NUM_GC; i++)
+            rt.gc();
+
+
         StringBuffer sb = new StringBuffer();
         synchronized (objs) {
             boolean passed = true;
@@ -168,7 +212,7 @@ public abstract class MemoryLeakTest  extends AbstractTest {
             }
             if (passed) return true;
         }
-        
+
         String objStr = sb.toString();
 
         TestReport.Entry entry = new TestReport.Entry
@@ -176,11 +220,11 @@ public abstract class MemoryLeakTest  extends AbstractTest {
              fmt(ERROR_OBJS_NOT_CLEARED, new Object[]{objStr}));
         entries.add(entry);
 
-        if (objStr.length() > 40) 
+        if (objStr.length() > 40)
             objStr = objStr.substring(0,40) + "..." ;
         System.err.println(">>>>> Objects not cleared: " + objStr);
-        // System.err.println("Waiting for heap dump...");
-        // try { Thread.sleep(60000); } catch (InterruptedException ie) { }
+        // System.err.println("Waiting for 5 seconds for heap dump...");
+        // try { Thread.sleep(5000); } catch (InterruptedException ie) { }
         return false;
     }
 
@@ -224,7 +268,7 @@ public abstract class MemoryLeakTest  extends AbstractTest {
                 objs.remove(desc);
             }
         }
-        
+
     }
-    
+
 }
