@@ -59,13 +59,13 @@ import org.apache.batik.test.TestReport;
 
 /**
  * Checks for regressions in rendering a specific SVG document.
- * The <tt>Test</tt> will rasterize and SVG document and
+ * The <code>Test</code> will rasterize and SVG document and
  * compare it to a reference image. The test passes if the
  * rasterized SVG and the reference image match exactly (i.e.,
  * all pixel values are the same).
  *
  * @author <a href="mailto:vhardy@apache.lorg">Vincent Hardy</a>
- * @version $Id: AbstractRenderingAccuracyTest.java 581141 2007-10-02 05:36:50Z cam $
+ * @version $Id: AbstractRenderingAccuracyTest.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     /**
@@ -349,7 +349,11 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
         if (this.variationURLs == null) {
             this.variationURLs = new LinkedList();
         }
-        this.variationURLs.add(resolveURL(variationURL));
+        try {
+            this.variationURLs.add(resolveURL(variationURL));
+        } catch (IllegalArgumentException iae) {
+            // no worries
+        }
     }
 
     /**
@@ -364,7 +368,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     }
 
     /**
-     * Returns this <tt>Test</tt>'s name. The name is the
+     * Returns this <code>Test</code>'s name. The name is the
      * URL of the SVG being rendered.
      */
     public String getName(){
@@ -375,7 +379,7 @@ public abstract class AbstractRenderingAccuracyTest extends AbstractTest {
     }
 
     /**
-     * Requests this <tt>Test</tt> to run and produce a
+     * Requests this <code>Test</code> to run and produce a
      * report.
      *
      */
