@@ -1,61 +1,41 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.test.svg;
 
 import java.io.File;
+import java.io.FilePermission;
 import java.net.MalformedURLException;
 import java.net.URL;
+import java.security.AccessControlContext;
+import java.security.AccessController;
+import java.security.CodeSource;
+import java.security.Permission;
+import java.security.PermissionCollection;
+import java.security.Permissions;
+import java.security.Policy;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
+import java.security.ProtectionDomain;
+import java.security.cert.Certificate;
+import java.util.Enumeration;
 
-import org.w3c.dom.Document;
-
+import org.apache.batik.anim.dom.SAXSVGDocumentFactory;
 import org.apache.batik.bridge.BaseScriptingEnvironment;
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.BridgeException;
@@ -70,24 +50,26 @@ import org.apache.batik.bridge.NoLoadScriptSecurity;
 import org.apache.batik.bridge.RelaxedExternalResourceSecurity;
 import org.apache.batik.bridge.RelaxedScriptSecurity;
 import org.apache.batik.bridge.ScriptSecurity;
-import org.apache.batik.bridge.UserAgent;
 import org.apache.batik.bridge.UserAgentAdapter;
-import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
 import org.apache.batik.test.AbstractTest;
 import org.apache.batik.test.TestReport;
+import org.apache.batik.util.ApplicationSecurityEnforcer;
 import org.apache.batik.util.ParsedURL;
 import org.apache.batik.util.XMLResourceDescriptor;
 
+import org.w3c.dom.Document;
+import org.w3c.dom.Element;
+
 /**
  * This test takes an SVG file as an input. It processes the input SVG
  * (meaning it turns it into a GVT tree) and then dispatches the 'onload'
  * event.
- * 
+ *
  * In that process, the test checks for the occurence of a specific
  * exception type and, for BridgeExceptions, for a given error code.
  *
  * If an exception of the given type (and, optionally, code) happens,
- * then the test passes. If an exception of an unexpected type 
+ * then the test passes. If an exception of an unexpected type
  * (or code, for BridgeExceptions) happens, or if no exception happens,
  * the test fails.
  *
@@ -100,9 +82,14 @@ import org.apache.batik.util.XMLResourceDescriptor;
  * - Validate (e.g., "true")
  *
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: SVGOnLoadExceptionTest.java,v 1.4 2003/08/09 16:58:43 deweese Exp $
+ * @version $Id: SVGOnLoadExceptionTest.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class SVGOnLoadExceptionTest extends AbstractTest {
+    /**
+     * Value for the script having successfully run.
+     */
+    public static final String RAN = "ran";
+
     /**
      * Error when the expected exception did not occur
      */
@@ -123,6 +110,12 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
     public static final String ERROR_UNEXPECTED_ERROR_CODE
         = "SVGOnLoadExceptionTest.error.unexpected.error.code";
 
+    /**
+     * Error when the script does not run as expected.
+     */
+    public static final String ERROR_SCRIPT_DID_NOT_RUN
+        = "SVGOnLoadExceptionTest.error.script.did.not.run";
+
     /**
      * Entry describing the unexpected exception
      */
@@ -147,6 +140,12 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
     public static final String ENTRY_KEY_EXPECTED_EXCEPTION
         = "SVGOnLoadExceptionTest.entry.key.expected.exception";
 
+    /**
+     * Entry describing the unexpected exception
+     */
+    public static final String ENTRY_KEY_UNEXPECTED_RESULT
+        = "SVGOnLoadExceptionTest.entry.key.unexpected.result";
+
     /**
      * Value used to disable error code check on BridgeExceptions
      */
@@ -167,7 +166,7 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
      * The allowed script types
      */
     protected String scripts = "text/ecmascript, application/java-archive";
-    
+
     /**
      * Name of the expected exception class
      */
@@ -188,10 +187,34 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
      */
     protected String resourceOrigin = "ANY";
 
+    /**
+     * True if the scripts are run securely (i.e., with a security manager)
+     */
+    protected boolean secure = false;
+
     /**
      * Controls whether or not the input SVG document should be validated
      */
-    protected Boolean validate = new Boolean(false);
+    protected Boolean validate = Boolean.FALSE;
+
+    /**
+     * The name of the test file
+     */
+    protected String fileName;
+
+    /**
+     * Controls whether on not the document should be processed from
+     * a 'restricted' context, one with no createClassLoader permission.
+     */
+    protected boolean restricted = false;
+
+    public boolean getRestricted() {
+        return restricted;
+    }
+
+    public void setRestricted(boolean restricted) {
+        this.restricted = restricted;
+    }
 
     public void setScripts(String scripts){
         this.scripts = scripts;
@@ -217,6 +240,14 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
         return this.resourceOrigin;
     }
 
+    public void setSecure(boolean secure){
+        this.secure = secure;
+    }
+
+    public boolean getSecure(){
+        return secure;
+    }
+
     public void setExpectedExceptionClass(String expectedExceptionClass){
         this.expectedExceptionClass = expectedExceptionClass;
     }
@@ -240,10 +271,10 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
     public void setValidate(Boolean validate) {
         this.validate = validate;
         if (this.validate == null) {
-            this.validate = new Boolean(false);
+            this.validate = Boolean.FALSE;
         }
     }
-    
+
     /**
      * Default constructor
      */
@@ -252,7 +283,15 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
 
     public void setId(String id){
         super.setId(id);
-        svgURL = resolveURL("test-resources/org/apache/batik/" + id + ".svg");
+
+        if (id != null) {
+            int i = id.indexOf("(");
+            if (i != -1) {
+                id = id.substring(0, i);
+            }
+            fileName = "test-resources/org/apache/batik/" + id + ".svg";
+            svgURL = resolveURL(fileName);
+        }
     }
 
     /**
@@ -273,7 +312,7 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
                 throw new IllegalArgumentException();
             }
         }
-        
+
         // url is not a file. It must be a regular URL...
         try{
             return (new URL(url)).toString();
@@ -287,14 +326,77 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
      * Run this test and produce a report.
      * The test goes through the following steps: <ul>
      * <li>load the input SVG into a Document</li>
-     * <li>build the GVT tree corresponding to the 
+     * <li>build the GVT tree corresponding to the
      *     Document and dispatch the 'onload' event</li>
      * </ul>
      *
      */
     public TestReport runImpl() throws Exception{
+        ApplicationSecurityEnforcer ase
+            = new ApplicationSecurityEnforcer(this.getClass(),
+                                              "org/apache/batik/apps/svgbrowser/resources/svgbrowser.policy");
+
+        if (secure) {
+            ase.enforceSecurity(true);
+        }
+
+        try {
+            if (!restricted) {
+                return testImpl();
+            } else {
+                // Emulate calling from restricted code. We create a
+                // calling context with only the permission to read
+                // the file.
+                Policy policy = Policy.getPolicy();
+                URL classesURL = (new File("classes")).toURL();
+                CodeSource cs = new CodeSource(classesURL, (Certificate[])null);
+                PermissionCollection permissionsOrig
+                    = policy.getPermissions(cs);
+                Permissions permissions = new Permissions();
+                Enumeration iter = permissionsOrig.elements();
+                while (iter.hasMoreElements()) {
+                    Permission p = (Permission)iter.nextElement();
+                    if (!(p instanceof RuntimePermission)) {
+                        if (!(p instanceof java.security.AllPermission)) {
+                            permissions.add(p);
+                        }
+                    } else {
+                        if (!"createClassLoader".equals(p.getName())) {
+                            permissions.add(p);
+                        }
+                    }
+                }
+
+                permissions.add(new FilePermission(fileName, "read"));
+                permissions.add(new RuntimePermission("accessDeclaredMembers"));
+
+                ProtectionDomain domain;
+                AccessControlContext ctx;
+                domain = new ProtectionDomain(null, permissions);
+                ctx = new AccessControlContext(new ProtectionDomain[]{domain});
+
+                try {
+                    return (TestReport)AccessController.doPrivileged
+                        (new PrivilegedExceptionAction() {
+                                public Object run() throws Exception {
+                                    return testImpl();
+                                }
+                            }, ctx);
+                } catch (PrivilegedActionException pae) {
+                    throw pae.getException();
+                }
+            }
+        } finally {
+            ase.enforceSecurity(false);
+        }
+    }
+
+    /**
+     * Implementation helper
+     */
+    protected TestReport testImpl() {
         //
-        // First step: 
+        // First step:
         //
         // Load the input SVG into a Document object
         //
@@ -306,46 +408,69 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
         try {
             doc = f.createDocument(svgURL);
         } catch(Exception e){
+            e.printStackTrace();
             return handleException(e);
-        } 
+        }
 
         //
         // Second step:
-        // 
+        //
         // Now that the SVG file has been loaded, build
         // a GVT Tree from it
         //
-        UserAgent userAgent = buildUserAgent();
+        TestUserAgent userAgent = buildUserAgent();
         GVTBuilder builder = new GVTBuilder();
         BridgeContext ctx = new BridgeContext(userAgent);
         ctx.setDynamic(true);
-
+        Exception e = null;
         try {
             builder.build(ctx, doc);
-            BaseScriptingEnvironment scriptEnvironment 
+            BaseScriptingEnvironment scriptEnvironment
                 = new BaseScriptingEnvironment(ctx);
             scriptEnvironment.loadScripts();
             scriptEnvironment.dispatchSVGLoadEvent();
-        } catch (Exception e){
-            return handleException(e);
-        } 
+        } catch (Exception ex){
+            e = ex;
+        } finally {
+            if (e == null && userAgent.e != null) {
+                e = userAgent.e;
+            }
+
+            if (e != null) {
+                return handleException(e);
+            }
+        }
 
         //
-        // If we got here, it means that the expected exception did not
-        // happen. Report an error
-        //
-        TestReport report = reportError(ERROR_EXCEPTION_DID_NOT_OCCUR);
-        report.addDescriptionEntry(ENTRY_KEY_EXPECTED_EXCEPTION,
-                                   expectedExceptionClass);
+        // If we got here, it means that an exception did not
+        // happen. Check if this is expected.
+        TestReport report = null;
+        if (expectedExceptionClass == null) {
+            // No error was expected then check that the script ran.
+            Element elem = doc.getElementById("testResult");
+            String s = elem.getAttributeNS(null, "result");
+            if (RAN.equals(s)) {
+                report = reportSuccess();
+            } else {
+                report = reportError(ERROR_SCRIPT_DID_NOT_RUN);
+                report.addDescriptionEntry(ENTRY_KEY_UNEXPECTED_RESULT,
+                                           s);
+            }
+        }
+        if (report == null) {
+            report = reportError(ERROR_EXCEPTION_DID_NOT_OCCUR);
+            report.addDescriptionEntry(ENTRY_KEY_EXPECTED_EXCEPTION,
+                                       expectedExceptionClass);
+        }
         return report;
     }
 
-    /** 
+    /**
      * Compares the input exception with the expected exception
      * If they match, then the test passes. Otherwise, the test fails
      */
     protected TestReport handleException(Exception e) {
-        if (!e.getClass().getName().equals(expectedExceptionClass)) {
+        if (!isMatch(e.getClass(), expectedExceptionClass)) {
             TestReport report = reportError(ERROR_UNEXPECTED_EXCEPTION);
             report.addDescriptionEntry(ENTRY_KEY_UNEXPECTED_EXCEPTION,
                                        e.getClass().getName());
@@ -368,15 +493,31 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
         }
     }
 
+    /**
+     * Check if the input class' name (or one of its base classes) matches
+     * the input name.
+     */
+    protected boolean isMatch(final Class cl, final String name) {
+        if (cl == null) {
+            return false;
+        } else if (cl.getName().equals(name)) {
+            return true;
+        } else {
+            return isMatch(cl.getSuperclass(), name);
+        }
+    }
+
     /**
      * Give subclasses a chance to build their own UserAgent
      */
-    protected UserAgent buildUserAgent(){
+    protected TestUserAgent buildUserAgent(){
         return new TestUserAgent();
     }
 
     class TestUserAgent extends UserAgentAdapter {
-        public ExternalResourceSecurity 
+        Exception e;
+
+        public ExternalResourceSecurity
             getExternalResourceSecurity(ParsedURL resourceURL,
                                         ParsedURL docURL) {
             if ("ANY".equals(resourceOrigin)) {
@@ -396,25 +537,31 @@ public class SVGOnLoadExceptionTest extends AbstractTest {
             getScriptSecurity(String scriptType,
                               ParsedURL scriptURL,
                               ParsedURL docURL) {
+            ScriptSecurity result = null;
             if (scripts.indexOf(scriptType) == -1) {
-                return new NoLoadScriptSecurity(scriptType);
+                result = new NoLoadScriptSecurity(scriptType);
             } else {
                 if ("ANY".equals(scriptOrigin)) {
-                    return new RelaxedScriptSecurity(scriptType,
+                    result = new RelaxedScriptSecurity(scriptType,
                                                      scriptURL,
                                                      docURL);
-                } else if ("DOCUMENT".equals(resourceOrigin)) {
-                    return new DefaultScriptSecurity(scriptType,
+                } else if ("DOCUMENT".equals(scriptOrigin)) {
+                    result = new DefaultScriptSecurity(scriptType,
                                                      scriptURL,
                                                      docURL);
-                } else if ("EMBEDED".equals(resourceOrigin)) {
-                    return new EmbededScriptSecurity(scriptType,
+                } else if ("EMBEDED".equals(scriptOrigin)) {
+                    result = new EmbededScriptSecurity(scriptType,
                                                      scriptURL,
                                                      docURL);
                 } else {
-                    return new NoLoadScriptSecurity(scriptType);
+                    result = new NoLoadScriptSecurity(scriptType);
                 }
             }
+            return result;
+        }
+
+        public void displayError(Exception e) {
+            this.e = e;
         }
     }
 
