@@ -1,53 +1,21 @@
 /*
 
- ============================================================================
-                   The Apache Software License, Version 1.1
- ============================================================================
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
- Copyright (C) 1999-2003 The Apache Software Foundation. All rights reserved.
+       http://www.apache.org/licenses/LICENSE-2.0
 
- Redistribution and use in source and binary forms, with or without modifica-
- tion, are permitted provided that the following conditions are met:
-
- 1. Redistributions of  source code must  retain the above copyright  notice,
-    this list of conditions and the following disclaimer.
-
- 2. Redistributions in binary form must reproduce the above copyright notice,
-    this list of conditions and the following disclaimer in the documentation
-    and/or other materials provided with the distribution.
-
- 3. The end-user documentation included with the redistribution, if any, must
-    include  the following  acknowledgment:  "This product includes  software
-    developed  by the  Apache Software Foundation  (http://www.apache.org/)."
-    Alternately, this  acknowledgment may  appear in the software itself,  if
-    and wherever such third-party acknowledgments normally appear.
-
- 4. The names "Batik" and  "Apache Software Foundation" must  not  be
-    used to  endorse or promote  products derived from  this software without
-    prior written permission. For written permission, please contact
-    apache@apache.org.
-
- 5. Products  derived from this software may not  be called "Apache", nor may
-    "Apache" appear  in their name,  without prior written permission  of the
-    Apache Software Foundation.
-
- THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES,
- INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- FITNESS  FOR A PARTICULAR  PURPOSE ARE  DISCLAIMED.  IN NO  EVENT SHALL  THE
- APACHE SOFTWARE  FOUNDATION  OR ITS CONTRIBUTORS  BE LIABLE FOR  ANY DIRECT,
- INDIRECT, INCIDENTAL, SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL  DAMAGES (INCLU-
- DING, BUT NOT LIMITED TO, PROCUREMENT  OF SUBSTITUTE GOODS OR SERVICES; LOSS
- OF USE, DATA, OR  PROFITS; OR BUSINESS  INTERRUPTION)  HOWEVER CAUSED AND ON
- ANY  THEORY OF LIABILITY,  WHETHER  IN CONTRACT,  STRICT LIABILITY,  OR TORT
- (INCLUDING  NEGLIGENCE OR  OTHERWISE) ARISING IN  ANY WAY OUT OF THE  USE OF
- THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-
- This software  consists of voluntary contributions made  by many individuals
- on  behalf of the Apache Software  Foundation. For more  information on the
- Apache Software Foundation, please see <http://www.apache.org/>.
-
-*/
+   Unless required by applicable law or agreed to in writing, software
+   distributed under the License is distributed on an "AS IS" BASIS,
+   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+   See the License for the specific language governing permissions and
+   limitations under the License.
 
+ */
 package org.apache.batik.test.svg;
 
 import java.io.File;
@@ -67,19 +35,19 @@ import org.apache.batik.transcoder.image.ImageTranscoder;
 import org.apache.batik.transcoder.image.PNGTranscoder;
 
 /**
- * Validates the operation of <tt>SVGRenderingAccuracyTest</tt>
+ * Validates the operation of <code>SVGRenderingAccuracyTest</code>
  * by forcing specific test case situations and checking that
  * they are handled properly by the class.
  * 
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: SVGRenderingAccuracyTestValidator.java,v 1.8 2003/08/09 16:58:43 deweese Exp $
+ * @version $Id: SVGRenderingAccuracyTestValidator.java 1629243 2014-10-03 16:00:05Z vhennebert $
  */
 public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
     /**
      * Simple valid SVG content used for this test
      */
     private static final String validSVG 
-        = "<svg width=\"450\" height=\"500\" viewBox=\"0 0 450 500\"> \n" +
+        = "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"450\" height=\"500\" viewBox=\"0 0 450 500\"> \n" +
         "    <rect x=\"25\" y=\"25\" width=\"400\" height=\"450\" fill=\"blue\" /> \n" +
         "</svg>\n";
 
@@ -87,7 +55,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
      * Simple valid SVG content used for this test
      */
     private static final String validSVGVariation
-        = "<svg width=\"450\" height=\"500\" viewBox=\"0 0 450 500\"> \n" +
+        = "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"450\" height=\"500\" viewBox=\"0 0 450 500\"> \n" +
         "    <rect x=\"25\" y=\"25\" width=\"400\" height=\"450\" fill=\"#0000cc\" /> \n" +
         "</svg>\n";
 
@@ -95,7 +63,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
      * Simple valid SVG content used for this test, small size
      */
     private static final String validSmallSVG 
-        = "<svg width=\"45\" height=\"50\" viewBox=\"0 0 45 50\"> \n" +
+        = "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"45\" height=\"50\" viewBox=\"0 0 45 50\"> \n" +
         "    <rect x=\"2.5\" y=\"2.5\" width=\"40\" height=\"45\" fill=\"blue\" /> \n" +
         "</svg>\n";
 
@@ -103,7 +71,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
      * Simple valid SVG content used for this test, red rectangle
      */
     private static final String validRedSVG 
-        = "<svg width=\"450\" height=\"500\" viewBox=\"0 0 450 500\"> \n" +
+        = "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"450\" height=\"500\" viewBox=\"0 0 450 500\"> \n" +
         "    <rect x=\"25\" y=\"25\" width=\"400\" height=\"450\" fill=\"red\" /> \n" +
         "</svg>\n";
 
@@ -113,7 +81,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
      * end of the width attribute value.
      */
     private static final String invalidSVG 
-        = "<svg width=\"450 height=\"500\" viewBox=\"0 0 450 500\"> \n" +
+        = "<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"450 height=\"500\" viewBox=\"0 0 450 500\"> \n" +
         "    <rect x=\"25\" y=\"25\" width=\"400\" height=\"450\" fill=\"blue\" /> \n" +
         "</svg>\n";
 
@@ -228,10 +196,21 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
                 return r;
             }
 
-            if(!t.variationURL.toString().endsWith(expectedVariationURL)){
+            if (t.variationURLs == null
+                    || t.variationURLs.size() != 3
+                    || !t.variationURLs.get(0).toString()
+                            .endsWith(expectedVariationURL)) {
                 TestReport r = reportError(ERROR_UNEXPECTED_VARIATION_URL);
                 r.addDescriptionEntry(ENTRY_KEY_EXPECTED_VALUE, expectedVariationURL);
-                r.addDescriptionEntry(ENTRY_KEY_FOUND_VALUE, "" + t.variationURL);
+                String found;
+                if (t.variationURLs == null) {
+                    found = "null";
+                } else if (t.variationURLs.size() != 1) {
+                    found = "<list of " + t.variationURLs.size() + " URLs>";
+                } else {
+                    found = t.variationURLs.get(0).toString();
+                }
+                r.addDescriptionEntry(ENTRY_KEY_FOUND_VALUE, found);
                 return r;
             }
 
@@ -249,7 +228,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
 
 
     /**
-     * Creates an <tt>SVGRenderingAccuracyTest</tt> with an
+     * Creates an <code>SVGRenderingAccuracyTest</code> with an
      * invalid URL for the source SVG. Checks that this 
      * error is reported as a failure.
      */
@@ -275,7 +254,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
     }
 
     /**
-     * Creates an <tt>SVGRenderingAccuracyTest</tt> with a
+     * Creates an <code>SVGRenderingAccuracyTest</code> with a
      * valid URL pointing to an invalid SVG document. Checks that this 
      * error is reported as a failure.
      */
@@ -298,7 +277,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
     }
 
     /**
-     * Creates an <tt>SVGRenderingAccuracyTest</tt> with an
+     * Creates an <code>SVGRenderingAccuracyTest</code> with an
      * valid URL for the source SVG but with an invalid 
      * URL for the reference image.
      */
@@ -325,7 +304,7 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
     }
 
     /**
-     * Creates an <tt>SVGRenderingAccuracyTest</tt> with an
+     * Creates an <code>SVGRenderingAccuracyTest</code> with an
      * valid URL for the source SVG valid 
      * URL for the reference image, but the reference image,
      * but the reference image does not exist
@@ -443,12 +422,10 @@ public class SVGRenderingAccuracyTestValidator extends DefaultTestSuite {
 
             super.runImpl();            
 
-            t.setVariationURL(tmpVariationFile.toURL().toString());
+            t.addVariationURL(tmpVariationFile.toURL().toString());
             t.setSaveVariation(null);
 
-            setConfig(t,
-                      true,
-                      null);
+            setConfig(t, true, null);
 
             return super.runImpl();
         }
