@@ -27,13 +27,13 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 
+import org.apache.batik.anim.dom.SAXSVGDocumentFactory;
 import org.apache.batik.bridge.BaseScriptingEnvironment;
 import org.apache.batik.bridge.BridgeContext;
 import org.apache.batik.bridge.BridgeException;
 import org.apache.batik.bridge.GVTBuilder;
 import org.apache.batik.bridge.UserAgent;
 import org.apache.batik.bridge.UserAgentAdapter;
-import org.apache.batik.dom.svg.SAXSVGDocumentFactory;
 import org.apache.batik.test.AbstractTest;
 import org.apache.batik.test.DefaultTestReport;
 import org.apache.batik.test.TestReport;
@@ -68,7 +68,7 @@ import org.apache.batik.util.XMLResourceDescriptor;
  * &lt;/svg&gt;</code>
  *
  * @author <a href="mailto:vhardy@apache.org">Vincent Hardy</a>
- * @version $Id: SelfContainedSVGOnLoadTest.java 475477 2006-11-15 22:44:28Z cam $
+ * @version $Id: SelfContainedSVGOnLoadTest.java 1664314 2015-03-05 11:45:15Z lbernardo $
  */
 public class SelfContainedSVGOnLoadTest extends AbstractTest {
     /**
