@@ -1,10 +1,11 @@
 /*
 
-   Copyright 2001-2003  The Apache Software Foundation 
-
-   Licensed under the Apache License, Version 2.0 (the "License");
-   you may not use this file except in compliance with the License.
-   You may obtain a copy of the License at
+   Licensed to the Apache Software Foundation (ASF) under one or more
+   contributor license agreements.  See the NOTICE file distributed with
+   this work for additional information regarding copyright ownership.
+   The ASF licenses this file to You under the Apache License, Version 2.0
+   (the "License"); you may not use this file except in compliance with
+   the License.  You may obtain a copy of the License at
 
        http://www.apache.org/licenses/LICENSE-2.0
 
@@ -20,33 +21,35 @@ package org.apache.batik.transcoder.image;
 import java.awt.image.BufferedImage;
 import java.awt.image.RenderedImage;
 
+import java.io.BufferedInputStream;
+import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.ByteArrayInputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.OutputStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 
 import java.net.MalformedURLException;
 import java.net.URL;
 
+import java.util.Arrays;
 import java.util.Map;
 
-import org.apache.batik.ext.awt.image.spi.ImageTagRegistry;
-import org.apache.batik.ext.awt.image.renderable.Filter;
+import org.apache.commons.io.IOUtils;
 
-import org.apache.batik.ext.awt.image.codec.PNGImageEncoder;
-import org.apache.batik.ext.awt.image.codec.PNGEncodeParam;
+import org.apache.batik.ext.awt.image.spi.ImageTagRegistry;
+import org.apache.batik.ext.awt.image.spi.ImageWriter;
+import org.apache.batik.ext.awt.image.spi.ImageWriterRegistry;
+import org.apache.batik.ext.awt.image.renderable.Filter;
 
 import org.apache.batik.transcoder.TranscoderException;
 import org.apache.batik.transcoder.TranscoderInput;
 import org.apache.batik.transcoder.TranscoderOutput;
 
-import org.apache.batik.transcoder.image.ImageTranscoder;
-import org.apache.batik.transcoder.image.PNGTranscoder;
-
 import org.apache.batik.test.AbstractTest;
 import org.apache.batik.test.DefaultTestReport;
 import org.apache.batik.test.TestReport;
@@ -56,7 +59,7 @@ import org.apache.batik.test.svg.SVGRenderingAccuracyTest;
  * The base class for the ImageTranscoder tests.
  *
  * @author <a href="mailto:Thierry.Kormann@sophia.inria.fr">Thierry Kormann</a>
- * @version $Id: AbstractImageTranscoderTest.java,v 1.7 2004/08/18 07:17:12 vhardy Exp $
+ * @version $Id: AbstractImageTranscoderTest.java 1627832 2014-09-26 17:12:57Z vhennebert $
  */
 public abstract class AbstractImageTranscoderTest extends AbstractTest {
 
@@ -64,22 +67,22 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
      * Error when the reference image and the generated image is different.
      */
     public static final String ERROR_IMAGE_DIFFER =
-	"AbstractImageTranscoderTest.error.image.differ";
+        "AbstractImageTranscoderTest.error.image.differ";
 
     /**
      * Tag for difference image URI.
      */
     public static final String DIFFERENCE_IMAGE =
-	"AbstractImageTranscoderTest.error.difference.image";
+        "AbstractImageTranscoderTest.error.difference.image";
 
     /**
      * Error when an exception occured while transcoding.
      */
     public static final String ERROR_TRANSCODING =
-	"AbstractImageTranscoderTest.error.transcoder.exception";
+        "AbstractImageTranscoderTest.error.transcoder.exception";
 
     /**
-     * Constructs a new <tt>AbstractImageTranscoderTest</tt>.
+     * Constructs a new <code>AbstractImageTranscoderTest</code>.
      */
     public AbstractImageTranscoderTest() {
     }
@@ -102,7 +105,7 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
                 throw new IllegalArgumentException();
             }
         }
-        
+
         // url is not a file. It must be a regular URL...
         try{
             return new URL(url);
@@ -117,31 +120,31 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
      * the test's internal operation fails.
      */
     public TestReport runImpl() throws Exception {
-	report = new DefaultTestReport(this);
+        report = new DefaultTestReport(this);
 
-	try {
-	    DiffImageTranscoder transcoder = 
-		new DiffImageTranscoder(getReferenceImageData());
+        try {
+            DiffImageTranscoder transcoder =
+                new DiffImageTranscoder(getReferenceImageData());
 
-	    Map hints = createTranscodingHints();
-	    if (hints != null) {
-		transcoder.setTranscodingHints(hints);
-	    }
+            Map hints = createTranscodingHints();
+            if (hints != null) {
+                transcoder.setTranscodingHints(hints);
+            }
 
-	    TranscoderInput input = createTranscoderInput();
-	    transcoder.transcode(input, null);
-	} catch (Exception ex) {
-	    report.setErrorCode(ERROR_TRANSCODING);
-	    report.addDescriptionEntry(ERROR_TRANSCODING, toString(ex));
+            TranscoderInput input = createTranscoderInput();
+            transcoder.transcode(input, null);
+        } catch (Exception ex) {
+            report.setErrorCode(ERROR_TRANSCODING);
+            report.addDescriptionEntry(ERROR_TRANSCODING, toString(ex));
             ex.printStackTrace();
-	    report.setPassed(false);
-	}
-	
-	return report;
+            report.setPassed(false);
+        }
+
+        return report;
     }
 
     /**
-     * Creates the <tt>TranscoderInput</tt>.
+     * Creates the <code>TranscoderInput</code>.
      */
     protected abstract TranscoderInput createTranscoderInput();
 
@@ -149,7 +152,7 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
      * Creates a Map that contains additional transcoding hints.
      */
     protected Map createTranscodingHints() {
-	return null;
+        return null;
     }
 
     /**
@@ -165,9 +168,9 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
      * Gives the specified exception as a string.
      */
     public static String toString(Exception ex) {
-	StringWriter trace = new StringWriter();
-	ex.printStackTrace(new PrintWriter(trace));
-	return trace.toString();
+        StringWriter trace = new StringWriter();
+        ex.printStackTrace(new PrintWriter(trace));
+        return trace.toString();
     }
 
     static String filename;
@@ -219,45 +222,48 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
      */
     protected class DiffImageTranscoder extends ImageTranscoder {
 
-	/** The result of the image comparaison. */
-	protected boolean state;
+        /** The result of the image comparaison. */
+        protected boolean state;
 
-	/** The reference image. */
-	protected byte [] refImgData;
+        /** The reference image. */
+        protected byte [] refImgData;
 
-	/**
-	 * Constructs a new <tt>DiffImageTranscoder</tt>.
-	 *
-	 * @param refImg the reference image
-	 * @param report the test report into which errors have been sent
-	 */
-	public DiffImageTranscoder(byte [] refImgData) {
-	    this.refImgData = refImgData;
-	}
+        /**
+         * Constructs a new <code>DiffImageTranscoder</code>.
+         *
+         * @param refImgData the reference image data
+         */
+        public DiffImageTranscoder(byte [] refImgData) {
+            this.refImgData = refImgData;
+        }
 
-	/**
-	 * Creates a new image with the specified dimension.
-	 * @param w the image width in pixels
-	 * @param h the image height in pixels
-	 */
-	public BufferedImage createImage(int w, int h) {
-	    return new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
-	}
+        /**
+         * Creates a new image with the specified dimension.
+         * @param w the image width in pixels
+         * @param h the image height in pixels
+         */
+        public BufferedImage createImage(int w, int h) {
+            return new BufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
+        }
 
-	/**
-	 * Compares the specified image with the reference image and set the
-	 * state flag.
-	 *
-	 * @param img the image to write
-	 * @param output the output (ignored)
-	 * @param TranscoderException if an error occured while storing the
-	 * image 
-	 */
-	public void writeImage(BufferedImage img, TranscoderOutput output)
-	    throws TranscoderException {
+        /**
+         * Compares the specified image with the reference image and set the
+         * state flag.
+         *
+         * @param img the image to write
+         * @param output the output (ignored)
+         * @throw TranscoderException if an error occured while storing the
+         * image
+         */
+        public void writeImage(BufferedImage img, TranscoderOutput output)
+            throws TranscoderException {
 
-	    compareImage(img);
-	}
+            try {
+                compareImage(img);
+            } catch (IOException e) {
+                throw new RuntimeException(e);
+            }
+        }
 
         protected void writeCandidateReference(byte [] imgData) {
             try {
@@ -272,33 +278,24 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
             return;
         }
 
-        protected void writeCandidateVariation(byte [] imgData, byte [] refData)
-        {
-            writeCandidateReference(imgData);
+        protected void writeCandidateVariation(byte[] diff) {
             try {
-                BufferedImage ref = getImage(new ByteArrayInputStream(refData));
-                BufferedImage img = getImage(new ByteArrayInputStream(imgData));
-                BufferedImage diff = 
-                    SVGRenderingAccuracyTest.buildDiffImage(ref, img);
                 String s = new File(filename).getName();
                 s = ("test-references/org/apache/batik/transcoder/image/"+
                      "candidate-variation/"+s);
-                PNGImageEncoder encoder 
-                    = new PNGImageEncoder
-                    (new FileOutputStream(s),
-                     PNGEncodeParam.getDefaultEncodeParam(diff));
-                encoder.encode(diff);
+                OutputStream out = new FileOutputStream(s);
+                IOUtils.copy(new ByteArrayInputStream(diff), out);
                 report.addDescriptionEntry(DIFFERENCE_IMAGE,new File(s));
             } catch (Exception e) { }
         }
 
-	/**
-	 * Compares both source and result images and set the state flag.
-	 */
-	protected void compareImage(BufferedImage img) 
-            throws TranscoderException {
-	    // compare the resulting image with the reference image
-	    // state = true if refImg is the same than img
+        /**
+         * Compares both source and result images and set the state flag.
+         */
+        protected void compareImage(BufferedImage img)
+            throws TranscoderException, IOException {
+            // compare the resulting image with the reference image
+            // state = true if refImg is the same than img
 
             ByteArrayOutputStream out = new ByteArrayOutputStream();
             TranscoderOutput output = new TranscoderOutput(out);
@@ -314,38 +311,50 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
                 state = false;
                 return;
             }
-            
-            if (refImgData.length != imgData.length) {
-                report.setErrorCode(ERROR_IMAGE_DIFFER);
-                report.addDescriptionEntry(ERROR_IMAGE_DIFFER, "");
-                report.setPassed(false);
-                writeCandidateVariation(imgData, refImgData);
-                return;
-            }
 
-            for (int i = 0; i < refImgData.length; ++i) {
-                if (refImgData[i] != imgData[i]) {
+            if (!Arrays.equals(refImgData, imgData)) {
+                byte[] actualDiff = createDiffImage(img);
+                File acceptedDiffFile = new File(
+                        "test-references/org/apache/batik/transcoder/image/accepted-variation/",
+                        new File(filename).getName());
+                if (!(acceptedDiffFile.exists() && dataFromFileEqual(acceptedDiffFile, actualDiff))) {
                     report.setErrorCode(ERROR_IMAGE_DIFFER);
                     report.addDescriptionEntry(ERROR_IMAGE_DIFFER, "");
                     report.setPassed(false);
-                    writeCandidateVariation(imgData, refImgData);
+                    writeCandidateReference(imgData);
+                    writeCandidateVariation(actualDiff);
                     return;
                 }
             }
-            
-	    state = true;
-	}
 
-	/**
-	 * Returns true if the reference image is the same than the generated
-	 * image, false otherwise.  
-	 */
-	public boolean isIdentical() {
-	    return state;
-	}
+            state = true;
+        }
+
+        private byte[] createDiffImage(BufferedImage actualImage) throws IOException {
+            BufferedImage referenceImage = getImage(new ByteArrayInputStream(refImgData));
+            BufferedImage diffImage = SVGRenderingAccuracyTest.buildDiffImage(referenceImage, actualImage);
+            ImageWriter writer = ImageWriterRegistry.getInstance().getWriterFor("image/png");
+            ByteArrayOutputStream diff = new ByteArrayOutputStream();
+            writer.writeImage(diffImage, diff);
+            return diff.toByteArray();
+        }
+
+        private boolean dataFromFileEqual(File file, byte[] data) throws IOException {
+            ByteArrayOutputStream out = new ByteArrayOutputStream();
+            IOUtils.copy(new BufferedInputStream(new FileInputStream(file)), out);
+            return Arrays.equals(out.toByteArray(), data);
+        }
+
+        /**
+         * Returns true if the reference image is the same than the generated
+         * image, false otherwise.
+         */
+        public boolean isIdentical() {
+            return state;
+        }
     }
 
-    protected BufferedImage getImage(InputStream is) 
+    protected BufferedImage getImage(InputStream is)
         throws IOException {
         ImageTagRegistry reg = ImageTagRegistry.getRegistry();
         Filter filt = reg.readStream(is);
@@ -355,7 +364,7 @@ public abstract class AbstractImageTranscoderTest extends AbstractTest {
         RenderedImage red = filt.createDefaultRendering();
         if(red == null)
             throw new IOException("Couldn't render Stream");
-        
+
         BufferedImage img = new BufferedImage(red.getWidth(),
                                               red.getHeight(),
                                               BufferedImage.TYPE_INT_ARGB);
